/*
 * DistortionPerformanceProcessor.h
 *
 *  Created on: October 2nd, 2013
 *      Author: amunnich
 */

#ifndef DISTORTIONPERFROMANCEPROSECCOR_H
#define DISTORTIONPERFROMANCEPROSECCOR_H

#include <marlin/Processor.h>
#include <lcio.h>

#include "ModuleDistortionCalculator.h"

// Aida
#ifdef MARLIN_USE_AIDA
//AIDA
#include <marlin/AIDAProcessor.h>
#endif

//GEAR
#include "gear/TPCParameters.h"

#include "TH1.h"
#include "TNtuple.h"


namespace marlintpc
{
  /**
   * Marlin processor for distortion analysis
   * Computes the distortion along a row for all hits on input tracks.
   * A histogram for each row is created an evaluated. Results are saved through the Aida processor.
   */
  class DistortionPerformanceProcessor : public marlin::Processor
  {
  public:

    virtual Processor*  newProcessor() { return new DistortionPerformanceProcessor; }
    
    DistortionPerformanceProcessor();
    
    virtual void init();
    
    virtual void processRunHeader(lcio::LCRunHeader* run);
    
    virtual void processEvent(lcio::LCEvent* evt);
    
    virtual void check(lcio::LCEvent* evt);
    
    virtual void end();
    
    
  protected:
    std::string _inputColName; ///< Name of the input collection with tracks


  private:
    
    ModuleDistortionCalculator * _myModuleDistortionCalculator;
    
    /**
     * Histograms containing distances for each module and row
     */
    std::map< std::pair<int,int>, TH1*> _HistogramsRow;
    std::map< std::pair<int,int>, TH1*> _HistogramsZ;
 
	TNtuple * _xyDistortionsTuple;
	TNtuple * _zDistortionsTuple;

   /**
     * Maps containing the results of the fitted histogram
     */
    std::map< std::pair<int,int>, std::pair<double,double> >  _ResultsRow;
    std::map< std::pair<int,int>, std::pair<double,double> >  _ResultsZ;
 
    const gear::TPCParameters* _GearParameters;

    //count failed intersections
    int _failedIntersections;
    int _intersectionCalls;
  };
}

#endif //DISTORTIONPERFORMANCEPROSECCOR_H
