#ifndef GEMPULSINGCALIBRATIONPROCESSOR_H
#define GEMPULSINGCALIBRATIONPROCESSOR_H

// Marlin
#include <marlin/ConditionsProcessor.h>
#include <marlin/Processor.h>
#include "marlin/Exceptions.h"
#include "marlin/Global.h"
#include <lcio.h>
#include <string>

//ROOT
#include "TH1F.h"
#include "TH2F.h"
#include "TMath.h"

//GEAR
#include "gear/TPCParameters.h"

#include "ChannelMappingProcessor.h"

// LCIO
#include "lcio.h"
#include <EVENT/LCCollection.h>
#include <IMPL/TrackerPulseImpl.h>
#include <IMPL/LCCollectionVec.h>
#include <IMPL/LCFlagImpl.h>
#include <UTIL/LCFixedObject.h>

#include "LCObjectCopier.h"

// MarlinTPC
#include "TPCCondDataExeptions.h"
#include "ADCChannelMapping.h"
#include "ChannelMappingListener.h"

// Aida
#ifdef MARLIN_USE_AIDA
//AIDA
#include <marlin/AIDAProcessor.h>
#endif


// C++
#include <iostream>
#include <fstream>
#include <string>

using namespace lcio ;
using namespace marlin ;
using namespace tpcconddata;
using namespace std;

namespace marlintpc
{
///  Short description of processor.

/**
 * description: GEMPulsingCalibrationProcessor calibrates the time of pulses with a textfile which is generated in the GEMPulsingTimeEventAnalysisProcessor and includes calculatet deviationtimes. This processor changes the time of the pulses.
 * @author Lisa Waldmueller, DESY
 *
 * \par Credits:
 * The processor skeleton was generated by the script \c createProcessor.py
 *
 */

class GEMPulsingCalibrationProcessor : public marlin::Processor
{
 public:
  
  virtual Processor*  newProcessor() { return new GEMPulsingCalibrationProcessor; }
  
  GEMPulsingCalibrationProcessor();
  
  virtual void init();
  
  virtual void processRunHeader(lcio::LCRunHeader* run);
  
  virtual void processEvent(lcio::LCEvent* evt);
  
  virtual void check(lcio::LCEvent* evt);
  
  virtual void end();
  

  
 	 protected:
		  /* the place for protected and private member data and functions */

		  /// the name of the input collection
		  std::string _inputColName ;
		  /// the name of the output collection
		  std::string _outputColName ;
		  /// the name of the ChannelMapping collection
//		  std::string _channelMappingColName ;

		  /// set the persistency flag of the output collection
		  bool _outputIsPersistent;

		  /** The timepan in seconds for which the last valid collection will be used if no conditions data are available via LCCD.
		   */
//		  int _missingConditionsDataOverrideTimespan;
//		  bool _removeChannelsWithoutMapping;

		  std::map< std::pair<int,int>, std::pair<float,float> > _timeDeviationMap;
		  std::map< std::pair<int,int>, std::pair<float,float> > _chargeDeviationMap;

		  std::string _inputfilenameTime;
		  std::string _inputfilenameCharge;



 	 private:

//		  ChannelMappingListener* _ChannelMappingListener;

		  /// Internal member to hold the time for the missing conditions overide. Unit: nanoseconds.
		  lcio::long64 _missingConditionsDataOverrideLCIOTime;

		  //fg: add a map of channels that have been removed from the collection as
		  //    their hardware address is not mapped to any known software address (padID)
		  typedef std::map< std::pair< int, int > , unsigned > MAP ;
		  MAP _cMap ;



};
} // namespace marlintpc
#endif // GEMPULSINGANALYSISPROCESSOR_H
