#ifndef GEMPULSINGCHARGEEVENTANALYSISPROCESSOR_H
#define GEMPULSINGCHARGEEVENTANALYSISPROCESSOR_H

#include <marlin/Processor.h>
#include "marlin/Global.h"
#include <lcio.h>
#include <string>

//ROOT
#include "TH1F.h"
#include "TH2F.h"
#include "TMath.h"
#include "TNtuple.h"

//GEAR
#include "gear/TPCParameters.h"

// MarlinTPC
#include "ADCChannelMapping.h"
#include "ChannelMappingListener.h"

// Aida
#ifdef MARLIN_USE_AIDA
//AIDA
#include <marlin/AIDAProcessor.h>
#endif

namespace marlintpc
{
///  Short description of processor.

/**
 * description: GEMPulsingChargeEventAnalysisProcessor analyses the charge of pulses and generates a tuple which is used to calibrate the charge of each pad in GEMChargeVoltage.C.
 * @author Lisa Waldmueller, DESY
 *
 * \par Credits:
 * The processor skeleton was generated by the script \c createProcessor.py
 *
 */

class GEMPulsingChargeEventAnalysisProcessor : public marlin::Processor
{
 public:
  
  virtual Processor*  newProcessor() { return new GEMPulsingChargeEventAnalysisProcessor; }
  
  GEMPulsingChargeEventAnalysisProcessor();
  
  virtual void init();
  
  virtual void processRunHeader(lcio::LCRunHeader* run);
  
  virtual void processEvent(lcio::LCEvent* evt);
  
  virtual void check(lcio::LCEvent* evt);
  
  virtual void end();
  
  
 protected:
  /* the place for protected and private member data and functions */

	std::string _inputColName; ///< Name of the input collection
    std::string _channelMappingColName ;
	std::map< std::pair<int,int>, std::vector<float> > _inputPulseMap;
	std::map< std::pair<int,int>, std::vector<float> > _inputPulseMapTuple;
	std::map< std::pair<int,int>, double > _inputPulseMapAverage;
	std::map< std::pair<int,int>, std::vector<double> > _chargeDeviationMap;
	std::map< std::pair<int,int>, std::pair<double,double> > _chargeDeviationMapAverageRMS;

	TNtuple * _chargeCalibrationTuple;


	// key: Modul, chip, 2: int: pad, vector: value
	std::map< std::pair<int,int>, std::pair<int, std::vector<double> > > _chipMap;
	// key: Modul, cable, 2: int: pad, vector: value
	std::map< std::pair<int,int>, std::pair<int, std::vector<double> >  > _cableMap;
	std::string _filename;
	std::string _filenameCharge;
	std::vector<double> _eventChargeAverages;
	bool _createOutput;
	int _chipToChipMin;
	int _chipToChipMax;
	int _chipToChipRMS;
	int _cableToCableMin;
	int _cableToCableMax;
	int _cableToCableRMS;
	int _chipToChipDeviationRange;
	int _chipToChipDeviationRMS;
	int _cableToCableDeviationRange;
	int _cableToCableDeviationRMS;
	int _eventChargeAverageMin;
	int _eventChargeAverageMax;
	int _eventChargeAverageRMS;
	double _voltage;


 private:
	TH1F * _chipToChipChargeAverage;
	TH1F * _cableToCableChargeAverage;
	TH1F * _RMSChip_Charge;
	TH1F * _RMSCable_Charge;
	TH1F * _eventAverage;
	TH1F * _eventRMS;
	TH1F * _eventAverageTemp;
	TH1F * _eventRMSTemp;

	TH1F * _chipToChipChargeAverageDeviation;
	TH1F * _cableToCableChargeAverageDeviation;
	TH1F * _RMSChipChargeDeviation;
	TH1F * _RMSCableChargeDeviation;

	TH2F * _padToPadChargeDeviationAverageXY;
	TH2F * _padToPadChargeDeviationRMSXY;
	TH2F * _chipToChipChargeAverageXY;
	TH2F * _cableToCableChargeAverageXY;


	const gear::TPCParameters* _gearParameters;
    ChannelMappingListener* _channelMappingListener;


};
} // namespace marlintpc
#endif // GEMPULSINGANALYSISPROCESSOR_H
