#ifndef GEMPULSINGTIMEEVENTANALYSISPROCESSOR_H
#define GEMPULSINGTIMEEVENTANALYSISPROCESSOR_H

#include <marlin/Processor.h>
#include "marlin/Global.h"
#include <lcio.h>
#include <string>

//ROOT
#include "TH1F.h"
#include "TH2F.h"
#include "TMath.h"
#include "TNtuple.h"

//GEAR
#include "gear/TPCParameters.h"

// MarlinTPC
#include "ADCChannelMapping.h"
#include "ChannelMappingListener.h"

// Aida
#ifdef MARLIN_USE_AIDA
//AIDA
#include <marlin/AIDAProcessor.h>
#endif

namespace marlintpc {
///  Short description of processor.

/**
 * description: GEMPulsingTimeEventAnalysisProcessor analyses the times of pulses on eventbase. It generates one outputfile, which is used to calibrate the pulsetimes for each channel in the GEMPulsingCalibrationProcessor.
 * @author Lisa Waldmueller, DESY
 *
 * \par Credits:
 * The processor skeleton was generated by the script \c createProcessor.py
 *
 */

class GEMPulsingTimeEventAnalysisProcessor: public marlin::Processor {
public:

	virtual Processor* newProcessor() {
		return new GEMPulsingTimeEventAnalysisProcessor;
	}

	GEMPulsingTimeEventAnalysisProcessor();

	virtual void init();

	virtual void processRunHeader(lcio::LCRunHeader* run);

	virtual void processEvent(lcio::LCEvent* evt);

	virtual void check(lcio::LCEvent* evt);

	virtual void end();

protected:
	/* the place for protected and private member data and functions */

	std::string _inputColName; ///< Name of the input collection
	std::string _channelMappingColName;
	std::map<std::pair<int, int>, std::vector<float> > _inputPulseMap;
	std::map<std::pair<int, int>, double> _inputPulseMapAverage;
	std::map<std::pair<int, int>, std::vector<double> > _timeDeviationMap;
	std::map<std::pair<int, int>, std::pair<double, double> > _timeDeviationMapAverageRMS;

	// key: Modul, chip, 2: int: pad, vectorpair: value Mean, value RMS
	std::map<std::pair<int, int>, std::pair<int, std::pair< std::vector<double>, std::vector<double> > > > _chipMap;
	// key: Modul, chip, 2: int: pad, vectorpair: value Mean, value RMS
	std::map<std::pair<int, int>, std::pair<int, std::pair< std::vector<double>, std::vector<double> > > > _cableMap;
	std::string _filename;
	bool _createOutput;
	int _chipToChipMin;
	int _chipToChipMax;
	int _chipToChipRMS;
	int _cableToCableMin;
	int _cableToCableMax;
	int _cableToCableRMS;
	int _chipToChipDeviationRange;
	int _chipToChipDeviationRMS;
	int _cableToCableDeviationRange;
	int _cableToCableDeviationRMS;
	int _eventTimeAverageMin;
	int _eventTimeAverageMax;
	int _eventTimeAverageRMS;


	TNtuple * _timeCalibrationTuple;

private:
	TH1F * _chipToChipTimeAverage;
	TH1F * _cableToCableTimeAverage;
	TH1F * _RMSChipTime;
	TH1F * _RMSCableTime;
	TH1F * _eventAverage;
	TH1F * _eventRMS;
	TH1F * _eventAverageTemp;
	TH1F * _eventRMSTemp;

	TH1F * _chipToChipTimeAverageDeviation;
	TH1F * _cableToCableTimeAverageDeviation;
	TH1F * _RMSChipTimeDeviation;
	TH1F * _RMSCableTimeDeviation;

	TH2F * _padToPadTimeDeviationAverageXY;
	TH2F * _padToPadTimeDeviationRMSXY;
	TH2F * _chipToChipTimeAverageXY;
	TH2F * _cableToCableTimeAverageXY;

	const gear::TPCParameters* _gearParameters;
	ChannelMappingListener* _channelMappingListener;

};
} // namespace marlintpc
#endif // GEMPULSINGANALYSISPROCESSOR_H
