#ifndef NOISEHITANDPULSECOUNTERPROCESSOR_H
#define NOISEHITANDPULSECOUNTERPROCESSOR_H

#include <marlin/Processor.h>
#include <lcio.h>
#include <string>

//GEAR
#include "gear/TPCParameters.h"

// Aida
#ifdef MARLIN_USE_AIDA
//AIDA
#include <marlin/AIDAProcessor.h>
#endif

//ROOT
#include "TH1.h"
#include "TH1I.h"
#include "TH2I.h"

namespace marlintpc
{
///  This processor counts hits and pulses not included in tracks

/**
 * This processor counts the number of hits and pulses, which are not included in tracks and separately also those included in tracks for comparison. It also maps the location of noise pulses.
 * @author P. Malek, Desy
 *
 * \par Credits:
 * The processor skeleton was generated by the script \c createProcessor.py
 *
 */

class NoiseHitAndPulseCounterProcessor : public marlin::Processor
{
 public:
  
  virtual Processor*  newProcessor() { return new NoiseHitAndPulseCounterProcessor; }
  
  NoiseHitAndPulseCounterProcessor();
  
  virtual void init();
  
  virtual void processRunHeader(lcio::LCRunHeader* run);
  
  virtual void processEvent(lcio::LCEvent* evt);
  
  virtual void check(lcio::LCEvent* evt);
  
  virtual void end();
  
 protected:
  /* the place for protected and private member data and functions */
  
  virtual void _getMeanRMS90(TH1* hist, double retVal[]);

  std::string _inputTrackColName; ///< Name of the input track collection
  std::string _inputHitColName; ///< Name of the input hit collection
  std::string _inputPulseColName; ///< Name of the input pulse collection

  int _noiseHitsHistoMax;
  int _noisePulsesHistoMax;
  int _notInHitPulsesHistoMax;
  int _trackPulsesHistoMax;

  TH1I* _noiseHitsHisto;

  TH1I* _noisePulsesHisto;

  TH1I* _notInHitPulsesHisto;

  TH1I* _noiseHitsNormHisto;

  TH1I* _noisePulsesNormHisto;

  TH1I* _notInHitPulsesNormHisto;

  TH1I* _trackHitsHisto;

  TH1I* _trackPulsesHisto;

  TH1I* _trackHitsNormHisto;

  TH1I* _trackPulsesNormHisto;

  TH2I* _noisePulsesDistribution;

  TH2I* _notInHitPulsesDistribution;

  const gear::TPCParameters* _gearParameters;

};
} // namespace marlintpc
#endif // NOISEHITANDPULSECOUNTERPROCESSOR_H
