#ifndef PRFPERFORMANCEPROCESSOR_H
#define PRFPERFORMANCEPROCESSOR_H

#include <marlin/Processor.h>
#include <lcio.h>
#include <string>

//GEAR
#include "gear/TPCParameters.h"

#include "TH2.h"
#include "TNtuple.h"
// Aida
#ifdef MARLIN_USE_AIDA
//AIDA
#include <marlin/AIDAProcessor.h>
#endif


namespace marlintpc
{
/**  Marlin processor for PRF analysis.
 * Computes the PRF of the hits on a track.
 * A histogram for each row is created an evaluated. Results are saved through the Aida processor.
 * 
 * @author F. Mueller (DESY)
 *
 */

class PRFPerformanceProcessor : public marlin::Processor
{
public:

	virtual Processor*  newProcessor() { return new PRFPerformanceProcessor; }

	PRFPerformanceProcessor();

	virtual void init();

	virtual void processRunHeader(lcio::LCRunHeader* run);

	virtual void processEvent(lcio::LCEvent* evt);

	virtual void check(lcio::LCEvent* evt);

	virtual void end();


protected:
	/* the place for protected and private member data and functions */
	std::string _inputColName; ///< Name of the input collection


private:


	/**
	 * Histograms containing PRF for each module and row
	 */
	std::map< std::pair<int,int>, TH2*> _histogramsRow;
	TH2 * _prfAll;  ///Histogram containing the PRF for all modules and rows

	const gear::TPCParameters* _gearParameters;

	TNtuple * _singleRowsTuple;
	TNtuple * _bestRowTuple;
	TNtuple * _allRowsTuple;
	TNtuple * _singleRowsTupleCorrected;
	TNtuple * _bestRowTupleCorrected;



	int _bestModule; ///< Module with the best row
	int _bestRow; ///< Row with the best resolution

	/**
	 * Maps containing the results of the fitted histogram
	 */
	std::map< std::pair<int,int>, std::pair<double,double> >  _resultsRow;



};
} // namespace marlintpc
#endif // PRFPERFORMANCEPROCESSOR_H
