#ifndef PER_PAD_PULSE_CHARGE_PROCESSOR_H
#define PER_PAD_PULSE_CHARGE_PROCESSOR_H 1

//LCIO
#include <lcio.h>

//MARLIN
#include "marlin/Processor.h"

namespace AIDA
{
	class IHistogram1D;
}



namespace marlintpc
{
	/** Fill charge per pulse into AIDA histograms for each channel.
	*  <h4>Input - Prerequisites</h4>
	*  \li A collection of non-mapped pulses.
	*
	*  <h4>Output</h4>
	*  \li Histograms with charge per pulse for each channel.
	*
	*  @param InputPulses Name of input TrackerPulses collection (default: TPCPulses)
	*  @param MinCharge Minimal charge value in histogram (default:  0)
	*  @param MaxCharge Maximal charge value in histogram (default:  300)
	*  @param Bins Number of bins in histogram (default:  300)
	*  @param Channels Number of channels (default: 96)
	*
	*  @author Oliver Sch&auml;fer, University of Rostock / DESY-FLC
	*  @version $Id:$
	*  @version $Rev: 3907 $
	*/

	class PerPadPulseChargeHistogrammerProcessor : public marlin::Processor
	{
		
		public:
		
			virtual Processor*  newProcessor()
			{
				return new PerPadPulseChargeHistogrammerProcessor;
			}

			/// The constructor. Parameters will be registered here.
			PerPadPulseChargeHistogrammerProcessor();

			/// The Destructor.
			~PerPadPulseChargeHistogrammerProcessor();

			/// Processor initialisation. Histograms are prepared here.
			virtual void init();

			/// RunHeader processing. Revision and parameter logging.
			virtual void processRunHeader(lcio::LCRunHeader* run);

			/// Event processing. The working horse, histograms are filled here.
			virtual void processEvent(lcio::LCEvent * evt);

		protected:
		
			std::string  _pulsesColName; ///< The name of the input collection of TrackerPulses.

			std::vector<AIDA::IHistogram1D *>  _chargePerPadHistos; ///< Vector of histograms of pulse-charges.

			int  _nBins; ///< Number of bins.
			int  _nChannels; ///< Number of channels. It corresponds to the number of generated histograms.
			float  _maxCharge; ///< Maximum charge in histogram.
			float  _minCharge; ///< Minimum charge in histogram.
	};
}// namespace

#endif
