#ifndef PULSE_COUNTER_PROCESSOR_H
#define PULSE_COUNTER_PROCESSOR_H 1

//LCIO
#include <lcio.h>

//MARLIN
#include "marlin/Processor.h"

namespace AIDA
{
	class IHistogram1D;
}

namespace marlintpc
{

	/** Fills number of pulses per channel into AIDA histogram.
	 *
	 *  <h4>Input - Prerequisites</h4>
	 *  \li A collection of unmapped pulses <br>
	 *
	 *  <h4>Output</h4>
	 *  \li Histogram with pulses per channel
	 *
	 *  @param InputPulses Name of collection of unmapped input TrackerPulses (default: TPCPulses)
	 *  @param Channels  Number of channels (default: 128 ), the histogram has zero as lowest channel number.
	 *
	*  @author Oliver Sch&auml;fer, University of Rostock
	*  @version $Id:$
	*  @version Revision: $Rev: 3907 $
	 */

	class PulseCounterProcessor : public marlin::Processor
	{

		public:

			virtual Processor*  newProcessor()
			{
				return new PulseCounterProcessor ;
			}

			/// The constructor. Parameters will be registered here.
			PulseCounterProcessor();

			/// The Destructor.
			~PulseCounterProcessor();

			/// Processor initialisation. Histogram is prepared here.
			virtual void init();

			/// RunHeader processing. Revision and parameter logging.
			virtual void processRunHeader(lcio::LCRunHeader* run) ;

			/// Event processing. The working horse, histogram is filled here.
			virtual void processEvent(lcio::LCEvent * evt) ;

		protected:

			std::string _pulsesColName ;    ///< The name of the input pulses collection.

			AIDA::IHistogram1D * _pulsesPerChannelHisto; ///< Histogram for pulses per channel.

			int _nChannels; ///< Number of channels displayed in histogram.

	};
}// namespace

#endif // ends definition of HIT_TRACK_CHARGE_PROCESSOR_H
