#ifndef PULSESHAPEPROCESSOR_H
#define PULSESHAPEPROCESSOR_H

#include <marlin/Processor.h>
#include <lcio.h>
#include <string>
#include <utility>
#include <cstdlib>
#include <map>

//GEAR
#include "gear/TPCParameters.h"

// Aida
#ifdef MARLIN_USE_AIDA
//AIDA
#include <marlin/AIDAProcessor.h>
#endif

//ROOT
#include "TH2.h"

namespace marlintpc
{
/**
 * This processor creates average pulse shapes for the maximum pulse in a hit and its neighbors.
 * @author P. Malek, DESY
 *
 * \par Credits:
 * The processor skeleton was generated by the script \c createProcessor.py
 *
 */

class PulseShapeProcessor : public marlin::Processor
{
 public:
  
  virtual Processor*  newProcessor() { return new PulseShapeProcessor; }
  
  PulseShapeProcessor();
  
  virtual void init();
  
  virtual void processRunHeader(lcio::LCRunHeader* run);
  
  virtual void processEvent(lcio::LCEvent* evt);
  
  virtual void check(lcio::LCEvent* evt);
  
  virtual void end();
  
  
 protected:
  /* the place for protected and private member data and functions */
  std::string _inputColName; ///< Name of the input collection

  float _normPulseHistoMax;///< maximum relative pulse height value of the normalized pulse histograms
  bool _usePolynomialInFit;///< determines if the polynomial is used in the gamma4 fit to approximate the underground

  int _nNeighborsUsed;///< number of neighbors used in _normPulseNeighbors on each side of the max
  bool _useOnlyHighQualityPulses;//< flag to set use of only high quality pulses in average pulse shapes

  TH2* _pulseHighestHisto2D;///< scatterplot to show the distribution of shapes of the maximum pulses
  TH2* _pulseNeighborsHisto2D;///< scatterplot to show the distribution of shapes of the neighboring pulses
  TH2* _normPulseHighestHisto2D;///< scatterplot to show the distribution of shapes of the normalized  maximum pulses
  TH2* _normPulseNeighborsHisto2D;///< scatterplot to show the distribution of shapes of the normalized neighboring pulses

  std::map<int, TH2*> _pulseHighestHistoVSCharge;///< scatterplots to show the average pulse shape for different charge intervals

};
} // namespace marlintpc
#endif // PULSESHAPEPROCESSOR_H
