#ifndef TIMEPIX_CLUSTER_SIZE_PROCESSOR_H
#define TIMEPIX_CLUSTER_SIZE_PROCESSOR_H 1

//LCIO
#include <lcio.h>

//MARLIN
#include "marlin/Processor.h"
#include "TimePixPixelMode.h"
#include "TimePixMapListener.h"

//Root
#include "TH2F.h"

namespace AIDA{
  class IHistogram1D;
  class IHistogram2D;
}


namespace marlintpc
{

    class TrackFitterFactory;

    /** Fill the number of pixels in a cluster and the cluster radius in a histogram
     *
     *  <h4>Input - Prerequisites</h4>
     *  \li A Collection of reconstructed hits
     *
     *  <h4>Output</h4>
     *  \li Histograms of the x, y and z position distributions
     *
     *  @param InputTrackes Name of input Hits collection (default: TPCHits)
     *  @param HistogramMinValuePixel  Minimum value in Pixel histogram (default: 0.)
     *  @param HistogramMaxValuePixel  Maximal value in Pixel histogram (default: 1000.) 
     *  @param HistogramMinValueRadius  Minimum value in Radius histogram (default: 0. mm)
     *  @param HistogramMaxValueRadius  Maximal value in Radius histogram (default: 15. mm)
     *
     *
     *  @author Simone Zimmermann, University of Bonn
     */
    class TimePixClusterSizeProcessor : public marlin::Processor{
			      
    public:
    
	virtual Processor*  newProcessor() { return new TimePixClusterSizeProcessor ; }

	TimePixClusterSizeProcessor();
    
	~TimePixClusterSizeProcessor();

	virtual void init();

	virtual void processRunHeader(lcio::LCRunHeader* run ) ;

	virtual void processEvent(lcio::LCEvent * evt ) ; 
    
    protected:

     std::string _inputColName ;    ///< the name of the input collection
     
     AIDA::IHistogram1D * _distributionPixelHisto; ///< The histogram of the clustersize in pixel
     AIDA::IHistogram1D * _distributionRadiusHisto; ///< The histogram of the clusterradius
     std::vector<AIDA::IHistogram1D *> _distributionPixelHistoVec;
     std::vector<AIDA::IHistogram1D *> _distributionRadiusHistoVec;

     TH2F * _clusterCharge_vs_clusterSizePlot;
     tpcconddata::TimePixMap * _timePixMap;
     
     float  _histogramMinValuePixel; ///< Minimum value in pixel histogram
     float  _histogramMaxValuePixel; ///< Maximum value in pixel histogram
     float  _histogramMinValueRadius; ///< Minimum value in radius histogram
     float  _histogramMaxValueRadius; ///< Maximum value in radius histogram

     float _pitch; ///< pitch of the pixel

     int _zDistributionBinning; ///< Binning of z distribution of histograms
     float _maxValueZ; ///< Minimum value in z binning
     float _minValueZ; ///< Maximal value in z binning
     bool _unseparatedCluster; ///< True if only unseparated clusters should be checked

    };

}

#endif // TOT_DISTRIBUTION_PROCESSOR_H
