#ifndef TRIPLETCALCULATIONTOOL
#define TRIPLETCALCULATIONTOOL

#include <utility>
#include <vector>
#include "EVENT/TrackerHit.h"
#include "gear/TPCModule.h"
#include "gear/TPCParameters.h"

#include "TVector3.h"

namespace marlintpc
{
/**
 * A collection of tools to evaluate the width of a distribution in a histogram.
 * Available methods:
 * 1. RMS90
 * 2. Gaussian fit within range of 3 sigma
 * 3. rms after 5 sigma iterative cut of tails
 */

class TripletCalculationTool
{
public:
	/**Constructor with histogram to be evaluated and chosen method.
	 * In case of method=0 all methods are evaluated.
	 * method=1: RMS90
	 * method=2: Gaussian fit within range of 3 sigma
	 * method 3: rms after 5 sigma iterative cut of tails
	 */
	TripletCalculationTool(const gear::TPCParameters * tpcParameters, EVENT::TrackerHit * currentHit, EVENT::TrackerHit * previousHit, EVENT::TrackerHit * nextHit, int method=0);


	///destructor
	virtual ~TripletCalculationTool();

	/**Function giving back the resolution from rms90.
	 *The factor 0.8 compensates the difference between the rms90 and a sigma
	 *of a gaussian distribution which is the value needed for a resolution.
	 */
	double getZTriplet() {return sqrt(3/2)*_tripletZ;};
	double getXYTriplet() {return sqrt(3/2)*_tripletXY;};


private:
	const gear::TPCParameters * _tpcParameters;
	EVENT::TrackerHit * _currentHit;
	EVENT::TrackerHit * _previousHit;
	EVENT::TrackerHit * _nextHit;
	double _tripletZ;
	double _tripletXY;

	void calculateRowDistance();

	void calculateXYDistance();

};


}
#endif
