#ifndef XrayLikelihoodProcessor_h
#define XrayLikelihoodProcessor_h 1

#include "GridPixXrayObject.h"
#include "XrayGeometryService.h"

//c++
#include <string>
#include <vector>
#include <sstream>
#include <limits>

//lcio
#include <lcio.h>
#include <EVENT/LCCollection.h>
#include <IMPL/LCCollectionVec.h>
#include <EVENT/LCRelation.h>
#include <IMPL/LCRelationImpl.h>
#include <EVENT/TrackerHit.h>

//marlin
#include <marlin/Processor.h>
#include <marlin/Global.h>

//root
#include "TH1D.h"
#include <TROOT.h>
#include <TFile.h>

// Aida
#include <AIDA/AIDA.h>
#include <marlin/AIDAProcessor.h>

namespace marlintpc{
  
  class XrayLikelihoodProcessor;
  
  class XrayLikelihoodProcessor : public marlin::Processor{
    
    public:
      
      virtual Processor* newProcessor() {return new XrayLikelihoodProcessor;}
      
      XrayLikelihoodProcessor();
      
      ~XrayLikelihoodProcessor();
      
      int energyBin( double energy );
      
      double logLikelihood(TH1D* histogram, double value);

      virtual void init();
      
      virtual void processRunHeader( lcio::LCRunHeader* run );
      
      virtual void processEvent( LCEvent * evt );
      
      virtual void end();
      
    protected:
      
      std::string _inputCollectionXray;
      
      std::string _outputXrays;
      
      std::string _outputCollection;
      
      int _outputIsTransient;
      
      std::vector<float> _energyBinsLowerEdges;
      std::vector<std::string> _referenceROOTFilesXray;
      std::vector<TFile*> _referenceTFilesXray;
      std::vector<float> _scaleFactors;

      int _numberEnergyBins;

      std::vector<TH1D*> _histogramsLikelihood;

      XrayGeometryService* _xrayGeoService;

      std::vector<TH1D*> _xrayExcentricity;

      std::vector<TH1D*> _xrayKurtosisL;

      std::vector<TH1D*> _xrayLength;

      std::vector<TH1D*> _xraySkewnessL;

      std::vector<TH1D*> _xrayRmsT;

      std::vector<TH1D*> _xrayLengthDivByRmsT;

      std::vector<TH1D*> _xrayFractionWithinRmsY;

  };//class XrayLikelihoodProcessor
  
}//namespace marlintpc

#endif
