#include <vector>
#include <iostream>
#include "TLatex.h"
#include "TCanvas.h"
#include "TH1.h"
#include "TGraphErrors.h"
#include "TMultiGraph.h"
#include "TColor.h"
#include "TFile.h"
#include "TMath.h"
#include "TFrame.h"


#include "TNtuple.h"
#include "TStyle.h"
#include "TROOT.h"
#include "sstream"
#include <cstdarg>

/*How to call this:
.L $MARLINTPC/analysis/rootscripts/DistortionPerformance.C++
DistortionPerformance("path for output","path to the aida file", number of runs in the following list, list of the runs)
e.g DistortionPerformance("../myplots","../AidaFiles/",3,18850,18851,18860)
ideally run umber are given already ordered in z for color coding to work
*/
void DistortionPerformance(char * pathToOutput, char * pathToAida, int nRuns, ...){

  //loading the style
  gROOT->ProcessLine(".L ${MARLINTPC}/analysis/rootscripts/LCTPCStyle.C");
  gROOT->ProcessLine("LCTPCStyle()");

  //initialize the input list
  va_list argList;
  va_start(argList,nRuns);

 TFile *outfile=new TFile(Form("%s%s",pathToOutput,"DistortionPerformance.root"),"RECREATE");


  int runNumber;

  // will contain the results from the processors
  TMultiGraph *runDistZGraph=new TMultiGraph();
  TMultiGraph *runDistRPhiGraph=new TMultiGraph();
  runDistZGraph->SetTitle("z Distortions;row radius [mm];distortion in z [mm] ");
  runDistRPhiGraph->SetTitle("rphi Distortions;row radius [mm];distortion in r#varphi [mm]");

  std::vector<double> runZPositions;
  	std::vector<double> runZPositionsError;


  	std::vector<double> runBestXYRes;
  	std::vector<double> runBestXYResError;

  	std::vector<double> runBestZRes;
  	std::vector<double> runBestZResError;

	std::vector<double> runSingleZResMean;
	std::vector<double> runSingleZResMeanError;
	std::vector<double> runSingleXYResMean;
	std::vector<double> runSingleXYResMeanError;

  //create colors
  const int BINNING=nRuns;
  Int_t colors[BINNING];

  const Int_t NRGBs = 5;

  Double_t stops[NRGBs] = { 0.00, 0.34, 0.61, 0.84, 1.00 };
  Double_t red[NRGBs]   = { 0.00, 0.00, 0.87, 1.00, 0.51 };
  Double_t green[NRGBs] = { 0.00, 0.81, 1.00, 0.20, 0.00 };
  Double_t blue[NRGBs]  = { 0.51, 1.00, 0.12, 0.00, 0.00 };
  Int_t FI = TColor::CreateGradientColorTable(NRGBs, stops, red, green, blue, BINNING);

  for (int i=0;i<BINNING;i++)
    colors[i] = FI+i;


  //loop over the given runs
  for (int i=0;i<nRuns;i++)
    {
      //get run number from input
      runNumber=va_arg(argList,int);

      //create stringstream containing the aida file name
      std::stringstream runNumberString;
      std::cout<<"run number: "<<runNumber<<std::endl;
      runNumberString <<pathToAida<<"aida_run" <<runNumber<< ".root";

      //open the aida file
      TFile *file  = TFile::Open(runNumberString.str().c_str(),"Read");
      //check that the file exists
      if (file==0)
	{
	  std::cout<<"This run will be skipped. The aida file does not exist. Please check this!"<<std::endl;
	  continue;
	}

      //get the tuples from the file
      TNtuple * zPosTuple=(TNtuple*)file->Get("MyAnalysisCutSelectorProcessor/RunZPositions");
      TNtuple * xyRowTuple=(TNtuple*)file->Get("MyDistortionPerformanceProcessor/xy distortions");
      TNtuple * zRowTuple=(TNtuple*)file->Get("MyDistortionPerformanceProcessor/z distortions");

      if (zPosTuple==0 ||zRowTuple==0 || xyRowTuple==0)
	{
	  std::cout<<"This run will be skipped. One of the necessary root tuples does not exist. Please check that you ran the corresponding processors and that the tuple is in the aida file!"<<std::endl;
	  continue;
	}
      //open the first and only entry of the tuples
      zPosTuple->GetEntry(0);

      std::vector<double> xy,z,wxy,wz;
      		double sumwxy=0;
      		double sumwz=0;

      		int nRows = xyRowTuple->GetEntries();
      		for (int RowIter=0;RowIter<nRows;RowIter++)
      		{
      			xyRowTuple->GetEntry(RowIter);
      			zRowTuple->GetEntry(RowIter);

    			wxy.push_back(1/(xyRowTuple->GetArgs()[5]*xyRowTuple->GetArgs()[5]));
    			wz.push_back(1/(zRowTuple->GetArgs()[5]*zRowTuple->GetArgs()[5]));
    			xy.push_back(xyRowTuple->GetArgs()[4]);
    			z.push_back(zRowTuple->GetArgs()[4]);

    			if(xyRowTuple->GetArgs()[0]==3 && xyRowTuple->GetArgs()[1]==17)
    			{
    				runBestXYRes.push_back(xyRowTuple->GetArgs()[4]);
    				runBestXYResError.push_back(xyRowTuple->GetArgs()[5]);
      				runBestZRes.push_back(zRowTuple->GetArgs()[4]);
        				runBestZResError.push_back(zRowTuple->GetArgs()[5]);

    			}

    			sumwxy+=wxy[RowIter];
    			sumwz+=wz[RowIter];

      		}

      		runSingleXYResMean.push_back(TMath::Mean(xy.size(),&xy[0],&wxy[0]));
      				runSingleZResMean.push_back(TMath::Mean(xy.size(),&z[0],&wz[0]));

      				runSingleXYResMeanError.push_back(sqrt(1/sumwxy));
      				runSingleZResMeanError.push_back(sqrt(1/sumwz));
      //get the graphs from the file
      TGraphErrors *g_rphi=(TGraphErrors*)file->Get("MyDistortionPerformanceProcessor/XYDistortionRow");
      TGraphErrors *g_z=(TGraphErrors*)file->Get("MyDistortionPerformanceProcessor/ZDistortionRow");
      //check that the tuples exists
      if(g_rphi==0 || g_z==0 )
	{
	  std::cout<<"This run will be skipped. One of the necessary root graphs does not exist. Please check that you ran the corresponding processors and that the graph is in the aida file!"<<std::endl;
	  continue;
	}

      //fill the vectors with the values from the tuples ([0]=mean,[2]=rms)
      //567.6-hitposition is the conversion into drift distance
      int zpos= int(567.6-zPosTuple->GetArgs()[0]) ;

      runZPositions.push_back(567.6-zPosTuple->GetArgs()[0]);
      runZPositionsError.push_back(zPosTuple->GetArgs()[2]);
      std::ostringstream legEntry;
      //legEntry << "z = "<<zpos<<" [mm]";
      legEntry << "z_{d} = "<<zpos;
      //assign color
      g_z->SetLineColor(colors[i]);
      g_z->SetMarkerColor(colors[i]);
      g_z->SetTitle(legEntry.str().c_str());
      g_rphi->SetLineColor(colors[i]);
      g_rphi->SetMarkerColor(colors[i]);
      g_rphi->SetTitle(legEntry.str().c_str());


      runDistZGraph->Add(g_z,"P");
      runDistRPhiGraph->Add(g_rphi,"P");
      
      file->Close();
      delete file;

    }

  va_end(argList);

  //save the results
  outfile->cd();
  gStyle->SetPadRightMargin(0.18);

  TCanvas *c1 = new TCanvas("Distortion rphi","Distortion rphi");
  runDistRPhiGraph->Draw("A");
  runDistRPhiGraph->SetName("distortion rphi");
  runDistRPhiGraph->Write();
  gPad->BuildLegend(0.85,0.2,0.99,0.99,"[mm]");
  c1->SaveAs(Form("%s%s",pathToOutput,"DistortionPerformance_rphi.png"));
  c1->SaveAs(Form("%s%s",pathToOutput,"DistortionPerformance_rphi.pdf"));
//  c1->Write(); //Peter -- Saving the canvas


  TCanvas *c2 = new TCanvas("Distortion z","Distortion z");
  runDistZGraph->Draw("A");
  gPad->BuildLegend(0.85,0.2,0.99,0.99,"[mm]");
  c2->SaveAs(Form("%s%s",pathToOutput,"DistortionPerformance_z.png"));
  c2->SaveAs(Form("%s%s",pathToOutput,"DistortionPerformance_z.pdf"));
  runDistZGraph->SetName("distortion z");
  runDistZGraph->Write();
//  delete c1;  //Keep these for now. --Peter
//  delete c2;

  std::pair<int,int> bestRow=std::make_pair(3,17);


  TCanvas *c3 = new TCanvas("Resolution rphi best row","Resolution rphi best row");
  TGraphErrors *runBestXYResGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runBestXYRes[0], &runZPositionsError[0], &runBestXYResError[0] );
  runBestXYResGraph->SetTitle(Form("module_%i row_%i",bestRow.first,bestRow.second));
  runBestXYResGraph->GetXaxis()->SetTitle("drift distance [mm]");
  runBestXYResGraph->GetYaxis()->SetTitle("#sigma_{r#varphi} [mm]");
  runBestXYResGraph->SetLineColor(colors[0]);
  runBestXYResGraph->SetFillColor(kWhite);
  runBestXYResGraph->Draw("AP");
  gPad->Update();
  runBestXYResGraph->GetYaxis()->SetRangeUser(0,gPad->GetFrame()->GetY2()*1.5);
  c3->SaveAs(Form("%s%s",pathToOutput,"XYResolutionBestRow.png"));
  c3->SaveAs(Form("%s%s",pathToOutput,"XYResolutionBestRow.pdf"));
  runBestXYResGraph->SetName("rphi best row");
  runBestXYResGraph->Write();

  TCanvas *c9 = new TCanvas("rphi Resolution","rphi Resolution");
  runBestXYResGraph->Draw("AP");
  TGraphErrors *meanSingleXYResGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runSingleXYResMean[0], &runZPositionsError[0], &runSingleXYResMeanError[0] );
  meanSingleXYResGraph->SetLineColor(colors[1]);
  meanSingleXYResGraph->SetFillColor(kWhite);
  meanSingleXYResGraph->SetTitle("mean all rows");
  meanSingleXYResGraph->Draw("sameP");
  gPad->BuildLegend(0.5515075,0.2008929,0.8994975,0.3839286);
  c9->SaveAs(Form("%s%s",pathToOutput,"XYResolution.png"));
  c9->SaveAs(Form("%s%s",pathToOutput,"XYResolution.pdf"));
  meanSingleXYResGraph->SetName("rphi mean all rows");
  meanSingleXYResGraph->GetXaxis()->SetTitle("drift distance [mm]");
  meanSingleXYResGraph->GetYaxis()->SetTitle("#sigma_{r#varphi} [mm]");
  meanSingleXYResGraph->Write();

  TCanvas *c4 = new TCanvas("Resolution z best row","Resolution z best row");
  TGraphErrors *runBestZResGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runBestZRes[0], &runZPositionsError[0], &runBestZResError[0] );
  runBestZResGraph->SetTitle(Form("module_%i row_%i",bestRow.first,bestRow.second));
  runBestZResGraph->GetXaxis()->SetTitle("drift distance [mm]");
  runBestZResGraph->GetYaxis()->SetTitle("#sigma_{z} [mm]");
  runBestZResGraph->SetLineColor(colors[0]);
  runBestZResGraph->SetFillColor(kWhite);
  runBestZResGraph->Draw("AP");
  gPad->Update();
  runBestZResGraph->GetYaxis()->SetRangeUser(0,gPad->GetFrame()->GetY2()*1.5);
  c4->SaveAs(Form("%s%s",pathToOutput,"ZResolutionBestRow.png"));
  c4->SaveAs(Form("%s%s",pathToOutput,"ZResolutionBestRow.pdf"));
  runBestZResGraph->SetName("z best row");
  runBestZResGraph->Write();

  TCanvas *c10 = new TCanvas("Z Resolution","Z Resolution");
  runBestZResGraph->Draw("AP");
  TGraphErrors *meanSingleZResGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runSingleZResMean[0], &runZPositionsError[0], &runSingleZResMeanError[0] );
  meanSingleZResGraph->SetLineColor(colors[1]);
  meanSingleZResGraph->SetFillColor(kWhite);
  meanSingleZResGraph->SetTitle("mean all rows");
  meanSingleZResGraph->Draw("SAMEP");
  gPad->BuildLegend(0.5515075,0.2008929,0.8994975,0.3839286);
  c10->SaveAs(Form("%s%s",pathToOutput,"ZResolution.png"));
  c10->SaveAs(Form("%s%s",pathToOutput,"ZResolution.pdf"));
  meanSingleZResGraph->SetName("z mean all rows");
  meanSingleZResGraph->GetXaxis()->SetTitle("drift distance [mm]");
  meanSingleZResGraph->GetYaxis()->SetTitle("#sigma_{z} [mm]");
  meanSingleZResGraph->Write();

  c1->Write(); //Saving the canvases for easier manipulation later --Peter
  c2->Write();
  c3->Write();
  c4->Write();
  c9->Write();
  c10->Write();

  outfile->Write();
  outfile->Close();
  gStyle->SetPadRightMargin(0.04);

}
