#include <vector>
#include <iostream>
#include <fstream>
#include "sstream"
#include <utility>
#include <cstdarg>
#include <cmath>

#include "TLatex.h"
#include "TCanvas.h"
#include "TAxis.h"
#include "TGraphErrors.h"
#include "TFile.h"
#include "TNtuple.h"
#include "TStyle.h"
#include "TROOT.h"
#include "TFrame.h"
#include "TColor.h"
#include "TMath.h"


/*How to call this:
.L $MARLINTPC/analysis/rootscripts/PulseShape.C++
PulseShape("path for output","path to the aida file", number of runs in the following list, list of the runs)
e.g PulseShape("../myplots","../AidaFiles/",3,18850,18851,18860)
 */
void PulseShape(char* pathToOutput, char* pathToAida, int nRuns, ...){

	//loading the style
	gROOT->ProcessLine(".L ${MARLINTPC}/analysis/rootscripts/LCTPCStyle.C");
	gROOT->ProcessLine("LCTPCStyle()");

	//open a root file to store the graphs in
	TFile* outfile = new TFile(Form("%s%s",pathToOutput,"PulseShape.root"),"RECREATE");

	//initialize the input list
	va_list argList;
	va_start(argList,nRuns);

	int runNumber;
	// will contain the results from the processors
	std::vector<double> runZPositions;
	std::vector<double> runZPositionsError;
	std::vector<double> runZPositionsSingleRow;
	std::vector<double> runZPositionsSingleRowError;

	std::vector<double> runHighestNormPulseProfilePar0;
	std::vector<double> runHighestNormPulseProfileParErr0;
	std::vector<double> runHighestNormPulseProfilePar2;
	std::vector<double> runHighestNormPulseProfileParErr2;
	std::vector<double> runHighestNormPulseProfilePar3;
	std::vector<double> runHighestNormPulseProfileParErr3;
	std::vector<double> runHighestNormPulseProfilePar4;
	std::vector<double> runHighestNormPulseProfileParErr4;
	std::vector<double> runHighestNormPulseProfilePar5;
	std::vector<double> runHighestNormPulseProfileParErr5;
	std::vector<double> runHighestNormPulseProfilePar2Plus3;
	std::vector<double> runHighestNormPulseProfileParErr2Plus3;

	std::vector<double> runHighestNormPulseHist2DPar0;
	std::vector<double> runHighestNormPulseHist2DParErr0;
	std::vector<double> runHighestNormPulseHist2DPar2;
	std::vector<double> runHighestNormPulseHist2DParErr2;
	std::vector<double> runHighestNormPulseHist2DPar3;
	std::vector<double> runHighestNormPulseHist2DParErr3;
	std::vector<double> runHighestNormPulseHist2DPar4;
	std::vector<double> runHighestNormPulseHist2DParErr4;
	std::vector<double> runHighestNormPulseHist2DPar5;
	std::vector<double> runHighestNormPulseHist2DParErr5;
	std::vector<double> runHighestNormPulseHist2DPar2Plus3;
	std::vector<double> runHighestNormPulseHist2DParErr2Plus3;

	std::vector<double> runNeighborsNormPulseProfilePar0;
	std::vector<double> runNeighborsNormPulseProfileParErr0;
	std::vector<double> runNeighborsNormPulseProfilePar2;
	std::vector<double> runNeighborsNormPulseProfileParErr2;
	std::vector<double> runNeighborsNormPulseProfilePar3;
	std::vector<double> runNeighborsNormPulseProfileParErr3;
	std::vector<double> runNeighborsNormPulseProfilePar4;
	std::vector<double> runNeighborsNormPulseProfileParErr4;
	std::vector<double> runNeighborsNormPulseProfilePar5;
	std::vector<double> runNeighborsNormPulseProfileParErr5;
	std::vector<double> runNeighborsNormPulseProfilePar2Plus3;
	std::vector<double> runNeighborsNormPulseProfileParErr2Plus3;

	std::vector<double> runNeighborsNormPulseHist2DPar0;
	std::vector<double> runNeighborsNormPulseHist2DParErr0;
	std::vector<double> runNeighborsNormPulseHist2DPar2;
	std::vector<double> runNeighborsNormPulseHist2DParErr2;
	std::vector<double> runNeighborsNormPulseHist2DPar3;
	std::vector<double> runNeighborsNormPulseHist2DParErr3;
	std::vector<double> runNeighborsNormPulseHist2DPar4;
	std::vector<double> runNeighborsNormPulseHist2DParErr4;
	std::vector<double> runNeighborsNormPulseHist2DPar5;
	std::vector<double> runNeighborsNormPulseHist2DParErr5;
	std::vector<double> runNeighborsNormPulseHist2DPar2Plus3;
	std::vector<double> runNeighborsNormPulseHist2DParErr2Plus3;

	//create colors
	const int BINNING=nRuns;
	Int_t colors[BINNING];

	const Int_t NRGBs = 5;

	Double_t stops[NRGBs] = { 0.00, 0.34, 0.61, 0.84, 1.00 };
	Double_t red[NRGBs]   = { 0.00, 0.00, 0.87, 1.00, 0.51 };
	Double_t green[NRGBs] = { 0.00, 0.81, 1.00, 0.20, 0.00 };
	Double_t blue[NRGBs]  = { 0.51, 1.00, 0.12, 0.00, 0.00 };
	Int_t FI = TColor::CreateGradientColorTable(NRGBs, stops, red, green, blue, BINNING);

	for(int i=0 ; i<BINNING ; i++) 
		colors[i] = FI+i;

	//loop over the given runs
	for(int i=0 ; i<nRuns ; i++)
	{
		//get run number from input
		runNumber=va_arg(argList,int);

		//create stringstream containing the aida file name
		std::stringstream runNumberString;
		std::cout << "run number: " << runNumber << std::endl;
		runNumberString << pathToAida << "aida_run" << runNumber << ".root";

		//open the aida file
		TFile* file  = TFile::Open(runNumberString.str().c_str(),"Read");
		//check that the file exists
		if(file==0)
		{
			std::cout << "This run will be skipped. The aida file does not exist. Please check this!" << std::endl;
			continue;
		}

		//cd out of the file to prevent root from automatically deleting the following, when closing the file
		gROOT->cd();

		//get the stuff from the file, check that everything exist
		TNtuple* zPosTuple = (TNtuple*)file->Get("MyAnalysisCutSelectorProcessor/RunZPositions");
		TNtuple* normPulseHighestTuple = (TNtuple*)file->Get("MyPulseShapeProcessor/NormPulseHighestFitParameters");
		TNtuple* normPulseNeighborsTuple = (TNtuple*)file->Get("MyPulseShapeProcessor/NormPulseNeighborsFitParameters");

		if( zPosTuple==0 || normPulseHighestTuple==0 || normPulseNeighborsTuple==0 )
		{
			std::cout << "This run will be skipped. One of the necessary root tuples does not exist. Please check that you ran the corresponding processors and that the tuple is in the aida file!" << std::endl;
			continue;
		}

		//open the first entry of the tuples
		zPosTuple->GetEntry(0);
		//entry 0 contains profile data
		normPulseHighestTuple->GetEntry(0);
		normPulseNeighborsTuple->GetEntry(0);

		//fill the vectors with the values from the tuples ([0]=mean,[2]=rms)
		runZPositions.push_back(567.6-zPosTuple->GetArgs()[0]);
		runZPositionsError.push_back(zPosTuple->GetArgs()[2]);

		runHighestNormPulseProfilePar0.push_back( normPulseHighestTuple->GetArgs()[0] );
		runHighestNormPulseProfileParErr0.push_back( normPulseHighestTuple->GetArgs()[1] );
		runHighestNormPulseProfilePar2.push_back( normPulseHighestTuple->GetArgs()[4] );
		runHighestNormPulseProfileParErr2.push_back( normPulseHighestTuple->GetArgs()[5] );
		runHighestNormPulseProfilePar3.push_back( normPulseHighestTuple->GetArgs()[6] );
		runHighestNormPulseProfileParErr3.push_back( normPulseHighestTuple->GetArgs()[7] );
		runHighestNormPulseProfilePar4.push_back( normPulseHighestTuple->GetArgs()[8] );
		runHighestNormPulseProfileParErr4.push_back( normPulseHighestTuple->GetArgs()[9] );
		runHighestNormPulseProfilePar5.push_back( normPulseHighestTuple->GetArgs()[10] );
		runHighestNormPulseProfileParErr5.push_back( normPulseHighestTuple->GetArgs()[11] );
		runHighestNormPulseProfilePar2Plus3.push_back( normPulseHighestTuple->GetArgs()[4] + normPulseHighestTuple->GetArgs()[6] );
		runHighestNormPulseProfileParErr2Plus3.push_back( sqrt( pow( normPulseHighestTuple->GetArgs()[5],2) + pow(normPulseHighestTuple->GetArgs()[7],2) ) );

		runNeighborsNormPulseProfilePar0.push_back( normPulseNeighborsTuple->GetArgs()[0] );
		runNeighborsNormPulseProfileParErr0.push_back( normPulseNeighborsTuple->GetArgs()[1] );
		runNeighborsNormPulseProfilePar2.push_back( normPulseNeighborsTuple->GetArgs()[4] );
		runNeighborsNormPulseProfileParErr2.push_back( normPulseNeighborsTuple->GetArgs()[5] );
		runNeighborsNormPulseProfilePar3.push_back( normPulseNeighborsTuple->GetArgs()[6] );
		runNeighborsNormPulseProfileParErr3.push_back( normPulseNeighborsTuple->GetArgs()[7] );
		runNeighborsNormPulseProfilePar4.push_back( normPulseNeighborsTuple->GetArgs()[8] );
		runNeighborsNormPulseProfileParErr4.push_back( normPulseNeighborsTuple->GetArgs()[9] );
		runNeighborsNormPulseProfilePar5.push_back( normPulseNeighborsTuple->GetArgs()[10] );
		runNeighborsNormPulseProfileParErr5.push_back( normPulseNeighborsTuple->GetArgs()[11] );
		runNeighborsNormPulseProfilePar2Plus3.push_back( normPulseNeighborsTuple->GetArgs()[4] + normPulseNeighborsTuple->GetArgs()[6] );
		runNeighborsNormPulseProfileParErr2Plus3.push_back( sqrt( pow( normPulseNeighborsTuple->GetArgs()[5],2) + pow(normPulseNeighborsTuple->GetArgs()[7],2) ) );

		//entry 1 contains scatterplot data
		normPulseHighestTuple->GetEntry(1);
		normPulseNeighborsTuple->GetEntry(1);

		runHighestNormPulseHist2DPar0.push_back( normPulseHighestTuple->GetArgs()[0] );
		runHighestNormPulseHist2DParErr0.push_back( normPulseHighestTuple->GetArgs()[1] );
		runHighestNormPulseHist2DPar2.push_back( normPulseHighestTuple->GetArgs()[4] );
		runHighestNormPulseHist2DParErr2.push_back( normPulseHighestTuple->GetArgs()[5] );
		runHighestNormPulseHist2DPar3.push_back( normPulseHighestTuple->GetArgs()[6] );
		runHighestNormPulseHist2DParErr3.push_back( normPulseHighestTuple->GetArgs()[7] );
		runHighestNormPulseHist2DPar4.push_back( normPulseHighestTuple->GetArgs()[8] );
		runHighestNormPulseHist2DParErr4.push_back( normPulseHighestTuple->GetArgs()[9] );
		runHighestNormPulseHist2DPar5.push_back( normPulseHighestTuple->GetArgs()[10] );
		runHighestNormPulseHist2DParErr5.push_back( normPulseHighestTuple->GetArgs()[11] );
		runHighestNormPulseHist2DPar2.push_back( normPulseHighestTuple->GetArgs()[4] + normPulseHighestTuple->GetArgs()[6] );
		runHighestNormPulseHist2DParErr2.push_back( sqrt( pow(normPulseHighestTuple->GetArgs()[5],2) + pow(normPulseHighestTuple->GetArgs()[7],2) ) );

		runNeighborsNormPulseHist2DPar0.push_back( normPulseNeighborsTuple->GetArgs()[0] );
		runNeighborsNormPulseHist2DParErr0.push_back( normPulseNeighborsTuple->GetArgs()[1] );
		runNeighborsNormPulseHist2DPar2.push_back( normPulseNeighborsTuple->GetArgs()[4] );
		runNeighborsNormPulseHist2DParErr2.push_back( normPulseNeighborsTuple->GetArgs()[5] );
		runNeighborsNormPulseHist2DPar3.push_back( normPulseNeighborsTuple->GetArgs()[6] );
		runNeighborsNormPulseHist2DParErr3.push_back( normPulseNeighborsTuple->GetArgs()[7] );
		runNeighborsNormPulseHist2DPar4.push_back( normPulseNeighborsTuple->GetArgs()[8] );
		runNeighborsNormPulseHist2DParErr4.push_back( normPulseNeighborsTuple->GetArgs()[9] );
		runNeighborsNormPulseHist2DPar5.push_back( normPulseNeighborsTuple->GetArgs()[10] );
		runNeighborsNormPulseHist2DParErr5.push_back( normPulseNeighborsTuple->GetArgs()[11] );
		runNeighborsNormPulseHist2DPar2.push_back( normPulseNeighborsTuple->GetArgs()[4] + normPulseNeighborsTuple->GetArgs()[6]);
		runNeighborsNormPulseHist2DParErr2.push_back( sqrt( pow(normPulseNeighborsTuple->GetArgs()[5],2) + pow(normPulseNeighborsTuple->GetArgs()[7],2) ) );

		//567.6-hitposition is the conversion into drift distance
		int zpos= int(567.6-zPosTuple->GetArgs()[0]) ;
		std::ostringstream legEntry;
		legEntry << "z_{d} = "<<zpos;

		std::ostringstream legEntryHist;
		legEntryHist << "z_{d} = "<<zpos;

		delete zPosTuple;
		delete normPulseHighestTuple;
		delete normPulseNeighborsTuple;

		file->Close();
		delete file;
	}

	va_end(argList);

	//save the results
	outfile->cd();

	TCanvas* c1 = new TCanvas("HighestNormPulseFitPar0","Highest Pulse Norm Fit Parameter 0");
	TGraphErrors* runHighestNormPulseProfilePar0Graph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runHighestNormPulseProfilePar0[0], &runZPositionsError[0], &runHighestNormPulseProfileParErr0[0] );
	runHighestNormPulseProfilePar0Graph->SetNameTitle("HighestNormPulseProfileFitPar0Graph","Highest Pulse Norm Profile Fit Parameter 0");
	runHighestNormPulseProfilePar0Graph->GetXaxis()->SetTitle("drift distance [mm]");
	runHighestNormPulseProfilePar0Graph->GetYaxis()->SetTitle("nomalized pulseheight");
	runHighestNormPulseProfilePar0Graph->Draw("AP");
	TGraphErrors* runHighestNormPulseHist2DPar0Graph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runHighestNormPulseHist2DPar0[0], &runZPositionsError[0], &runHighestNormPulseHist2DParErr0[0] );
	runHighestNormPulseHist2DPar0Graph->SetNameTitle("HighestNormPulseHist2DFitPar0Graph","Highest Pulse Norm Hist2D Fit Parameter 0");
	runHighestNormPulseHist2DPar0Graph->GetXaxis()->SetTitle("drift distance [mm]");
	runHighestNormPulseHist2DPar0Graph->GetYaxis()->SetTitle("nomalized pulseheight");
	runHighestNormPulseHist2DPar0Graph->SetLineColor(kRed);
//	runHighestNormPulseHist2DPar0Graph->Draw("sameP");
	gPad->Update();
	c1->SaveAs(Form("%s%s",pathToOutput,"HighestNormPulseFitPar0.png"));
	c1->SaveAs(Form("%s%s",pathToOutput,"HighestNormPulseFitPar0.pdf"));
	runHighestNormPulseProfilePar0Graph->Write();
	runHighestNormPulseHist2DPar0Graph->Write();

	TCanvas* c2 = new TCanvas("HighestNormPulseFitPar2","Highest Pulse Norm Fit Parameter 2");
	TGraphErrors* runHighestNormPulseProfilePar2Graph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runHighestNormPulseProfilePar2[0], &runZPositionsError[0], &runHighestNormPulseProfileParErr2[0] );
	runHighestNormPulseProfilePar2Graph->SetNameTitle("HighestNormPulseProfileFitPar2Graph","Highest Pulse Norm Profile Fit Parameter 2");
	runHighestNormPulseProfilePar2Graph->GetXaxis()->SetTitle("drift distance [mm]");
	runHighestNormPulseProfilePar2Graph->GetYaxis()->SetTitle("start time [time bins]");
	runHighestNormPulseProfilePar2Graph->Draw("AP");
	TGraphErrors* runHighestNormPulseHist2DPar2Graph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runHighestNormPulseHist2DPar2[0], &runZPositionsError[0], &runHighestNormPulseHist2DParErr2[0] );
	runHighestNormPulseHist2DPar2Graph->SetNameTitle("HighestNormPulseHist2DFitPar2Graph","Highest Pulse Norm Hist2D Fit Parameter 2");
	runHighestNormPulseHist2DPar2Graph->GetXaxis()->SetTitle("drift distance [mm]");
	runHighestNormPulseHist2DPar2Graph->GetYaxis()->SetTitle("start time [time bins]");
	runHighestNormPulseHist2DPar2Graph->SetLineColor(kRed);
//	runHighestNormPulseHist2DPar2Graph->Draw("sameP");
	gPad->Update();
	c2->SaveAs(Form("%s%s",pathToOutput,"HighestNormPulseFitPar2.png"));
	c2->SaveAs(Form("%s%s",pathToOutput,"HighestNormPulseFitPar2.pdf"));
	runHighestNormPulseProfilePar2Graph->Write();
	runHighestNormPulseHist2DPar2Graph->Write();

	TCanvas* c3 = new TCanvas("HighestNormPulseFitPar3","Highest Pulse Norm Fit Parameter 3");
	TGraphErrors* runHighestNormPulseProfilePar3Graph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runHighestNormPulseProfilePar3[0], &runZPositionsError[0], &runHighestNormPulseProfileParErr3[0] );
	runHighestNormPulseProfilePar3Graph->SetNameTitle("HighestNormPulseProfileFitPar3Graph","Highest Pulse Norm Profile Fit Parameter 3");
	runHighestNormPulseProfilePar3Graph->GetXaxis()->SetTitle("drift distance [mm]");
	runHighestNormPulseProfilePar3Graph->GetYaxis()->SetTitle("rise time [time bins]");
	runHighestNormPulseProfilePar3Graph->Draw("AP");
	TGraphErrors* runHighestNormPulseHist2DPar3Graph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runHighestNormPulseHist2DPar3[0], &runZPositionsError[0], &runHighestNormPulseHist2DParErr3[0] );
	runHighestNormPulseHist2DPar3Graph->SetNameTitle("HighestNormPulseHist2DFitPar3Graph","Highest Pulse Norm Hist2D Fit Parameter 3");
	runHighestNormPulseHist2DPar3Graph->GetXaxis()->SetTitle("drift distance [mm]");
	runHighestNormPulseHist2DPar3Graph->GetYaxis()->SetTitle("rise time [time bins]");
	runHighestNormPulseHist2DPar3Graph->SetLineColor(kRed);
//	runHighestNormPulseHist2DPar3Graph->Draw("sameP");
	gPad->Update();
	c3->SaveAs(Form("%s%s",pathToOutput,"HighestNormPulseFitPar3.png"));
	c3->SaveAs(Form("%s%s",pathToOutput,"HighestNormPulseFitPar3.pdf"));
	runHighestNormPulseProfilePar3Graph->Write();
	runHighestNormPulseHist2DPar3Graph->Write();

	TCanvas* c11 = new TCanvas("HighestNormPulseFitPar2Plus3","Highest Pulse Norm Fit Parameter 2+3");
	TGraphErrors* runHighestNormPulseProfilePar2Plus3Graph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runHighestNormPulseProfilePar2Plus3[0], &runZPositionsError[0], &runHighestNormPulseProfileParErr2Plus3[0] );
	runHighestNormPulseProfilePar2Plus3Graph->SetNameTitle("HighestNormPulseProfileFitPar2Plus3Graph","Neighbors Pulse Norm Profile Fit Parameter 2+3");
	runHighestNormPulseProfilePar2Plus3Graph->GetXaxis()->SetTitle("drift distance [mm]");
	runHighestNormPulseProfilePar2Plus3Graph->GetYaxis()->SetTitle(" start + rise time [time bins]");
	runHighestNormPulseProfilePar2Plus3Graph->Draw("AP");
	TGraphErrors* runHighestNormPulseHist2DPar2Plus3Graph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runHighestNormPulseHist2DPar2Plus3[0], &runZPositionsError[0], &runHighestNormPulseHist2DParErr2Plus3[0] );
	runHighestNormPulseHist2DPar2Plus3Graph->SetNameTitle("HighestNormPulseHist2DFitPar2Plus3Graph","Neighbors Pulse Norm Hist2D Fit Parameter 2+3");
	runHighestNormPulseHist2DPar2Plus3Graph->GetXaxis()->SetTitle("drift distance [mm]");
	runHighestNormPulseHist2DPar2Plus3Graph->GetYaxis()->SetTitle(" start + rise time [time bins]");
	runHighestNormPulseHist2DPar2Plus3Graph->SetLineColor(kRed);
//	runHighestNormPulseHist2DPar2Plus3Graph->Draw("sameP");
	gPad->Update();
	c11->SaveAs(Form("%s%s",pathToOutput,"HighestNormPulseFitPar2Plus3.png"));
	c11->SaveAs(Form("%s%s",pathToOutput,"HighestNormPulseFitPar2Plus3.pdf"));
	runHighestNormPulseProfilePar2Plus3Graph->Write();
	runHighestNormPulseHist2DPar2Plus3Graph->Write();

	TCanvas* c4 = new TCanvas("HighestNormPulseFitPar4","Highest Pulse Norm Fit Parameter 4");
	TGraphErrors* runHighestNormPulseProfilePar4Graph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runHighestNormPulseProfilePar4[0], &runZPositionsError[0], &runHighestNormPulseProfileParErr4[0] );
	runHighestNormPulseProfilePar4Graph->SetNameTitle("HighestNormPulseProfileFitPar4Graph","Highest Pulse Norm Profile Fit Parameter 4");
	runHighestNormPulseProfilePar4Graph->GetXaxis()->SetTitle("drift distance [mm]");
	runHighestNormPulseProfilePar4Graph->GetYaxis()->SetTitle("offset");
	runHighestNormPulseProfilePar4Graph->Draw("AP");
	TGraphErrors* runHighestNormPulseHist2DPar4Graph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runHighestNormPulseHist2DPar4[0], &runZPositionsError[0], &runHighestNormPulseHist2DParErr4[0] );
	runHighestNormPulseHist2DPar4Graph->SetNameTitle("HighestNormPulseHist2DFitPar4Graph","Highest Pulse Norm Hist2D Fit Parameter 4");
	runHighestNormPulseHist2DPar4Graph->GetXaxis()->SetTitle("drift distance [mm]");
	runHighestNormPulseHist2DPar4Graph->GetYaxis()->SetTitle("offset");
	runHighestNormPulseHist2DPar4Graph->SetLineColor(kRed);
//	runHighestNormPulseHist2DPar4Graph->Draw("sameP");
	gPad->Update();
	c4->SaveAs(Form("%s%s",pathToOutput,"HighestNormPulseFitPar4.png"));
	c4->SaveAs(Form("%s%s",pathToOutput,"HighestNormPulseFitPar4.pdf"));
	runHighestNormPulseProfilePar4Graph->Write();
	runHighestNormPulseHist2DPar4Graph->Write();

	TCanvas* c5 = new TCanvas("HighestNormPulseFitPar5","Highest Pulse Norm Fit Parameter 5");
	TGraphErrors* runHighestNormPulseProfilePar5Graph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runHighestNormPulseProfilePar5[0], &runZPositionsError[0], &runHighestNormPulseProfileParErr5[0] );
	runHighestNormPulseProfilePar5Graph->SetNameTitle("HighestNormPulseProfileFitPar5Graph","Highest Pulse Norm Profile Fit Parameter 5");
	runHighestNormPulseProfilePar5Graph->GetXaxis()->SetTitle("drift distance [mm]");
	runHighestNormPulseProfilePar5Graph->GetYaxis()->SetTitle("slope");
	runHighestNormPulseProfilePar5Graph->Draw("AP");
	TGraphErrors* runHighestNormPulseHist2DPar5Graph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runHighestNormPulseHist2DPar5[0], &runZPositionsError[0], &runHighestNormPulseHist2DParErr5[0] );
	runHighestNormPulseHist2DPar5Graph->SetNameTitle("HighestNormPulseHist2DFitPar5Graph","Highest Pulse Norm Hist2D Fit Parameter 5");
	runHighestNormPulseHist2DPar5Graph->GetXaxis()->SetTitle("drift distance [mm]");
	runHighestNormPulseHist2DPar5Graph->GetYaxis()->SetTitle("slope");
	runHighestNormPulseHist2DPar5Graph->SetLineColor(kRed);
//	runHighestNormPulseHist2DPar5Graph->Draw("sameP");
	gPad->Update();
	c5->SaveAs(Form("%s%s",pathToOutput,"HighestNormPulseFitPar5.png"));
	c5->SaveAs(Form("%s%s",pathToOutput,"HighestNormPulseFitPar5.pdf"));
	runHighestNormPulseProfilePar5Graph->Write();
	runHighestNormPulseHist2DPar5Graph->Write();

	TCanvas* c6 = new TCanvas("NeighborsNormPulseFitPar0","Neighbors Pulse Norm Fit Parameter 0");
	TGraphErrors* runNeighborsNormPulseProfilePar0Graph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runNeighborsNormPulseProfilePar0[0], &runZPositionsError[0], &runNeighborsNormPulseProfileParErr0[0] );
	runNeighborsNormPulseProfilePar0Graph->SetNameTitle("NeighborsNormPulseProfileFitPar0Graph","Neighbors Pulse Norm Profile Fit Parameter 0");
	runNeighborsNormPulseProfilePar0Graph->GetXaxis()->SetTitle("drift distance [mm]");
	runNeighborsNormPulseProfilePar0Graph->GetYaxis()->SetTitle("normalized pulseheight");
	runNeighborsNormPulseProfilePar0Graph->Draw("AP");
	TGraphErrors* runNeighborsNormPulseHist2DPar0Graph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runNeighborsNormPulseHist2DPar0[0], &runZPositionsError[0], &runNeighborsNormPulseHist2DParErr0[0] );
	runNeighborsNormPulseHist2DPar0Graph->SetNameTitle("NeighborsNormPulseHist2DFitPar0Graph","Neighbors Pulse Norm Hist2D Fit Parameter 0");
	runNeighborsNormPulseHist2DPar0Graph->GetXaxis()->SetTitle("drift distance [mm]");
	runNeighborsNormPulseHist2DPar0Graph->GetYaxis()->SetTitle("normalized pulseheight");
	runNeighborsNormPulseHist2DPar0Graph->SetLineColor(kRed);
//	runNeighborsNormPulseHist2DPar0Graph->Draw("sameP");
	gPad->Update();
	c6->SaveAs(Form("%s%s",pathToOutput,"NeighborsNormPulseFitPar0.png"));
	c6->SaveAs(Form("%s%s",pathToOutput,"NeighborsNormPulseFitPar0.pdf"));
	runNeighborsNormPulseProfilePar0Graph->Write();
	runNeighborsNormPulseHist2DPar0Graph->Write();

	TCanvas* c7 = new TCanvas("NeighborsNormPulseFitPar2","Neighbors Pulse Norm Fit Parameter 2");
	TGraphErrors* runNeighborsNormPulseProfilePar2Graph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runNeighborsNormPulseProfilePar2[0], &runZPositionsError[0], &runNeighborsNormPulseProfileParErr2[0] );
	runNeighborsNormPulseProfilePar2Graph->SetNameTitle("NeighborsNormPulseProfileFitPar2Graph","Neighbors Pulse Norm Profile Fit Parameter 2");
	runNeighborsNormPulseProfilePar2Graph->GetXaxis()->SetTitle("drift distance [mm]");
	runNeighborsNormPulseProfilePar2Graph->GetYaxis()->SetTitle("start time [time bins]");
	runNeighborsNormPulseProfilePar2Graph->Draw("AP");
	TGraphErrors* runNeighborsNormPulseHist2DPar2Graph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runNeighborsNormPulseHist2DPar2[0], &runZPositionsError[0], &runNeighborsNormPulseHist2DParErr2[0] );
	runNeighborsNormPulseHist2DPar2Graph->SetNameTitle("NeighborsNormPulseHist2DFitPar2Graph","Neighbors Pulse Norm Hist2D Fit Parameter 2");
	runNeighborsNormPulseHist2DPar2Graph->GetXaxis()->SetTitle("drift distance [mm]");
	runNeighborsNormPulseHist2DPar2Graph->GetYaxis()->SetTitle("start time [time bins]");
	runNeighborsNormPulseHist2DPar2Graph->SetLineColor(kRed);
//	runNeighborsNormPulseHist2DPar2Graph->Draw("sameP");
	gPad->Update();
	c7->SaveAs(Form("%s%s",pathToOutput,"NeighborsNormPulseFitPar2.png"));
	c7->SaveAs(Form("%s%s",pathToOutput,"NeighborsNormPulseFitPar2.pdf"));
	runNeighborsNormPulseProfilePar2Graph->Write();
	runNeighborsNormPulseHist2DPar2Graph->Write();

	TCanvas* c8 = new TCanvas("NeighborsNormPulseFitPar3","Neighbors Pulse Norm Fit Parameter 3");
	TGraphErrors* runNeighborsNormPulseProfilePar3Graph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runNeighborsNormPulseProfilePar3[0], &runZPositionsError[0], &runNeighborsNormPulseProfileParErr3[0] );
	runNeighborsNormPulseProfilePar3Graph->SetNameTitle("NeighborsNormPulseProfileFitPar3Graph","Neighbors Pulse Norm Profile Fit Parameter 3");
	runNeighborsNormPulseProfilePar3Graph->GetXaxis()->SetTitle("drift distance [mm]");
	runNeighborsNormPulseProfilePar3Graph->GetYaxis()->SetTitle("rise time [time bins]");
	runNeighborsNormPulseProfilePar3Graph->Draw("AP");
	TGraphErrors* runNeighborsNormPulseHist2DPar3Graph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runNeighborsNormPulseHist2DPar3[0], &runZPositionsError[0], &runNeighborsNormPulseHist2DParErr3[0] );
	runNeighborsNormPulseHist2DPar3Graph->SetNameTitle("NeighborsNormPulseHist2DFitPar3Graph","Neighbors Pulse Norm Hist2D Fit Parameter 3");
	runNeighborsNormPulseHist2DPar3Graph->GetXaxis()->SetTitle("drift distance [mm]");
	runNeighborsNormPulseHist2DPar3Graph->GetYaxis()->SetTitle("rise time [time bins]");
	runNeighborsNormPulseHist2DPar3Graph->SetLineColor(kRed);
//	runNeighborsNormPulseHist2DPar3Graph->Draw("sameP");
	gPad->Update();
	c8->SaveAs(Form("%s%s",pathToOutput,"NeighborsNormPulseFitPar3.png"));
	c8->SaveAs(Form("%s%s",pathToOutput,"NeighborsNormPulseFitPar3.pdf"));
	runNeighborsNormPulseProfilePar3Graph->Write();
	runNeighborsNormPulseHist2DPar3Graph->Write();

	TCanvas* c12 = new TCanvas("NeighborsNormPulseFitPar2Plus3","Neighbors Pulse Norm Fit Parameter 2+3");
	TGraphErrors* runNeighborsNormPulseProfilePar2Plus3Graph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runNeighborsNormPulseProfilePar2Plus3[0], &runZPositionsError[0], &runNeighborsNormPulseProfileParErr2Plus3[0] );
	runNeighborsNormPulseProfilePar2Plus3Graph->SetNameTitle("NeighborsNormPulseProfileFitPar2Plus3Graph","Neighbors Pulse Norm Profile Fit Parameter 2+3");
	runNeighborsNormPulseProfilePar2Plus3Graph->GetXaxis()->SetTitle("drift distance [mm]");
	runNeighborsNormPulseProfilePar2Plus3Graph->GetYaxis()->SetTitle(" start + rise time [time bins]");
	runNeighborsNormPulseProfilePar2Plus3Graph->Draw("AP");
	TGraphErrors* runNeighborsNormPulseHist2DPar2Plus3Graph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runNeighborsNormPulseHist2DPar2Plus3[0], &runZPositionsError[0], &runNeighborsNormPulseHist2DParErr2Plus3[0] );
	runNeighborsNormPulseHist2DPar2Plus3Graph->SetNameTitle("NeighborsNormPulseHist2DFitPar2Plus3Graph","Neighbors Pulse Norm Hist2D Fit Parameter 2+3");
	runNeighborsNormPulseHist2DPar2Plus3Graph->GetXaxis()->SetTitle("drift distance [mm]");
	runNeighborsNormPulseHist2DPar2Plus3Graph->GetYaxis()->SetTitle(" start + rise time [time bins]");
	runNeighborsNormPulseHist2DPar2Plus3Graph->SetLineColor(kRed);
//	runNeighborsNormPulseHist2DPar2Plus3Graph->Draw("sameP");
	gPad->Update();
	c12->SaveAs(Form("%s%s",pathToOutput,"NeighborsNormPulseFitPar2Plus3.png"));
	c12->SaveAs(Form("%s%s",pathToOutput,"NeighborsNormPulseFitPar2Plus3.pdf"));
	runNeighborsNormPulseProfilePar2Plus3Graph->Write();
	runNeighborsNormPulseHist2DPar2Plus3Graph->Write();

	TCanvas* c9 = new TCanvas("NeighborsNormPulseFitPar4","Neighbors Pulse Norm Fit Parameter 4");
	TGraphErrors* runNeighborsNormPulseProfilePar4Graph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runNeighborsNormPulseProfilePar4[0], &runZPositionsError[0], &runNeighborsNormPulseProfileParErr4[0] );
	runNeighborsNormPulseProfilePar4Graph->SetNameTitle("NeighborsNormPulseProfileFitPar4Graph","Neighbors Pulse Norm Profile Fit Parameter 4");
	runNeighborsNormPulseProfilePar4Graph->GetXaxis()->SetTitle("drift distance [mm]");
	runNeighborsNormPulseProfilePar4Graph->GetYaxis()->SetTitle("offset");
	runNeighborsNormPulseProfilePar4Graph->Draw("AP");
	TGraphErrors* runNeighborsNormPulseHist2DPar4Graph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runNeighborsNormPulseHist2DPar4[0], &runZPositionsError[0], &runNeighborsNormPulseHist2DParErr4[0] );
	runNeighborsNormPulseHist2DPar4Graph->SetNameTitle("NeighborsNormPulseHist2DFitPar4Graph","Neighbors Pulse Norm Hist2D Fit Parameter 4");
	runNeighborsNormPulseHist2DPar4Graph->GetXaxis()->SetTitle("drift distance [mm]");
	runNeighborsNormPulseHist2DPar4Graph->GetYaxis()->SetTitle("offset");
	runNeighborsNormPulseHist2DPar4Graph->SetLineColor(kRed);
//	runNeighborsNormPulseHist2DPar4Graph->Draw("sameP");
	gPad->Update();
	c9->SaveAs(Form("%s%s",pathToOutput,"NeighborsNormPulseFitPar4.png"));
	c9->SaveAs(Form("%s%s",pathToOutput,"NeighborsNormPulseFitPar4.pdf"));
	runNeighborsNormPulseProfilePar4Graph->Write();
	runNeighborsNormPulseHist2DPar4Graph->Write();

	TCanvas* c10 = new TCanvas("NeighborsNormPulseFitPar5","Neighbors Pulse Norm Fit Parameter 5");
	TGraphErrors* runNeighborsNormPulseProfilePar5Graph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runNeighborsNormPulseProfilePar5[0], &runZPositionsError[0], &runNeighborsNormPulseProfileParErr5[0] );
	runNeighborsNormPulseProfilePar5Graph->SetNameTitle("NeighborsNormPulseProfileFitPar5Graph","Neighbors Pulse Norm Profile Fit Parameter 5");
	runNeighborsNormPulseProfilePar5Graph->GetXaxis()->SetTitle("drift distance [mm]");
	runNeighborsNormPulseProfilePar5Graph->GetYaxis()->SetTitle("slope");
	runNeighborsNormPulseProfilePar5Graph->Draw("AP");
	TGraphErrors* runNeighborsNormPulseHist2DPar5Graph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runNeighborsNormPulseHist2DPar5[0], &runZPositionsError[0], &runNeighborsNormPulseHist2DParErr5[0] );
	runNeighborsNormPulseHist2DPar5Graph->SetNameTitle("NeighborsNormPulseHist2DFitPar5Graph","Neighbors Pulse Norm Hist2D Fit Parameter 5");
	runNeighborsNormPulseHist2DPar5Graph->GetXaxis()->SetTitle("drift distance [mm]");
	runNeighborsNormPulseHist2DPar5Graph->GetYaxis()->SetTitle("slope");
	runNeighborsNormPulseHist2DPar5Graph->SetLineColor(kRed);
//	runNeighborsNormPulseHist2DPar5Graph->Draw("sameP");
	gPad->Update();
	c10->SaveAs(Form("%s%s",pathToOutput,"NeighborsNormPulseFitPar5.png"));
	c10->SaveAs(Form("%s%s",pathToOutput,"NeighborsNormPulseFitPar5.pdf"));
	runNeighborsNormPulseProfilePar5Graph->Write();
	runNeighborsNormPulseHist2DPar5Graph->Write();

	//Saving these to save plot details for future use.  --Peter
	c1->Write();
	c2->Write();
	c3->Write();
	c4->Write();
	c5->Write();
	c6->Write();
	c7->Write();
	c8->Write();
	c9->Write();
	c10->Write();
	c11->Write();
	c12->Write();

	outfile->Write();
	outfile->Close();
	gStyle->SetPadRightMargin(0.04);
}
