#include <vector>
#include <iostream>
#include "TLatex.h"
#include "TCanvas.h"
#include "TH1.h"
#include "TGraphErrors.h"
#include "TMultiGraph.h"
#include "TFile.h"
#include "TMath.h"
#include "TNtuple.h"
#include "TStyle.h"
#include "TROOT.h"
#include "TFrame.h"
#include "sstream"
#include <utility>
#include <cstdarg>
#include "TColor.h"

/*How to call this:
.L $MARLINTPC/analysis/rootscripts/TripletResolutionPerformance.C++
TripletResolutionPerformance("path for output","path to the aida file", number of runs in the following list, list of the runs)
e.g TripletResolutionPerformance("../myplots","../AidaFiles/",3,18850,18851,18860)
 */
void TripletResolutionPerformance(char * pathToOutput,char * pathToAida, int nRuns, ...){

	//loading the style
	gROOT->ProcessLine(".L ${MARLINTPC}/analysis/rootscripts/LCTPCStyle.C");
	gROOT->ProcessLine("LCTPCStyle()");

	//open a root file to store the graphs in
	TFile *outfile=new TFile(Form("%s%s",pathToOutput,"TripletResolutionPerformance.root"),"RECREATE");
	
	//initialize the input list
	va_list argList;
	va_start(argList,nRuns);

	int runNumber;
	int nRows=0;
	// will contain the results from the processors
	TMultiGraph *runResZGraph=new TMultiGraph();
	TMultiGraph *runResRPhiGraph=new TMultiGraph();
	runResZGraph->SetTitle("z Resolution;r_{row} [mm];#sigma_{z} [mm]");
	runResRPhiGraph->SetTitle("r phi Resolution;r_{row} [mm];#sigma_{r#varphi} [mm]");

	std::vector<double> runZPositions;
	std::vector<double> runZPositionsError;
	std::vector<double> runZPositionsSingleRow;
	std::vector<double> runZPositionsSingleRowError;

	std::vector<double> runSingleXYRes;
	std::vector<double> runSingleXYResError;
	std::vector<double> runBestXYRes;
	std::vector<double> runBestXYResError;
	std::vector<double> runAllXYRes;
	std::vector<double> runAllXYResError;

	std::vector<double> runSingleZRes;
	std::vector<double> runSingleZResError;
	std::vector<double> runBestZRes;
	std::vector<double> runBestZResError;
	std::vector<double> runAllZRes;
	std::vector<double> runAllZResError;

	std::vector<double> runSingleZResMean;
	std::vector<double> runSingleZResMeanError;
	std::vector<double> runSingleXYResMean;
	std::vector<double> runSingleXYResMeanError;

	std::pair<int,int> bestRow;

	//create colors
	const int BINNING=nRuns;
	Int_t colors[BINNING];

	const Int_t NRGBs = 5;

	Double_t stops[NRGBs] = { 0.00, 0.34, 0.61, 0.84, 1.00 };
	Double_t red[NRGBs]   = { 0.00, 0.00, 0.87, 1.00, 0.51 };
	Double_t green[NRGBs] = { 0.00, 0.81, 1.00, 0.20, 0.00 };
	Double_t blue[NRGBs]  = { 0.51, 1.00, 0.12, 0.00, 0.00 };
	Int_t FI = TColor::CreateGradientColorTable(NRGBs, stops, red, green, blue, BINNING);

	for (int i=0;i<BINNING;i++)
		colors[i] = FI+i;
	// 	int nCOLORS=20;
	// 	int colors[nCOLORS];
	// 	int index[5]={10,5,2,0,-3};
	// 	for(int g=0;g<5;g++)
	// 		colors[g]=kAzure+index[g];
	// 	for(int g=0;g<5;g++)
	// 		colors[g+5]=kViolet+index[g];
	// 	for(int g=0;g<5;g++)
	// 		colors[g+10]=kOrange+index[g];
	// 	for(int g=0;g<5;g++)
	// 		colors[g+15]=kTeal+index[g];

	//loop over the given runs
	for (int i=0;i<nRuns;i++)
	{
		//get run number from input
		runNumber=va_arg(argList,int);

		//create stringstream containing the aida file name
		std::stringstream runNumberString;
		std::cout<<"run number: "<<runNumber<<std::endl;
		runNumberString <<pathToAida<<"aida_run" <<runNumber<< ".root";

		//open the aida file
		TFile *file  = TFile::Open(runNumberString.str().c_str(),"Read");
		//check that the file exists
		if (file==0)
		{
			std::cout<<"This run will be skipped. The aida file does not exist. Please check this!"<<std::endl;
			continue;
		}

		//get the tuples from the file
		TNtuple * zPosTuple=(TNtuple*)file->Get("MyAnalysisCutSelectorProcessor/RunZPositions");
		TNtuple * allRowTuple=(TNtuple*)file->Get("MyTripletResolutionPerformanceProcessor/allRowsTripletResolution");
		TNtuple * bestRowTuple=(TNtuple*)file->Get("MyTripletResolutionPerformanceProcessor/bestRowTripletResolution");
		TNtuple * singleRowTuple=(TNtuple*)file->Get("MyTripletResolutionPerformanceProcessor/singleRowsTripletResolution");

		if (zPosTuple==0 || allRowTuple==0 || bestRowTuple==0 || singleRowTuple==0 )
		{
			std::cout<<"This run will be skipped. One of the necessary root tuples does not exist. Please check that you ran the corresponding processors and that the tuple is in the aida file!"<<std::endl;
			continue;
		}
		//open the first and only entry of the tuples
		zPosTuple->GetEntry(0);
		allRowTuple->GetEntry(0);
		bestRowTuple->GetEntry(0);

		runZPositions.push_back(567.6-zPosTuple->GetArgs()[0]);
		runZPositionsError.push_back(zPosTuple->GetArgs()[2]);
		runBestXYRes.push_back(bestRowTuple->GetArgs()[2]);
		runBestXYResError.push_back(bestRowTuple->GetArgs()[3]);
		runAllXYRes.push_back(allRowTuple->GetArgs()[0]);
		runAllXYResError.push_back(allRowTuple->GetArgs()[1]);


		runBestZRes.push_back(bestRowTuple->GetArgs()[4]);
		runBestZResError.push_back(bestRowTuple->GetArgs()[5]);
		runAllZRes.push_back(allRowTuple->GetArgs()[2]);
		runAllZResError.push_back(allRowTuple->GetArgs()[3]);

		bestRow = std::make_pair(static_cast<int>(bestRowTuple->GetArgs()[0]),static_cast<int>(bestRowTuple->GetArgs()[1]));

		std::vector<double> xy,z,wxy,wz;
		double sumwxy=0;
		double sumwz=0;

		nRows = singleRowTuple->GetEntries();
		for (int RowIter=0;RowIter<nRows;RowIter++)
		{
			runZPositionsSingleRow.push_back(567.6-zPosTuple->GetArgs()[0]);
			runZPositionsSingleRowError.push_back(zPosTuple->GetArgs()[2]);
			singleRowTuple->GetEntry(RowIter);
			runSingleXYRes.push_back(singleRowTuple->GetArgs()[2]);
			runSingleXYResError.push_back(singleRowTuple->GetArgs()[3]);
			runSingleZRes.push_back(singleRowTuple->GetArgs()[4]);
			runSingleZResError.push_back(singleRowTuple->GetArgs()[5]);

			wxy.push_back(1/(singleRowTuple->GetArgs()[3]*singleRowTuple->GetArgs()[3]));
			wz.push_back(1/(singleRowTuple->GetArgs()[5]*singleRowTuple->GetArgs()[5]));
			xy.push_back(singleRowTuple->GetArgs()[2]);
			z.push_back(singleRowTuple->GetArgs()[4]);

			sumwxy+=wxy[RowIter];
			sumwz+=wz[RowIter];
		}


		runSingleXYResMean.push_back(TMath::Mean(xy.size(),&xy[0],&wxy[0]));
		runSingleZResMean.push_back(TMath::Mean(xy.size(),&z[0],&wz[0]));

		runSingleXYResMeanError.push_back(sqrt(1/sumwxy));
		runSingleZResMeanError.push_back(sqrt(1/sumwz));

		//get the graphs from the file
		TGraphErrors *g_rphi=(TGraphErrors*)file->Get("MyTripletResolutionPerformanceProcessor/XYTripletResolutionRow");
		TGraphErrors *g_z=(TGraphErrors*)file->Get("MyTripletResolutionPerformanceProcessor/ZTripletResolutionRow");
		//check that the tuples exists
		if(g_rphi==0 || g_z==0 )
		{
			std::cout<<"This run will be skipped. One of the necessary root graphs does not exist. Please check that you ran the corresponding processors and that the graph is in the aida file!"<<std::endl;
			continue;
		}

		//fill the vectors with the values from the tuples ([0]=mean,[2]=rms)
		//567.6-hitposition is the conversion into drift distance
		int zpos= int(567.6-zPosTuple->GetArgs()[0]) ;
		std::ostringstream legEntry;
		legEntry << "z_{d} = "<<zpos;
		//assign color
		g_z->SetLineColor(colors[i]);
		g_z->SetMarkerColor(colors[i]);
		g_z->SetTitle(legEntry.str().c_str());
		g_rphi->SetLineColor(colors[i]);
		g_rphi->SetMarkerColor(colors[i]);
		g_rphi->SetTitle(legEntry.str().c_str());

		runResZGraph->Add(g_z,"P");
		runResRPhiGraph->Add(g_rphi,"P");

		file->Close();
		delete file;
	}

	va_end(argList);

	//save the results
	outfile->cd();

	TCanvas *c3 = new TCanvas("Triplet Resolution rphi best row","Triplet Resolution rphi best row");
	TGraphErrors *runBestXYResGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runBestXYRes[0], &runZPositionsError[0], &runBestXYResError[0] );
	runBestXYResGraph->SetTitle(Form("module_%i row_%i",bestRow.first,bestRow.second));
	runBestXYResGraph->GetXaxis()->SetTitle("drift distance [mm]");
	runBestXYResGraph->GetYaxis()->SetTitle("#sigma_{r#varphi} [mm]");
	runBestXYResGraph->SetLineColor(colors[0]);
	runBestXYResGraph->SetFillColor(kWhite);
	runBestXYResGraph->Draw("AP");
	gPad->Update();
	runBestXYResGraph->GetYaxis()->SetRangeUser(0,gPad->GetFrame()->GetY2()*1.5);
	c3->SaveAs(Form("%s%s",pathToOutput,"XYTripletResolutionBestRow.png"));
	c3->SaveAs(Form("%s%s",pathToOutput,"XYTripletResolutionBestRow.pdf"));
	runBestXYResGraph->SetName("rphi best row");
	runBestXYResGraph->Write();

	TCanvas *c9 = new TCanvas("rphi triplet Resolution","rphi triplet Resolution");
	runBestXYResGraph->Draw("AP");
	TGraphErrors *meanSingleXYResGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runSingleXYResMean[0], &runZPositionsError[0], &runSingleXYResMeanError[0] );
	meanSingleXYResGraph->SetLineColor(colors[1]);
	meanSingleXYResGraph->SetFillColor(kWhite);
	meanSingleXYResGraph->SetTitle("mean all rows");
	meanSingleXYResGraph->Draw("sameP");
	gPad->BuildLegend(0.5515075,0.2008929,0.8994975,0.3839286);
	c9->SaveAs(Form("%s%s",pathToOutput,"XYTripletResolution.png"));
	c9->SaveAs(Form("%s%s",pathToOutput,"XYTripletResolution.pdf"));
	meanSingleXYResGraph->SetName("rphi mean all rows");
	meanSingleXYResGraph->GetXaxis()->SetTitle("drift distance [mm]");
	meanSingleXYResGraph->GetYaxis()->SetTitle("#sigma_{r#varphi} [mm]");
	meanSingleXYResGraph->Write();

	TCanvas *c4 = new TCanvas("Triplet Resolution z best row","Triplet Resolution z best row");
	TGraphErrors *runBestZResGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runBestZRes[0], &runZPositionsError[0], &runBestZResError[0] );
	runBestZResGraph->SetTitle(Form("module_%i row_%i",bestRow.first,bestRow.second));
	runBestZResGraph->GetXaxis()->SetTitle("drift distance [mm]");
	runBestZResGraph->GetYaxis()->SetTitle("#sigma_{z} [mm]");
	runBestZResGraph->SetLineColor(colors[0]);
	runBestZResGraph->SetFillColor(kWhite);
	runBestZResGraph->Draw("AP");
	gPad->Update();
	runBestZResGraph->GetYaxis()->SetRangeUser(0,gPad->GetFrame()->GetY2()*1.5);
	c4->SaveAs(Form("%s%s",pathToOutput,"ZTripletResolutionBestRow.png"));
	c4->SaveAs(Form("%s%s",pathToOutput,"ZTripletResolutionBestRow.pdf"));
	runBestZResGraph->SetName("z best row");
	runBestZResGraph->Write();

	TCanvas *c10 = new TCanvas("Z Triplet Resolution","Z Triplet Resolution");
	runBestZResGraph->Draw("AP");
	TGraphErrors *meanSingleZResGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runSingleZResMean[0], &runZPositionsError[0], &runSingleZResMeanError[0] );
	meanSingleZResGraph->SetLineColor(colors[1]);
	meanSingleZResGraph->SetFillColor(kWhite);
	meanSingleZResGraph->SetTitle("mean all rows");
	meanSingleZResGraph->Draw("SAMEP");
	gPad->BuildLegend(0.5515075,0.2008929,0.8994975,0.3839286);
	c10->SaveAs(Form("%s%s",pathToOutput,"ZTripletResolution.png"));
	c10->SaveAs(Form("%s%s",pathToOutput,"ZTripletResolution.pdf"));
	meanSingleZResGraph->SetName("z mean all rows");
	meanSingleZResGraph->GetXaxis()->SetTitle("drift distance [mm]");
	meanSingleZResGraph->GetYaxis()->SetTitle("#sigma_{z} [mm]");
	meanSingleZResGraph->Write();

	TCanvas *c5 = new TCanvas("Triplet Resolution rphi all row","Triplet Resolution rphi all row");
	TGraphErrors *runAllXYResGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runAllXYRes[0], &runZPositionsError[0], &runAllXYResError[0] );
	runAllXYResGraph->SetTitle("all rows");
	runAllXYResGraph->GetXaxis()->SetTitle("drift distance [mm]");
	runAllXYResGraph->GetYaxis()->SetTitle("#sigma_{r#varphi} [mm]");
	runAllXYResGraph->Draw("AP");
	runAllXYResGraph->GetYaxis()->SetRangeUser(0,gPad->GetFrame()->GetY2()*1.5);
	c5->SaveAs(Form("%s%s",pathToOutput,"XYTripletResolutionAllRows.png"));
	c5->SaveAs(Form("%s%s",pathToOutput,"XYTripletResolutionAllRows.pdf"));
	runAllXYResGraph->SetName("rphi all rows");
	runAllXYResGraph->Write();

	TCanvas *c6 = new TCanvas("Triplet Resolution z all row","Triplet Resolution z all row");
	TGraphErrors *runAllZResGraph = new TGraphErrors( runZPositions.size(), &runZPositions[0], &runAllZRes[0], &runZPositionsError[0], &runAllZResError[0] );
	runAllZResGraph->SetTitle("all rows");
	runAllZResGraph->GetXaxis()->SetTitle("drift distance [mm]");
	runAllZResGraph->GetYaxis()->SetTitle("#sigma_{z} [mm]");
	runAllZResGraph->Draw("AP");
	runAllZResGraph->GetYaxis()->SetRangeUser(0,gPad->GetFrame()->GetY2()*1.5);
	c6->SaveAs(Form("%s%s",pathToOutput,"ZTripletResolutionAllRows.png"));
	c6->SaveAs(Form("%s%s",pathToOutput,"ZTripletResolutionAllRows.pdf"));
	runAllZResGraph->SetName("z all rows");
	runAllZResGraph->Write();

	TCanvas *c7 = new TCanvas("Triplet Resolution rphi single row","Triplet Resolution rphi single row");
	TGraphErrors *runSingleXYResGraph = new TGraphErrors( runZPositionsSingleRow.size(), &runZPositionsSingleRow[0], &runSingleXYRes[0], &runZPositionsSingleRowError[0], &runSingleXYResError[0] );
	runSingleXYResGraph->SetTitle("single rows");
	runSingleXYResGraph->GetXaxis()->SetTitle("drift distance [mm]");
	runSingleXYResGraph->GetYaxis()->SetTitle("#sigma_{r#varphi} [mm]");
	runSingleXYResGraph->Draw("AP");
	c7->SaveAs(Form("%s%s",pathToOutput,"XYTripletResolutionSingleRows.png"));
	c7->SaveAs(Form("%s%s",pathToOutput,"XYTripletResolutionSingleRows.pdf"));

	TCanvas *c8 = new TCanvas("Triplet Resolution z single row","Triplet Resolution z single row");
	TGraphErrors *runSingleZResGraph = new TGraphErrors( runZPositionsSingleRow.size(), &runZPositionsSingleRow[0], &runSingleZRes[0], &runZPositionsSingleRowError[0], &runSingleZResError[0] );
	runSingleZResGraph->SetTitle("single rows");
	runSingleZResGraph->GetXaxis()->SetTitle("drift distance [mm]");
	runSingleZResGraph->GetYaxis()->SetTitle("#sigma_{z} [mm]");
	runSingleZResGraph->Draw("AP");
	c8->SaveAs(Form("%s%s",pathToOutput,"ZTripletResolutionSingleRows.png"));
	c8->SaveAs(Form("%s%s",pathToOutput,"ZTripletResolutionSingleRows.pdf"));

	gStyle->SetPadRightMargin(0.18);

	TCanvas *c1 = new TCanvas("Triplet Resolution rphi ","Triplet Resolution rphi");
	runResRPhiGraph->Draw("A");
	gPad->BuildLegend(0.85,0.2,0.99,0.99,"[mm]");
	c1->SaveAs(Form("%s%s",pathToOutput,"XYTripletResolutionRow.png"));
	c1->SaveAs(Form("%s%s",pathToOutput,"XYTripletResolutionRow.pdf"));


	TCanvas *c2 = new TCanvas("Triplet Resolution z","Triplet Resolution z");
	runResZGraph->Draw("A");
	gPad->BuildLegend(0.85,0.2,0.99,0.99,"[mm]");
	c2->SaveAs(Form("%s%s",pathToOutput,"ZTripletResolutionRow.png"));
	c2->SaveAs(Form("%s%s",pathToOutput,"ZTripletResolutionRow.pdf"));

	outfile->Write();
	outfile->Close();
	gStyle->SetPadRightMargin(0.04);

	//	delete c1;
	//	delete c2;
	//	delete c3;
	//	delete c4;
	//	delete c5;
	//	delete c6;
	//	delete c7;
	//	delete c8;

}
