#include <vector>
#include <iostream>
#include "TLatex.h"
#include "TCanvas.h"
#include "TH1.h"
#include "TGraphErrors.h"
#include "TMultiGraph.h"
#include "TColor.h"
#include "TFile.h"
#include "TNtuple.h"
#include "TStyle.h"
#include "TROOT.h"
#include "sstream"
#include <cstdarg>

/*How to call this:
.L $MARLINTPC/analysis/rootscripts/dEdxPerformance.C++
DistortionPerformance("path for output","path to the aida file", number of runs in the following list, list of the runs)
e.g dEdxPerformance("../myplots","../AidaFiles/",3,18850,18851,18860)
ideally run umber are given already ordered in z for color coding to work
*/
void dEdxPerformance(char * pathToOutput,char * pathToAida, int nRuns, ...){

  //loading the style
  gROOT->ProcessLine(".L ${MARLINTPC}/analysis/rootscripts/LCTPCStyle.C");
  gROOT->ProcessLine("LCTPCStyle()");

  //open a root file to store the graphs in
  TFile *outfile=new TFile(Form("%s%s",pathToOutput,"dEdxPerformance.root"),"RECREATE");
  
  //initialize the input list
  va_list argList;
  va_start(argList,nRuns);

  int runNumber;

  // will contain the results from the processors
  TMultiGraph *run_dEdx=new TMultiGraph();
  TMultiGraph *run_dEdx2=new TMultiGraph();
  run_dEdx->SetName("dEdx");
  run_dEdx->SetTitle("dEdx; track curvature; dEdx [arb. units]");
  run_dEdx2->SetName("dEdx");
  run_dEdx2->SetTitle("dEdx; run number; dEdx [arb. units]");

  std::vector<double> runMean90;
  std::vector<double> runMean90Err;
  std::vector<double> runXPosition;
  std::vector<double> runXPosition2;
  std::vector<double> runXPositionErr;

  //create colors
  const int BINNING=nRuns;
  Int_t colors[BINNING];

  const Int_t NRGBs = 5;

  Double_t stops[NRGBs] = { 0.00, 0.34, 0.61, 0.84, 1.00 };
  Double_t red[NRGBs]   = { 0.00, 0.00, 0.87, 1.00, 0.51 };
  Double_t green[NRGBs] = { 0.00, 0.81, 1.00, 0.20, 0.00 };
  Double_t blue[NRGBs]  = { 0.51, 1.00, 0.12, 0.00, 0.00 };
  Int_t FI = TColor::CreateGradientColorTable(NRGBs, stops, red, green, blue, BINNING);

  for (int i=0;i<BINNING;i++)
    colors[i] = FI+i;


  //loop over the given runs
  for (int i=0;i<nRuns;i++)
    {
      //get run number from input
      runNumber=va_arg(argList,int);

      //create stringstream containing the aida file name
      std::stringstream runNumberString;
      std::cout<<"run number: "<<runNumber<<std::endl;
      runNumberString <<pathToAida<<"aida_run" <<runNumber<< ".root";

      //open the aida file
      TFile *file  = TFile::Open(runNumberString.str().c_str(),"Read");
      //check that the file exists
      if (file==0)
	{
	  std::cout<<"This run will be skipped. The aida file does not exist. Please check this!"<<std::endl;
	  continue;
	}

      //get the tuples from the file
      TNtuple * Tuple=(TNtuple*)file->Get("MydEdxProcessor/tuple");
      if (Tuple==0 )
	{
	  std::cout<<"This run will be skipped. One of the necessary root tuples does not exist. Please check that you ran the corresponding processors and that the tuple is in the aida file!"<<std::endl;
	  continue;
	}
      //open the first and only entry of the tuples
      Tuple->GetEntry(0);
      for(int n=0;n<Tuple->GetEntries();n++)
	{
	  Tuple->GetEntry(n);
	  runMean90.push_back(Tuple->GetArgs()[2]);
	  runMean90Err.push_back(Tuple->GetArgs()[3]);
	  // runXPosition.push_back(fabs(0.3/Tuple->GetArgs()[6])/1000.);
	  runXPosition.push_back(Tuple->GetArgs()[6]);
	  runXPosition2.push_back(runNumber);
	  runXPositionErr.push_back(0);
	}
      
      TGraphErrors *g=new TGraphErrors(Tuple->GetEntries(),&runXPosition[0],&runMean90[0],&runXPositionErr[0],&runMean90Err[0]);
      TGraphErrors *g2=new TGraphErrors(Tuple->GetEntries(),&runXPosition2[0],&runMean90[0],&runXPositionErr[0],&runMean90Err[0]);
      //assign color
      g->SetLineColor(colors[i]);
      g->SetMarkerColor(colors[i]);
      g2->SetLineColor(colors[i]);
      g2->SetMarkerColor(colors[i]);
     
      run_dEdx->Add(g,"P");
      run_dEdx2->Add(g2,"P");
      runMean90.clear();
      runMean90Err.clear();
      runXPosition.clear();
      runXPosition2.clear();
      runXPositionErr.clear();
    }

  va_end(argList);

  TCanvas *c1 = new TCanvas("dEdx","dEdx");
  run_dEdx->Draw("A");
  c1->SaveAs(Form("%s%s",pathToOutput,"dEdx_vsOmega.png"));
  c1->SaveAs(Form("%s%s",pathToOutput,"dEdx_vsOmega.pdf"));
  TCanvas *c2 = new TCanvas("dEdx2","dEdx2");
  run_dEdx2->Draw("A");
  c2->SaveAs(Form("%s%s",pathToOutput,"dEdx_vsRun.png"));
  c2->SaveAs(Form("%s%s",pathToOutput,"dEdx_vsRun.pdf"));

  //save the results
  outfile->cd();
  run_dEdx->Write();
  run_dEdx2->Write();
  outfile->Write();
  outfile->Close();
  gStyle->SetPadRightMargin(0.04);
  delete c1;
  delete c2;


}
