#include "GEMPulsingCalibrationProcessor.h"
#include <iostream>
#include <string>
#include <vector>

// include what you need from lcio
#include <EVENT/LCEvent.h>
#include <IMPL/LCCollectionVec.h>
#include <EVENT/TrackerPulse.h>
#include <Exceptions.h>

//MarliNTPC
#include "TPCCondDataExeptions.h"

//GEAR
#include "gear/TPCModule.h"



using namespace lcio;
using namespace marlin;
using namespace std;


namespace marlintpc
{

	GEMPulsingCalibrationProcessor aGEMPulsingCalibrationProcessor;


	GEMPulsingCalibrationProcessor::GEMPulsingCalibrationProcessor() : Processor("GEMPulsingCalibrationProcessor")
	{
	  // the processor description
	  _description = "GEMPulsingCalibrationProcessor calibrates the time of pulses with a textfile which is generated in the GEMPulsingTimeEventAnalysisProcessor and includes calculatet deviationtimes. This processor changes the time of the pulses.";

	  /* register steering parameters: name, description, class-variable, default value

		for example:
	  */

		    registerInputCollection(LCIO::TRACKERPULSE,
		                            "InputCollectionName" ,
		                            "Name of the input collection"  ,
		                            _inputColName ,
		                            std::string("TPCPulses")
		                           );

		    registerOutputCollection(LCIO::TRACKERPULSE,
		                             "OutputCollectionName" ,
		                             "Name of the output collection"  ,
		                             _outputColName ,
		                             std::string("TPCPulsesTimeCalibrated")
		                            );

		    registerProcessorParameter("WriteOutputToStorage",
		                               "if true the output collection is written",
		                               _outputIsPersistent,
		                               true);

		    registerOptionalParameter("deviationInputFileNameTime",
		                              "Input file name for the time-deviation",
		                              _inputfilenameTime,
		                              std::string("DeviationTime.txt"));

//		    registerOptionalParameter("deviationInputFileNameCharge",
//		                              "Input file name for the charge-deviation",
//		                              _inputfilenameCharge,
//		                              std::string("DeviationCharge.txt"));



	  /*
			registerOutputCollection(   LCIO::TRACKERDATA,
			  "OutputCollectionName" ,
			  string("Name of the output TrackerData collection")+
			  string("only the data related to a pulse will be stored")+
			  string(\, default: TPCDataInPulses),
			  _outputCollectionName ,
			  string("TPCDataInPulses")) ;

			// It's nive to have the default value in the comment
			registerProcessorParameter( "MinPulsesPerHit" ,
			   "Minimum number of pulses for a valid hit, default: 3",
			   _minPulsesPerHit,
			   int(3));

			// Optional parameters should only be used when set
			// in the steering file.
			// The default value is there only for technical reasons,
			// it is meaningless.
			//
			// If properly implemented the cut is not applied when the parameter is not set:
			//   if (parameterSet("ChageCut")) { //apply the cut }
			//
			registerOptionalParameter( "ChargeCut" ,
				"If the charge is below this value the data will be ignored",
				_chargeCut,
			   float(17.8));
	  */
	}


	void GEMPulsingCalibrationProcessor::init()
	{
		streamlog_out(DEBUG) << " init() called  " << endl;

	    printParameters();


	    // read in time-file

	    ifstream deviationInTime;
	string lineInTime;

	streamlog_out( MESSAGE ) << "inputfilename_Time: " << _inputfilenameTime << endl;

	deviationInTime.open(_inputfilenameTime.c_str());
	if( !deviationInTime )
			{
				streamlog_out( ERROR ) << "TimeDeviationData could not be opened!" << endl;
				throw StopProcessingException(this);
			}

	int line = 0;
	int moduleTime, rowTime;
	float timeDeviationAverage, timeDeviationRMS;

	while( deviationInTime.good() )
			{

				getline( deviationInTime, lineInTime );
				stringstream lineInTimeStream ( lineInTime );

				if( !( lineInTimeStream >> moduleTime >> rowTime >> timeDeviationAverage >> timeDeviationRMS ) )
				{
					streamlog_out( DEBUG4 ) << "Skipping line" << endl;
					continue;
				}

				streamlog_out( DEBUG4 ) << "Reading Line " << line << endl;

				std::pair<int, int> key = std::make_pair( moduleTime, rowTime );
				std::pair<float, float> valueRow = std::make_pair( timeDeviationAverage, timeDeviationRMS );
				_timeDeviationMap.insert( std::make_pair( key, valueRow ) );

				streamlog_out( DEBUG ) << timeDeviationAverage << " " << timeDeviationRMS << std::endl;
				++line;


				if( deviationInTime.bad() || line==0 )
				{
					streamlog_out( ERROR ) << "Error while reading the deviation file DeviationTime.txt" << endl;
					throw StopProcessingException(this);
				}

			}
	deviationInTime.close();

	}


	void GEMPulsingCalibrationProcessor::processRunHeader(LCRunHeader* run)
	{
	  // do something for each run

	    run->parameters().setValue(_processorName + "_revision", "$Rev: 2453 $");
	    for (ProcParamMap::iterator i = _map.begin(); i != _map.end(); i++)
	      {
	        if (! i->second->isOptional() || i->second->valueSet())
	          {
	            	run->parameters().setValue(_processorName + "_" + i->second->name(), i->second->value());
	          }
	      }
	}


	void GEMPulsingCalibrationProcessor::processEvent(LCEvent* evt)
	{
	  // do something for each event, usually access the input collection:

	  // use an LCCollectionVec instead of LCCollection so we can use iterators later

	    LCCollectionVec *inputCol = NULL;
	    // get the input collection from the event
	    try
	      {
	        inputCol = dynamic_cast<LCCollectionVec*>(evt->getCollection(_inputColName)) ;
	        if (inputCol->size() == 0)
	          {
	            throw DataNotAvailableException(std::string("Empty pulses collection ") + _inputColName);
	          }
	      }
	    catch (DataNotAvailableException &e)
	      {
	        streamlog_out(WARNING) << _processorName << " : Empty event " << evt->getEventNumber() << endl;
	        //      throw marlin::SkipEventException( this);
	        return;
	      }

	    LCCollectionVec* outputCol = new LCCollectionVec(LCIO::TRACKERPULSE);
	    outputCol->setTransient(!_outputIsPersistent);   // transience is the opposite of persistency

	    if (!parameterSet("OutputCollectionName"))
	      {
	        cerr << " ERROR: no output collection is provided; this will very probably result in something you don't want!" << endl;
	        cerr << " please provide an output collection name that is *ALSO* used in the succeeding processor(s)! " << endl;
	        // slight abuse of the type of exception, but this is the least misleading
	        throw lcio::EventException("No output collection for the ChannelMappingProcessor given, please provide a name!");
	      }

	    // set the flags that cellID1 should be stored
	    //switch on error covariance matrix
	    lcio::LCFlagImpl trkFlag(0) ;
	    trkFlag.setBit(lcio::LCIO::TRAWBIT_ID1) ;
	    trkFlag.setBit(lcio::LCIO::TRAWBIT_CM);
	    outputCol->setFlag(trkFlag.getFlag());


	    int nData = inputCol->getNumberOfElements();

	        // loop over the number of elements of the input collection
	        for (int iData = 0; iData < nData ; ++iData)
	          {
	            /* getElement return only a LCObject
	                * you have to cast it the get the right type
	                * please use dynamic_cast which will return 0 if the object can not be casted
	                */
	            TrackerPulseImpl* pulse = LCObjectCopier::copy(dynamic_cast<TrackerPulse*>(inputCol->getElementAt(iData)));

	            try
	              {

				std::pair<int,int> key = make_pair((pulse->getCellID1()), (pulse->getCellID0()));
				float deviationTime = ((*_timeDeviationMap.find(key)).second).first;
				float newTime = (pulse->getTime())- deviationTime;

				streamlog_out(DEBUG) << "deviationTime = "<< deviationTime << std::endl;
				streamlog_out(DEBUG) << "old time = "<< pulse->getTime() << std::endl;
				streamlog_out(DEBUG) << "new time = "<< newTime << std::endl;

				pulse ->setTime(newTime);
				outputCol->addElement(pulse);

	              }
	            catch (std::exception &e)
	              {


	                    	streamlog_out(MESSAGE) << "no map entry for cellid " << pulse->getCellID0() << "," << pulse->getCellID1() << std::endl;

	              }
	          }

	        // do not write empty collections (can happen if only unmapped channels were in the input)
	        if (!outputCol->empty())
				{
				  evt->addCollection(outputCol, _outputColName);
				}
	        else
				{
				  // delete the CollectionVec if not added to the event to avoid a memory leak
				  delete outputCol;
				}

	}


	void GEMPulsingCalibrationProcessor::check(LCEvent*  /*evt*/)
	{
	  // check what is needed here
	}


	void GEMPulsingCalibrationProcessor::end()
	{
	  // do something at the end of the processing, e.g. cleaning up


	}
} // close namespace brackets

