#include "GEMPulsingTimeEventAnalysisProcessor.h"
#include <iostream>
#include <fstream>
#include <string>
#include <vector>

// include what you need from lcio
#include <EVENT/LCEvent.h>
#include <IMPL/LCCollectionVec.h>
#include <EVENT/TrackerPulse.h>
#include <Exceptions.h>

//MarliNTPC
#include "ChannelMappingListener.h"
#include "ChannelMappingProcessor.h"
#include "TPCCondDataExeptions.h"

//GEAR
#include "gear/TPCModule.h"



using namespace lcio;
using namespace marlin;
using namespace std;


namespace marlintpc
{

	GEMPulsingTimeEventAnalysisProcessor aGEMPulsingTimeEventAnalysisProcessor;


	GEMPulsingTimeEventAnalysisProcessor::GEMPulsingTimeEventAnalysisProcessor() : Processor("GEMPulsingTimeEventAnalysisProcessor")
	{
	  // the processor description
	  _description = "GEMPulsingTimeEventAnalysisProcessor analyses the times of pulses on eventbase. It generates one outputtextfile, which is used to calibrate the pulsetimes for each channel in the GEMPulsingCalibrationProcessor.";

	  /* register steering parameters: name, description, class-variable, default value

		for example:
	  */

			registerInputCollection(    LCIO::TRACKERPULSE,
				"InputCollectionName" ,
				"Name of the input TrackerData collection, default: TPCData"  ,
				_inputColName ,
				string("TPCPulses")) ;

		    registerProcessorParameter("ChannelMappingCollectionName" ,
			   "Name of the collection containing the ChannelMappings",
			   _channelMappingColName,
			   tpcconddata::ADCChannelMapping::getDefaultColName());

		    registerOptionalParameter("DeviationOutputFileName",
		                              "Output file name for the deviation",
		                              _filename,
		                              std::string("DeviationTime.txt"));

		    registerOptionalParameter("CreateOutputFile",
		                              "create an output file yes/no",
									  _createOutput,
		                              bool(true));


		    registerOptionalParameter("ChipToChipMin",
		                              "Minimum for Chip to Chip Histogram",
									  _chipToChipMin,
		                              int(0));


		    registerOptionalParameter("ChipToChipMax",
                    				  "Maximum for Chip to Chip Histogram",
									  _chipToChipMax,
		                              int(0));

		    registerOptionalParameter("ChipToChipRMS",
		                              "Maximum for Chip to Chip Histogram RMS",
									  _chipToChipRMS,
		                              int(0));

		    registerOptionalParameter("CableToCableMin",
		                              "Minimum for Cable to Cable Histogram",
									  _cableToCableMin,
		                              int(0));

		    registerOptionalParameter("CableToCableMax",
		                              "Maximum for Cable to Cable Histogram",
									  _cableToCableMax,
		                              int(0));

		    registerOptionalParameter("CableToCableRMS",
		                              "Maximum for Cable to Cable Histogram RMS",
									  _cableToCableRMS,
		                              int(0));

		    registerOptionalParameter("ChipToChipDeviationRange",
                    				  "Range for Chip to Chip Deviation Histogram",
									  _chipToChipDeviationRange,
		                              int(0));


		    registerOptionalParameter("ChipToChipDeviationRMS",
                    				  "Range for Chip to Chip Deviation Histogram RMS",
									  _chipToChipDeviationRMS,
		                              int(0));

		    registerOptionalParameter("CableToCableDeviationRange",
                    				  "Range for Cable to Cable Deviation Histogram",
									  _cableToCableDeviationRange,
		                              int(0));

		    registerOptionalParameter("CableToCableDeviationRMS",
                    				  "Range for Cable to Cable Deviation Histogram RMS",
									  _cableToCableDeviationRMS,
		                              int(0));

		    registerOptionalParameter("eventTimeAverageMin",
                    				  "Minimum for event Time Average",
									  _eventTimeAverageMin,
		                              int(0));

		    registerOptionalParameter("eventTimeAverageMax",
                    				  "Maximum for event Time Average",
									  _eventTimeAverageMax,
		                              int(0));

		    registerOptionalParameter("eventTimeAverageRMS",
                    				  "Maximum for event Time Average RMS",
									  _eventTimeAverageRMS,
		                              int(0));

	  /*
			registerOutputCollection(   LCIO::TRACKERDATA,
			  "OutputCollectionName" ,
			  string("Name of the output TrackerData collection")+
			  string("only the data related to a pulse will be stored")+
			  string(\, default: TPCDataInPulses),
			  _outputCollectionName ,
			  string("TPCDataInPulses")) ;

			// It's nive to have the default value in the comment
			registerProcessorParameter( "MinPulsesPerHit" ,
			   "Minimum number of pulses for a valid hit, default: 3",
			   _minPulsesPerHit,
			   int(3));

			// Optional parameters should only be used when set
			// in the steering file.
			// The default value is there only for technical reasons,
			// it is meaningless.
			//
			// If properly implemented the cut is not applied when the parameter is not set:
			//   if (parameterSet("ChageCut")) { //apply the cut }
			//
			registerOptionalParameter( "ChargeCut" ,
				"If the charge is below this value the data will be ignored",
				_chargeCut,
			   float(17.8));
	  */
	}


	void GEMPulsingTimeEventAnalysisProcessor::init()
	{
	  streamlog_out(DEBUG) << " init() called  " << endl;

	  // you might want to list the used parameters by invoking
	  printParameters();
		AIDAProcessor::tree(this);


		_timeCalibrationTuple = new TNtuple("timeCalibrationTuple","timeCalibrationTuple","module:pad:deviation:RMS");

		_chipToChipTimeAverage = new TH1F ( "ChipToChip_Time_Average", "Time average of chips", 100, _chipToChipMin, _chipToChipMax );
		_chipToChipTimeAverage->GetXaxis()->SetTitle("Average");
		_chipToChipTimeAverage->GetYaxis()->SetTitle("Chips");

		_RMSChipTime = new TH1F ( "TimeRMSChip", "Time-RMS of chips", 100, 0, _chipToChipRMS );
		_RMSChipTime ->GetXaxis()->SetTitle("Time-RMS");
		_RMSChipTime ->GetYaxis()->SetTitle("Chips");

		_cableToCableTimeAverage = new TH1F ( "CableToCable_Time_Average", "Time average of cables", 100, _cableToCableMin, _cableToCableMax);
		_cableToCableTimeAverage->GetXaxis()->SetTitle("Average");
		_cableToCableTimeAverage->GetYaxis()->SetTitle("Cables");

		_RMSCableTime = new TH1F ( "TimeRMSCable", "Time-RMS of Cable", 100, 0, _cableToCableRMS );
		_RMSCableTime ->GetXaxis()->SetTitle("Time-RMS");
		_RMSCableTime ->GetYaxis()->SetTitle("Cable");

		_chipToChipTimeAverageDeviation = new TH1F ( "ChipToChip_Time_Average_Deviation", "Time average_Deviation of chips", 100, -_chipToChipDeviationRange, _chipToChipDeviationRange );
		_chipToChipTimeAverageDeviation->GetXaxis()->SetTitle("Average (Deviation)");
		_chipToChipTimeAverageDeviation->GetYaxis()->SetTitle("Chips");

		_RMSChipTimeDeviation = new TH1F ( "TimeRMSChip_Deviation", "Time-RMS_Deviation of chips", 100, 0, _chipToChipDeviationRMS );
		_RMSChipTimeDeviation ->GetXaxis()->SetTitle("Time-RMS (Deviation)");
		_RMSChipTimeDeviation ->GetYaxis()->SetTitle("Chips");

		_cableToCableTimeAverageDeviation = new TH1F ( "CableToCable_Time_Average_Deviation", "Time average_Deviation of cables", 100, -_cableToCableDeviationRange, _cableToCableDeviationRange);
		_cableToCableTimeAverageDeviation->GetXaxis()->SetTitle("Average (Deviation)");
		_cableToCableTimeAverageDeviation->GetYaxis()->SetTitle("Cables");

		_RMSCableTimeDeviation = new TH1F ( "TimeRMSCable_Deviation", "Time-RMS_Deviation of Cable", 100, 0, _cableToCableDeviationRMS );
		_RMSCableTimeDeviation ->GetXaxis()->SetTitle("Time-RMS (Deviation)");
		_RMSCableTimeDeviation ->GetYaxis()->SetTitle("Cables");

		_eventAverage = new TH1F ("event_timeAverages", "time-average per event", 50, _eventTimeAverageMin, _eventTimeAverageMax);
		_eventAverage ->GetXaxis()->SetTitle("Time [ns]");
		_eventAverage ->GetYaxis()->SetTitle("#entries");

		_eventRMS = new TH1F ("event_timeRMS", "time-RMS per event", 50, 0, _eventTimeAverageRMS);
		_eventRMS ->GetXaxis()->SetTitle("RMS [ns]");
		_eventRMS ->GetYaxis()->SetTitle("#entries");

		_eventAverageTemp = new TH1F ("event_timeAveragesTemp", "time-averageTemp per event", 50, _eventTimeAverageMin, _eventTimeAverageMax);
		_eventRMSTemp = new TH1F ("event_timeRMSTemp", "time-RMSTemp per event", 50, 0, _eventTimeAverageRMS);

		_padToPadTimeDeviationAverageXY = new TH2F ( "PulseTimeDeviationMeanVSXY", "Time of pulses (Deviation Mean)", 100, 0, 0, 100, 0, 0 );
		_padToPadTimeDeviationAverageXY ->GetXaxis()->SetTitle("x [mm]");
		_padToPadTimeDeviationAverageXY ->GetYaxis()->SetTitle("y [mm]");
		_padToPadTimeDeviationAverageXY ->GetZaxis()->SetTitle("Deviation [ns]");

		_padToPadTimeDeviationRMSXY = new TH2F ( "PulseTimeDeviationRMSVSXY", "Time of pulses (Deviation RMS)", 100, 0, 0, 100, 0, 0 );
		_padToPadTimeDeviationRMSXY ->GetXaxis()->SetTitle("x [mm]");
		_padToPadTimeDeviationRMSXY ->GetYaxis()->SetTitle("y [mm]");
		_padToPadTimeDeviationRMSXY ->GetZaxis()->SetTitle("Deviation [ns]");

		_chipToChipTimeAverageXY = new TH2F ( "PulseTimeVSXY_Chip", "Time of pulses (chip)", 100, 0, 0, 100, 0, 0 );
		_chipToChipTimeAverageXY ->GetXaxis()->SetTitle("x [mm]");
		_chipToChipTimeAverageXY ->GetYaxis()->SetTitle("y [mm]");
		_chipToChipTimeAverageXY ->GetZaxis()->SetTitle("Deviation [ns]");

		_cableToCableTimeAverageXY = new TH2F ( "PulseTimeVSXY_Cable", "Time of pulses (cable)", 100, 0, 0, 100, 0, 0 );
		_cableToCableTimeAverageXY ->GetXaxis()->SetTitle("x [mm]");
		_cableToCableTimeAverageXY ->GetYaxis()->SetTitle("y [mm]");
		_cableToCableTimeAverageXY ->GetZaxis()->SetTitle("Deviation [ns]");

		_channelMappingListener = new ChannelMappingListener(_channelMappingColName);

		_gearParameters= &marlin::Global::GEAR->getTPCParameters();
	}


	void GEMPulsingTimeEventAnalysisProcessor::processRunHeader(LCRunHeader*  /*run*/)
	{
	  // do something for each run
	}


	void GEMPulsingTimeEventAnalysisProcessor::processEvent(LCEvent* evt)
	{
		AIDAProcessor::tree(this);

		  // do something for each event, usually access the input collection:

		  // use an LCCollectionVec instead of LCCollection so we can use iterators later
		  LCCollectionVec* inputColVec = 0;

		  // do a try / catch in case there is an empty event
		  try
		  {
			inputColVec = dynamic_cast<LCCollectionVec *>(evt->getCollection(_inputColName) );
		  }
		  catch (DataNotAvailableException &e)
		  {
			// no input data, just return and process the next event
			streamlog_out(MESSAGE) << "No pulses available in event " << evt->getEventNumber() << endl;
			return;
		  }

		  // loop the data collection and do something
		  for( LCCollectionVec::iterator inputObjectIter = inputColVec->begin() ;
			   inputObjectIter < inputColVec->end() ;
			   inputObjectIter++ )
		  {


				// dynamic cast the input object (LCObject) to what it really is e.g.

				TrackerPulse * myTrackerPulse = dynamic_cast<TrackerPulse *>( *inputObjectIter );

				if(myTrackerPulse!=0)
				{
					if(myTrackerPulse->getQuality() != 0)
						continue;
						// for objects derived from LCGenericObject or LCFixedObject (like conditions data)
						// you have to use special constructor
						// TPCConditions myTPCConditions( dynamic_cast<LCGenericObject*>( *inputObjectIter ) );
						//now do something with the data ...

						int modul = myTrackerPulse->getCellID1();
						int row   = myTrackerPulse->getCellID0();
						std::pair<int, int> key(modul, row);
						std::map< std::pair<int,int>, std::vector<float> >::iterator vectoriter;
						vectoriter = _inputPulseMap.find(key);

	//						insert vector at key
						if(vectoriter == _inputPulseMap.end())
							{
							std::vector<float> tempVec;
							_inputPulseMap.insert( std::make_pair(key,tempVec));
							}

	//						insert value in vector at key
						vectoriter = _inputPulseMap.find(key);
						try
						{
							float time = myTrackerPulse->getTime();
							((*vectoriter).second).push_back(time);
						}
						catch(std::exception &e)
						{

						}
				}
		  }



//			build Map with time-averages of each pad each event
			std::map< std::pair<int,int>, std::vector<float> >::iterator it;
			for (it=_inputPulseMap.begin(); it!= _inputPulseMap.end(); ++it)
			{
					try
					{
						std::vector <float> tempvec = (*it).second;
						double tempaverage = TMath::Mean(tempvec.begin(), tempvec.end());
						_inputPulseMapAverage.insert(std::make_pair((*it).first, tempaverage));
					}
					catch (tpcconddata::CondDataNotAvailableException& e)
					{

					}
			}

//			Calculate Average of Pads per event
			std::vector<double> eventTimesTemp;

			std::map< std::pair<int,int>, double >::iterator ii;
			for (ii=_inputPulseMapAverage.begin(); ii!= _inputPulseMapAverage.end(); ++ii)
			{
					eventTimesTemp.push_back((*ii).second);
			}

			double eventAverageTemp = TMath::Mean(eventTimesTemp.begin(), eventTimesTemp.end());
			_eventAverageTemp -> Fill(eventAverageTemp);

			double eventRMSTemp = TMath::RMS(eventTimesTemp.begin(), eventTimesTemp.end());
			_eventRMSTemp -> Fill(eventRMSTemp);

			std::vector<double> eventTimesTemp2;
			std::map< std::pair<int,int>, double >::iterator iii;


			for (iii=_inputPulseMapAverage.begin(); iii!= _inputPulseMapAverage.end(); ++iii)
			{
				if(abs((*iii).second-eventAverageTemp) < 5*eventRMSTemp)
				{
					eventTimesTemp2.push_back((*iii).second);
				}
			}

			if((TMath::RMS(eventTimesTemp2.begin(), eventTimesTemp2.end()))>20)
					{
						streamlog_out(DEBUG) << "second if-loop RMS" << endl;
						std::vector<double> eventTimesTemp2;
						std::map< std::pair<int,int>, double >::iterator iii;
						for (iii=_inputPulseMapAverage.begin(); iii!= _inputPulseMapAverage.end(); ++iii)
						{
							if(abs((*iii).second-eventAverageTemp) < 5*eventRMSTemp)
							{
								eventTimesTemp2.push_back((*iii).second);
							}
						}
					}


			double event_average = TMath::Mean(eventTimesTemp2.begin(), eventTimesTemp2.end());
			_eventAverage -> Fill(event_average);

			double eventRMS = TMath::RMS(eventTimesTemp2.begin(), eventTimesTemp2.end());
			_eventRMS -> Fill(eventRMS);


			eventTimesTemp.clear();
			eventTimesTemp2.clear();


//			calculate deviation of each pad to the average
			std::map< std::pair<int,int>, double >::iterator i;
			for (i=_inputPulseMapAverage.begin(); i!= _inputPulseMapAverage.end(); ++i)
			{

					std::map< std::pair<int,int>, std::vector<double> >::iterator vectoriter1;
					vectoriter1 = _timeDeviationMap.find((*i).first);

//					insert vector at key
					if(vectoriter1 == _timeDeviationMap.end())
						{
							std::vector<double> tempVec1;
							_timeDeviationMap.insert( std::make_pair((*i).first,tempVec1));
						}


//					insert value in vector at key
					vectoriter1 = _timeDeviationMap.find((*i).first);
					try
					{
						double deviationtemp = ((*i).second) - event_average;
						if (abs(deviationtemp) < 5* eventRMS)
						{
							((*vectoriter1).second).push_back(deviationtemp);
						}

					}

					catch (std::exception &e)
					{
						streamlog_out(WARNING)<<"calculateDistanceRow for distances failed. Value ignored!"<<std::endl;
					}
			}


//			chip to chip and cable to cable
			vector<double> chipAverage;
			vector<double> cableAverage;
			int CounterNew = 0;
			int CounterOld = 0;

			streamlog_out(DEBUG) << "MapSize: " << _inputPulseMap.size() << endl;

			std::map< std::pair<int,int>, double >::iterator iter;
			for (iter=_inputPulseMapAverage.begin(); iter!= _inputPulseMapAverage.end(); ++iter)
			{
				CounterNew = (*iter).first.second;

//				Chip to chip (16 pads are one chip)
				if ((int)(CounterNew/16) == (int)(CounterOld/16))
				{
					chipAverage.push_back((*iter).second);
				}
				else
				{
					std::pair<int,int> key = make_pair(((*iter).first).first, (int)(CounterOld/16) );
					std::map< std::pair<int,int>, std::pair<int, std::pair< std::vector<double>, std::vector<double> > > >::iterator itermap;

					itermap = _chipMap.find(key);

//					insert vector at key
					if (itermap == _chipMap.end())
					{
						std::vector<double> tempVecchipMean;
						std::vector<double> tempVecchipRMS;
						std::pair<std::vector<double>, std::vector<double> > tempVecChip = std::make_pair(tempVecchipMean, tempVecchipRMS);
						std::pair<int, std::pair<std::vector<double>, std::vector<double> > > tempValue = std::make_pair((*iter).first.second, tempVecChip);
						_chipMap.insert( std::make_pair(key,tempValue));
					}

					itermap = _chipMap.find(key);
//					insert value in vector at key
					try
					{
						double chipAveragetemp = TMath::Mean(chipAverage.begin(), chipAverage.end());
						double ChipRMStemp = TMath::RMS(chipAverage.begin(), chipAverage.end());
						((*itermap).second.second.first).push_back(chipAveragetemp);
						((*itermap).second.second.second).push_back(ChipRMStemp);
					}

					catch (std::exception &e)
					{

					}

					chipAverage.clear();
					chipAverage.push_back((*iter).second);

					streamlog_out(DEBUG) << (*iter).first.second << endl;
				}

//				Cable to cable (32 pads are one cable)
				if ((int)(CounterNew/32) == (int)(CounterOld/32))
				{
					cableAverage.push_back((*iter).second);
				}
				else
				{
					std::pair<int,int> key = make_pair(((*iter).first).first, (int)(CounterOld/32) );
					std::map< std::pair<int,int>, std::pair<int, std::pair< std::vector<double>, std::vector<double> > > >::iterator itermap2;

					itermap2 = _cableMap.find(key);

//					insert vector at key
					if (itermap2 == _cableMap.end())
					{
						std::vector<double> tempVecCableMean;
						std::vector<double> tempVecCableRMS;
						std::pair<std::vector<double>, std::vector<double> > tempVecCable = std::make_pair(tempVecCableMean, tempVecCableRMS);
						std::pair<int, std::pair<std::vector<double>, std::vector<double> > > tempValueCable = std::make_pair((*iter).first.second, tempVecCable);
						_cableMap.insert( std::make_pair(key,tempValueCable));
					}

					itermap2 = _cableMap.find(key);
//					insert value in vector at key
					try
					{
						double cableAveragetemp = TMath::Mean(cableAverage.begin(), cableAverage.end());
						double CableRMStemp = TMath::RMS(cableAverage.begin(), cableAverage.end());
						((*itermap2).second.second.first).push_back(cableAveragetemp);
						((*itermap2).second.second.second).push_back(CableRMStemp);
					}

					catch (std::exception &e)
					{

					}

					cableAverage.clear();
					cableAverage.push_back((*iter).second);
				}
				CounterOld = CounterNew;
		}

		_inputPulseMap.clear();
		_inputPulseMapAverage.clear();
	}


	void GEMPulsingTimeEventAnalysisProcessor::check(LCEvent*  /*evt*/)
	{
	  // check what is needed here
	}


	void GEMPulsingTimeEventAnalysisProcessor::end()
	{
	  // do something at the end of the processing, e.g. cleaning up


//		Chip to Chip --> 1D Histogram Mean of Mean & RMS of Mean
		streamlog_out(DEBUG) << "Chip_map: " << _chipMap.size() << endl;

		std::map< std::pair<int,int>, std::pair<int, std::pair< std::vector<double>, std::vector<double> > > >::iterator it;
		for (it=_chipMap.begin(); it!= _chipMap.end(); ++it)
		{

			_chipToChipTimeAverage-> Fill(TMath::Mean(((*it).second.second.first).begin(), ((*it).second.second.first).end()));
			_RMSChipTime-> Fill(TMath::Mean(((*it).second.second.second).begin(), ((*it).second.second.second).end()));
		}

//		Chip to Chip --> 2D Histogram

		std::map< std::pair<int,int>, std::pair<int, std::pair< std::vector<double>, std::vector<double> > > >::iterator it1;
		for (it1=_chipMap.begin(); it1!= _chipMap.end(); ++it1)
		{
				try
				{
					std::pair<int, int> tempkey_gear = _channelMappingListener->getGearMapping(std::make_pair((*it1).second.first, (*it1).first.first));

					const gear::TPCModule& tempModule = _gearParameters->getModule(tempkey_gear.second);
					gear::Vector2D padcenter=tempModule.getPadCenter(tempkey_gear.first);

					_chipToChipTimeAverageXY -> Fill(padcenter[0], padcenter[1] , TMath::Mean(((*it1).second.second.first).begin(), ((*it1).second.second.first).end()));

				}
				catch (tpcconddata::CondDataNotAvailableException& e)
				{

				}
		}


//		Cable to Cable --> 1D Histogram
		std::map< std::pair<int,int>, std::pair<int, std::pair< std::vector<double>, std::vector<double> > > >::iterator it2;
		for (it2=_cableMap.begin(); it2!= _cableMap.end(); ++it2)
		{
			_cableToCableTimeAverage-> Fill(TMath::Mean(((*it2).second.second.first).begin(), ((*it2).second.second.first).end()));
			_RMSCableTime-> Fill(TMath::Mean(((*it2).second.second.second).begin(), ((*it2).second.second.second).end()));
		}

//		Cable to Cable --> 2D Histogram

		std::map< std::pair<int,int>, std::pair<int, std::pair< std::vector<double>, std::vector<double> > > >::iterator it3;
		for (it3=_cableMap.begin(); it3!= _cableMap.end(); ++it3)
		{
				try
				{
					std::pair<int, int> tempkey_gear = _channelMappingListener->getGearMapping(std::make_pair((*it3).second.first, (*it3).first.first));
					const gear::TPCModule& tempModule = _gearParameters->getModule(tempkey_gear.second);
					gear::Vector2D padcenter=tempModule.getPadCenter(tempkey_gear.first);

					_cableToCableTimeAverageXY -> Fill(padcenter[0], padcenter[1], TMath::Mean((*it3).second.second.first.begin(), (*it3).second.second.first.end()));
				}
				catch (tpcconddata::CondDataNotAvailableException& e)
				{

				}
		}



//			calculate averages of deviations
		vector<double> chipAverageDeviation;
		vector<double> cableAverageDeviation;
		int CounterNew_Deviation = 0;
		int CounterOld_Deviation = 0;

		std::map< std::pair<int,int>, std::vector<double> >::iterator iter;
		for (iter=_timeDeviationMap.begin(); iter!= _timeDeviationMap.end(); ++iter)
		{

			std::pair<double, double> Average_RMS = make_pair(TMath::Mean(((*iter).second).begin(), ((*iter).second).end()), TMath::RMS(((*iter).second).begin(), ((*iter).second).end()));
			_timeDeviationMapAverageRMS.insert(make_pair((*iter).first, Average_RMS));


//			Pad to pad deviation (Mean + RMS) --> 2D Histogram
			try
			{
				std::pair<int, int> tempkey_gear = _channelMappingListener->getGearMapping(std::make_pair((*iter).first.second, (*iter).first.first));
				std::vector <double> tempvec = (*iter).second;
				double tempaverage = TMath::Mean(tempvec.begin(), tempvec.end());
				double tempRMS = TMath::RMS(tempvec.begin(), tempvec.end());

				const gear::TPCModule& tempModule = _gearParameters->getModule(tempkey_gear.second);
				gear::Vector2D padcenter=tempModule.getPadCenter(tempkey_gear.first);

				streamlog_out(DEBUG) << "x= " << padcenter[0] << "; y= " << padcenter[1] << endl;

				_padToPadTimeDeviationAverageXY -> Fill(padcenter[0], padcenter[1], tempaverage);
				_padToPadTimeDeviationRMSXY -> Fill(padcenter[0], padcenter[1], tempRMS);
			}
			catch (tpcconddata::CondDataNotAvailableException& e)
			{

			}

			CounterNew_Deviation = (*iter).first.second;


//			Chip to chip deviation
			if ((int)CounterNew_Deviation/16 == (int)CounterOld_Deviation/16)
			{
				double tempDeviationChip = TMath::Mean(((*iter).second).begin(), ((*iter).second).end());
				chipAverageDeviation.push_back(tempDeviationChip);
				tempDeviationChip = 0;
			}
			else
			{
				_chipToChipTimeAverageDeviation-> Fill(TMath::Mean(chipAverageDeviation.begin(), chipAverageDeviation.end()));
				_RMSChipTimeDeviation-> Fill(TMath::RMS(chipAverageDeviation.begin(), chipAverageDeviation.end()));
				chipAverageDeviation.clear();
				chipAverageDeviation.push_back(TMath::Mean(((*iter).second).begin(), ((*iter).second).end()));
			}


//			Cable to cable deviation
			if ((int)CounterOld_Deviation/32 == (int)CounterNew_Deviation/32)
			{

				double tempDeviationCable = TMath::Mean(((*iter).second).begin(), ((*iter).second).end());
				cableAverageDeviation.push_back(tempDeviationCable);
				tempDeviationCable = 0;
			}
			else
			{
				_cableToCableTimeAverageDeviation-> Fill(TMath::Mean(cableAverageDeviation.begin(), cableAverageDeviation.end()));
				_RMSCableTimeDeviation-> Fill(TMath::RMS(cableAverageDeviation.begin(), cableAverageDeviation.end()));
				cableAverageDeviation.clear();
				cableAverageDeviation.push_back(TMath::Mean(((*iter).second).begin(), ((*iter).second).end()));
			}
			CounterOld_Deviation = CounterNew_Deviation;
		}

			_chipToChipTimeAverageDeviation-> Fill(TMath::Mean(chipAverageDeviation.begin(), chipAverageDeviation.end()));
			_RMSChipTimeDeviation-> Fill(TMath::RMS(chipAverageDeviation.begin(), chipAverageDeviation.end()));

			_cableToCableTimeAverageDeviation-> Fill(TMath::Mean(cableAverageDeviation.begin(), cableAverageDeviation.end()));
			_RMSCableTimeDeviation-> Fill(TMath::StdDev(cableAverageDeviation.begin(), cableAverageDeviation.end()));


//			Fill Tuple/Outputfile
			if (_createOutput == true)
			{

				std::map< std::pair<int,int>, std::pair<double,double> >::iterator outputiter;

				std::ofstream outputFile;
				outputFile.open(_filename.c_str());

				for(outputiter= _timeDeviationMapAverageRMS.begin(); outputiter != _timeDeviationMapAverageRMS.end(); ++outputiter)
				  {
					int module = ((*outputiter).first).first;
					int pad = ((*outputiter).first).second;
					double deviationaverage = ((*outputiter).second).first;
					double deviationRMS = ((*outputiter).second).second;

					outputFile << Form("%i %i %f %f \n",module,pad,deviationaverage,deviationRMS);

					// Tupel fuellen
					_timeCalibrationTuple -> Fill(module, pad, deviationaverage, deviationRMS);

				  }
				outputFile.close();
			}


	}
} // close namespace brackets

