 #include "OctoboardHitAndTrackChargeProcessor.h"

//C++
#include <iostream>
#include <cmath>
#include <sstream>
using namespace std;

//LCIO
#include <IMPL/LCCollectionVec.h>
#include <EVENT/Track.h>
#include <EVENT/TrackerHit.h>

// Aida
//#ifdef MARLIN_USE_AIDA // the whole processor does not make sense without aida
#include <AIDA/AIDA.h>
#include <marlin/AIDAProcessor.h>
#include <AIDA/IHistogramFactory.h>
#include <AIDA/IHistogram1D.h>
// endif

#include <marlin/Global.h>

using namespace lcio ;
using namespace marlin ;

namespace marlintpc
{

OctoboardHitAndTrackChargeProcessor aOctoboardHitAndTrackChargeProcessor;
    
OctoboardHitAndTrackChargeProcessor::OctoboardHitAndTrackChargeProcessor(): Processor("OctoboardHitAndTrackChargeProcessor")
{
    // register steering parameters: name, description, class-variable, default value
    registerInputCollection(    LCIO::TRACKERHIT,
				"InputHits" , 
				"Name of the input hits collection (default: TimePixHits)"  ,
				_hitsColName ,
				std::string("TimePixHits") ) ;
    // modify processor description
    _description = "Count hits in events and tracks, and track per event" ;

  registerProcessorParameter( "MaxChargePerHit" , 
			      "Maximum value in ChargePerHit histograms (default: ?? )"  ,
			       _maxChargePerHit,
			      float(10000) );

  registerProcessorParameter( "nChips" ,
			      "Number of Chips (default: 8)",
			       _nChips,
			      int(8) );

  registerProcessorParameter( "nBins" ,
			      "Number of Bins (default: 100)",
			       _nBins,
			      int(100) );

}

OctoboardHitAndTrackChargeProcessor::~OctoboardHitAndTrackChargeProcessor()
{
}

void OctoboardHitAndTrackChargeProcessor::init()
{ 
	// usually a good idea to
	printParameters() ;
	



	_chargePerHitHistos.resize(_nChips +1);


	// check parameter:
	// _nZBins >=0

	// _minZ < _maxZ

	// get a histograms from the AIDAProcessor
	
	// first the histos in z slices
	for (int chips=0; chips < _nChips; chips++)
	{




	    std::stringstream chipnr;
	    chipnr << " Chip " << chips+1;
	    string histoTitle = string("Charge per Hit") + chipnr.str();
	    string histoName  = string("ChargePerHit") + chipnr.str();

	    streamlog_out(DEBUG0) << histoName << " : " << histoTitle << endl;
	    _chargePerHitHistos[chips] =   AIDAProcessor::histogramFactory(this)->
		createHistogram1D( histoName, 
				   histoTitle, 
				   _nBins, -0.5, _maxChargePerHit+0.5);



	}
	streamlog_out(DEBUG0)<<"_chargePerHitHistos.size() + "<<_chargePerHitHistos.size()<<std::endl;
	
}

void OctoboardHitAndTrackChargeProcessor::processRunHeader( EVENT::LCRunHeader*  /*run*/ )
{
}
    
void OctoboardHitAndTrackChargeProcessor::processEvent( LCEvent * evt )
{ 
    //read input data
    streamlog_out(MESSAGE3) << "Event: " << evt->getEventNumber() << std::endl;

    LCCollectionVec* hitsCollection=0;
    try{
	 hitsCollection = dynamic_cast<LCCollectionVec*>( evt->getCollection( _hitsColName ) ) ;
    }
    catch(DataNotAvailableException &e)
    {
	// there are no hits
	// continue with next event
	return;
    }

    // iterate the hits to fill the charge into histograms
    if( hitsCollection != 0 )
    {
	for( LCCollectionVec::iterator hitIter = hitsCollection->begin() ; 
	    hitIter < hitsCollection->end(); hitIter ++ )
	{
	    TrackerHit *hit = dynamic_cast<TrackerHit *>(*hitIter) ;

	    int cellID1 = hit->getCellID1();
	
	    // a hit is point like , it's not dEdx but charge
		_chargePerHitHistos[cellID1]->fill(hit->getEDep () );
		streamlog_out(DEBUG0) << "CellID1: " << cellID1 << std::endl;
	}
    }
    
}


}// namespace 
