#include "TimePixTOTDistributionProcessor.h"
#include "TimePixMapHandlerProcessor.h"
#include "TimePixPixelMode.h"

//C++
#include <iostream>
using namespace std;

//LCIO
#include <IMPL/LCCollectionVec.h>
//#include <EVENT/Track.h>
#include <EVENT/TrackerData.h>
//#include "IntersectionCalculator.h"

// Aida
//#ifdef MARLIN_USE_AIDA // the whole processor does not make sense without aida
#include <AIDA/AIDA.h>
#include <marlin/AIDAProcessor.h>
#include <AIDA/IHistogramFactory.h>
#include <AIDA/IHistogram1D.h>
#include <AIDA/IHistogram2D.h>
// endif

#include <marlin/Global.h>

#include "TrackFitterFactory.h"
#include "TrackFitterBase.h"

using namespace lcio ;
using namespace marlin ;
using namespace tpcconddata;

namespace marlintpc
{

TimePixTOTDistributionProcessor aTimePixTOTDistributionProcessor;
    
TimePixTOTDistributionProcessor::TimePixTOTDistributionProcessor(): Processor("TimePixTOTDistributionProcessor")
						    , //_distributionTOTHisto(0) //,
						      //_distributionYHisto(0), 
						      //_distributionZHisto(0), 
						      _distributionXYHisto(0),
						      _NrOfHitsXYHisto(0)
{
    // register steering parameters: name, description, class-variable, default value
    registerInputCollection(    LCIO::TRACKERDATA,
				"InputData" , 
				"Name of the input data collection"  ,
				_inputColName ,
				std::string("TimePixInterpolData") ) ;
    // modify processor description
    _description = "TimePixTOTDistributionProcessor: Creates XY Histo with mean Charge per Pixel" ;

  registerProcessorParameter( "HistogramMinValueX" , 
			      "Minimum value in x histogram (default: -0.5 Pixel)"  ,
			      _histogramMinValueX ,
			      float(0) ) ;

  registerProcessorParameter( "HistogramMaxValueX" , 
			      "Maximal value in x histogram (default:  255 Pixel)"  ,
			      _histogramMaxValueX ,
			      float(255) ) ;

  registerProcessorParameter( "HistogramMinValueY" , 
			      "Minimum value in y histogram (default: -0.5 Pixel)"  ,
			      _histogramMinValueY ,
			      float(0) ) ;

  registerProcessorParameter( "HistogramMaxValueY" , 
			      "Maximal value in y histogram (default:  255 Pixel)"  ,
			      _histogramMaxValueY ,
			      float(255) ) ;
			      
  registerProcessorParameter( "MaxMeanCharge" , 
			      "Maximal value in x MeanChargeHistogram (default:  8000 electrons)"  ,
			       _maxMeanCharge,
			      int(8000) );
			      
  registerProcessorParameter( "Mode" , 
			      "mode that should be plotted (TOT, TIMEPIX, MEDIPIX, ONEHIT or ANY), default: ANY"  ,
			      _selectedMode ,
			      std::string("TOT") ) ;			      

//  registerProcessorParameter( "HistogramMinValueZ" , 
//			      "Minimum value in z histogram (default: 0.0 mm)"  ,
//			      _histogramMinValueZ ,
//			      float(0.0) ) ;
//
//  registerProcessorParameter( "HistogramMaxValueZ" , 
//			      "Maximal value in z histogram (default:  260.0 mm)"  ,
//			      _histogramMaxValueZ ,
//			      float(260.0) ) ;
}

TimePixTOTDistributionProcessor::~TimePixTOTDistributionProcessor()
{
}

void TimePixTOTDistributionProcessor::init()
{ 
	// usually a good idea to
	printParameters() ;
	
	// get a histograms from the AIDAProcessor
//	_distributionTOTHisto =   AIDAProcessor::histogramFactory(this)->
//	  createHistogram1D( "TimePixTOTDistributionHisto", 
//			     "TimePixTOTDistributionHisto", 
//			     TOTbins, _histogramMinValueTOT, _histogramMaxValueTOT);
//
//	_distributionYHisto =   AIDAProcessor::histogramFactory(this)->
//	  createHistogram1D( "yDistributionHisto", 
//			     "yDistributionHisto", 
//			     100, _histogramMinValueY, _histogramMaxValueY);
//
//	_distributionZHisto =   AIDAProcessor::histogramFactory(this)->
//	  createHistogram1D( "zDistributionHisto", 
//			     "zDistributionHisto", 
//			     100, _histogramMinValueZ, _histogramMaxValueZ);

	_nBinsX = 256;
	_nBinsY = _nBinsX;
	AIDAProcessor::tree(this);	//changes into directory of this processor in aida-output-file 
	_distributionXYHisto =   new TH2F( "SumOfAllToT", 
					    "SumOfAllToT", 
					    _nBinsX, _histogramMinValueX, _histogramMaxValueX,
					    _nBinsY, _histogramMinValueY, _histogramMaxValueY);
	
	_NrOfHitsXYHisto =   new TH2F( "_NrOfHits", 
					"_NrOfHits", 
					_nBinsX, _histogramMinValueX, _histogramMaxValueX,
					_nBinsY, _histogramMinValueY, _histogramMaxValueY);			     

	_MeanChargePerPixelXY =   new TH2F( "MeanChargePerPixel", 
					     "MeanChargePerPixel", 
					     _nBinsX, _histogramMinValueX, _histogramMaxValueX,
					     _nBinsY, _histogramMinValueY, _histogramMaxValueY);		

	_MeanChargeHisto =  new TH1F("MeanChargeHisto",
				      "MeanChargeHisto",
				      2000, 0, _maxMeanCharge
				      );
				      
	if ( _selectedMode == "TOT" ) 
	    _plotMode = TimePixPixelMode::TOT;
	else if ( _selectedMode == "TIMEPIX" )
	    _plotMode = TimePixPixelMode::TIMEPIX;
	else if ( _selectedMode == "MEDIPIX" )
	    _plotMode = TimePixPixelMode::MEDIPIX;
	else if ( _selectedMode == "ONEHIT" )
	    _plotMode = TimePixPixelMode::ONEHIT;
	else
	    _plotMode = -1; // plot all modes			      
				      
					     
    }

void TimePixTOTDistributionProcessor::processRunHeader(lcio::LCRunHeader*  /*run*/ )
{
    _timePixMap = TimePixMapHandlerProcessor::getTimePixMap();
	
}
    
void TimePixTOTDistributionProcessor::processEvent(lcio::LCEvent * evt ) 
{ 
    //read input data
    streamlog_out(MESSAGE2) << "Event: " << evt->getEventNumber() << std::endl;


    // we need a LCCollectionVec to use iterators, as LCCollectio is not a std::vector
    LCCollectionVec* InputCollection=0;
    try{
	InputCollection = dynamic_cast<LCCollectionVec*>(evt->getCollection( _inputColName )) ;
    }
    catch(DataNotAvailableException &e)
    {
	streamlog_out(DEBUG0)<<"no imput data"<<std::endl;
	// no input data
	// just continue ...
	return;
    }


    // cast did not succeed
    if( InputCollection == 0 )
    {
	streamlog_out(ERROR2) << "InputCollection vector is 0" << std::endl;
	return;
    }

    for(LCCollectionVec::iterator it = InputCollection->begin() ; 
	it < InputCollection->end(); it ++ )
    {
	streamlog_out(DEBUG0)<<"casting data"<<std::endl;
	TrackerData* data = dynamic_cast<TrackerData *>(*it);
	
	if ( data == 0 ){
	  streamlog_out(DEBUG0)<<"cast failed"<<std::endl;
	}
	else {
	  streamlog_out(DEBUG0)<<"cast succesfull"<<std::endl;
	}

	if ( data == 0 ) break;
	else {
	    
	    

	    int pixelID = data->getCellID0();
	    int chipID  = data->getCellID1();

	    streamlog_out(DEBUG0)<<"pixelID : "<<pixelID<<"  chipId "<<chipID<<std::endl;

	    FloatVec ChargeVec = data->getChargeValues();
	    
	    for ( FloatVec::iterator ch = ChargeVec.begin(); ch < ChargeVec.end(); ch++ ) {
	    
		streamlog_out(DEBUG0)<<"ChargeVecIteration "<<pixelID<<"  "<<*ch<<std::endl;

	        streamlog_out(DEBUG0)<<"mode = "<<((_timePixMap->find( TimePixPixelMode::PixelKey( chipID, pixelID ) ))
						   ->second)->getPixelMode() <<std::endl;

		if ( ((_timePixMap->find( TimePixPixelMode::PixelKey( chipID, pixelID ) ))
		      ->second)->getPixelMode() == _plotMode )
		{
		    _NrOfHitsXYHisto->Fill( pixelID % _nBinsX, pixelID / _nBinsX );
		    _distributionXYHisto->Fill( pixelID % _nBinsX, pixelID / _nBinsX , *ch );
		}
		pixelID++;
	    
	    }
	
	}

//	_distributionYHisto->fill(pos[1]);
//	_distributionZHisto->fill(pos[2]);
////	    if (pos[2] < 10)
//	_distributionTOTYHisto->fill(pos[0],pos[1]);
	
    }
    
}

void TimePixTOTDistributionProcessor::end()
{
  for (int x = 0; x < _nBinsX; x++)
    for (int y = 0; y < _nBinsY; y++)  
      if (_NrOfHitsXYHisto->GetBinContent( x, y) > 0 )
	{ _MeanChargePerPixelXY->Fill( x, y,  _distributionXYHisto->GetBinContent( x, y) / (	1.0 * _NrOfHitsXYHisto->GetBinContent( x, y) 	) 	);
	  _MeanChargeHisto->Fill(_distributionXYHisto->GetBinContent( x, y) / (	1.0 * _NrOfHitsXYHisto->GetBinContent( x, y) ) );
	}
      else 
      _MeanChargePerPixelXY->Fill( x, y, 0);

}



}// namespace 
