#include "TrackFinderEvaluateProcessor.h"
#include <iostream>
#include <string>

// include what you need from lcio
#include <EVENT/LCEvent.h>
#include <IMPL/LCCollectionVec.h>
#include <EVENT/Track.h>
#include <Exceptions.h>

using namespace lcio;
using namespace marlin;
using namespace std;

// Use AIDA histograms
#include <AIDA/AIDA.h>
#include <marlin/AIDAProcessor.h>

namespace marlintpc
{

TrackFinderEvaluateProcessor aTrackFinderEvaluateProcessor;


TrackFinderEvaluateProcessor::TrackFinderEvaluateProcessor() : Processor("TrackFinderEvaluateProcessor"),hitsTrackNumber(0),hitsAddedTrackNumber(0),tracksPerEvent(0),hitsPerTrack(0)
{
	// the processor description
	_description = "Processor to evaluate the performance of the track finding algorithm. It plots the number of track versus the number of of hits on a track.";

	/* register steering parameters: name, description, class-variable, default value

    for example:
	 */

	registerInputCollection(    LCIO::TRACK,
			"InputCollectionName" ,
			"Name of the input Track collection, default: TPCTracks"  ,
			_inputColName ,
			string("TPCTracks")) ;

}


void TrackFinderEvaluateProcessor::init()
{ 
	streamlog_out(DEBUG) << " init() called  " << endl;

	// you might want to list the used parameters by invoking
	printParameters();
	AIDAProcessor::tree(this);
	hitsTrackNumber =new TH2D("hit number vs track number","hit number vs track number",10,0,10,100,0,100);
	hitsTrackNumber->GetXaxis()->SetTitle("number of tracks");
	hitsTrackNumber->GetYaxis()->SetTitle("number of hits on a track");

	hitsAddedTrackNumber =new TH2D("hit number added vs track number","hit number added vs track number",10,0,10,256,0,256);
	hitsAddedTrackNumber->GetXaxis()->SetTitle("number of tracks");
	hitsAddedTrackNumber->GetYaxis()->SetTitle("number of hits on all tracks");

	tracksPerEvent = new TH1D("tracksPerEvent","tracks per event",10,0,10);
	tracksPerEvent->GetXaxis()->SetTitle("number of tracks");
	hitsPerTrack = new TH1D("hitsPerTrack","hits per track",100,0,100);
	hitsPerTrack->GetXaxis()->SetTitle("number of hits per track");

}


void TrackFinderEvaluateProcessor::processRunHeader(LCRunHeader*  /*run*/)
{
	// do something for each run
} 


void TrackFinderEvaluateProcessor::processEvent(LCEvent* evt)
{ 
	// do something for each event, usually access the input collection:

	// use an LCCollectionVec instead of LCCollection so we can use iterators later
	LCCollectionVec* inputColVec = 0;

	// do a try / catch in case there is an empty event
	try
	{
		inputColVec = dynamic_cast<LCCollectionVec *>(evt->getCollection( _inputColName ) );
	}
	catch (DataNotAvailableException &e)
	{
		// no input data, just return and process the next event
		return;
	}
	int numberOfTracks= inputColVec->size();
	tracksPerEvent->Fill(numberOfTracks);
	int numberOfHitsCombined=0;
	// loop the data collection and do something
	for( LCCollectionVec::iterator inputObjectIter = inputColVec->begin() ;
			inputObjectIter < inputColVec->end() ;
			inputObjectIter++ )
	{
		// dynamic cast the input object (LCObject) to what it really is e.g.
		Track * inputTrack = dynamic_cast<Track *>( *inputObjectIter );
		int numberOfHits=(inputTrack->getTrackerHits()).size();
		numberOfHitsCombined+=numberOfHits;
		hitsTrackNumber->Fill(numberOfTracks,numberOfHits);
		hitsPerTrack->Fill(numberOfHits);

	}
	hitsAddedTrackNumber->Fill(numberOfTracks,numberOfHitsCombined);
}


void TrackFinderEvaluateProcessor::check(LCEvent*  /*evt*/)
{ 
	// check what is needed here
}


void TrackFinderEvaluateProcessor::end()
{ 
	// do something at the end of the processing, e.g. cleaning up
}
} // close namespace brackets
