#ifndef ADC2PRIMARYELECTRONFACTORCALCULATOR_H
#define ADC2PRIMARYELECTRONFACTORCALCULATOR_H

// C++
#include <string>
#include <fstream>

// LCIO
#include "lcio.h"

// Marlin
#include "marlin/Processor.h"
#include "lccd/ConditionsMap.hh"
//ROOT
#include "TH1F.h"
#include "TFile.h"
#include "TNtuple.h"

namespace marlintpc
{

/**
 *  A processor to calculate hitcharge per length unit 
 *  taking into account the track length over the measurement row ofthe hit.
 *
 *  <h4>Input - Prerequisites</h4>
 *  a collection of Tracks
 *
 *  <h4>Output</h4>
 *  a double value and its error written into the collection paramters of the hits
 *
 * @param InputCollectionName Name of the input collection (default: ClupatraTracks)
 *
 * @author Astrid Muennich, DESY
 */

class ADC2PrimaryElectronFactorCalculatorProcessor : public marlin::Processor
{

    public:

        /* This method will be called by the marlin package
         * It returns a processor of the currend type 
         */
        virtual Processor*  newProcessor()
        {
            return new ADC2PrimaryElectronFactorCalculatorProcessor ;
        }

        /* the default constructor
         * here the processor parameters are registered to the marlin package
         * other initialisation should be placed in the init method
         */
        ADC2PrimaryElectronFactorCalculatorProcessor() ;

        /* Called at the beginning of the job before anything is read.
         * Use to initialize the processor, e.g. book histograms
         */
        virtual void init() ;

        /* Called for every run.
         * in this processor it is not used
         */
        virtual void processRunHeader(lcio::LCRunHeader* run) ;

        /* Called for every event - the working horse.
         * Here the real conversion take place
         */
        virtual void processEvent(lcio::LCEvent * evt) ;

        /* This method is only called if the check flag is set (default) in the main processor
         * (where the input files are given)
         * It should be used to create check plots which are not needed in a complete analysis
         * (where normaly the check flag is deactivated)
         */
      //FIXME: not needed:  virtual void check(lcio::LCEvent * evt) ;


        /* Called after data processing for clean up.
         * e.g. saving control histogramms, delete created objects
         * (which are not stored in a collection/event), etc.
         */
        virtual void end() ;




    protected:

        std::string _inputColName ;  ///<- the name of the input collection
   
        int _nRun ; ///<- counts the processed runs

    private:
	double _primEl;
	TNtuple *_tuple;
        TH1F* _histoL;
        TH1F* _histoQ;
        TH1F* _histodEdx;
	TFile *_outfile_root;
	std::ofstream _outfile;
	std::string _outfileName;
	std::string _rootoutfileName;

} ;

}

#endif



