#ifndef TimePixDigitisationProcessor_h
#define TimePixDigitisationProcessor_h 1

#include "VoxelTPC.h"

//LCIO
#include "lcio.h"

//MARLIN
#include "marlin/Processor.h"
#include "TimePixMapListener.h"
#include "TimePixPixelMode.h"

//C++
#include <string>

//GEAR
#include <gear/TPCParameters.h>

namespace AIDA{
  class IHistogram1D;

}

namespace marlintpc{

/** 
 *  
 *  DOCU!!!!!!!!!!!!!
 *  
 *  
 */

class TimePixDigitisationProcessor : public marlin::Processor {
  
 public:
  
  virtual Processor*  newProcessor() { return new TimePixDigitisationProcessor ; }
  
  
  TimePixDigitisationProcessor() ;
  virtual ~TimePixDigitisationProcessor(){}
  
  /** Called at the begin of the job before anything is read.
   *  Use to initialize the processor, e.g. book histograms.
   */
  virtual void init() ;
  
  /** Called for every run.
   */
  virtual void processRunHeader(lcio::LCRunHeader* run ) ;
  
  /** Called for every event - the working horse.
   */
  virtual void processEvent(lcio::LCEvent * evt ) ; 
  
  
  virtual void check(lcio::LCEvent * evt ) ; 
  
  
  /** Called after data processing for clean up.
   */
  virtual void end() ;

  /** gives back the tot - counts from this voxel
   */
  virtual int getTOT( TPCVoxel voxel );

  /** gives back the sum of all tot counts in the vector of voxel
   */
  virtual int getTOT( std::vector< TPCVoxel > VoxelVec );

   /** gives back the medipix - counts from this voxel
   */ 
  virtual int getMEDIPIX( std::vector< TPCVoxel > VoxelVec );

  /** gives back the time - counts from this voxel
   */ 
  virtual int getTIME( std::vector< TPCVoxel > VoxelVec );


protected:

  /** Input/output collection names.
   */
  std::string _inputCollectionName;
  std::string _outputCollectionName;

  int _transient;
  int _nColumn; ///< number of columns on chip
  int _nRow; ///< number of rows on chip
  int _chipID; ///< chipID of current chip
  float _readoutFrequency; ///< the readout frequency of chip in MHz
  float _shutter; ///< the shutter time of the measurement in us
  float _propFactorElectronsADC; ///< conversion factor from ADC counts to electrons
  int _appliedThreshold; ///< the threshold of the TOT counts, applied by experimenter
  int _hardwareThreshold; ///< the internal ADCcounts threshold of the chip

  float _a; ///< parameter to calculate TOT from electrons TOT = c/(ln(Q-a)-b)
  float _b; ///< parameter to calculate TOT from electrons TOT = c/(ln(Q-a)-b)
  float _c; ///< parameter to calculate TOT from electrons TOT = c/(ln(Q-a)-b)
 

  gear::PadRowLayout2D const * _padLayout;

  VoxelTPC* _tpc;

  tpcconddata::TimePixMap * _timePixMap;

  AIDA::IHistogram1D *  _distributionTimeValues;
};

}
#endif 
