#include "TPCGas.h"
#include <cmath>
#include <cstring>

namespace marlintpc{

  TPCGas::TPCGas(std::string gas)
  {
    //dependend on the gas fill the parametrisation values 
    _gasName=gas;
    if( strcmp(gas.c_str(),"TDR")==0)
      initTDRGas();
    else if (strcmp(gas.c_str(),"P5")==0)
      initP5Gas();
    else if ( strcmp(gas.c_str(),"P10")==0)
      initP10Gas();
    else if ( strcmp(gas.c_str(),"T2K")==0)
      initT2KGas();
    else
      throw "Unknown gas type";
    
  }
  
  void TPCGas::CalculateGasParameters()
  {
    _vdrift=10*EvaluateEFieldDependence(_vd_params[0],_vd_params[1],_vd_params[2],_vd_params[3],_vd_params[4],
				     _vd_params[5],_vd_params[6],_vd_params[7],_vd_params[8],_vd_params[9],
				     _vd_params[10]);
    _diffLong=EvaluateEFieldDependence(_dL_params[0],_dL_params[1],_dL_params[2],_dL_params[3],_dL_params[4],
				       _dL_params[5],_dL_params[6],_dL_params[7],_dL_params[8],_dL_params[9],
				       _dL_params[10]);
    CalculateTransverseDiffusion();
  }
  
  void TPCGas::CalculateTransverseDiffusion()
  {
    double a=0.0, b=0.0, c=0.0, d=0.0, e=0.0, f=0.0, g=0.0,h=0.0,i=0,j=0;
    a=EvaluateBFieldDependence( _dT_params[0][0], _dT_params[0][1], _dT_params[0][2], 
				_dT_params[0][3], _dT_params[0][4], _dT_params[0][5]);
    
    b=EvaluateBFieldDependence( _dT_params[1][0], _dT_params[1][1], _dT_params[1][2], 
				_dT_params[1][3], _dT_params[1][4], _dT_params[1][5]);
    
    c=EvaluateBFieldDependence( _dT_params[2][0], _dT_params[2][1], _dT_params[2][2], 
				_dT_params[2][3], _dT_params[2][4], _dT_params[2][5]);
    
    d=EvaluateBFieldDependence( _dT_params[3][0], _dT_params[3][1], _dT_params[3][2], 
				_dT_params[3][3], _dT_params[3][4], _dT_params[3][5]);
    
    e=EvaluateBFieldDependence( _dT_params[4][0], _dT_params[4][1], _dT_params[4][2], 
				_dT_params[4][3], _dT_params[4][4], _dT_params[4][5]);
    
    f=EvaluateBFieldDependence( _dT_params[5][0], _dT_params[5][1], _dT_params[5][2], 
				_dT_params[5][3], _dT_params[5][4], _dT_params[5][5]);
    
    g=EvaluateBFieldDependence( _dT_params[6][0], _dT_params[6][1], _dT_params[6][2], 
				_dT_params[6][3], _dT_params[6][4], _dT_params[6][5]);
    
    h=EvaluateBFieldDependence( _dT_params[7][0], _dT_params[7][1], _dT_params[7][2], 
				_dT_params[7][3], _dT_params[7][4], _dT_params[7][5]);
    
    i=EvaluateBFieldDependence( _dT_params[8][0], _dT_params[8][1], _dT_params[8][2],
				_dT_params[8][3], _dT_params[8][4], _dT_params[8][5]);

    j=EvaluateBFieldDependence( _dT_params[9][0], _dT_params[9][1], _dT_params[9][2],
				_dT_params[9][3], _dT_params[9][4], _dT_params[9][5]);

    //now d_t dependend on E
    _diffTrans=EvaluateEFieldDependence(a,b,c,d,e,f,g,h,i,j,0);
				
  }
   
  double TPCGas::EvaluateEFieldDependence(double a, double b ,double c, double d,
				 double e, double f, double g, double h,
				 double i, double j, double k)
  {
    double logE=std::log(_EField);
    
    return a+b*logE+c*pow(logE,2)+d*pow(logE,3)
      +e*pow(logE,4)+f*pow(logE,5)+g*pow(logE,6)
      +h*pow(logE,7)+i*pow(logE,8)+j*pow(logE,9)
      +k*pow(logE,10);
  }

  double TPCGas::EvaluateBFieldDependence(double a, double b, double c, double d,double e, double f)
  {
    return a + b*_BField + c*pow(_BField,2) + d*pow(_BField,3) + e*pow(_BField,4)+ f*pow(_BField,5);
  }


  double TPCGas::GetAttachment(double E, double dist)
  {
    double attachment=0.;
    //attachment only significant above a certain EField strength
    if(E>1000 && E<10000)
       attachment=_att[0] + _att[1]*E + _att[2]*pow(E,2) + _att[3]*pow(E,3) + _att[4]*pow(E,4) + _att[5]*pow(E,5) + _att[6]*pow(E,6);
    
    return exp(-dist*attachment);
  }

  void  TPCGas::initTDRGas()
  {
    _dL_params[0] =-110369.279057625;
    _dL_params[1] =220779.365529786;
    _dL_params[2] =-187931.357154949;
    _dL_params[3] =90465.0565843025; 
    _dL_params[4] =-27363.2494470196;
    _dL_params[5] =5451.70005605478; 
    _dL_params[6] =-727.005814951371;
    _dL_params[7] =64.2889479781   ; 
    _dL_params[8] =-3.61907342719664;
    _dL_params[9] =0.117440351004847;
    _dL_params[10]=-0.00167233389877851;
  
    _vd_params[0] =-2087.14481157477;
    _vd_params[1] =4988.19775908129  ;
    _vd_params[2] =-5213.28050066342 ;
    _vd_params[3] =3134.02364965672  ;
    _vd_params[4] =-1199.06293662325 ;
    _vd_params[5] =304.93453476299   ;
    _vd_params[6] =-52.2070052208005 ;
    _vd_params[7] =5.94607746435271  ;
    _vd_params[8] =-0.431707055905293;
    _vd_params[9] =0.0180715488714349;
    _vd_params[10]=-0.000331805483109844;
	
    _dT_params[0][0]= -106.046108535431;
    _dT_params[0][1]= -7925.61400273791;
    _dT_params[0][2]=  6686.92350165348 ;
    _dT_params[0][3]=  -1736.42761722403;
    _dT_params[0][4]=  146.611430723786;
    _dT_params[0][5]= 0;

    _dT_params[1][0]= 1970.33307314786;
    _dT_params[1][1]= 8539.99540224613;
    _dT_params[1][2]= -8165.5604789030;
    _dT_params[1][3]= 2185.06605798116;
    _dT_params[1][4]= -186.64910130239;
    _dT_params[1][5]= 0;
    
    _dT_params[2][0]=   -1310.89853174629;
    _dT_params[2][1]=   -3859.80438359763;
    _dT_params[2][2]=   4067.95823611071;
    _dT_params[2][3]=   -1112.18595370353;
    _dT_params[2][4]=   95.733291666645;
    _dT_params[2][5]= 0;
    
    _dT_params[3][0]= 359.575535714393;
    _dT_params[3][1]=906.582285713412;
    _dT_params[3][2]=-1055.48393749909;
    _dT_params[3][3]=294.561624999709;
    _dT_params[3][4]=-25.5341874999711;
    _dT_params[3][5]= 0;
    
    _dT_params[4][0]=-47.8321031745661;
    _dT_params[4][1]= -121.309271693408;
    _dT_params[4][2]= 156.069644444719;
    _dT_params[4][3]= -44.4134370371213;
    _dT_params[4][4]= 3.87508333334145;
    _dT_params[4][5]= 0;
    
    _dT_params[5][0]= 3.08740047617949;
    _dT_params[5][1]=  9.46675825402465;
    _dT_params[5][2]=  -13.3412950000488;
    _dT_params[5][3]=  3.86233555556974;
    _dT_params[5][4]=  -0.338861666667986;
    _dT_params[5][5]= 0;
    
    _dT_params[6][0]= -0.0802989230166763;
    _dT_params[6][1]=  -0.406835040606432;
    _dT_params[6][2]=  0.616783563887318;
    _dT_params[6][3]=  -0.181050885184737;
    _dT_params[6][4]=  0.0159530999999581;
    _dT_params[6][5]= 0;

    _dT_params[7][0]= 0.000202925869483748;
    _dT_params[7][1]=  0.00751979708915846;
    _dT_params[7][2]=  -0.0119714449267601;
    _dT_params[7][3]=  0.00355000019899887;
    _dT_params[7][4]=  -0.000313752165998087;
    _dT_params[7][5]= 0;

    _dT_params[8][0]= 0;
    _dT_params[8][1]= 0;
    _dT_params[8][2]= 0;
    _dT_params[8][3]= 0;
    _dT_params[8][4]= 0;
    _dT_params[8][5]= 0;

    _dT_params[9][0]= 0;
    _dT_params[9][1]= 0;
    _dT_params[9][2]= 0;
    _dT_params[9][3]= 0;
    _dT_params[9][4]= 0;
    _dT_params[9][5]= 0;

    _att[0]=-0.343716243708895;
    _att[1]= 0.000310654263375315;
    _att[2]=-1.43700791338589e-08;
    _att[3]= 7.13759804300208e-12;
    _att[4]=-1.20510943273379e-15;
    _att[5]= 4.88792048657487e-20;
    _att[6]=0;
  }
  
  void  TPCGas::initP5Gas()
  {
    _dL_params[0] = 66540.3253522381  ;
    _dL_params[1] =	-125892.973218309	;
    _dL_params[2] =	105109.552186917	;
    _dL_params[3] =	-49900.9805699735	;
    _dL_params[4] =	14901.2375078537	;
    _dL_params[5] =	-2932.47405365043	;
    _dL_params[6] =	386.37974056973	;
    _dL_params[7] =	-33.7549355914578	;
    _dL_params[8] =	1.87617799102579	;
    _dL_params[9] =	-0.060056848900623;
    _dL_params[10]=	0.000842629436663657;
    
    _vd_params[0] = 434.076969292862    ;
    _vd_params[1] =	-1074.66184491203   ;
    _vd_params[2] =	1149.72785706044    ;
    _vd_params[3] =	-698.444581229727   ;
    _vd_params[4] =	266.473335099845    ;
    _vd_params[5] =	-66.6913114740444   ;
    _vd_params[6] =	11.1071873889043    ;
    _vd_params[7] =	-1.21924046183097   ;
    _vd_params[8] =	0.0847232468170763  ;
    _vd_params[9] =	-0.00337737394355272;
    _vd_params[10]=	5.88460170933486e-05;
    
    _dT_params[0][0]= 1717.55091269849 ;
    _dT_params[0][1]=   -11862.4161798943;
    _dT_params[0][2]=   7645.31818055561 ;
    _dT_params[0][3]=   -1711.3409351852 ;
    _dT_params[0][4]=   131.273458333334;
    _dT_params[0][5]= 0;
	
    _dT_params[1][0]= -1187.37916666688;
    _dT_params[1][1]=   15041.8556111118 ;
    _dT_params[1][2]=   -10250.1270416673;
    _dT_params[1][3]=   2349.85980555573 ;
    _dT_params[1][4]=   -183.043375000017;
    _dT_params[1][5]= 0;
	
    _dT_params[2][0]=749.58074206344;
    _dT_params[2][1]=  -8386.30572089888;
    _dT_params[2][2]=  5915.78454861053;
    _dT_params[2][3]=  -1385.18655092575;
    _dT_params[2][4]=  109.588270833316;
    _dT_params[2][5]= 0;
    
    _dT_params[3][0]= -277.356452380926;
    _dT_params[3][1]=  2393.05499206331;
    _dT_params[3][2]=  -1741.8060416665;
    _dT_params[3][3]=  415.234138888839;
    _dT_params[3][4]=  -33.2723749999953;
    _dT_params[3][5]= 0;

    _dT_params[4][0]= 58.7842686507959;
    _dT_params[4][1]=  -378.769551719536;
    _dT_params[4][2]=  284.079220138844;
    _dT_params[4][3]=  -68.8185782407264;
    _dT_params[4][4]=  5.57579791666526;
    _dT_params[4][5]= 0;
	
    _dT_params[5][0]= -6.89263670634272;
    _dT_params[5][1]=  33.816481256577;
    _dT_params[5][2]=  -26.0894200694113;
    _dT_params[5][3]=  6.41412967591606;
    _dT_params[5][4]=  -0.524940624999067;
    _dT_params[5][5]= 0;

    _dT_params[6][0]=0.411706769841139;
    _dT_params[6][1]= -1.59868090608373;
    _dT_params[6][2]= 1.26615805555469;
    _dT_params[6][3]= -0.315568351851591;
    _dT_params[6][4]= 0.0260675833333084;
    _dT_params[6][5]= 0;

    _dT_params[7][0]= -0.00974956896824944;
    _dT_params[7][1]=  0.031175398716901;
    _dT_params[7][2]=  -0.0252933602777505;
    _dT_params[7][3]=  0.00638410037036229;
    _dT_params[7][4]=  -0.00053190499999924;
    _dT_params[7][5]= 0;

    _dT_params[8][0]= 0;
    _dT_params[8][1]= 0;
    _dT_params[8][2]= 0;
    _dT_params[8][3]= 0;
    _dT_params[8][4]= 0;
    _dT_params[8][5]= 0;

    _dT_params[9][0]= 0;
    _dT_params[9][1]= 0;
    _dT_params[9][2]= 0;
    _dT_params[9][3]= 0;
    _dT_params[9][4]= 0;
    _dT_params[9][5]= 0;
 
    _att[0]=0.569805421256637;
    _att[1]=-0.00091365767373601;
    _att[2]=4.37189301033502e-07;
    _att[3]=-7.06077170050197e-11;
    _att[4]=4.75904934032274e-15;
    _att[5]=-1.15317373111484e-19;
    _att[6]=0;

 }
  
  void  TPCGas::initP10Gas()
  {  
    _dL_params[0] =56323.2298716282   ;
    _dL_params[1] =	-104137.055803	 ;
    _dL_params[2] =	84443.4053447754	 ;
    _dL_params[3] =	-38744.2017170156	 ;
    _dL_params[4] =	11144.8199782006	 ;
    _dL_params[5] =	-2108.00545694589	 ;
    _dL_params[6] =	266.523751314122	 ;
    _dL_params[7] =	-22.3152100099685	 ;
    _dL_params[8] =	1.18752302003322	 ;
    _dL_params[9] =	-0.0363634262572915;
    _dL_params[10]=	0.000487691075825722;
    
    _vd_params[0] = 1862.50727805909    ;
    _vd_params[1] =	 -4100.98694313903	;
    _vd_params[2] =	 3932.6230354755	;
    _vd_params[3] =	 -2160.5991694089	;
    _vd_params[4] =	 752.872704920645	;
    _vd_params[5] =	 -173.939322553779	;
    _vd_params[6] =	 27.0266629604996	;
    _vd_params[7] =	 -2.79534719623288	;
    _vd_params[8] =	 0.18467738994886	;
    _vd_params[9] =	 -0.00705629626191505;
    _vd_params[10]=	 0.00011870800945113;
    
    _dT_params[0][0]= 5275.68159650525 ;
    _dT_params[0][1]=  -16880.2740313074;
    _dT_params[0][2]=  12867.7142585106 ;
    _dT_params[0][3]=  -3363.51052406353;
    _dT_params[0][4]=  292.460617554278;
    _dT_params[0][5]= 0;
		 
    _dT_params[1][0]= -5706.28654761803;
    _dT_params[1][1]=  20937.2623412812 ;
    _dT_params[1][2]=  -16759.4706250131;
    _dT_params[1][3]=  4464.23652778203 ;
    _dT_params[1][4]=  -392.358958333753;
    _dT_params[1][5]= 0;
    
    _dT_params[2][0]= 3047.80333333317;
    _dT_params[2][1]=  -11063.7030555512;
    _dT_params[2][2]=  9116.48541666194;
    _dT_params[2][3]=  -2460.36027777624;
    _dT_params[2][4]=  217.967916666513;
    _dT_params[2][5]= 0;

    _dT_params[3][0]=-899.991821429039;
    _dT_params[3][1]= 3046.22859523997;
    _dT_params[3][2]= -2579.98914583488;
    _dT_params[3][3]= 705.574708333773;
    _dT_params[3][4]= -63.0282291667072;
    _dT_params[3][5]= 0;

    _dT_params[4][0]= 154.97143253985;
    _dT_params[4][1]= -473.445552646092;
    _dT_params[4][2]= 412.008873611581;
    _dT_params[4][3]= -114.223045370503;
    _dT_params[4][4]= 10.2919291666788;
    _dT_params[4][5]= 0;
	
    _dT_params[5][0]=-15.3491762500078;
    _dT_params[5][1]= 41.9985363334113;
    _dT_params[5][2]= -37.5006902292425;
    _dT_params[5][3]= 10.53647404169;
    _dT_params[5][4]= -0.957505645835577;
    _dT_params[5][5]= 0;

    _dT_params[6][0]=0.805057706349255;
    _dT_params[6][1]= -1.98744670661367;
    _dT_params[6][2]= 1.81693607777767;
    _dT_params[6][3]= -0.516984125925895;
    _dT_params[6][4]= 0.0473617166666638;
    _dT_params[6][5]= 0;

    _dT_params[7][0]=-0.0172623238095152;
    _dT_params[7][1]= 0.0389708056348575;
    _dT_params[7][2]= -0.0363897941666099;
    _dT_params[7][3]= 0.0104751738888722;
    _dT_params[7][4]= -0.000966802499998446;
    _dT_params[7][5]= 0;

    _dT_params[8][0]= 0;
    _dT_params[8][1]= 0;
    _dT_params[8][2]= 0;
    _dT_params[8][3]= 0;
    _dT_params[8][4]= 0;
    _dT_params[8][5]= 0;

    _dT_params[9][0]= 0;
    _dT_params[9][1]= 0;
    _dT_params[9][2]= 0;
    _dT_params[9][3]= 0;
    _dT_params[9][4]= 0;
    _dT_params[9][5]= 0;

    _att[0]=0.383157426324749;
    _att[1]=-0.000468682252407133;
    _att[2]=1.45306944419336e-07;
    _att[3]=-6.60161997388468e-12;
    _att[4]=-6.98791967319079e-16;
    _att[5]=4.27452327695256e-20 ;
    _att[6]=0;
  }
  
  void TPCGas::initT2KGas()
  { 
   
    _dL_params[0] = 4056.36087909 ;
    _dL_params[1] = -3376.00345441;
    _dL_params[2] = 1326.25362669;
    _dL_params[3] = -247.019635862;
    _dL_params[4] = 15.4905714968;
    _dL_params[5] = 1.09869391304;
    _dL_params[6] = -0.139913612555;
    _dL_params[7] = -0.00605166882624;
    _dL_params[8] = 0.00116373462378;
    _dL_params[9] = -3.64924219222e-05;
    _dL_params[10]= 5.83448384413e-09;
  
    _vd_params[0] = 274.292045725;
    _vd_params[1] =-465.816877198	;
    _vd_params[2] =316.291326425	;
    _vd_params[3] =-106.857085941	;
    _vd_params[4] =17.2161530739	;
    _vd_params[5] =-0.557620091269	;
    _vd_params[6] =-0.182719493571	;
    _vd_params[7] =0.0126890150338	;
    _vd_params[8] =0.00218019067494	;
    _vd_params[9] =-0.000307558560924;
    _vd_params[10]=1.08668003287e-05;

    _dT_params[0][0]= -3338.99778302;
    _dT_params[0][1]= 20739.971345  ;
    _dT_params[0][2]= -26047.5189285;
    _dT_params[0][3]= 11497.5959353 ;
    _dT_params[0][4]= -2174.53266669;
    _dT_params[0][5]= 149.629186607 ;
  
    _dT_params[1][0]= 5690.93807321 ;
    _dT_params[1][1]= -27382.8519757;
    _dT_params[1][2]= 33775.4261902 ;
    _dT_params[1][3]= -14897.151834 ;
    _dT_params[1][4]= 2823.76070688 ;
    _dT_params[1][5]= -194.924318109;
      		      
    _dT_params[2][0]=  -2893.96374094 ;
    _dT_params[2][1]=  12940.8647563  ;
    _dT_params[2][2]=  -16133.4898009 ;
    _dT_params[2][3]=  7170.62115605  ;
    _dT_params[2][4]=  -1367.62771922 ;
    _dT_params[2][5]=  94.928419111   ;
      		      
    _dT_params[3][0]= 611.175363604 ;
    _dT_params[3][1]= -2598.53387206;
    _dT_params[3][2]= 3317.10406066 ;
    _dT_params[3][3]= -1492.8759095 ;
    _dT_params[3][4]= 287.284992397 ;
    _dT_params[3][5]= -20.0891215482;
      		      
    _dT_params[4][0]= -29.5156665766;
    _dT_params[4][1]= 106.381550495 ;
    _dT_params[4][2]= -148.036503573;
    _dT_params[4][3]= 69.3830005406 ;
    _dT_params[4][4]= -13.7145850898;
    _dT_params[4][5]= 0.979494941106;
    		      
    _dT_params[5][0]= -8.00617042512;
    _dT_params[5][1]= 34.4463472142 ;
    _dT_params[5][2]= -43.9789357232;
    _dT_params[5][3]= 19.7899654518 ;
    _dT_params[5][4]= -3.80805704946;
    _dT_params[5][5]= 0.266278327894;
      		      
    _dT_params[6][0]= 0.742005483054  ;
    _dT_params[6][1]= -2.68116961731  ;
    _dT_params[6][2]= 3.68644052349   ;
    _dT_params[6][3]= -1.71885913176  ;
    _dT_params[6][4]= 0.338571142498  ;
    _dT_params[6][5]= -0.0241129595712;
  		      
    _dT_params[7][0]= 0.123119908113   ;
    _dT_params[7][1]= -0.514574075224  ;
    _dT_params[7][2]= 0.66632442837    ;
    _dT_params[7][3]= -0.301891114093  ;
    _dT_params[7][4]= 0.0583564283108  ;
    _dT_params[7][5]= -0.00409537801912;
  
    _dT_params[8][0]=  -0.0212615167245 ;
    _dT_params[8][1]=  0.0789280649855  ;
    _dT_params[8][2]=  -0.106565523646  ;
    _dT_params[8][3]=  0.0492956310445  ;
    _dT_params[8][4]=  -0.00966074330017;
    _dT_params[8][5]=  0.000685333588906;	
 
    _dT_params[9][0]= 0.000877827765939	;
    _dT_params[9][1]= -0.00302583485186	;
    _dT_params[9][2]= 0.00418392472699	;
    _dT_params[9][3]= -0.00195860767246	;
    _dT_params[9][4]= 0.000386829155947	;
    _dT_params[9][5]= -2.76074116205e-05;
      

    _att[0]= -0.0093571083454;
    _att[1]= 0.000207651287462;
    _att[2]= -6.98766979618e-07;
    _att[3]= 6.47405302031e-10;
    _att[4]= -1.29100002282e-13;
    _att[5]= 9.80204293902e-18;
    _att[6]= -2.57371468197e-22;
 }

 
}//marlintpc
