#include "TPCVoxel.h"
#include <iostream>

using namespace lcio ;

namespace marlintpc{

  TPCVoxel::TPCVoxel(int padIndex, int timeIndex, int charge, int moduleID )
{
    setPadIndex  ( padIndex  );
    setTimeIndex ( timeIndex );
    setCharge    ( charge    );
    setModuleID  ( moduleID  );
}

TPCVoxel::~TPCVoxel(){ }

int TPCVoxel::getPadIndex() const
{
    return getIntVal( 0 ) ;
}

int TPCVoxel::getTimeIndex() const
{
    return getIntVal( 1 ) ;
}

int TPCVoxel::getCharge() const
{
    return getIntVal( 2 ) ;
}

int TPCVoxel::getModuleID() const
{
    return getIntVal( 3 ) ;
}

bool TPCVoxel::operator==(const TPCVoxel &right) const
{
  return (  (this->getModuleID()  == right.getModuleID() ) && 
            (this->getPadIndex()  == right.getPadIndex() ) && 
            (this->getTimeIndex() == right.getTimeIndex()) 
         );
}
  
bool TPCVoxel::operator<(const TPCVoxel &right) const
{
    //TODO::make it easy to read
    return ( this->getModuleID() != right.getModuleID() ) ?
           ( this->getModuleID() <  right.getModuleID() )  : 
           (   ( this->getPadIndex()  == right.getPadIndex()  ) ? 
               ( this->getTimeIndex() <  right.getTimeIndex() ) : 
               ( this->getPadIndex()  <  right.getPadIndex()  ) 
           );
}

TPCVoxel& TPCVoxel::operator=(const TPCVoxel& that)
{
  if (this == &that) return *this; // avoid fatal self-assignment

  if (_createdObject)
    delete _obj;

  _createdObject=that._createdObject;

  if (_createdObject)
    _obj = new  LCGenericObjectImpl(*(that._obj));
  else
    _obj = that._obj;

  return *this;
}

void TPCVoxel::print ( std::ostream& os ) const {
    os << "TPCVoxel:  "
       << "; ModuleID:"   << getModuleID()
       << "; PadIndeX: "  << getPadIndex()
       << "; TimeIndex: " << getTimeIndex()
       << "; Charge: "    << getCharge();
} 

std::ostream &operator<<(std::ostream &os, const TPCVoxel &tv) {
    tv.print(os);
    return os;
}

void TPCVoxel::setPadIndex(int padIndex){
    obj()->setIntVal( 0, padIndex) ;
}

void TPCVoxel::setTimeIndex(int timeIndex){
    obj()->setIntVal( 1, timeIndex) ;
}

void TPCVoxel::setCharge(int charge){
    obj()->setIntVal( 2, charge) ;
}

void TPCVoxel::setModuleID(int moduleid){
    obj()->setIntVal( 3, moduleid);
}

std::string TPCVoxel::getRevision(){
    return std::string("$Rev:$");
}

}
