/*
 * ModuleDistortionCorrectionProcessor.h
 *
 *  Created on: July 11, 2013
 *      Author: amunnich
 *
 *  Processor takes the hits and shifts them along the row and/or z according to
 *  input file given. This input file can be obtained with ModuleDistortionCalculatorProcessor
 *  or by a fit. The format is
 *  module row distortion error
 *
 *  Alternatively one can use an input file generated by Millepede from output of
 *  SimpleHelixGeneralBrokenLinesInterfaceProcessor
 */

#ifndef MODULEDISTORIONCORRECTIONPROCESSOR
#define MODULEDISTORIONCORRECTIONPROCESSOR


//STD
#include <string>
#include <vector>
//LCIO
#include "lcio.h"
#include "EVENT/TrackerHit.h"

//MARLIN
#include <marlin/Processor.h>

// GEAR
namespace gear
{
  class TPCParameters;
  class GearMgr;
}

namespace  marlintpc
{
class ModuleDistortionCorrectionProcessor : public marlin::Processor
{
public:

    /**
     * Returns an instance of this processor to the Marlin program
     */
    Processor*  newProcessor() { return new ModuleDistortionCorrectionProcessor ; }

    /**
     * Default constructor
     */
    ModuleDistortionCorrectionProcessor();

    /**
     * Virtual destructor
     */
    ~ModuleDistortionCorrectionProcessor();

    /** Called at the begin of the job before anything is read.
     * Use to initialize the processor, e.g. book histograms.
     */

    void init() ;

    /**
     * Called for every run.
     */
    void processRunHeader( lcio::LCRunHeader* run ) ;

    /**
     * Called for every event - the working horse.
     */
    void processEvent( lcio::LCEvent * evt ) ;


    void check( lcio::LCEvent * evt ) ;

    /**
     * Called after data processing for clean up.
     */
     void end() ;

 
private:
      /// Name of the output collection where the shifted hit data are stored
     std::string _HitDataCollName_OUT;
     /// Name of the input collection where the hit data are stored
     std::string _HitDataCollName;
     /// Name of the file containing distortions in z for each row
     std::string _DistortionZFileName;
     /// Name of the file containing distortions in row for each row
     std::string _DistortionRowFileName;
     /// Name of the file containing distortions calculated by Millepede
     std::string _MillepedeDistortionFileName;
     
     ///Maps containing the distortion corrections. Key is a pair consisting of module number and row number
     ///Distortion Correction for z
     std::map< std::pair<int,int> , std::pair<double,double> > _ZDistortionCorrection;
     ///Distortion Correction for row
     std::map< std::pair<int,int> , std::pair<double,double> > _RowDistortionCorrection;
     ///Distortion Correction depending on charge for z
     std::vector< std::pair<double,double> > _ZChargeCorrection;
     ///Distortion Correction depending on charge for row
     std::vector< std::pair<double,double> > _RowChargeCorrection;
     
     lcio::TrackerHitVec _AllShiftedHits;
     const gear::TPCParameters* _GearParameters;
     /**
     * Function shifting hits
     */
     void ShiftHits(const double* hitPos, double* newPos, int module, int row, double radius, double charge, int coordinate);
};

}

#endif 
