#ifndef PULSEFILTERLPROCESSOR_H
#define PULSEFILTERPROCESSOR_H

#include <marlin/Processor.h>
#include <lcio.h>
#include <string>


namespace marlintpc
{
/**  Processor which removes pulses with certain properties
 * 
 * @author I. Heinze, DESY
 * @author Ch. Rosemann, DESY
 * @author M. Killenberg, CERN
 *
 */

class PulseFilterProcessor : public marlin::Processor
{
 public:
  
  virtual Processor*  newProcessor() { return new PulseFilterProcessor; }
  
  PulseFilterProcessor();
  
  virtual void init();
  
  virtual void processRunHeader(lcio::LCRunHeader* run);
  
  virtual void processEvent(lcio::LCEvent* evt);
  
  virtual void check(lcio::LCEvent* evt);
  
  virtual void end();
  
  
 protected:
  /* the place for protected and private member data and functions */
  std::string _inputColName; ///< Name of the input collection
  std::string _outputCollectionName; ///<Name of the output collection

  float _chargeVariation; ///Number by which the charges in the spectrum must vary to call it a good pulse. Only pulses with a charge variations larger than this value will be accepted.

  float _slopeVariation; ///Number which the slope in the spectrum has to reach at some point to call it a good pulse. Only pulses with a slope variations larger than this value will be accepted. Otherwise we have a flat or to slow rising or falling pulse.
};
} // namespace marlintpc
#endif // PULSEFILTERPROCESSOR_H
