#ifdef USE_LCCD

#ifndef AFTERRAWDATACONVERTERPROCESSOR_H
#define AFTERRAWDATACONVERTERPROCESSOR_H 1

// Marlin
#include <marlin/Processor.h>

// LCIO
#include <lcio.h>

// stl
#include <string>

namespace marlintpc
{

/** AFTERRawDataConverter: splits the input raw data into smaller raw data chunks if it is suppressed. This needs to happen because the (AFTER?) Front End Mezzanine (FEM) card puts sentinal values into the ADC values which split the data. These need to be removed and the correct starting time for each collection of ADC values needs to be calculated.
 *    @author J. Abernathy (UVic)<br>
 */
class AFTERRawDataConverterProcessor : public marlin::Processor
{

    public:

        AFTERRawDataConverterProcessor() ;

        virtual void init() ;

        virtual Processor*  newProcessor()
        {
            return new AFTERRawDataConverterProcessor();
        }

 //FIXME: not used:       virtual void processRunHeader(lcio::LCRunHeader* run);

        virtual void processEvent(lcio::LCEvent * evt);

 //FIXME: not used:       virtual void check(lcio::LCEvent * evt);

        virtual void end();

    private:

        /** the name of the collection the input pulses will be stored under
        */
        std::string _input_tracker_raw_data_collection_name;

        /** the name of the output collection which will contain the photoelectric pulses
         */
        std::string _output_tracker_data_collection_name;

        /** determines whether the output of this processor will be stored in a file after program termination (false) or not (true - default)
        */
        bool _output_is_transient;

        /** specifies which bit signals a new pulse in the raw data, defaults to 13
         */
        int _new_pulse_sentinal_bit;

        /** specifies the number of bins which are stored before the pulse started, defaults to 10
         */
        int _bin_count_before_pulse;

        /** the number of bits per ADC sample (Default - 12). This overrides any information provided by LCCD. Currently this information can't be provided by LCCD so this parameter must be used
         */
        int _bits_per_sample_override;

};

}

#endif // AFTERRAWDATACONVERTERPROCESSOR_H
#endif // USE_LCCD
