#ifndef PHOTOELECTRICLIKELIHOODSEEDERPROCESSOR_H
#define PHOTOELECTRICLIKELIHOODSEEDERPROCESSOR_H 1

// Marlin
#include <marlin/Processor.h>

// GEAR
#include <gear/TPCParameters.h>
#include <gearimpl/GlobalPadIndex.h>

// LCIO
#include <EVENT/TrackerHit.h>

// CLHEP
#include <CLHEP/Vector/TwoVector.h>

// stl
#include <string>
#include <set>
#include <list>

namespace EVENT
{

class TrackerPulse;
}

namespace AIDA
{

class ITuple;
}

namespace marlintpc
{

/** PhotoelectricLikelihoodSeederProcessor: prepares the photoelectric data for fitting
 *    @author J. Abernathy (UVic)<br>
 */
class PhotoelectricLikelihoodSeederProcessor : public marlin::Processor
{

    public:

        PhotoelectricLikelihoodSeederProcessor() ;

        virtual void init() ;

        virtual Processor*  newProcessor()
        {
            return new PhotoelectricLikelihoodSeederProcessor();
        }

        virtual void processRunHeader(lcio::LCRunHeader* run);

        virtual void processEvent(lcio::LCEvent * evt);

        virtual void check(lcio::LCEvent * evt);

        virtual void end();

    private:

        /** the name of the input collection of TPCHits which represent photodots
        */
        std::string _input_photoelectric_tracker_pulses_collection_name;

        /** the name of the output collection of TPCHits which represent fit photodots
         */
        std::string _output_photodot_collection_name;

        /** the name of the output collection of Tracks which represent fit photolines
         */
        std::string _output_photoline_collection_name;

        /** determines whether the output of this processor will be stored in a file after program termination (false) or not (true - default)
        */
        bool _output_is_transient;

        /** specifies the name of the collection which stores the results of the projection finding
         */
        std::string _projected_photogeometry_file_name;

        /** determines whether the seed positions are written to the root file
         */
        bool _write_seed_tuple;

        /**
         */
        double _sigma;


        /**  tuple to store the seeds found by the seeder
         */
        AIDA::ITuple *_seed_position_tuple;

        /** this map contains a pointer to the set of TrackerPulses for each photodot
         */
        std::map< gear::GlobalPadIndex, std::list< std::set< EVENT::TrackerPulse* > >::iterator > _photodot_pulse_set_by_global_index;

        /** the actual collection of sets of pulses, one for each photodot
         */
        std::list< std::set< EVENT::TrackerPulse* > > _photodot_pulse_sets;

        /** this map contains a pointer to the set of TrackerPulses for each photoline
         */
        std::map< gear::GlobalPadIndex, std::list< std::set< EVENT::TrackerPulse* > >::iterator > _photoline_pulse_set_by_global_index;

        /** the actual collection of sets of pulses, one for each photoline
         */
        std::list< std::set< EVENT::TrackerPulse* > > _photoline_pulse_sets;
};

}

#endif // PHOTOELECTRICLIKELIHOODSEEDERPROCESSOR_H
