#ifndef INGRID_SIMPLE_CLUSTER_FINDER_H
#define INGRID_SIMPLE_CLUSTER_FINDER_H 1

// C++
#include <string>
#include <vector>

// LCIO
#include "lcio.h"
#include <EVENT/TrackerData.h>

// Marlin
#include "marlin/Processor.h"
#include "TimePixPixelMode.h"
#include "TimePixMapHandlerProcessor.h"
#include "lccd/ConditionsMap.hh"

//AIDA
#include <AIDA/AIDA.h>




namespace marlintpc
{
  /* the following comment is tranfered to doxygen documentation
  * which begins with a second asterix, or in case of on line comment an extra third slash 
  */
  /**  
  *  Converts tracker data objects to tracker hits by dividing every multihit into single hits.
  *   
  *   
  *
  *  <h4>Input - Prerequisites</h4>
  * @li a collection of zero suppressed TrackerData
  *
  *  <h4>Output</h4> 
  * @li Collection containing TrackerHits of single Pixels
  * @li Collection of TrackerData objects which has been split because it belonged to several clusters
  *  
  * @param InputCollectionName Name of the input collection with zero suppressed TrackerData (default "TimePixZeroSuppressedData")Name of input collection, TrackerData
  * @param OutputCollectionName Name of the output containing with sepatated hits (default "TimePixHits")
  * @param OutputCollectionName Name of the output containing with sepatated hits (default "TimePixHits")
  * @param Shutter length of the shutter signal
  * @param ReadoutFrequency readout frequency
  * @param DriftVelocity drift velocity of the given gas
  * @param SetOutputTransient If not 0 the output collection is set transient
  * @author Felix Müller
  */
  class InGridSimpleClusterFinder : public marlin::Processor
  {
    public:
      
      virtual Processor*  newProcessor() { return new InGridSimpleClusterFinder; }
      
      InGridSimpleClusterFinder();
      
      virtual void init();
      
      virtual void processRunHeader(EVENT::LCRunHeader* run);
      
      virtual void processEvent(EVENT::LCEvent* evt);
      
      virtual void check(EVENT::LCEvent* evt);
      
      virtual void end();
      
      
    protected:
      /* the place for protected and private member data and functions */
      std::string _inputColName; ///< Name of the input collection
      std::string _outputColName ; ///<- the name of the output collection
      std::string _outputTrackerDataColName ; ///<- the name of the output tracker data collection
      double _readoutFrequency; ///<- readout frequency of TimePix chip in MHz, optional, should come from gear
      double _driftVelocity; ///<- drift velocity of gas in mm/us
      double _shutter; ///lenght of the shutter in us
      int _medianfilter; ///use median filter on time mode pixels or not
      /** give the status which is set to the transient flag of the output collection
      * it is an int instead of a boolean, because the processor parameter cannot be a boolean
      */
      int _outputIsTransient;
      /** the actual mode map is a lccd::ConditionsMap
      *  the key is a PixelKey, specifically a struct which contains
      *  the ChipID and the PixelID
      *  the elements are TimePixPixelModes
      */
      tpcconddata::TimePixMap* _timePixMap;
  };
} // namespace marlintpc
#endif // INGRIDSIMPLECLUSTERFINDER_H
