#include "TimePixClusterProjectionSeparatorProcessor.h"
#include "TimePixMapHandlerProcessor.h"
#include "TimePixPixelMode.h"

using std::string;
using namespace tpcconddata;
using namespace lcio ;
using namespace marlin ;

namespace marlintpc{

  
    TimePixClusterProjectionSeparatorProcessor aTimePixClusterProjectionSeparatorProcessor ;
    

    TimePixClusterProjectionSeparatorProcessor::TimePixClusterProjectionSeparatorProcessor() : 
    Processor("TimePixClusterProjectionSeparatorProcessor") {
	/* the constructor calls first the super constructor which is the name 
	 * of the processor
	 * this must by the same as the class name.
	 */
	
	/* description of the processor which will be displayed in the
	 * steering file autometicly generated by 
	 * marlin (use MarlinTPC -x (XML-format) or MarlinTPXC -l (text format))
	 */
      _description = "TimePixClusterProjectionSeparatorProcessor further seperates " 
	"the clusters found in the TimePixClusterFinderProcessor "
	"by projecting them onto their own axis" ;
          	
	
	/* register steering parameters: name, description, class-variable, default value
	 * the type will by definded by the type of the default value
	 * string, double, float and int are possible
	 * plaese note to get a float you must cast in C++: (float) 1.3 
	 */
	registerInputCollection( LCIO::TRACKERHIT,
				 "InputCollectionName" , 
				 "Name of the input collection"  ,
				 _inputColName ,
				 std::string("TimePixHitCandidates") ) ;
	
	registerOutputCollection( LCIO::TRACKERHIT,
				  "OutputCollectionName" , 
				  "Name of the output collection"  ,
				  _outputColName ,
				  std::string("TimePixSepHitCandidates") ) ;

	registerOutputCollection( LCIO::TRACKERDATA,
				  "OutputTrackerDataCollectionName" , 
				  "Name of the output collection with sepatated tracker data objects (default TimePixSepData)"  ,
				  _outputTrackerDataColName ,
				  std::string("TimePixSepData") ) ;

	registerOutputCollection( LCIO::LCRELATION,
				  "OutputRelationsCollectionName" , 
				  "Name of the collection with relations between clusters and tracker data (default TimePixRelationSepHit)"  ,
				  _relationsColName ,
				  std::string("TimePixRelationSepHit") ) ;

//	registerInputCollection( LCIO::LCGENERICOBJECT,
//				  "ModeMapCollectionName" , 
//				  "Name of the mode map collection"  ,
//				  _modeMapColName ,
//				  std::string("TimePixModeMap") ) ;
//
//	registerInputCollection( LCIO::LCGENERICOBJECT,
//				 "ModeMapCorrCollectionName" , 
//				 "Name of the collection containing the correction "
//				 "to the mode map collection"  ,
//				 _modeMapCorrName ,
//				 std::string("TimePixModeMapCorr") ) ;

	registerProcessorParameter( "nColumn" , 
				    "number of columns of the chip"  ,
				    _nColumn ,
				    (int) 256 ) ;

	registerProcessorParameter( "nRow" , 
				    "number of rows of the chip"  ,
				    _nRow ,
				    (int) 256 ) ;

	registerProcessorParameter( "clusterSizeToSep" , 
				    std::string("number of pixels in a cluster in order for it ") + 
				    std::string("to be seperated (default 9)"),
				    _clusterSizeToSep ,
				    (int) 9 ) ;
				    
	registerProcessorParameter( "minResultingClusterSize" , 
				    "number of pixels in a split cluster ",
				    _minResultingClusterSize ,
				    (int) 9 ) ;				    
	    

	registerProcessorParameter( "SetOutputTransient" , 
				    "if not 0 the output collection is set transient"  ,
				    _outputIsTransient,
				    0);

				    
	registerProcessorParameter( "Keep A 3d histo with all Clusters of an event" , 
				    "if true large data amounts may be produced"  ,
				    _keep_hs_sep,
				    bool(false) ) ;			    
				    
				    
				    

	_timePixMap = 0;
    }
    
    
    void TimePixClusterProjectionSeparatorProcessor::init() { 
	
	// usually a good idea to
	printParameters() ;
	
//	highTOT =   AIDAProcessor::histogramFactory(this)->
//	  createHistogram1D( "highTOT", "TOT values above 1000",
//			     20000, 0, 20000);

	AIDAProcessor::tree(this);	//changes into directory of this processor in aida-output-file 
	_pixelCountHisto = new TH1F("PixelCounter","PixelCounter", (_nRow+_nColumn), -0.5, (_nRow+_nColumn)-0.5 );


    }
    
    void TimePixClusterProjectionSeparatorProcessor::processRunHeader(lcio::LCRunHeader* run) { 
	

      run->parameters().setValue( _processorName + "_revision", "$Rev: 3215 $");

      for ( ProcParamMap::iterator i = _map.begin(); i != _map.end(); i++ ) {
	if ( ! i->second->isOptional() || i->second->valueSet() ) {
	  run->parameters().setValue
	    ( _processorName + "_" + i->second->name(), i->second->value() );
	}
      }

					       

    } 
    
    void TimePixClusterProjectionSeparatorProcessor::processEvent(lcio::LCEvent * evt ) { 

//      streamlog_out(MESSAGE1)<<"Event "<<evt->getEventNumber()<<std::endl;

      streamlog_out(DEBUG3)<<"processEvent( "<<evt->getEventNumber()<<")"<<std::endl;

      //get the current TimePixMap from the MapHandlerProcessor
      _timePixMap = TimePixMapHandlerProcessor::getTimePixMap();

      // get the input collection from the event
      LCCollection* inputCol=0;
      try{      
	inputCol = evt->getCollection( _inputColName ) ;
      } //end of try {

      catch(DataNotAvailableException){
	// if there is not input collection give a warning to the user
	streamlog_out(WARNING2)
	  <<"processEvent: input collection "
	  <<_inputColName<<" cannot be found in event "
	  <<evt->getEventNumber()<<std::endl;	

	return;
      }	

	
      // create the  output collections with a string defining the TYPE (see lcio.h)
      LCCollectionVec* outputCol = new LCCollectionVec( LCIO::TRACKERHIT ); 
	
      LCCollectionVec* DataCol = new LCCollectionVec( LCIO::TRACKERDATA ); 
      
      LCCollectionVec* RelationsCol = new LCCollectionVec( LCIO::LCRELATION );
      
      // set the flags that cellID1 should be stored in the TrackerData
      lcio::LCFlagImpl trkFlag(0) ;
      trkFlag.setBit( lcio::LCIO::TRAWBIT_ID1 ) ;
      DataCol->setFlag( trkFlag.getFlag() );

      // set the transient flag as definded by the processor parameter
      // Do this after setting the cellID1 flag, which seems to delete the transient flag
      outputCol->setTransient( (_outputIsTransient != 0) );
      DataCol->setTransient( (_outputIsTransient != 0) );
      RelationsCol->setTransient( (_outputIsTransient != 0) );
      
      for ( std::vector<AIDA::IHistogram1D*>::iterator i = hProjectXVec.begin();
	    i < hProjectXVec.end(); i++ ) {
	
	AIDAProcessor::histogramFactory(this)->destroy( *i );	

      }


      hProjectXVec.clear();

      
      for(int iHit=0; iHit < inputCol->getNumberOfElements() ; iHit++){
	
	streamlog_out(DEBUG0)<<"Separate cluster "<<iHit+1<<" of "
			     <<inputCol->getNumberOfElements()<<std::endl;
	
	TrackerHit* RawHit = 
	  dynamic_cast<TrackerHit*>( inputCol->getElementAt( iHit ) ) ;
	
	if( RawHit != 0 ) {
	  
	  std::vector<LCObject*> DataVec = RawHit->getRawHits();
	  std::vector<LCObject*>::iterator j;
	  
	  //axis of projection parameters
	  a = 0;
	  b = 0;
	  double sumx = 0;
	  double sumy = 0;
	  double sumxy = 0;
	  double sumxx = 0;
	  double sumyy = 0;
	  double n = 0;
	  
	  int xmin = _nColumn;
	  int xmax = 0;
	  int ymin = _nRow;
	  int ymax = 0;
	  
	  //calculate axis of projection   
	  streamlog_out(DEBUG0)<<"calculate axis of projection"<<std::endl;
	  for ( j = DataVec.begin(); j != DataVec.end(); j++) {
	    
	    TrackerData* Data = dynamic_cast<TrackerData*>( *j ) ;
	    if ( Data != 0 ) {
	      int cellid0 = Data->getCellID0();
	      int x = cellid0%_nColumn;
	      int y = static_cast<int>((cellid0-x)/_nColumn); 
	      
	      FloatVec ChargeValVec = Data->getChargeValues();
	      n += ChargeValVec.size();
	      
	      FloatVec::iterator iChargeVal;
	      for ( iChargeVal = ChargeValVec.begin(); 
		    iChargeVal != ChargeValVec.end(); iChargeVal++ ) {
		
		//determine parameters to calculate
		sumx += x ;
		sumy += y;
		sumxy += x*y;
		sumxx += x*x;
		sumyy += y*y;
		
		if ( x < xmin ) xmin = x;
		if ( x > xmax ) xmax = x;
		if ( y < ymin ) ymin = y;
		if ( y > ymax ) ymax = y;
		
		x++;
		if ( x == _nColumn ) {
		  x = 0;
		    y++;
		}
		
	      } //end of for( iChargeVal = ChargeValVec.begin(); 
	      
	      ChargeValVec.clear();
	      
	    } //end of if ( Data != 0 ) {
	    
	  } //end of for ( j = DataVec.begin(); j != DataVec.end(); j++
	  
	  if ( n >= _clusterSizeToSep ) { 
	    //only calculate axis if cluster has more than _clusterSizeToSep pixels
	    
	    
	    //now calculate
	    if ( xmax - xmin >= ymax - ymin ) {
	      //cluster bigger in x-direction
	      //fit should be y = ax + b
	      
	      a = ( ( n * sumxy ) - ( sumx * sumy ) ) /
		( ( n * sumxx ) - ( sumx * sumx ) );
	      
	      b = ( ( sumxx * sumy ) - ( sumx * sumxy ) ) /
		( ( n * sumxx ) - ( sumx * sumx ) );
	      
	    }
	    
	    else {
	      //cluster bigger in y-direction
	      //fit should be x = ay + b
	      
	      a = ( ( n * sumxy ) - ( sumx * sumy ) ) /
		( ( n * sumyy ) - ( sumy * sumy ) );
	      
	      b = ( ( sumyy * sumx ) - ( sumy * sumxy ) ) /
		( ( n * sumyy ) - ( sumy * sumy ) );
	      
	    }
	    
	    
	    //Fill projection histogramm
	    streamlog_out(DEBUG0)<<"fill separation histograms"<<std::endl;
	    
	    	    
	    std::stringstream sEvt;
	    std::stringstream sHit;
	    sEvt << evt->getEventNumber();
	    sHit << iHit;
	    
	    std::string histoname = "HistoPro" + sEvt.str() + "_" + sHit.str();
	    std::string cluster = ": Projection on x-axis";
	    std::string histotitle = histoname + cluster;
	    
	    //if ( xmax - xmin >= ymax - ymin ) {    		// this if is useless since the histos have the same size anyway
	      
	      hProjectXVec.push_back(
		AIDAProcessor::histogramFactory(this)->
		createHistogram1D( histoname.c_str(), 
				   histotitle.c_str(), 
				   _nColumn+_nRow , -0.5, _nColumn+_nRow-0.5) );
	      
	   /*    }
	    
	    else  {
	      
	      hProjectXVec.push_back(
		AIDAProcessor::histogramFactory(this)->
		createHistogram1D( histoname.c_str(), 
				   histotitle.c_str(), 
				   _nRow, 0, _nRow) );
	      
	    }
	    */
	   
	    AIDA::IHistogram1D *hProjectX = *(hProjectXVec.end()-1);
	    _pixelCountHisto->Reset();

	    for ( j = DataVec.begin(); j != DataVec.end(); j++) {
	      
	      TrackerData* data = dynamic_cast<TrackerData*>( *j ) ;
	      if ( data != 0 ) {
		
		int cellid0 = data->getCellID0();
		int chipID  = data->getCellID1();
		int x = cellid0%_nColumn;
		int y = static_cast<int>((cellid0-x)/_nColumn);
		double d = 0;
		
		
		FloatVec ChargeValVec = data->getChargeValues();
		FloatVec::iterator iChargeVal;
		
		for ( iChargeVal = ChargeValVec.begin(); 
		      iChargeVal != ChargeValVec.end(); iChargeVal++ ) {
		    
		    
		  if ( xmax - xmin >= ymax - ymin )  
		      d = ( x + a * y ) / ( sqrt( 1 + a*a ) );
		  else 
		      d = ( y + a * x ) / ( sqrt( 1 + a*a ) );

		  if (d < 0 ){
		      //  streamlog_out(DEBUG0) <<"d is: "<<d <<std::endl;
		      d = -d;
		      // streamlog_out(DEBUG0) <<"d is new: "<<d <<std::endl;
		  }

		  int mode = 
		      ((_timePixMap->find( TimePixPixelMode::PixelKey( chipID, cellid0 ) ))
			 ->second)->getPixelMode();
		  
		  cellid0++;
		  
		  float weigh = 0;
		  
		  //if Pixel is in TOT Mode weigh the projection with the TOT value
		  if ( mode == TimePixPixelMode::TOT ) weigh = *iChargeVal; 
		  else weigh = 1;
		  
		  //some pixels have negative values, 
		  //those are broken and should be weighted with 1
		  //if ( weigh < 0 ) weigh = 1;
		  
		  hProjectX->fill(d, weigh);
		  _pixelCountHisto->Fill(d);
		  
		  x++;
		  if ( x == _nColumn ) {
		    
		    x = 0;
		    y++;
		    
		  }
		  
		  
		}
		ChargeValVec.clear();
	      }
	    }
	    
	    //_pixelCountHisto->Write();		// FIXME I'm just a debug line - remove me if debuging done
	    //separate clusters
	    //------------------------
	    
	    int locmin_index = 0;
	    int oldlocmin_index = locmin_index;
	    
	    double locmin = //(hProjectX->axis()).lowerEdge() + (hProjectX->axis()).binWidth(1)*
			    locmin_index;
	    double oldlocmin = locmin;
	    
	    bool separated = false;

	    while ( oldlocmin_index != _nColumn + _nRow -1 ) {
	      
	      //this is the distance on the straight line fit from x
	      streamlog_out(DEBUG0)<<"sepBinFinder( "<<locmin_index<<", "
				   <<histoname.c_str()<<" )"<<std::endl;
	      
	      locmin_index = sepBinFinder( locmin_index, hProjectX);
	      locmin = //(hProjectX->axis()).lowerEdge() + (hProjectX->axis()).binWidth(1)*
			locmin_index; 
	      
	      streamlog_out(DEBUG0)<<"locmin_index = "<<locmin_index
				   <<"     locmin = "<<locmin<<std::endl;

	      TrackerHitImpl* hitCandidate = new TrackerHitImpl();
	      std::vector<LCObject*> DataVec = RawHit->getRawHits();
	      std::vector<LCObject*>::iterator j;
	      
	      if ( locmin_index == 0 )  { //cluster needs not to be separated
		locmin_index = _nColumn + _nRow - 1;
		locmin = //(hProjectX->axis()).lowerEdge() + (hProjectX->axis()).binWidth(1)*
			  locmin_index;

		if ( separated ) { //but originates from a cluster that has already benn split
		    //set LCRelation of this new hit to the hit it originates from
		    //LCRelationImpl (EVENT::LCObject *from, EVENT::LCObject *to, float weight=1.0f)
		    LCRelationImpl* relation = new LCRelationImpl( RawHit, hitCandidate );
		    RelationsCol->addElement( relation );
		}

	      }
	      else { //cluster needs to be separated

		  //set LCRelation of this new hit to the hit it originates from
		  //LCRelationImpl (EVENT::LCObject *from, EVENT::LCObject *to, float weight=1.0f)
		  LCRelationImpl* relation = new LCRelationImpl( RawHit, hitCandidate );
		  RelationsCol->addElement( relation );

		  separated = true;
	      }

	      

	      for ( j = DataVec.begin(); j != DataVec.end(); j++) {
		
		TrackerData* Data = dynamic_cast<TrackerData*>( *j ) ;
		if ( Data != 0 ) {
		  
		  int cellid0 = Data->getCellID0();
		  int x = cellid0%_nColumn;
		  int y = static_cast<int>((cellid0-x)/_nColumn);
		  double d = 0;    
		  
		  FloatVec ChargeVal = Data->getChargeValues();
		  FloatVec::iterator iChargeVal;
		    
		  for ( iChargeVal = ChargeVal.begin(); 
			iChargeVal != ChargeVal.end(); iChargeVal++ ) {
		      
		      if ( xmax - xmin >= ymax - ymin )  
		      d = ( x + a * y ) / ( sqrt( 1 + a*a ) );
		  else 
		      d = ( y + a * x ) / ( sqrt( 1 + a*a ) );

		  if (d < 0 ){
		      // streamlog_out(DEBUG0) <<"d2 is: "<<d <<std::endl;
		      d = -d;
		      // streamlog_out(DEBUG0) <<"d2 is new: "<<d <<std::endl;
		  }
		      
		  		
		    //FIXME I'm a debugline
		    //std::cout << std::setprecision(5) << "d is" << d << std::endl;
		    
		    d += 0.5; // needed because we compare floats to ints
		    if ( d >= oldlocmin && d < locmin ) { 
		      
		      //FIXME I'm a debugline
		      //std::cout << "which is between: " << oldlocmin << " and " << locmin << std::endl;
		      
		      TrackerDataImpl* NewData = new TrackerDataImpl();
		      FloatVec NewChargeValVec;
		      NewData->setCellID1( Data->getCellID1() );
		      NewData->setCellID0( cellid0 );
		      
		      while ( d >= oldlocmin && 
			      d < locmin     && 
			      iChargeVal != ChargeVal.end()-1 ) {
			
			NewChargeValVec.push_back( *iChargeVal );
			
			iChargeVal++;
			
			cellid0++; 
			x++;
			if ( x == _nColumn ) {
			  x = 0;
			  y++;
			}
			
			if ( xmax - xmin >= ymax - ymin )  
			    d = ( x + a * y ) / ( sqrt( 1 + a*a ) );
			else 
			    d = ( y + a * x ) / ( sqrt( 1 + a*a ) );

			if (d < 0 )
			{
			    // streamlog_out(DEBUG0) <<"d3 is: "<<d <<std::endl;
			    d = -d;
			    // streamlog_out(DEBUG0) <<"d3 is new: "<<d <<std::endl;
			}
		
			
		      }//end of while( x >= oldlocmin && x < locmin && iChargeVal != ChargeVal.end()-1 )
			
		      if ( iChargeVal == ChargeVal.end()-1 && 
			   d >= oldlocmin                  && 
			   d < locmin                         ) { 
			
			NewChargeValVec.push_back( *iChargeVal );
			x++;
			if ( x == _nColumn ) {
			  x = 0;
			  y++;
			}
			
			if ( xmax - xmin >= ymax - ymin )  
			  d = ( x + a * y ) / ( sqrt( 1 + a*a ) );
			else 
			  d = ( y + a * x ) / ( sqrt( 1 + a*a ) );		
			if (d < 0 )
			{
			    //   streamlog_out(DEBUG0) <<"d4 is: "<<d <<std::endl;
			    d = -d;
			    //  streamlog_out(DEBUG0) <<"d4 is new: "<<d <<std::endl;
			}	
			
		      }
		      
		      if (NewChargeValVec.size() >0)
		      // only add the cluster if it is not empty
		      {
			  NewData->setChargeValues( NewChargeValVec );
			  DataCol->addElement( NewData );
			  hitCandidate->rawHits().push_back( NewData );
		      }
		      else // cluster is empty. In principle this should not happen
		      {
			  streamlog_out(WARNING4) << "Empty data not added to hit" << std::endl
						<< "oldlocmin " << oldlocmin << " ; locmin "<< locmin << std::endl;
			  
			  delete NewData;
		      }
			


		      
		      
		    } //end of if ( x >= oldlocmin && x < locmin ) {
		    
		    cellid0++;	     
		    x++;
		    if ( x == _nColumn ) {
		      x = 0;
		      y++;
		    }
		    
		    if ( xmax - xmin >= ymax - ymin )  
		      d = ( x + a * y ) / ( sqrt( 1 + a*a ) );
		    else 
		      d = ( y + a * x ) / ( sqrt( 1 + a*a ) );		
		    	if (d < 0 )
			{
			    //  streamlog_out(DEBUG0) <<"d5 is: "<<d <<std::endl;
			    d = -d;
			    // streamlog_out(DEBUG0) <<"d5 is new: "<<d <<std::endl;
			}
		    
		  } //end of for ( iChargeVal = ChargeVal.begin();
		  
		  
		} //end of if ( Data != 0 ) {
		
	      }//end of for ( j = DataVec.begin(); j != DataVec.end(); j++) {
	      
	      oldlocmin_index = locmin_index;
	      oldlocmin = locmin;
	      
	      if (hitCandidate->rawHits().size() !=0)
	      {
		  outputCol->addElement( hitCandidate  );
	      }
	      else
	      {
		  streamlog_out(DEBUG0) << "Empty hit not added to collection" << std::endl;
		  delete hitCandidate;
	      }
	      
	    }//end of while ( oldlocmin != _nColumn ) {    
	    
	  } //end of if ( n > clusterSizeToSep ) {
	  
	  
	    //add the unseparated cluster to output collection
	  else{ 
	    
	    TrackerHitImpl* NewRawHit = new TrackerHitImpl();
	    std::vector<LCObject*> DataVec = RawHit->getRawHits();
	    std::vector<LCObject*>::iterator j;
	    
	    for ( j = DataVec.begin(); j != DataVec.end(); j++) {
	      
	      NewRawHit->rawHits().push_back( *j );
	      
	    }
	    
	    outputCol->addElement( NewRawHit ); 
	    
	  }
	  
	  

	  
	} //end of if ( RawHit != 0 ) 
	
      } //end of for(int iHit=0; iHit< nHit ; iHit++){
      
      
	// save the processor parameters in the first collection stored
//      if (isFirstCollection){
//	outputCol->parameters().setValue("InputCollectionName",_inputColName);
//	// TO DO: does this make sence? 
//	outputCol->parameters().setValue("SetOutputTransient",_outputIsTransient);
//	isFirstCollection = false;
//      }
      
      evt->addCollection( DataCol, _outputTrackerDataColName );
      
      evt->addCollection( outputCol, _outputColName );

      if ( RelationsCol->getNumberOfElements() != 0 )
	  evt->addCollection( RelationsCol, _relationsColName);
      else delete RelationsCol;

      streamlog_out(DEBUG1)<<"end process event "<<evt->getEventNumber()<<std::endl;


    }
    
    
    void TimePixClusterProjectionSeparatorProcessor::check(lcio::LCEvent * evt ) { 

      streamlog_out(DEBUG1)<<"check( evt "<<evt->getEventNumber()<<")"<<std::endl;

      LCCollection* HitCol;

      try{
	  HitCol = evt->getCollection( _outputColName );
      }
      catch(DataNotAvailableException)
      {
	  return;
      }


      std::stringstream sEvt;
      sEvt << evt->getEventNumber();

      std::string aidahistoname = 
	"aidahistocluster" + sEvt.str();
      std::string aidahistotitle = 
	"010707_001.siy.300.310 TOT values of Event " + sEvt.str();
	      
      AIDAProcessor::histogramFactory(this)->destroy(aidahistocluster);
      aidahistocluster = AIDAProcessor::histogramFactory(this)->
	createHistogram2D( aidahistoname.c_str(), 
			   aidahistotitle.c_str(), 
			   256, 0, 256, 256, 0, 256);

      std::string stackname = "clustersevt" + sEvt.str();


      int n = HitCol->getNumberOfElements();

      AIDAProcessor::tree(this);
      TH2F** histoCluster = new TH2F*[n];
      THStack* hs_sep = new THStack( stackname.c_str() , stackname.c_str() );


      for( int i = 0; i < n; i++ ) {
	TrackerHit* Hit = dynamic_cast<TrackerHit*>( HitCol->getElementAt( i ) ) ;

	std::stringstream si;
	si << i;

	std::string histoname = "histo" + sEvt.str() + si.str();

	histoCluster[i] = new TH2F (histoname.c_str() , histoname.c_str() ,
				    256, 0.5, 256.5, 256, 0.5, 256.5);

	int color = i%10+1;
	if ( color == 0 || color == 10 || color == 18 ) color = 46;
	histoCluster[i]->SetLineColor(color);

	if ( Hit != 0 ) {
	  std::vector<LCObject*> DataVec = Hit->getRawHits();
	  std::vector<LCObject*>::iterator j;
	  for ( j = DataVec.begin(); j != DataVec.end(); j++) {


	    TrackerData* data = dynamic_cast<TrackerData*>( *j ) ;

	    if ( data!=0 ) {
	      int cellid0 = data->getCellID0();
	      int chipID  = data->getCellID1();
	      int x = cellid0%_nColumn;
	      int y = static_cast<int>((cellid0-x)/_nColumn);

	      FloatVec ChargeVec  = data->getChargeValues();
	      FloatVec::iterator sv;
	      for ( sv = ChargeVec.begin(); sv != ChargeVec.end(); sv++ ) {

		  int mode = 
		      ((_timePixMap->find( TimePixPixelMode::PixelKey( chipID, cellid0 ) ))
		       ->second)->getPixelMode();

		  
		  float weight = 0;
		  if ( mode == TimePixPixelMode::TOT ) weight = *sv;
		  
//		  if ( weight > 1000 ) highTOT->fill( weight );
		  
		  streamlog_out(DEBUG0)<<"weigh = "<<weight<<std::endl;
		  
		  histoCluster[i]->Fill( x, y, weight);
		  
		  if ( weight > 0 ) aidahistocluster->fill( x, y, weight);
		  
		  cellid0++;
		  x++;
		  
		  if ( x == _nColumn ) {
		      x = 0;
		      y++;
		  }
		  
	      } //end of for ( sv = ChargeVec.begin(); sv != ChargeVec.end(); sv++ ) {
	      
	      ChargeVec.clear();
	      
	    } //end of if ( Data!=0 ) {
	    
	    
	  } //end of for ( j = DataVec.begin(); j != DataVec.end(); j++) {

	} //end of if ( Hit != 0 ) {


	hs_sep->Add(histoCluster[i]);

      } //end of for( int i = 0; i < n; i++ ) {

      if ( _keep_hs_sep == true ) hs_sep->Write();
      delete hs_sep;

      for( int i = 0; i < n; i++ ) {
	histoCluster[i]->SetDirectory(0);
	delete histoCluster[i];
      }

      delete[] histoCluster;


     streamlog_out(DEBUG1)<<"end check( evt "<<evt->getEventNumber()<<")"<<std::endl;
    }
    
    
    void TimePixClusterProjectionSeparatorProcessor::end(){ 

//      delete _corrModeMap;
//      _corrModeMap = 0;
    
      //std::cout<<"end()"<<std::endl;
      /*      	
      streamlog_out(MESSAGE4) << "TimePixClusterProjectionSeparatorProcessor::end()  " 
      << name() 
      << " processed " << evt->getEventNumber() << " events in " << _nRun << " runs "
      << std::endl ;
      */
    }



  int TimePixClusterProjectionSeparatorProcessor::locMaxFinder
  (int startbin, AIDA::IHistogram1D * h1 ) {

    
    int locmax = startbin;


    double upperedge = (h1->axis()).upperEdge();
    int upperedge_index = (h1->axis()).coordToIndex( upperedge );

    double loweredge = (h1->axis()).lowerEdge();
    int loweredge_index = (h1->axis()).coordToIndex( loweredge );

    for ( int ih1 = startbin; ih1 < upperedge_index-2; ih1++ ) {

      double binheight = h1->binHeight( ih1 );
      double nextbinheight = h1->binHeight( ih1+1 );
      double nextnextbinheight = h1->binHeight( ih1+2 );
      double nextnextnextbinheight = h1->binHeight( ih1+3 );

      //allow one fluctuation in between if the general trend is acending
      //but only if no binheight ever goes down to zero
      if ( ( (binheight > nextbinheight &&  nextbinheight > nextnextbinheight) ||
	     (binheight > nextbinheight && nextbinheight > nextnextnextbinheight) ) && 

	   ( binheight != 0 && nextbinheight != 0 && 
	     nextnextbinheight != 0 && nextnextnextbinheight != 0 )  ){

	//binheight might be a local maximum
	locmax = ih1;

	//test if the preceding bins where of equal height 
	//and choose the middle one of the equals
	double prebinheight = h1->binHeight( ih1-1 );
	int count = 1;
	
	if ( prebinheight == binheight ) {
	  for ( int i = ih1; i > loweredge_index+1; i-- ) {
	    
	    prebinheight = h1->binHeight( i-1 );
	    if ( prebinheight == binheight ) {
	      locmax += i-1;
	      count++;
	    }
	    else break;
	    
	  }
	}

	locmax = static_cast<int>(locmax/count);
	return locmax;
      }

      //if there is a minimum followed by the end of the chip and raising bins, 
      //one should cut at that minimum too, 
      //so return the upperedge and not 0
      if ( ih1 == static_cast<int>(upperedge)-3 ) {
	//&& binheight < nextbinheight && nextbinheight < nextnextbinheight ) {

	locmax = static_cast<int>(upperedge);

	return locmax;


      }
      

    }
    locmax = 0;
    return locmax;
    
  }
  
  int TimePixClusterProjectionSeparatorProcessor::locMinFinder
  (int startbin, AIDA::IHistogram1D * h1 ) {
 

    int locmin = startbin;

    double upperedge = (h1->axis()).upperEdge();
    int upperedge_index = (h1->axis()).coordToIndex( upperedge );

    //double loweredge = (h1->axis()).lowerEdge();
    //int loweredge_index = (h1->axis()).coordToIndex( loweredge );
  
    for ( int ih1 = startbin; ih1 < upperedge_index-3; ih1++ ) {

      double binheight = h1->binHeight( ih1 );
      double nextbinheight = h1->binHeight( ih1+1 );
      double nextnextbinheight = h1->binHeight( ih1+2 );
      double nextnextnextbinheight = h1->binHeight( ih1+3 );
      
      //allow one fluctuation in between if the geberal trend is acending       
      if ( (binheight < nextbinheight && nextbinheight < nextnextbinheight) ||
	  (binheight < nextbinheight && nextbinheight < nextnextnextbinheight) )
      {	  
	//binheight might be a local minimum
	locmin = ih1;
	
	//test if the preceding bins where of equal height 
	//and choose the middle one of the equals
	double prebinheight = h1->binHeight( ih1-1 );
	int count = 1;
	
	if ( prebinheight == binheight && binheight != 0 ) {
	  //for ( int i = ih1; i > loweredge_index+1; i-- ) {
	    for ( int i = ih1; i > startbin+1; i-- ) {
	      
	    prebinheight = h1->binHeight( i-1 );
	    
	    if ( prebinheight == binheight) {
	      locmin += i-1;
	      count++;
	      }
	    else break;
	    
	    }
	  }
	
	locmin = static_cast<int>(locmin/count);
	return locmin;
	
	} // end if general trend is acending
      //} // end min cluster size

    }

    locmin = 0;
    return locmin;
 
  }


  int TimePixClusterProjectionSeparatorProcessor::sepBinFinder
  ( int startlocmin, AIDA::IHistogram1D * h1) {
  
    int locmin = startlocmin;
    int locmax = locMaxFinder( locmin , h1 );
       
    double upperedge = (h1->axis()).upperEdge();
    int upperedge_index = (h1->axis()).coordToIndex( upperedge );
    
    if ( locmax != 0 ) {

	//std::cout << "locmin1 (startvalue) is now: " << locmin << std::endl;

	/*
	while ( _pixelCountHisto->Integral(locmin+1, locmax) < _minResultingClusterSize )   // this is in BIN numbers KEEP IN MIND used to be <=
	  {locmax++; 					// sep bin has to be large enogh that any resulting cluster will have enogh pixels
	   if (locmax >= upperedge_index-2) return 0;	// no sep. bin found;

	   //std::cout << "There will be at least " << _pixelCountHisto->Integral(locmin, locmax-1) << "Pixels in the cluster" << std::endl ;

	  }
	*/
	  
	//std::cout << "locmax1 is now: " << locmax << std::endl;
      
	locmin = locMinFinder( locmax, h1);	
	
	if( locmin != 0 ) {
	  
	  while ( _pixelCountHisto->Integral(startlocmin+1, locmin) < _minResultingClusterSize )
	  { 
	    //std::cout << "to few pixels so far - get next min" << locmin << std::endl;
	    //std::cout << "next locmin is: " << locMinFinder( locmin+1, h1) << std::endl;
	    locmin = locMinFinder( locmin+1, h1);		// to few pixels so far -> get next possible splitting point
	    
	    if ( locmin == 0 ) 
	    {
	      return 0;						// no sep. bin found!
	      //std::cout << "to few pixels no sep. bin found!" << std::endl;
	    }
	  }   
	  
	  //std::cout << "locmin2 is now: " << locmin << std::endl;
	  //std::cout << "pixels found following locmin2 is: " << _pixelCountHisto->Integral(locmin+1, upperedge_index) << std::endl;
	  if ( _pixelCountHisto->Integral(locmin+1, upperedge_index) < _minResultingClusterSize ) return 0; // sep. not allowed since remaining Cluster is to small
	  locmax = locMaxFinder( locmin , h1 );
	  
	  //std::cout << "locmax2 is now: " << locmax << std::endl;
	  
	  //locmax = 0;
	  if( locmax != 0 ) {
	    //std::cout << "sep. advised at" << locmin << std::endl;
	    return locmin;
	    
	  }
	  
	}
	
      
    }

    locmin = 0;
    return locmin;

  }
    




}
