#ifndef GENERALBROKENLINEINTERFACEHELPERS_H
#define GENERALBROKENLINEINTERFACEHELPERS_H

#include "SimpleHelixTrackModel.h"

#include <marlin/Processor.h>
#include <marlin/Global.h>

#include <lcio.h>
#include <string>

// GBL:
#include "GblTrajectory.h"
#include "MilleBinary.h"

// LCIO:
#include <EVENT/Track.h>
#include "EVENT/TrackerHit.h"

#include "TFile.h"
#include "TTree.h"
#include "TMath.h"

namespace marlintpc {

/// Helper hit
class gblHelperHit {
	public:
		gblHelperHit(const EVENT::TrackerHit&, const simpleHelix&, const bool, const bool, const int, const double);
		gblHelperHit(const double);

		/// Get projection from local to measurement system.
		TMatrixD getLocalToMeasurementProjection() const {
			return _l2M;
		}

		/// Get residuals.
		TVectorD getResiduals() const {
			return _res;
		}

		/// Get precision.
		TMatrixDSym getPrecision() const {
			return _prec;
		}

		/// Get arc-length.
		double getS() const {
			return _s;
		}

		/// Get phi (direction )of seed track.
		double getPhi() const {
			return _seedPhi;
		}

		/// Get scaled Edep.
		double getEScaled() const {
			return _eScaled;
		}

		/// Get measurement flag.
		bool isMeasurement() const {
			return _isMeas;
		}

		/// Get flag for valid prediction.
		bool isValid() const {
			return _isValid;
		}

		// access global derivatives for Millepede-II
		unsigned int getNumGlobals() const;
		std::vector<int> getGlobalLabels() const;
		const TMatrixD& getGlobalDerivatives() const;

	private:
		const bool _isMeas; ///< flag for measurements (else reference point)
		double _hitPhi; ///< phi of hit (with respect to module offset)
		bool _isValid; ///< hit has valid prediction from track
		double _seedPhi; ///< phi (XY direction) from seed track
		double _seedLambda; ///< lambda (ZS direction) from seed track
		double _xyPred; ///< XY residual (measurement - prediction)
		double _zPred; ///< Z residual (measurement - prediction)
		double _s; ///< arc-length
		double _eScaled; ///< Edep scaled to relative RowHeights
		TMatrixD _l2M; ///< transformation from local to measurement system
		TMatrixD _getL2M();
		TVectorD _res; ///< residual vector
		TVectorD _getRes();
		TMatrixDSym _prec; ///< precision matrix
		TMatrixDSym _getPrec(FloatVec);
		double _calcLocalPhi(const EVENT::TrackerHit&, const bool, const double);
		double _calcEScaled(const EVENT::TrackerHit&);
		// global derivatives for Millepede-II
		unsigned int _numGlobals; ///< number of global derivatives
		std::vector<int> _globalLab; ///< labels for global derivatives
		TMatrixD _globalDer; ///< global derivatives
};

/// User defined class to calculate global derivatives for Millepede-II.
class trackerAlcaCalculator {
	public:
		trackerAlcaCalculator();
		unsigned int getNumGlobals() const;
		std::vector<int> getGlobalLabels() const;
		const TMatrixD& getGlobalDerivatives() const;
		void calcModuleAlignment(const EVENT::TrackerHit& aHit, const TVectorD& tdir);
		void calcTrackDistortions(const EVENT::TrackerHit& aHit);
		void calcHitCharge(const EVENT::TrackerHit& aHit);
		void calcDr(const EVENT::TrackerHit& aHit, double phi);

	private:
		unsigned int _numDer; ///< number of global derivatives
		std::vector<int> _globalLab; ///< labels for global derivatives
		TMatrixD _globalDer; ///< global derivatives
};

/// User defined class to select tracks for Millepede-II.
class trackerAlcaSelector {
	public:
		trackerAlcaSelector(const EVENT::Track&, const int);
		bool isSelected() const;

	private:
		bool _selectionFlag; ///< selction flag
};

} // namespace marlintpc
#endif // GENERALBROKENLINEINTERFACEHELPERS_H
