#ifndef PATHFINDERINTERFACEPROCESSOR_H
#define PATHFINDERINTERFACEPROCESSOR_H

#include <marlin/Processor.h>
#include <marlin/Global.h>
#include <lcio.h>
#include <string>

#include <gear/TPCModule.h>

#include "FinderParameter.h"

#include "TFile.h"
#include "TTree.h"

/// The PathFinder tracking package
using namespace pathfinder;

namespace marlintpc
{
///  Short description of processor.

/**
 * Lengthy and detailed description of processor ...
 * @author Fn. Lastname, Affiliation
 *
 * \par Credits:
 * The processor skeleton was generated by the script \c createProcessor.py
 *
 */

class PathFinderInterfaceProcessor : public marlin::Processor
{
 public:
  
  virtual Processor*  newProcessor() { return new PathFinderInterfaceProcessor; }
  
  PathFinderInterfaceProcessor();
  
  virtual void init();
  
  virtual void processRunHeader(lcio::LCRunHeader* run);
  
  virtual void processEvent(lcio::LCEvent* evt);
  
  virtual void check(lcio::LCEvent* evt);
  
  virtual void end();
  

  
 protected:
  /* the place for protected and private member data and functions */
  std::string _inputColName; ///< Name of the input collection
  std::string _outputColName;  ///< Name of the output collection
  std::string _outputHitColName;

  int _nEvt;
  TFile* _rootFile;
  TTree* _rootTree;
  double trackD0, trackPhi, trackOmega, trackZ0, trackTanL;
  double trackD0Error, trackPhiError, trackOmegaError, trackZ0Error, trackTanLError;
  double trackSlopeXY, trackOffsetXY;
  double trackSlopeXYError, trackOffsetXYError;
  int nHitsOnTrack;
  double hitX[1000], hitY[1000], hitZ[1000], xError[1000], yError[1000], zError[1000], hitPhi[1000], hitR[1000], phiError[1000];

 private:

  gear::Vector3D _referencePoint;

  int track_type;
  bool usevertex;
  bool useconformalmapping;

  double xydistance;
  double szdistance;
  double xydistancefit;
  double szdistancefit;
  int minhitnumber;
  int maxhits;
  // double mind0xy;
  //double maxd0xy;
  //double minomegaxy;
  //double maxomegaxy;
  //double mind0sz;
  //double maxd0sz;
  int xythetabins;
  int xydzerobins;
  int xyomegabins;
  int szthetabins;
  int szdzerobins;
  //double xyd0binwidth;
  //double xyomegabinwidth;
  //double szd0binwidth;  
  double maxdxy;
  double maxdsz;
  bool searchneighborhood;
  bool saverootfile;
  bool shifthits;

  //  std::vector<double> vertexposition;

  //currently vertex constraint is only used for the xy circle search, so only these compnents are needed
  double vertexpositionx;
  double vertexpositiony;

  double referencepointx;
  double referencepointy;
  double referencepointz;
  
  FinderParameter* TrackFinderParameter;

  gear::Vector3D FindModuleCenter(const std::vector<gear::TPCModule*> theModules);
  
};

} // namespace marlintpc
#endif // PATHFINDERINTERFACEPROCESSOR_H
