#ifndef TRACKFITTERSIMPLEHELIX_H
#define TRACKFITTERSIMPLEHELIX_H

#include <marlin/Processor.h>
#include <lcio.h>
#include <string>
#include "TrackFitterBase.h"
#include <EVENT/Track.h>
#include <IMPL/TrackImpl.h>
#include <EVENT/TrackerHit.h>


namespace marlintpc
{
/**
 * 
 * @author F. Mueller
 *
 */

class TrackFitterSimpleHelix : public TrackFitterBase
{
public:

    /// Constructor for the simple helix fitter, initializes the track fitter with the initial values
	TrackFitterSimpleHelix();

	virtual ~TrackFitterSimpleHelix() {}

	/// Well defined interface to create a new instance of the track fitter
	static TrackFitterBase * getInstance(LCParameters const * parameters);

	virtual std::string getRevision() const;

	/// Information what fitter implementation this is.
	virtual unsigned char getFitterType() const;

	/// The implementation using the fitter function with all errors set to 1.
	virtual IMPL::TrackImpl *fitTrack(EVENT::Track const * seedTrack) const;


protected:
	//helper function to perform the linear regression in the xy-plane
	EVENT::DoubleVec doXYCircularFit(EVENT::Track const * seedTrack) const;

	//helper function to perform the linear regression in the sz-plane
	EVENT::DoubleVec doRegressionSZ(EVENT::Track const * seedTrack) const;

	//helper function to calculate the point of closest approach of the track
	EVENT::DoubleVec calculatePCA(EVENT::Track const * seedTrack) const;

	//helper function to calculate the arc length
	double calculateS(EVENT::TrackerHit * aHit, EVENT::DoubleVec pca, EVENT::Track const * seedTrack ) const;

};
} // namespace marlintpc
#endif // TRACKFITTERSIMPLEHELIX_H
