#ifndef TRACK_FITTER_SIMPLE_MINIMIZER_PROCESSOR_H
#define TRACK_FITTER_SIMPLE_MINIMIZER_PROCESSOR_H 1

#include "marlin/Processor.h"
//#include "lcio.h"
//#include <string>
#include <vector>
#include "TrackFitterFactory.h"

//#include <EVENT/TrackerHit.h>

//namespace AIDA {
//
//  class IHistogram1D;
//  class IHistogram2D;
//
//
//}


namespace marlintpc
{

/** A processor which instantiates a TrackFitterSimpleMinimizer and fits the tracks with it.
 *
 *  Optionally an itterative outlier rejection can be performed. The residual distribution
 *  is calculated from the hits on the track and a \c CutValue * RMS cut is applied. Afterwards
 *  the track is refit. This procedure is repeated until the cut converges or the MaxIterations
 *  is reached.
 *
 *  @author M. Killenberg
 *
 *  <h4>Input - Prerequisites</h4>
 *  \li A collection of track candidates (lcio::Tracks) containing the TrackerHits.<br>
 *  Note: The LCIO::TRBIT_HITS flag has to be set when this collection is created so
 *      the hits are stored in the track.
 *
 *  <h4>Output</h4>
 *  \li Collection of fitted tracks.
 *
 *  @param InputSeedTracks The the name of the input collection of track candidates 
 *         (default: TPCSeedTracks)
 *  @param OutputTracks The name of the output collection with the fitted tracks
 *         (default: TPCTracks)
 *  @param WriteOutputToStorage If true the output collection is written to file (default: true) *
 *  @param MaxIterations Maximum number of iterations for outlier rejection (default: 10)
 *  @param CutValue Cut outliers wich are _cutValue*RMS away from the fitted track, 
 *         optional: Setting this value turns on this feature.
 *
 */
class TrackFitterSimpleMinimizerProcessor : public marlin::Processor
{

    public:

        /** Default constructor
        */
        TrackFitterSimpleMinimizerProcessor();


        /** Destructor
        */
        ~TrackFitterSimpleMinimizerProcessor();


        /** Return a new instance of this processor
        */
        Processor * newProcessor();


        /** In this processor the init function set the
         *  collection parameters for the track collection
        */
        void init();


        /** Called once per run to process the event's header
        */
        void processRunHeader(lcio::LCRunHeader *run);


        /** Process event requests a track fitter from the
         *  TrackFitterFactory for each event (to update possible
         *  conditions data).
         *  Afterwards it loops all tracks and fits them.
         */
        virtual void processEvent(lcio::LCEvent *evt);


        /** Called for every event - right after processEvent()
         *  has been called.
        */
        virtual void check(lcio::LCEvent *evt);


        /** Called after data processing for clean up in the
         *  inverse order of the init() method so that resources
         *  allocated in the first processor also will be available for
         *  all following processors.
        */
        virtual void end();


    protected:

        /** The name of the input collection of seed tracks
        */
        std::string _input_seed_tracks_collection_name;

        /** The name of the output collection with the fitted tracks
        */
        std::string _output_tracks_collection_name;

        bool _writeOutput; //< Flag wheter to write the output to storage

	int _maxIterations; //< Maximum number of iterations for the outlier rejection

	bool _rejectOutliers; //< Flag whether to reject outliers

	float _cutValue; //< Cut outliers wich are _cutValue*RMS away from the fitted track.

        EVENT::LCParameters * _trackCollectionParameters;

	/// Returns meanXY, meanZ, rmsXY, rmsZ (in this order)
	std::vector<double> calculateMeanAndRMS( EVENT::Track *track, TrackFitterBase *fitter);
};

}// namespace marlintoc

#endif // TRACKSEEDERPROCESSOR_H



