#ifndef TrackRecoTools_h
#define TrackRecoTools_h 1

// C++
#include <string>
#include <cmath>
#include <algorithm>
#include <CLHEP/Vector/ThreeVector.h>

// Marlin
#include "marlin/Processor.h"

// LCIO
#include "lcio.h"
#include <EVENT/TrackerHit.h>
#include <EVENT/Track.h>

using namespace marlintpc;

namespace tracktools{

static bool compareHit(const std::pair<TrackerHit*, double> & pair1,
			       const std::pair<TrackerHit*, double> & pair2)
	  {
	    return pair1.second < pair2.second;
	  }

static double CalculateS(EVENT::Track* currentTrack, EVENT::TrackerHit * currentHit)
{

	double pca_x = ( -1.0 * currentTrack->getD0() ) * std::sin(currentTrack->getPhi());
	double pca_y = ( currentTrack->getD0() ) * std::cos(currentTrack->getPhi());

	double s = 0.;
	const double* HitPosition=currentHit->getPosition();

	if(fabs(currentTrack->getOmega()) < 0.000000001)
	{
		s = std::sqrt(
				static_cast<long double>(( HitPosition[0] - pca_x ) * ( HitPosition[0] - pca_x )
						+ ( HitPosition[1] - pca_y ) * ( HitPosition[1] - pca_y )));

		//make sure the sign of s is right
		if(( ( ( -1.25 * M_PI ) <= currentTrack->getPhi() ) && ( currentTrack->getPhi() <= ( -0.75 * M_PI ) ) )
				|| ( ( ( -0.25 * M_PI ) <= currentTrack->getPhi() ) && ( currentTrack->getPhi() <= ( 0.25 * M_PI ) ) )
				|| ( ( ( 0.75 * M_PI ) <= currentTrack->getPhi() ) && currentTrack->getPhi() <= ( 1.25 * M_PI ) ))
		{
			if(pca_x < HitPosition[0])
			{
				s = ( -1.0 ) * s;
			}
		}
		else
		{
			if(pca_y < HitPosition[1])
			{
				s = ( -1.0 ) * s;
			}
		}
		s = (-1.)*s; //Needed to make tracks look correctly in CEDViewer
	}
	else
	{
		double r = 1. / currentTrack->getOmega();
		//center of circle
		double xm, ym;
		xm = (r - currentTrack->getD0()) * sin(currentTrack->getPhi());
		ym = -(r - currentTrack->getD0()) * cos(currentTrack->getPhi());
		s = r * ( atan2(pca_y - ym , pca_x - xm)
				-
				atan2(HitPosition[1]- ym , HitPosition[0] - xm)
		);

		double s_1, s_2;
		s_1 = s;
		if(s < 0)
		{
			s_2 = s + fabs(2.*M_PI/currentTrack->getOmega());
		}
		else
		{
			s_2 = s - fabs(2.*M_PI/currentTrack->getOmega());
		}

		if(fabs(s_1) <= fabs(s_2)) s = s_1;
		else s = s_2;

	}
	return s;
}

/** TODO: If uncommented, the following function causes
 * warning: 'double tracktools::calculateMaxTrackLength(EVENT::Track*)'
 * defined but not used:
 */
//static double calculateMaxTrackLength(EVENT::Track* track)
//{
//	typedef std::vector< std::pair<EVENT::TrackerHit*, double> > HitDouble;
//	HitDouble HitPairVec;
//	//	    int hitNr=0;
//
//	// first calculate some track parameters from track:
//	double phi =  track->getPhi(); // just to improve readability
//
//	// unit vector from reference point to point of closest approach (pca) (xy plane only)
//	CLHEP::Hep3Vector n_pca( -sin( phi ), cos ( phi ) );
//
//	// reference point in xy projection
//	CLHEP::Hep3Vector reference_point( track->getReferencePoint ()[0], track->getReferencePoint ()[1],
//			track->getReferencePoint ()[2]);
//
//	// point of closest approach (3D point which is closest in the xy plane)
//	CLHEP::Hep3Vector pca =  reference_point
//			+  track->getD0() *  n_pca  // the xy part
//			+  track->getZ0() * CLHEP::Hep3Vector(0,0,1); // the z part
//
//	streamlog_out(DEBUG4) << "PCA is " << pca[0] << " , " << pca[1] << " , " << pca[2] << std::endl;
//
//	// psi is the angle of the pca wrt the helix center
//	// (Note: in the paper from Lasiuk et al. from 1998 this is Phi_0,
//	//  while Psi is the angle in track direktion.
//	//  As Phi is the angle in track direction in LCIO,
//	//  Phi and psi are exchanged wrt. the Lasiuk paper.)
//	double psi = track->getPhi() + ( track->getOmega()<0 ? -1 : 1 ) * M_PI_2;
//
//	// wrap psi to -2pi .. 2pi to prevent the while loop running for ages
//	// in case there are nonsense values for the track's phi
//	if ( std::fabs(psi) > 20 )
//	{
//		streamlog_out(WARNING3) << "Psi is " << psi << ", check phi of the input tracks!"
//				<< std::endl;
//	}
//
//	psi = std::fmod(psi, 2*M_PI);
//
//	TrackerHitVec hitVec = track->getTrackerHits();
//	float sumZ=0;
//	for ( TrackerHitVec::iterator hitIter = hitVec.begin(); hitIter < hitVec.end(); hitIter++ )
//			{
//		TrackerHit* trackerHit = *hitIter;
//					// A Hep3Vector for the test hit
//					CLHEP::Hep3Vector hit(trackerHit->getPosition()[0],trackerHit->getPosition()[1],
//							trackerHit->getPosition()[2] );
//
//					sumZ+=hit[2];
//					double s; // the track length in the xy-plane (projection)
//					if (  track->getOmega() != 0. )
//					{
//						double co = std::fabs(track->getOmega())*(hit[0] - pca[0]) + cos(psi);
//						double si = std::fabs(track->getOmega())*(hit[1] - pca[1]) + sin(psi);
//						streamlog_out(DEBUG2)<< "TrackFitterBase: co "<<co <<" : si " << si
//								<< " : psi " << psi << std::endl;
//
//						double phi =  ( co == 0 ? ( si < 0 ? - M_PI_2    : M_PI_2 ) : ( co > 0 ? - atan(si/co) :
//								M_PI - atan(si/co) ) );
//
//						// wrap phi to be in +- pi wrt. -psi (minus because the pca is at -psi wrt helix centre)
//						while ( -psi - phi >  M_PI ) phi+=2*M_PI ;
//						while ( -psi - phi < -M_PI ) phi-=2*M_PI ;
//
//						s = (-psi - phi) / track->getOmega();
//						//	  s = ( atan(si/co) + ( co < 0 ? M_PI : 0 ) - psi ) / track->getOmega();
//
//						//	  //This is the method from Lasiuk et al. It does not work because
//						//	  // the atan only works for +- pi/2, but we need a full circle
//						//	  s = atan( ( (hit[1] - pca[1] ) * sin(phi) + (hit[0] -pca[0]) * cos(phi) ) /
//						//		    ( 1/std::fabs( track->getOmega() ) +
//						//		      (hit[0] - pca[0]) * sin(phi) -
//						//		      (hit[1] - pca[1]) * cos(phi)
//						//		    )
//						//		  )
//						//	      /  track->getOmega() ;
//						streamlog_out(DEBUG3)<< "TrackFitterBase: Hit "<<hit[0] <<" : " <<hit[1] <<" : "<<hit[2]
//						                                                                              <<" ; s is " << s << std::endl;
//					}
//					else
//					{
//						s =  (hit[1] - pca[1] ) * sin(phi) + (hit[0] -pca[0]) * cos(phi);	// tracklength in xy projection
//
//						streamlog_out(DEBUG3)<< "TrackFitterBase: Hit "<<hit[0] <<" : " <<hit[1] <<" : "<<hit[2]
//						                                                                              <<" ; s is " << s << std::endl;
//					}
//
//					s = s * std::sqrt( 1 + track->getTanLambda()*track->getTanLambda()); // here comes the third dimension
//					std::pair<TrackerHit*, double> HitPair( trackerHit , s);
//					HitPairVec.push_back( HitPair );
//				}// for hitIter
//
//				sort( HitPairVec.begin(), HitPairVec.end(), compareHit );
//				sort( HitPairVec.begin(), HitPairVec.end(), compareHit );
//
//				//effective length of track is distance between outer hits times n/(n-1):
//				//distance between outer points on track
//				double l = fabs( (*HitPairVec.begin()).second - (*(HitPairVec.end()-1)).second );
//				//effective length
//				double l_eff = l*hitVec.size()/(hitVec.size()-1);
//			return l_eff;
//}
}

#endif
