#ifdef USE_PATHFINDER
#include "PathFinderInterfaceProcessor.h"
#include <iostream>
#include <string>

//GEAR
#include <gear/TPCParameters.h>

// include what you need from lcio
#include <EVENT/LCEvent.h>
#include <IMPL/LCCollectionVec.h>
#include <IMPL/TrackerHitImpl.h>
#include <IMPL/LCRelationImpl.h>
#include <EVENT/SimTrackerHit.h>
#include <IMPL/TrackImpl.h>
#include <IMPL/LCFlagImpl.h>
#include <Exceptions.h>

// this is the inserted part
#include "FinderParameter.h"
#include "basicHit.h"
#include "HoughTrafoTrackFinder.h"
#include <iostream>
#include <vector>

#include <cmath>
#include <ctime>

using namespace lcio;
using namespace marlin;
using namespace std;
using namespace gear;

using namespace pathfinder;

namespace marlintpc
{

PathFinderInterfaceProcessor aPathFinderInterfaceProcessor;


PathFinderInterfaceProcessor::PathFinderInterfaceProcessor() : Processor("PathFinderInterfaceProcessor")
{
    // the processor description
    _description = "PathFinderInterfaceProcessor is intended to ...";

    registerInputCollection(LCIO::TRACKERHIT,
                            "InputCollectionName",
                            "Name of the input Hit Collection",
                            _inputColName,
                            string("TPCHit"));
    registerOutputCollection(LCIO::TRACK,
			     "OutputCollectionName",
			     "Name of the output Track Collection",
			     _outputColName,
			     string("HoughTrafoTrack"));
    registerOutputCollection(LCIO::TRACKERHIT,
                             "OutputHitCollectionName",
                             "Name of the output Hit Collection containing all hits on Track",
                             _outputHitColName,
                             string("HitsOnTrack"));

    registerProcessorParameter("TrackModel", "straight line = 0, helix = 1, curler = 2", track_type, int(1));
    registerProcessorParameter("UseConformalMapping","true if conformal mapping should be used", useconformalmapping, bool(false));
    registerProcessorParameter("XYDistance", "maximum distance of hits from track in xy projection", xydistance, double(10.));//3.                          
    registerProcessorParameter("SZDistance", "maximum distance of hits from track in sz projection", szdistance, double(5.));//5.                          
    registerProcessorParameter("MinHitNumber", "minimum number of hits on track", minhitnumber, int(5));
    registerProcessorParameter("XYThetaBins", "HoughSpace binning theta xy", xythetabins, int(1000));
    registerProcessorParameter("XYDZeroBins", "HoughSpace binning d0 xy", xydzerobins, int(1000));//500 vor straight lines
    registerProcessorParameter("XYOmegaBins", "HoughSpace binning Omega xy", xyomegabins, int(300));//500
    registerProcessorParameter("SZThetaBins", "HoughSpace binning theta sz", szthetabins, int(300));
    registerProcessorParameter("SZDZeroBins", "HoughSpace binning d0 sz", szdzerobins, int(500));//500
    registerProcessorParameter("MaxDxy", "max possible distance between origin and center of circle/ max possible distance of closest approach for straight lines", maxdxy, double(3200.));
    registerProcessorParameter("MaxDsz", "max possible distance of closest approach for straight lines", maxdsz, double(3200.));
    registerProcessorParameter("SearchNeighborhood","is true if the neighborhood of maxima in Hough space are to be searched", searchneighborhood, bool(false));
    registerProcessorParameter("SaveRootFile","is true if root file should be saved", saverootfile, bool(false));
    registerProcessorParameter("ShiftHits","is true if hits should be shifted closer to the origin of track finding", shifthits, bool(false));
	registerProcessorParameter("MaxHitsPerEvent", "Limits the number of hits allowed in event for track reco to run.", maxhits, int(1000));

    registerOptionalParameter("XYDistanceFit", "maximum distance of hits from track in xy projection after fit", xydistancefit, double(3.));//3.                          
    registerOptionalParameter("SZDistanceFit", "maximum distance of hits from track in sz projection after fit", szdistancefit, double(3.));//5.                          
   
    registerOptionalParameter("VertexPositionX",
			      "Optional parameter to set vertex x position VertexPositionY must be set to have an effect",
			      vertexpositionx, double(0.));
    registerOptionalParameter("VertexPositionY",
			      "Optional parameter to set vertex y position VertexPositionX must be set to have an effect",
			      vertexpositiony, double(0.));

    registerOptionalParameter("ReferencePointXOverride",
			      "Optional Parameter to set a point to shift the hits. ReferencePointY and ReferencePointZ must be set and ShiftHits must be true to have any effect",
			      referencepointx, double(1503.615));	
    registerOptionalParameter("ReferencePointYOverride",
			      "Optional Parameter to set a point to shift the hits. ReferencePointX and ReferencePointZ must be set and ShiftHits must be true to have any effect",
			      referencepointy, double(0.));	
    registerOptionalParameter("ReferencePointZOverride",
			      "Optional Parameter to set a point to shift the hits. ReferencePointX and ReferencePointY must be set and ShiftHits must be true to have any effect",
			      referencepointz, double(0.));	

    //stefanos steering parameter
}

void PathFinderInterfaceProcessor::init()
{
    streamlog_out(DEBUG) << " init() called  " << endl;

    if(!parameterSet("XYDistanceFit"))
      {
	xydistancefit = xydistance;
      }
    if(!parameterSet("SZDistanceFit"))
      {
	szdistancefit = szdistance;
      }

    // you might want to list the used parameters by invoking
    printParameters();

    const gear::TPCParameters& tpcParameters = Global::GEAR->getTPCParameters();
    const std::vector<TPCModule*> moduleVector = tpcParameters.getModules();
	_nEvt=0;
    if(shifthits)
      {
	if(parameterSet("ReferencePointXOverride") && parameterSet("ReferencePointYOverride") && parameterSet("ReferencePointZOverride"))
	  {
	    _referencePoint[0] = referencepointx;
	    _referencePoint[1] = referencepointy;
	    _referencePoint[2] = referencepointz;
	  }
	else
	  {
	    _referencePoint = FindModuleCenter(moduleVector);
	  }
      }
    else
      {
	_referencePoint[0] = 0.;
	_referencePoint[1] = 0.;
	_referencePoint[2] = 0.;
      }

    streamlog_out(MESSAGE)<<"Reference Point: "<<_referencePoint[0]<<" "<<_referencePoint[1]<<" "<<_referencePoint[2]<<endl;

    //create root tree
    _rootFile = new TFile("trackparameters.root","RECREATE");

    _rootTree = new TTree("theTracks","theTracks");

    _rootTree->Branch("trackD0", &trackD0, "D0/D");
    _rootTree->Branch("trackPhi", &trackPhi, "Phi/D");
    _rootTree->Branch("trackOmega", &trackOmega, "Omega/D");
    _rootTree->Branch("trackZ0", &trackZ0, "Z0/D");
    _rootTree->Branch("trackTanL", &trackTanL, "TanL/D");
    _rootTree->Branch("trackD0Error", &trackD0Error, "D0Error/D");
    _rootTree->Branch("trackPhiError", &trackPhiError, "PhiError/D");
    _rootTree->Branch("trackOmegaError", &trackOmegaError, "OmegaError/D");
    _rootTree->Branch("trackZ0Error", &trackZ0Error, "Z0Error/D");
    _rootTree->Branch("trackTanLError", &trackTanLError, "TanLError/D");

    _rootTree->Branch("trackSlopeXY", &trackSlopeXY, "Mxy/D");
    _rootTree->Branch("trackOffsetXY", &trackOffsetXY, "Bxy/D");
    _rootTree->Branch("trackSlopeXYError", &trackSlopeXYError, "MxyError/D");
    _rootTree->Branch("trackOffsetXYError", &trackOffsetXYError, "BxyError/D");

    _rootTree->Branch("nHitsOnTrack", &nHitsOnTrack, "nHitsOnTrack/I");
    _rootTree->Branch("hitX", hitX, "X[nHitsOnTrack]/D");
    _rootTree->Branch("hitY", hitY, "Y[nHitsOnTrack]/D");
    _rootTree->Branch("hitZ", hitZ, "Z[nHitsOnTrack]/D");
    _rootTree->Branch("xError", xError, "ErrorX[nHitsOnTrack]/D");
    _rootTree->Branch("yError", yError, "ErrorY[nHitsOnTrack]/D");
    _rootTree->Branch("zError", zError, "ErrorZ[nHitsOnTrack]/D");

}


void PathFinderInterfaceProcessor::processRunHeader(lcio::LCRunHeader*  /*run*/)
{
    // do something for each run
}
   

void PathFinderInterfaceProcessor::processEvent(lcio::LCEvent* evt)
{
  std::clock_t start1 = std::clock();
  
  streamlog_out(DEBUG)<<"Event Number: "<<evt->getEventNumber()<<endl;
  
  //get input collection
  LCCollection* inputCol = 0;
  
  LCCollectionVec* outputCol = new LCCollectionVec( LCIO::TRACK );
  LCFlagImpl trkFlag(0) ;
  trkFlag.setBit( LCIO::TRBIT_HITS ) ;
  outputCol->setFlag( trkFlag.getFlag() );
  LCCollectionVec* hitsOnTrackCollection = new LCCollectionVec(LCIO::TRACKERHIT);
  hitsOnTrackCollection->setSubset();
  
  //these collections will not be saved
  LCCollectionVec* shiftedHitCollection = new LCCollectionVec(LCIO::TRACKERHIT);
  LCCollectionVec* hitRelations = new LCCollectionVec(LCIO::LCRELATION);
  
  // do a try / catch in case there is an empty event
  try
    {
      inputCol = evt->getCollection( _inputColName ) ;
    }
  catch (DataNotAvailableException &e)
    {
      // no input data, just return and process the next event
      return;
    }
  
  
  vector<basicHit> basicHitVector;
  vector<TrackerHit*> inputHitVector;
  
  int nInput = inputCol->getNumberOfElements();
  streamlog_out(DEBUG)<<"Number of input hits: "<<nInput<<endl;
  for ( int iInput = 0; iInput < nInput; iInput++)
    {
      TrackerHit* Hit =dynamic_cast<TrackerHit*>( inputCol->getElementAt( iInput ) ) ;
      TrackerHitImpl* trackerHit = new TrackerHitImpl();
      trackerHit->setPosition(Hit->getPosition());
      
      const double *pos = Hit->getPosition(); 
      double newPos[3];
      newPos[0] = pos[0] - _referencePoint[0];
      newPos[1] = pos[1] - _referencePoint[1];
      newPos[2] = pos[2];
      
      //calculate shifted hits
      TrackerHitImpl* shiftedHit = new TrackerHitImpl();
      shiftedHit->setPosition(newPos);
      shiftedHitCollection->addElement(shiftedHit);
      //add lc relations
      LCRelationImpl* HitShiftedHitRelation = new LCRelationImpl();
      HitShiftedHitRelation->setFrom(Hit);
      HitShiftedHitRelation->setTo(shiftedHit);
      hitRelations->addElement(HitShiftedHitRelation);

      if (!shifthits)
	{
	  basicHit tempHit(pos[0], pos[1], pos[2]);
	  basicHitVector.push_back(tempHit);
	  inputHitVector.push_back(Hit);
	}
      else
	{
	  basicHit tempHit(newPos[0], newPos[1],newPos[2]);
	  basicHitVector.push_back(tempHit);
	  inputHitVector.push_back(shiftedHit);
	}
    }//for ( int iInput = 0; iInput < nInput; iInput++)
  
  //prepare parameters for track finding
  if(track_type == 0)
    {
      TrackFinderParameter = new FinderParameter(true, false);
      TrackFinderParameter->setFindCurler(false);
    }
  if (track_type == 1)
    {
      TrackFinderParameter = new FinderParameter(false, true);
      TrackFinderParameter->setFindCurler(false);
    }
  if(track_type == 2)
    {
      TrackFinderParameter = new FinderParameter(false, true);
      TrackFinderParameter->setFindCurler(true);
    }

  
  if(parameterSet("VertexPositionX") && parameterSet("VertexPositionY"))
    {
      //if parameter VertexPosition is set set the vertex constraint, shift the vertex shifthits = true
      std::pair<double, double> vertex(vertexpositionx - _referencePoint[0], vertexpositiony - _referencePoint[1]);
      TrackFinderParameter->setVertex(vertex);
    }
  
  TrackFinderParameter->setUseConformalMapping(useconformalmapping);
  TrackFinderParameter->setMaxXYDistance(xydistance);
  TrackFinderParameter->setMaxSZDistance(szdistance);
  TrackFinderParameter->setMaxXYDistanceFit(xydistancefit);
  TrackFinderParameter->setMaxSZDistanceFit(szdistancefit);
  TrackFinderParameter->setMinimumHitNumber(minhitnumber);
  TrackFinderParameter->setNumberXYThetaBins(xythetabins);
  TrackFinderParameter->setNumberXYDzeroBins(xydzerobins);
  TrackFinderParameter->setNumberXYOmegaBins(xyomegabins);
  TrackFinderParameter->setNumberSZThetaBins(szthetabins);
  TrackFinderParameter->setNumberSZDzeroBins(szdzerobins);
  TrackFinderParameter->setMaxDxy(maxdxy);
  TrackFinderParameter->setMaxDsz(maxdsz);
  TrackFinderParameter->setSearchNeighborhood(searchneighborhood);
  TrackFinderParameter->setSaveRootFile(saverootfile);
  
  //TrackFinderParameter->print();
  
  
  streamlog_out(DEBUG) << " the hit vector has " << basicHitVector.size() << " elements " << endl;
  
  




        //streamlog_out(DEBUG) << "Now do same thing but use function HoughTrafoTrackFinder::find ..." << endl;
        HoughTrafoTrackFinder mySecondHoughTrafoTrackFinder;
        bool trackFound = false;
        mySecondHoughTrafoTrackFinder.setFinderParameter(*TrackFinderParameter);
        mySecondHoughTrafoTrackFinder.setInitialHits(basicHitVector);

	if(nInput < maxhits)
            {
                trackFound = mySecondHoughTrafoTrackFinder.find();
		streamlog_out(DEBUG1) <<_nEvt<<" "<<nInput<<" "<<time(NULL)<<endl;
            }
	else
	  streamlog_out(DEBUG1) <<"Rejecting :"<<_nEvt<<" "<<nInput<<" "<<time(NULL)<<endl;
        //delete TrackFinderParameter;
        streamlog_out(DEBUG) << "#################################" << endl;
        streamlog_out(DEBUG) << "Find is done. Print Parameters:" << endl;
        streamlog_out(DEBUG) << "#################################" << endl;
        if(trackFound == true)
            {
                vector<TrackFinderTrack> foundTracks = mySecondHoughTrafoTrackFinder.getTracks();


            streamlog_out(DEBUG) << "Number of Tracks found: " << foundTracks.size() << endl;
            streamlog_out(DEBUG) << "Number of Noise Hits: " << ((foundTracks.back()).getRejectedHits()).size() << endl;

            streamlog_out(DEBUG) << "******************" << endl;
            for (unsigned int i = 0; i < foundTracks.size(); i++)
                {
		  //TrackFinderParameter->print();
                    streamlog_out(DEBUG) << "Track " << i << endl;

		    //conversion of TrackFinderTrack to lcio::Track
		    TrackParameterFull currentTrackParameters = foundTracks[i].getTrackParameter();
		    TrackImpl* tempTrack = new TrackImpl();
		    if(!shifthits)
		      {
			//TrackFinderParameter->print();
			tempTrack->setPhi(currentTrackParameters.getPhi());
			tempTrack->setD0(currentTrackParameters.getDZero());
			tempTrack->setOmega(currentTrackParameters.getOmega());
			tempTrack->setTanLambda(currentTrackParameters.getTanLambda());
			tempTrack->setZ0(currentTrackParameters.getZZero());


			//set errors of track parameters
			FloatVec CovMatrix;
			streamlog_out(DEBUG)<<"Track Uncertainties: "<<currentTrackParameters.getDZeroError()<<" "<<currentTrackParameters.getPhiError()
			    <<currentTrackParameters.getOmegaError()<<" "<<currentTrackParameters.getZZeroError()<<" "<<currentTrackParameters.getTanLambdaError()<<std::endl;
			CovMatrix.push_back(currentTrackParameters.getDZeroError()*currentTrackParameters.getDZeroError());//d0*d0
			CovMatrix.push_back(currentTrackParameters.getPhiError()*currentTrackParameters.getDZeroError());//d0*phi
			CovMatrix.push_back(currentTrackParameters.getPhiError()*currentTrackParameters.getPhiError());//phi*phi
			CovMatrix.push_back(currentTrackParameters.getOmegaError()*currentTrackParameters.getDZeroError());//omega*d0
			CovMatrix.push_back(currentTrackParameters.getOmegaError()*currentTrackParameters.getPhiError());//omega*phi
			CovMatrix.push_back(currentTrackParameters.getOmegaError()*currentTrackParameters.getOmegaError());//omega*omega
			CovMatrix.push_back(currentTrackParameters.getZZeroError()*currentTrackParameters.getDZeroError());//z0*d0
			CovMatrix.push_back(currentTrackParameters.getZZeroError()*currentTrackParameters.getPhiError());//z0*phi
			CovMatrix.push_back(currentTrackParameters.getZZeroError()*currentTrackParameters.getOmegaError());//z0*omega
			CovMatrix.push_back(currentTrackParameters.getZZeroError()*currentTrackParameters.getZZeroError());//z0*z0
			CovMatrix.push_back(currentTrackParameters.getTanLambdaError()*currentTrackParameters.getDZeroError());//tanl*d0
			CovMatrix.push_back(currentTrackParameters.getTanLambdaError()*currentTrackParameters.getPhiError());//tanl*phi
			CovMatrix.push_back(currentTrackParameters.getTanLambdaError()*currentTrackParameters.getOmegaError());//tanl*omega
			CovMatrix.push_back(currentTrackParameters.getTanLambdaError()*currentTrackParameters.getZZeroError());//tanl*z0
			CovMatrix.push_back(currentTrackParameters.getTanLambdaError()*currentTrackParameters.getTanLambdaError());//tanl*tanl

			tempTrack->setCovMatrix(CovMatrix);

			trackD0 = currentTrackParameters.getDZero();
			trackPhi = currentTrackParameters.getPhi();
			trackOmega = currentTrackParameters.getOmega();
			trackZ0 = currentTrackParameters.getZZero();
			trackTanL = currentTrackParameters.getTanLambda();
                        trackD0Error = currentTrackParameters.getDZeroError();
                        trackPhiError = currentTrackParameters.getPhiError();
                        trackOmegaError = currentTrackParameters.getOmegaError();
                        trackZ0Error = currentTrackParameters.getZZeroError();
                        trackTanLError = currentTrackParameters.getTanLambdaError();

			trackSlopeXY = currentTrackParameters.getSlopeXY();
			trackOffsetXY = currentTrackParameters.getOffsetXY();
			trackSlopeXYError = currentTrackParameters.getSlopeXYError();
			trackOffsetXYError = currentTrackParameters.getOffsetXYError();
			streamlog_out(DEBUG)<<"Root Tree Test: slope "<<trackSlopeXY<<" +- "<<trackSlopeXYError<<endl;
			streamlog_out(DEBUG)<<"Root Tree Test: offset "<<trackOffsetXY<<" +- "<<trackOffsetXYError<<endl;

		      }
		    else
		      {
			//TrackFinderParameter->print();
			if(parameterSet("ReferencePointXOverride") && parameterSet("ReferencePointYOverride") && parameterSet("ReferencePointZOverride"))
			  {			
			    tempTrack->setPhi(currentTrackParameters.getPhi());
			    tempTrack->setD0(currentTrackParameters.getDZero());
			    tempTrack->setOmega(currentTrackParameters.getOmega());
			    tempTrack->setTanLambda(currentTrackParameters.getTanLambda());
			    tempTrack->setZ0(currentTrackParameters.getZZero());
			    
			    //set errors of track parameters
			    FloatVec CovMatrix;
			    streamlog_out(DEBUG)<<"Track Uncertainties: "<<currentTrackParameters.getDZeroError()<<" "<<currentTrackParameters.getPhiError()
				     <<currentTrackParameters.getOmegaError()<<" "<<currentTrackParameters.getZZeroError()<<" "<<currentTrackParameters.getTanLambdaError()<<std::endl;
			    CovMatrix.push_back(currentTrackParameters.getDZeroError()*currentTrackParameters.getDZeroError());//d0*d0
			    CovMatrix.push_back(currentTrackParameters.getPhiError()*currentTrackParameters.getDZeroError());//d0*phi
			    CovMatrix.push_back(currentTrackParameters.getPhiError()*currentTrackParameters.getPhiError());//phi*phi
			    CovMatrix.push_back(currentTrackParameters.getOmegaError()*currentTrackParameters.getDZeroError());//omega*d0
			    CovMatrix.push_back(currentTrackParameters.getOmegaError()*currentTrackParameters.getPhiError());//omega*phi
			    CovMatrix.push_back(currentTrackParameters.getOmegaError()*currentTrackParameters.getOmegaError());//omega*omega
			    CovMatrix.push_back(currentTrackParameters.getZZeroError()*currentTrackParameters.getDZeroError());//z0*d0
			    CovMatrix.push_back(currentTrackParameters.getZZeroError()*currentTrackParameters.getPhiError());//z0*phi
			    CovMatrix.push_back(currentTrackParameters.getZZeroError()*currentTrackParameters.getOmegaError());//z0*omega
			    CovMatrix.push_back(currentTrackParameters.getZZeroError()*currentTrackParameters.getZZeroError());//z0*z0
			    CovMatrix.push_back(currentTrackParameters.getTanLambdaError()*currentTrackParameters.getDZeroError());//tanl*d0
			    CovMatrix.push_back(currentTrackParameters.getTanLambdaError()*currentTrackParameters.getPhiError());//tanl*phi
			    CovMatrix.push_back(currentTrackParameters.getTanLambdaError()*currentTrackParameters.getOmegaError());//tanl*omega
			    CovMatrix.push_back(currentTrackParameters.getTanLambdaError()*currentTrackParameters.getZZeroError());//tanl*z0
			    CovMatrix.push_back(currentTrackParameters.getTanLambdaError()*currentTrackParameters.getTanLambdaError());//tanl*tanl
			    
			    tempTrack->setCovMatrix(CovMatrix);
			    
			    float refPoint[3];
			    refPoint[0]=_referencePoint[0];
			    refPoint[1]=_referencePoint[1];
			    refPoint[2]=_referencePoint[2];
			    
			    tempTrack->setReferencePoint(refPoint);
			    
			    trackD0 = currentTrackParameters.getDZero();
			    trackPhi = currentTrackParameters.getPhi();
			    trackOmega = currentTrackParameters.getOmega();
			    trackZ0 = currentTrackParameters.getZZero();
			    trackTanL = currentTrackParameters.getTanLambda();
			    trackD0Error = currentTrackParameters.getDZeroError();
			    trackPhiError = currentTrackParameters.getPhiError();
			    trackOmegaError = currentTrackParameters.getOmegaError();
			    trackZ0Error = currentTrackParameters.getZZeroError();
			    trackTanLError = currentTrackParameters.getTanLambdaError();
			    
			    trackSlopeXY = currentTrackParameters.getSlopeXY();
			    trackOffsetXY = currentTrackParameters.getOffsetXY();
			    trackSlopeXYError = currentTrackParameters.getSlopeXYError();
			    trackOffsetXYError = currentTrackParameters.getOffsetXYError();
			    streamlog_out(DEBUG)<<"Root Tree Test: slope "<<trackSlopeXY<<" +- "<<trackSlopeXYError<<endl;
			    streamlog_out(DEBUG)<<"Root Tree Test: offset "<<trackOffsetXY<<" +- "<<trackOffsetXYError<<endl;
			  }
			else
			  {
			
			    double phi = currentTrackParameters.getPhi();
			    double d0 = currentTrackParameters.getDZero();
			    double omega = currentTrackParameters.getOmega();
			    double tanl = currentTrackParameters.getTanLambda();
			    double z0 = currentTrackParameters.getZZero();
                            double phiError = currentTrackParameters.getPhiError();
                            double d0Error = currentTrackParameters.getDZeroError();
                            double omegaError = currentTrackParameters.getOmegaError();
                            double tanlError = currentTrackParameters.getTanLambdaError();
                            double z0Error = currentTrackParameters.getZZeroError();

			    double deltaX = -_referencePoint[0] - 0.;
			    double deltaY = -_referencePoint[1] - 0.;

			    double phiprime = 0.;
			    double d0prime = 0.;
			    double omegaprime = 0.;
			    double z0prime = 0.;
			    double tanlprime = 0.;
                            double phiErrorprime = 0.;
                            double d0Errorprime = 0.;
                            double omegaErrorprime = 0.;
                            double z0Errorprime = 0.;
                            double tanlErrorprime = 0.;

			    //shift phi
			    if(track_type == 0)//is straight line
			      {
				phiprime = phi;
				phiErrorprime = phiError;
			      }
			    else
			      {
				if(std::abs(omega) > 0.0000001)
				  {
				    double alpha = sin(phi) - deltaX/(1./omega - d0);
				    double beta = cos(phi)  + deltaY/(1/omega - d0);

				    double dalphadphi = cos(phi);
				    double dalphadomega = deltaX/(1-d0*omega)/(1-d0*omega);
				    double dalphadd0 = deltaX/(1./omega - d0)/(1./omega - d0); 
				    double deltaalpha = sqrt((dalphadphi*phiError)*(dalphadphi*phiError) +
							     (dalphadomega*omegaError)*(dalphadomega*omegaError)+
							     (dalphadd0*d0Error)*(dalphadd0*d0Error));

				    double dbetadphi = -sin(phi);
				    double dbetadomega = deltaY/(1-d0*omega)/(1-d0*omega);
				    double dbetadd0 = deltaY/(1./omega - d0)/(1./omega - d0); 
				    double deltabeta = sqrt((dbetadphi*phiError)*(dbetadphi*phiError) +
							    (dbetadomega*omegaError)*(dbetadomega*omegaError)+
							    (dbetadd0*d0Error)*(dbetadd0*d0Error));

				    phiprime = atan2(alpha, beta);
				    phiErrorprime = sqrt((deltaalpha/beta/((alpha/beta)*(alpha/beta)+1))*(deltaalpha/beta/((alpha/beta)*(alpha/beta)+1))+
							 (-alpha*deltabeta/beta/beta/((alpha/beta)*(alpha/beta)+1))*(-alpha*deltabeta/beta/beta/((alpha/beta)*(alpha/beta)+1)));
							 

				  }
				else
				  {
				    phiprime = phi;
				    phiErrorprime = phiError;
				  }
			      }
			    //shift d0
			    d0prime = d0 + deltaX*sin(phi) - deltaY*cos(phi)
			      +(deltaX*cos(phi) + deltaY*sin(phi)) * tan( (phiprime - phi)/ 2.);
			    
			    double dd0primedd0 = 1.;
			    double dd0primedphi = deltaX*cos(phi)+deltaY*sin(phi) +
			      deltaX*(-sin(phi)*tan((phiprime - phi)/ 2.) - cos(phi)/2/cos((phiprime - phi)/ 2.)/cos((phiprime - phi)/ 2.)) +
			      deltaY*(cos(phi)*tan((phiprime - phi)/ 2.) - sin(phi)/2/cos((phiprime - phi)/ 2.)/cos((phiprime - phi)/ 2.));
			    double dd0primedphiprime = (deltaX*cos(phi)+deltaY*sin(phi))/2/cos((phiprime - phi)/ 2.)/cos((phiprime - phi)/ 2.);

			    d0Errorprime = sqrt((dd0primedd0*d0Error)*(dd0primedd0*d0Error)+
						(dd0primedphi*phiError)*(dd0primedphi*phiError)+
						(dd0primedphiprime*phiErrorprime)*(dd0primedphiprime*phiErrorprime));

			    //shift omega
			    omegaprime = omega;
			    omegaErrorprime = omegaError;
			    
			    //shift z0
			    double pcax = 0. - d0*sin(phi);
			    double pcay = 0. + d0*cos(phi);
			    double pcaxError = sqrt((sin(phi)*d0Error)*(sin(phi)*d0Error)+
						    (d0*cos(phi)*phiError)*(d0*cos(phi)*phiError));
			    double pcayError = sqrt((cos(phi)*d0Error)*(cos(phi)*d0Error)+
						    (d0*sin(phi)*phiError)*(d0*sin(phi)*phiError));
			    double pcaxprime = -_referencePoint[0] - d0prime*sin(phiprime);
			    double pcayprime = -_referencePoint[1] + d0prime*cos(phiprime);
                            double pcaxprimeError = sqrt((sin(phiprime)*d0Errorprime)*(sin(phiprime)*d0Errorprime)+
                                                    (d0prime*cos(phiprime)*phiErrorprime)*(d0prime*cos(phiprime)*phiErrorprime));
                            double pcayprimeError = sqrt((cos(phiprime)*d0Errorprime)*(cos(phiprime)*d0Errorprime)+
                                                    (d0prime*sin(phiprime)*phiErrorprime)*(d0prime*sin(phiprime)*phiErrorprime));
			    double s = 0.;
			    double sError = 0.;;
			    if(track_type == 0)
			      {
				s = (pcaxprime - pcax)*cos(phi) + (pcayprime - pcay)*sin(phi);
				double dsdphi = (pcax - pcaxprime)*sin(phi) - (pcay - pcayprime)*cos(phi);

				sError = sqrt((dsdphi*phiError)*(dsdphi*phiError)+
					      (cos(phi)*pcaxprimeError)*(cos(phi)*pcaxprimeError)+
					      (-cos(phi)*pcaxError)*(-cos(phi)*pcaxError)+
					      (sin(phi)*pcayprimeError)*(sin(phi)*pcayprimeError)+
					      (-sin(phi)*pcayError)*(-sin(phi)*pcayError));
			      }
			    else
			      {
				if(std::abs(omega) > 0.0000001)
				  {
				    double alpha = -omega*(pcaxprime - pcax)*cos(phi) -omega*(pcayprime - pcay)*sin(phi);
				    double beta = 1- omega*(pcaxprime - pcax)*sin(phi) + omega*(pcayprime - pcay)*cos(phi);
				    s = atan2(-alpha, beta)/omega;

				    double dalphadomega = -(pcaxprime - pcax)*cos(phi) - (pcayprime - pcay)*sin(phi);
				    double dalphadphi = omega*(pcaxprime - pcax)*sin(phi) -omega*(pcayprime - pcay)*cos(phi);

				    double dbetadomega = -(pcaxprime - pcax)*sin(phi) + (pcayprime - pcay)*cos(phi);
				    double dbetadphi = -omega*(pcaxprime - pcax)*cos(phi) - omega*(pcayprime - pcay)*sin(phi);

				    double deltaalpha = sqrt((dalphadomega*omegaError)*(dalphadomega*omegaError)+
							     (dalphadphi*phiError)*(dalphadphi*phiError)+
							     (-omega*cos(phi)*pcaxprimeError)*(-omega*cos(phi)*pcaxprimeError)+
							     (omega*cos(phi)*pcaxError)*(omega*cos(phi)*pcaxError)+
							     (-omega*sin(phi)*pcayprimeError)*(-omega*sin(phi)*pcayprimeError)+
							     (omega*sin(phi)*pcayError)*(omega*sin(phi)*pcayError));
				    
                                    double deltabeta = sqrt((dbetadomega*omegaError)*(dbetadomega*omegaError)+
                                                             (dbetadphi*phiError)*(dbetadphi*phiError)+
                                                             (-omega*sin(phi)*pcaxprimeError)*(-omega*sin(phi)*pcaxprimeError)+
                                                             (omega*sin(phi)*pcaxError)*(omega*sin(phi)*pcaxError)+
                                                             (omega*cos(phi)*pcayprimeError)*(omega*cos(phi)*pcayprimeError)+
                                                             (-omega*cos(phi)*pcayError)*(-omega*cos(phi)*pcayError));

				    sError = sqrt((atan2(-alpha, beta)*omegaError/omega/omega)*(atan2(-alpha, beta)*omegaError/omega/omega)+
						  (-deltaalpha/beta/((-alpha/beta)*(-alpha/beta)+1))*(-deltaalpha/beta/((-alpha/beta)*(-alpha/beta)+1))+
						  (alpha*deltabeta/beta/beta/((-alpha/beta)*(-alpha/beta)+1))*(alpha*deltabeta/beta/beta/((-alpha/beta)*(-alpha/beta)+1)));

				  }
				else
				  {
				    s = (pcaxprime - pcax)*cos(phi) + (pcayprime - pcay)*sin(phi); 
				    double dsdphi = (pcax - pcaxprime)*sin(phi) - (pcay - pcayprime)*cos(phi);

				    sError = sqrt((dsdphi*phiError)*(dsdphi*phiError)+
						  (cos(phi)*pcaxprimeError)*(cos(phi)*pcaxprimeError)+
						  (-cos(phi)*pcaxError)*(-cos(phi)*pcaxError)+
						  (sin(phi)*pcayprimeError)*(sin(phi)*pcayprimeError)+
						  (-sin(phi)*pcayError)*(-sin(phi)*pcayError));
				  }
			      }
			    z0prime = z0 + s* tanl;

			    z0Errorprime = sqrt(z0Error*z0Error + (tanl*sError)*(tanl*sError) + (s*tanlError)*(s*tanlError));

			    //shift tanl
			    tanlprime = tanl;
			    tanlErrorprime = tanlError;
			    
			    tempTrack->setPhi(phiprime);
			    tempTrack->setD0(d0prime);
			    tempTrack->setOmega(omegaprime);
			    tempTrack->setTanLambda(tanlprime);
			    tempTrack->setZ0(z0prime);

			    //TrackFinderParameter->print();
			    streamlog_out(DEBUG)<<"Track Parameters: "<<d0prime<<" "<<phiprime<<" "<<omegaprime<<" "<<z0prime<<" "<<tanlprime<<std::endl;
			    FloatVec CovMatrix;
			    streamlog_out(DEBUG)<<"Track Uncertainties: "<<d0Errorprime<<" "<<phiErrorprime<<" "<<omegaErrorprime<<" "<<z0Errorprime<<" "<<tanlErrorprime<<std::endl;
			    CovMatrix.push_back(d0Errorprime*d0Errorprime);//d0*d0
			    CovMatrix.push_back(phiErrorprime*d0Errorprime);//d0*phi
			    CovMatrix.push_back(phiErrorprime*phiErrorprime);//phi*phi
			    CovMatrix.push_back(omegaErrorprime*d0Errorprime);//omega*d0
			    CovMatrix.push_back(omegaErrorprime*phiErrorprime);//omega*phi
			    CovMatrix.push_back(omegaErrorprime*omegaErrorprime);//omega*omega
			    CovMatrix.push_back(z0Errorprime*d0Errorprime);//z0*d0
			    CovMatrix.push_back(z0Errorprime*phiErrorprime);//z0*phi
			    CovMatrix.push_back(z0Errorprime*omegaErrorprime);//z0*omega
			    CovMatrix.push_back(z0Errorprime*z0Errorprime);//z0*z0
			    CovMatrix.push_back(tanlErrorprime*d0Errorprime);//tanl*d0
			    CovMatrix.push_back(tanlErrorprime*phiErrorprime);//tanl*phi
			    CovMatrix.push_back(tanlErrorprime*omegaErrorprime);//tanl*omega
			    CovMatrix.push_back(tanlErrorprime*z0Errorprime);//tanl*z0
			    CovMatrix.push_back(tanlErrorprime*tanlErrorprime);//tanl*tanl
			    
			    tempTrack->setCovMatrix(CovMatrix);			    

			    trackD0 = d0prime;
			    trackPhi = phiprime;
			    trackOmega = omegaprime;
			    trackZ0 = z0prime;
			    trackTanL = tanlprime;
			    trackD0Error = d0Errorprime;
			    trackPhiError = phiErrorprime;
			    trackOmegaError = omegaErrorprime;
			    trackZ0Error = z0Errorprime;
			    trackTanLError = tanlErrorprime;
						
			  }
		      }//if hits shifted
                    vector<basicHit> hitsOnTrack = foundTracks[i].getHitsOnTrack();
                    streamlog_out(DEBUG) << "Number of Hits on Track: " << hitsOnTrack.size() << endl;
		    nHitsOnTrack = hitsOnTrack.size();
                    streamlog_out(DEBUG) << "Number of rejected hits: " << (foundTracks[i].getRejectedHits()).size() << endl;
		    //TrackFinderParameter->print();
		    //loop over basic hit vector
		    int counter = 0;
                    for (unsigned int j = 0; j < hitsOnTrack.size(); j++)
                        {
			  //			  streamlog_out(DEBUG)<<"Loop over hits on track: "<<j<<endl;
			  //TrackFinderParameter->print();
			  //loop over TrackerHit vector
			  for(unsigned int k = 0; k<inputHitVector.size(); k++)
			    {
			      // streamlog_out(DEBUG) << hitsOnTrack[j].getX() << " " << hitsOnTrack[j].getY() << " " << hitsOnTrack[j].getZ() << endl;
			      //double pos[3];
			      const double *pos = inputHitVector[k]->getPosition();
			      //~ double pos1[3];
			      //~ for(int l = 0; l<3;l++)
				//~ {
				  //~ pos1[l] = pos[l];
				//~ }
			      //streamlog_out(DEBUG)<<pos[0]<<" "<<pos[1]<<" "<<pos[2]<<endl;
			      //streamlog_out(DEBUG)<<hitsOnTrack[j].getX()<<" "<<hitsOnTrack[j].getY()<<" "<<hitsOnTrack[j].getZ()<<endl;
			      if(pos[0] == hitsOnTrack[j].getX() &&
				 pos[1] == hitsOnTrack[j].getY() &&
				 pos[2] == hitsOnTrack[j].getZ())
				{
				  //streamlog_out(DEBUG)<<"Hit is on track!"<<endl;
				  //TrackerHitImpl* inputHit = new TrackerHitImpl();
				  //inputHit = dynamic_cast<TrackerHitImpl*>(inputHitVector[k]);
				  //     TrackerHitImpl* currentHit1 = new TrackerHitImpl();
				  //TrackerHitImpl* currentHit2 = new TrackerHitImpl();
				  //currentHit1->setType(inputHit->getType());
				  //currentHit1->setPosition(pos1);
				  //currentHit1->setCovMatrix(inputHit->getCovMatrix());
				  //currentHit1->setdEdx(inputHit->getdEdx());
				  //currentHit1->setEDep(inputHit->getEDep());
				  //currentHit1->setEDepError(inputHit->getEDepError());
				  //currentHit1->setTime(inputHit->getTime());
				  //currentHit1->setQuality(inputHit->getQuality());

				  if(!shifthits)
				    {

				      hitsOnTrackCollection->addElement(dynamic_cast<TrackerHitImpl*>(inputHitVector.at(k)));
				      //				      tempTrack->addHit(dynamic_cast<TrackerHitImpl*>(inputHitVector.at(k)));
				      tempTrack->addHit(dynamic_cast<TrackerHitImpl*>( (*hitsOnTrackCollection).back() ));

				      TrackerHit* theHit = dynamic_cast<TrackerHit*>( (*hitsOnTrackCollection).back() );
				      hitX[counter] = theHit->getPosition()[0] ;
				      hitY[counter] = theHit->getPosition()[1] ;
				      hitZ[counter] = theHit->getPosition()[2];
				      xError[counter] = (theHit)->getCovMatrix()[0] ;
				      yError[counter] = (theHit)->getCovMatrix()[2] ;
				      zError[counter] = (theHit)->getCovMatrix()[5] ;

				      counter++;
				    }
				  else
				    {
				      //loop over LCRelations
				      for(LCCollectionVec::iterator LCRelIter =  hitRelations->begin();
					  LCRelIter<hitRelations->end();
					  LCRelIter++)
					{
					  LCRelation* currentRelation = dynamic_cast<LCRelation*>(*LCRelIter);
					  if((currentRelation->getTo())->id() == (inputHitVector[k])->id())
					    {
					      streamlog_out(DEBUG)<<"relations: current Hit: "<<(inputHitVector[k])->id()<<" From: "<<(currentRelation->getFrom())->id()<<" To: "<<(currentRelation->getTo())->id()<<endl;
					      hitsOnTrackCollection->addElement(dynamic_cast<TrackerHitImpl*>(currentRelation->getFrom()));
					      //hitsOnTrackCollection->addElement(dynamic_cast<TrackerHitImpl*>(inputHitVector.at(k)));
					      streamlog_out(DEBUG)<<"current Hit id: "<<(*hitsOnTrackCollection).back()->id()<<" "<<(currentRelation->getFrom())->id()<<endl;
					      //      tempTrack->addHit(dynamic_cast<TrackerHitImpl*>(currentRelation->getFrom()));
					      tempTrack->addHit(dynamic_cast<TrackerHitImpl*>( (*hitsOnTrackCollection).back() ));

					      TrackerHit* theHit = dynamic_cast<TrackerHit*>( (*hitsOnTrackCollection).back() );
					      hitX[counter] = theHit->getPosition()[0] ;
					      hitY[counter] = theHit->getPosition()[1] ;
					      hitZ[counter] = theHit->getPosition()[2];
					      xError[counter] = (theHit)->getCovMatrix()[0] ;
					      yError[counter] = (theHit)->getCovMatrix()[2] ;
					      zError[counter] = (theHit)->getCovMatrix()[5] ;

					      counter++;
				      
					      break;

					    }
					  if(LCRelIter == hitRelations->end()-1 )streamlog_out(WARNING)<<"Relation was not found"<<endl;
					}
				    }
				}
			    }
                        }//for (unsigned int j = 0; j < hitsOnTrack.size(); j++) 

		    //TrackFinderParameter->print();

		    outputCol->addElement(tempTrack);
		    _rootTree->Fill();
                }//loop over found tracks
            streamlog_out(DEBUG) << "******************" << endl;

            streamlog_out(DEBUG) << "Number of Tracks found: " << foundTracks.size() << endl;
            streamlog_out(DEBUG) << "Number of Noise Hits: " << ((foundTracks.back()).getRejectedHits()).size() << endl;

            streamlog_out(DEBUG) << "******************" << endl;

	    //TrackFinderParameter->print();

        }//if(trackFound == true)
    else
        {
	  streamlog_out(DEBUG) << "Not enough hits on track... " << endl;
	  streamlog_out(DEBUG) << "Or more than "<<maxhits<<" in the tpc"<<endl;
        }

    streamlog_out(DEBUG) << "HoughTrafoTrackFinder::find ... done!" << endl;


    if( !outputCol->empty() )
      {
	evt->addCollection( outputCol, _outputColName );
      }
    if(!hitsOnTrackCollection->empty())
      {
        evt->addCollection(hitsOnTrackCollection, _outputHitColName);
	// evt->addCollection(allHitsCollection,"allHitsCollection");
	//evt->addCollection(hitRelations, "HitShiftedHitRelation");
	//evt->addCollection(shiftedHitCollection, "ShiftedHitCollection");
      }

   
    delete TrackFinderParameter;

    basicHitVector.clear();
    inputHitVector.clear();

    streamlog_out(DEBUG) << ((std::clock() - start1) / (double)CLOCKS_PER_SEC) << " sec" << endl;

    //    }
    // catch (...)
    // {
    //cerr << " something bad must have happened " << endl;
    //}

    //    if(inputCol !=0) delete inputCol;
    //if(outputCol !=0) delete outputCol;
    //if(hitsOnTrackCollection!=0) delete hitsOnTrackCollection;

    _nEvt++;
}


void PathFinderInterfaceProcessor::check(lcio::LCEvent*  /*evt*/)
{
    // check what is needed here
}


void PathFinderInterfaceProcessor::end()
{
    // do something at the end of the processing, e.g. cleaning up
  _rootFile->Write();
  _rootFile->Close();
}


  Vector3D PathFinderInterfaceProcessor::FindModuleCenter(const std::vector<TPCModule*> theModules)
  {
    Vector3D moduleCenter;
    moduleCenter[0] = 0.;
    moduleCenter[1] = 0.;
    moduleCenter[2] = 0.;

    for(unsigned int m = 0; m< theModules.size(); m++)
      {
	TPCModule* theModule = theModules.at(m);

	int nRows = theModule->getNRows();
	int nPads = theModule->getNPads();

	for( int r = 0; r<nRows; r++)
	  {
	    const std::vector<int > padsInRow = theModule->getPadsInRow(r);

	    for(unsigned int p = 0; p<padsInRow.size(); p++)
	      {
		Vector2D padCenter = theModule->getPadCenter(padsInRow.at(p));
		moduleCenter[0]+=padCenter[0]/(double)nPads/(double)theModules.size();
		moduleCenter[1]+=padCenter[1]/(double)nPads/(double)theModules.size();

		if(r == 4)
		  {
		    streamlog_out(DEBUG)<<"Pad Centers in Row 4: "<<padCenter[0]<<" "<<padCenter[1]<<endl;
		  }


	      }

	  }
      }

    return moduleCenter;
  }



} // close namespace brackets

#endif //#ifdef USE_PATHFINDER
