#include "TrackFitterFactory.h"
#include "TrackFitterBase.h"
#include "TrackFitterSimpleChiSquare.h"
#include "TrackFitterSimpleMinimizer.h"
#include "TrackFitterSimpleChiSquarePads.h"
#include "TrackFitterLinearRegression.h"
#include "TrackFitterSimpleHelix.h"
#include "TrackFitterKalman.h"

#include <Exceptions.h>


#include <streamlog/streamlog.h>

namespace marlintpc
{

TrackFitterFactory::FitterTypeException::FitterTypeException(std::string const & initString)
        : EVENT::Exception(initString) {}



TrackFitterBase * TrackFitterFactory::getTrackFitter(unsigned char fitterType,
                                                     EVENT::LCParameters const * collectionParameters,
                                                     EVENT::LCEvent const *  /*event*/)
{

    switch (fitterType)
        {
        case TrackFitterBase::FitterTypes::SIMPLECHISQUARE :
            return TrackFitterSimpleChiSquare::getInstance(collectionParameters);
        case TrackFitterBase::FitterTypes::SIMPLECHISQUAREPADS :
            return TrackFitterSimpleChiSquarePads::getInstance(collectionParameters);
        case TrackFitterBase::FitterTypes::SIMPLEMINIMIZER :
            return TrackFitterSimpleMinimizer::getInstance(collectionParameters);
        case TrackFitterBase::FitterTypes::KALMAN :
            return TrackFitterKalman::getInstance(collectionParameters);
        case TrackFitterBase::FitterTypes::LIKELIHOOD :
        case TrackFitterBase::FitterTypes::LINEARREGRESSION :
            return TrackFitterLinearRegression::getInstance(collectionParameters);
        case TrackFitterBase::FitterTypes::SIMPLEHELIXFIT :
        	return TrackFitterSimpleHelix::getInstance(collectionParameters);
        {
            std::stringstream errorText;
            errorText << "TrackFitterFactory: Fitter type " << fitterType << " not implemented yet!" ;

            throw FitterTypeException(errorText.str());
        }

        default:
        {
            std::stringstream errorText;
            errorText << "TrackFitterFactory: Fitter type " << fitterType << " unknown!" ;

            throw FitterTypeException(errorText.str());
        }
        }

}

} // namespace marlintpc
