#include "TrackFitterSimpleHelix.h"
#include <iostream>
#include <string>
#include <cmath>
#include "LCObjectCopier.h"
#include <streamlog/streamlog.h>
#include "TMatrixD.h"
#include "SimpleHelixTrackModel.h"

// include what you need from lcio
#include <Exceptions.h>

//ROOT
#include "TVectorD.h"


using namespace lcio;
using namespace marlin;
using namespace std;


namespace marlintpc
{

TrackFitterBase * TrackFitterSimpleHelix::getInstance(LCParameters const *)
{
	static TrackFitterSimpleHelix theSimpleHelixFitter;
	return &theSimpleHelixFitter;
}

TrackFitterSimpleHelix::TrackFitterSimpleHelix()
{
	streamlog_out(DEBUG2) << "TrackFitterSimpleHelix:: constructor" << std::endl;
}

std::string TrackFitterSimpleHelix::getRevision() const
{
	return std::string("$Rev:$");
}

unsigned char TrackFitterSimpleHelix::getFitterType() const
{
	return TrackFitterBase::FitterTypes::SIMPLEHELIXFIT;
}

IMPL::TrackImpl * TrackFitterSimpleHelix::fitTrack(EVENT::Track const * seedTrack) const
{
	//do the circle fit in the xy projection
//	simpleFitXY xyFit(1, seedTrack->getReferencePoint()[0], seedTrack->getReferencePoint()[1]);
//	xyFit.addTrack(seedTrack);
//	double chi2XY;
//	int i = xyFit.fit(chi2XY, (int&)seedTrack->getTrackerHits().size());
//	TVector3 xyPar = xyFit.getPar();
//
//	// create return track
//	IMPL::TrackImpl * fittedTrack=LCObjectCopier::copy(seedTrack);
//	fittedTrack->setD0(xyPar[2]);
//	fittedTrack->setPhi(xyPar[1]);
//	fittedTrack->setOmega(xyPar[0]);
//
//	simpleFitZS zsFit;
//	TrackerHitVec hitVec = seedTrack->getTrackerHits();
//	for(unsigned int iHit = 0 ; iHit < hitVec.size(); iHit++)
//			{
//		      zsFit.addPoint(hitVec[iHit]->getPosition()[0], inputHitVec[iHit]->getPosition()[1],1);
//			}
//
//	double chi2ZS;
//	i = zsFit.fit(chi2ZS, (int&)seedTrack->getTrackerHits().size());

	//do the circle fit in the xy projection
	EVENT::DoubleVec xyCircularFit=TrackFitterSimpleHelix::doXYCircularFit(seedTrack);
	//create return track
	IMPL::TrackImpl * fittedTrack=LCObjectCopier::copy(seedTrack);
	fittedTrack->setD0(xyCircularFit[0]);
	fittedTrack->setPhi(xyCircularFit[1]);
	fittedTrack->setOmega(xyCircularFit[2]);

	//perform a linear regression in the sz projection
	EVENT::DoubleVec szRegression=TrackFitterSimpleHelix::doRegressionSZ(fittedTrack);
	fittedTrack->setZ0(szRegression[0]);
	fittedTrack->setTanLambda(szRegression[1]);
	float covMatrix[15]={0};
	covMatrix[0]=xyCircularFit[3]*xyCircularFit[3];
	covMatrix[2]=xyCircularFit[4]*xyCircularFit[4];
	covMatrix[5]=xyCircularFit[5]*xyCircularFit[5];
	covMatrix[9]=szRegression[2]*szRegression[2];
	covMatrix[14]=szRegression[3]*szRegression[3];
	fittedTrack->setCovMatrix(covMatrix);

	fittedTrack->setChi2(xyCircularFit[6] + szRegression[4]); 

	return fittedTrack;

}

EVENT::DoubleVec TrackFitterSimpleHelix::doXYCircularFit(EVENT::Track const * seedTrack) const
{

	TrackerHitVec inputHitVec=seedTrack->getTrackerHits();

	vector<double> XPos;
	vector<double> YPos;

	double mean_x = 0.;
	double mean_y = 0.;
	double mean_x2 = 0.;
	double mean_xy = 0.;
	double mean_y2 = 0.;
	double mean_xr2 = 0.;
	double mean_yr2 = 0.;
	double mean_r2 = 0.;
	double mean_r4 = 0.;
	double Sx = 0.;
	double Sy = 0.;
	double Sxy = 0.;
	double Sxx = 0.;
	double Syy = 0.;
	double Sxr2 = 0;
	double Syr2 = 0.;
	double Sr2 = 0.;
	double Sr4 = 0.;
	double Sw = 0.;
	for(unsigned int iHit = 0 ; iHit < inputHitVec.size(); iHit++)
	{
		double pos[2];
		pos[0] = inputHitVec[iHit]->getPosition()[0] - seedTrack->getReferencePoint()[0];
		pos[1] = inputHitVec[iHit]->getPosition()[1] - seedTrack->getReferencePoint()[1];

		double r2 = pos[0] * pos[0] + pos[1] * pos[1];

		//mean_x += pos[0];
		//mean_y += pos[1];
		Sx += pos[0];
		Sy += pos[1];
		//mean_x2 += pos[0] * pos[0];
		Sxx += pos[0] * pos[0];
		//mean_xy += pos[0] * pos[1];
		Sxy += pos[0] * pos[1];
		//mean_y2 += pos[1] * pos[1];
		Syy += pos[1] * pos[1];
		//mean_xr2 += pos[0] * r2;
		Sxr2 += pos[0] * r2;
		//mean_yr2 += pos[1] * r2;
		Syr2 += pos[1] * r2;
		//mean_r2 += r2;
		Sr2 += r2;
		//mean_r4 += r2 * r2;
		Sr4  += r2 * r2;

		Sw += 1.;

		XPos.push_back(pos[0]);
		YPos.push_back(pos[1]);
	}

	mean_x = Sx / (double)inputHitVec.size();
	mean_y = Sy / (double)inputHitVec.size();
	mean_x2 = Sxx / (double)inputHitVec.size();
	mean_xy = Sxy / (double)inputHitVec.size();
	mean_y2 = Syy / (double)inputHitVec.size();
	mean_xr2 = Sxr2 / (double)inputHitVec.size();
	mean_yr2 = Syr2 / (double)inputHitVec.size();
	mean_r2 = Sr2 / (double)inputHitVec.size();
	mean_r4 = Sr4 / (double)inputHitVec.size();

	double Cxx = mean_x2 - mean_x * mean_x;
	double Cxy = mean_xy - mean_x * mean_y;
	double Cyy = mean_y2 - mean_y * mean_y;
	double Cxr2 = mean_xr2 - mean_x * mean_r2;
	double Cyr2 = mean_yr2 - mean_y * mean_r2;
	double Cr2r2 = mean_r4 - mean_r2 * mean_r2;

	double q1 = Cr2r2 * Cxy - Cxr2 * Cyr2;
	double q2 = Cr2r2 * (Cxx - Cyy) - Cxr2 * Cxr2 + Cyr2 * Cyr2;

	//    double phi_fit = 0.5*atan(2*q1/q2);
	double phi_fit = 0.5 * atan2(2 * q1, q2);

	double kappa = (sin(phi_fit) * Cxr2 - cos(phi_fit) * Cyr2) / Cr2r2;

	double delta = -kappa * mean_r2 + sin(phi_fit) * mean_x - cos(phi_fit) * mean_y;

	double omega_fit = 2 * kappa / sqrt(1 - 4 * delta * kappa);

	double d0_fit = 2 * delta / (1 + sqrt(1 - 4 * delta * kappa));

	//calculation of errors
	double u = 1 + omega_fit * d0_fit;
	double Sa = sin(phi_fit) * Sx - cos(phi_fit) * Sy;
	double Sb = cos(phi_fit) * Sx + sin(phi_fit) * Sy;
	double Sg = (sin(phi_fit) * sin(phi_fit) - cos(phi_fit) * cos(phi_fit)) * Sxy + sin(phi_fit) * cos(phi_fit) * (Sxx - Syy);
	double Sd = sin(phi_fit) * Sxr2 - cos(phi_fit) * Syr2;
	double Saa = sin(phi_fit) * sin(phi_fit) * Sxx - 2 * sin(phi_fit) * cos(phi_fit) * Sxy + cos(phi_fit) * cos(phi_fit) * Syy;

	TMatrixD V_inv(3, 3);

	V_inv(0, 0) = 0.25 * Sr4 - d0_fit * (Sd - d0_fit * (Saa = 0.5 * Sr2 - d0_fit * (Sa - 0.25 * d0_fit * Sw)));
	V_inv(0, 1) = -u * (0.5 * (cos(phi_fit) * Sxr2 + sin(phi_fit) * Syr2) - d0_fit * (Sg - 0.5 * d0_fit * Sb));
	V_inv(0, 2) = omega_fit * (-0.5 * Sd + d0_fit * Saa) + 0.5 * u * Sr2 - 0.5 * d0_fit * ((2 * u + omega_fit * d0_fit) * Sa - u * d0_fit * Sw);
	V_inv(1, 0) = -u * (0.5 * (cos(phi_fit) * Sxr2 + sin(phi_fit) * Syr2) - d0_fit * (Sg - 0.5 * d0_fit * Sb));
	V_inv(1, 1) = u * u * (cos(phi_fit) * cos(phi_fit) * Sxx + sin(2 * phi_fit) * Sxy + sin(phi_fit) * sin(phi_fit) * Syy);
	V_inv(1, 2) = u * (omega_fit * Sg - u * Sb);
	V_inv(2, 0) = omega_fit * (-0.5 * Sd + d0_fit * Saa) + 0.5 * u * Sr2 - 0.5 * d0_fit * ((2 * u + omega_fit * d0_fit) * Sa - u * d0_fit * Sw);
	V_inv(2, 1) = u * (omega_fit * Sg - u * Sb);
	V_inv(2, 2) = omega_fit * (omega_fit * Saa - 2 * u * Sa) + u * u * Sw;
	/*
    cout<<V_inv(0,0)<<"\t"<<V_inv(0,1)<<"\t"<<V_inv(0,2)<<endl;
    cout<<V_inv(1,0)<<"\t"<<V_inv(1,1)<<"\t"<<V_inv(1,2)<<endl;
    cout<<V_inv(2,0)<<"\t"<<V_inv(2,1)<<"\t"<<V_inv(2,2)<<endl;*/
	TMatrixD V = V_inv.InvertFast();//Attention: This line changes V_inv!!!
	/*  cout<<V(0,0)<<"\t"<<V(0,1)<<"\t"<<V(0,2)<<endl;
    cout<<V(1,0)<<"\t"<<V(1,1)<<"\t"<<V(1,2)<<endl;
    cout<<V(2,0)<<"\t"<<V(2,1)<<"\t"<<V(2,2)<<endl;
	 */
	double phiErr_fit = sqrt(abs(V(1, 1)));
	double d0Err_fit = sqrt(abs(V(2, 2)));
	double omegaErr_fit = sqrt(abs(V(0, 0)));
	/*
    cout << "Karimaeki Fit Phi: " << phi_fit <<" +- "<<phiErr_fit<< endl;
    cout << "Karimaeki Fit D0: " << d0_fit <<" +- "<<d0Err_fit<< endl;
    cout << "Karimaeki Fit Omega: " << omega_fit <<" +- "<<omegaErr_fit<< endl;
	 */
	//fittedXYTrackParameter = new TrackParameterXY(phi, d0, omega);

	double chi2 = Sw*Sw*(1 + omega_fit*d0_fit)*(1 + omega_fit*d0_fit)*(sin(phi_fit)*sin(phi_fit)*Cxx - 2*sin(phi_fit)*cos(phi_fit)*Cxy
			+ cos(phi_fit)*cos(phi_fit)*Cyy - kappa*kappa*Cr2r2);

	EVENT::DoubleVec returnVector;
	returnVector.push_back(-d0_fit);
	returnVector.push_back(phi_fit);
	returnVector.push_back(omega_fit);
	returnVector.push_back(d0Err_fit);
	returnVector.push_back(phiErr_fit);
	returnVector.push_back(omegaErr_fit);
	returnVector.push_back(chi2);
	return returnVector;
}

EVENT::DoubleVec TrackFitterSimpleHelix::doRegressionSZ(EVENT::Track const * seedTrack) const
{
	double slope;
	if (_useTanLambdaStart)
		slope=_tanLambdaStart;
	else
		slope=seedTrack->getTanLambda();
	//  slope = 0.;//I am not sure if the linear regression is correct for other cases...
	// cout << "Slope of straight line sz: " << slope << endl;

	TrackerHitVec inputHitVec=seedTrack->getTrackerHits();
	double mean_x = 0.;
	double mean_y = 0.;
	double mean_x2 = 0.;
	double mean_xy = 0.;
	double mean_y2 = 0.;
	EVENT::DoubleVec pca=TrackFitterSimpleHelix::calculatePCA(seedTrack);
	for(unsigned int iHit = 0 ; iHit < inputHitVec.size(); iHit++)
	{
		double pos[2];
		pos[0] = TrackFitterSimpleHelix::calculateS(inputHitVec.at(iHit),pca,seedTrack);
		pos[1] = inputHitVec.at(iHit)->getPosition()[2]-seedTrack->getReferencePoint()[2];

		mean_x += pos[0];
		mean_y += pos[1];
		mean_x2 += pos[0] * pos[0];
		mean_xy += pos[0] * pos[1];
		mean_y2 += pos[1] * pos[1];

	}

	double tanl_fit;
	double z0_fit;

	double tanlErr_fit;
	double z0Err_fit;

	double m = ((double)inputHitVec.size() * mean_xy - mean_x * mean_y) /
			((double)inputHitVec.size() * mean_x2 - mean_x * mean_x);
	//double mprime = ((double)inputHitVec.size() * mean_x2 - mean_x * mean_x)/((double)inputHitVec.size() * mean_xy - mean_x * mean_y);
	double mprime = ((double)inputHitVec.size() * mean_xy - mean_x * mean_y) /
			((double)inputHitVec.size() * mean_y2 - mean_y * mean_y);
	double b = (mean_x2 * mean_y - mean_x * mean_xy) / ((double)inputHitVec.size() * mean_x2 - mean_x * mean_x);
	//  double bprime = (-mean_y2 * mean_x + mean_y * mean_xy) / ((double)inputHitVec.size() * mean_y2 - mean_y * mean_y);
	double bprime = (mean_y2 * mean_x - mean_y * mean_xy) / ((double)inputHitVec.size() * mean_y2 - mean_y * mean_y);

	double chi2 = 0.; 

	//if slope is smaller than 1, minimize the y-distance of the hits to the track
	if(slope >= -1. && slope <= 1.)
	{
		tanl_fit = m;
		z0_fit = b;
		/*
	            tanl_fit = (((double)inputHitVec.size() * mean_xy - mean_x * mean_y) /
	                        ((double)inputHitVec.size() * mean_x2 - mean_x * mean_x));

	            z0_fit = (mean_x2 * mean_y - mean_x * mean_xy) / ((double)inputHitVec.size() * mean_x2 - mean_x * mean_x);
		 */
	}
	//if slope is larger than 1, minimize the x-distance of the hits to the track
	else
	{
		tanl_fit = -1. / mprime;
		z0_fit = bprime / mprime;
		/*
	            tanl_fit = ((mean_y * mean_y - (double)inputHitVec.size() * mean_y2) * (-1.) /
	                        ((double)inputHitVec.size() * mean_xy - mean_x * mean_y));

	            z0_fit = tanl_fit * (mean_y * mean_xy - mean_x * mean_y2) / (mean_y * mean_y - (double)inputHitVec.size() * mean_y2) * (-1.);
		 */
	}


	double mean_ybmx2 = 0.;
	double mean_ybmx2prime = 0.;
	for(unsigned int iHit = 0 ; iHit < inputHitVec.size(); iHit++)
	{
		double pos[2];
		pos[0] = TrackFitterSimpleHelix::calculateS(inputHitVec.at(iHit),pca,seedTrack);
		pos[1] = inputHitVec.at(iHit)->getPosition()[2]-seedTrack->getReferencePoint()[2];

		mean_ybmx2 += (pos[1] - b - m * pos[0]) * (pos[1] - b - m * pos[0]);
		mean_ybmx2prime += (pos[0] - bprime - mprime * pos[1]) * (pos[0] - bprime - mprime * pos[1]);
	}

	double mErr = sqrt(((double)inputHitVec.size() / ((double)inputHitVec.size() - 2.) * mean_ybmx2) /
			((double)inputHitVec.size() * mean_x2 - mean_x * mean_x));
	//  double mprimeErr = mErr/m/m;
	double mprimeErr = sqrt(((double)inputHitVec.size() / ((double)inputHitVec.size() - 2.) * mean_ybmx2prime) /
			((double)inputHitVec.size() * mean_y2 - mean_y * mean_y));
	double bErr = sqrt((mean_x2 * mean_ybmx2 / ((double)inputHitVec.size() - 2.)) / ((double)inputHitVec.size() * mean_x2 - mean_x * mean_x));
	//  double bprimeErr = sqrt( (mean_y2*mean_ybmx2prime/((double)inputHitVec.size()-2.))/((double)inputHitVec.size()* mean_y2 - mean_y*mean_y));
	double bprimeErr = sqrt((mean_y2 * mean_ybmx2prime / ((double)inputHitVec.size() - 2.)) / ((double)inputHitVec.size() * mean_y2 - mean_y * mean_y));

	if(slope >= -1. && slope <= 1.)
	{
		tanlErr_fit = mErr;
		z0Err_fit = bErr;

		chi2 = ((double)inputHitVec.size() / ((double)inputHitVec.size() - 2.) * mean_ybmx2); //PETER
	}
	else
	{
		//Note: no guarantee that this is correct...
		tanlErr_fit = mprimeErr / mprime / mprime;
		z0Err_fit = sqrt((bprime * mprimeErr / mprime / mprime) + (bprimeErr / mprime) * (bprimeErr / mprime));

		chi2 = ((double)inputHitVec.size() / ((double)inputHitVec.size() - 2.) * mean_ybmx2prime); //PETER
	}


	// cout << "tanl_fit: " << tanl_fit <<" +- "<<tanlErr_fit<< " z0_fit: " << z0_fit <<" +- "<<z0Err_fit<< endl;
	EVENT::DoubleVec returnVec;
	returnVec.push_back(z0_fit);
	returnVec.push_back(tanl_fit);
	returnVec.push_back(z0Err_fit);
	returnVec.push_back(tanlErr_fit);
	returnVec.push_back(chi2); 
	return returnVec;
}

EVENT::DoubleVec TrackFitterSimpleHelix::calculatePCA(EVENT::Track const * seedTrack) const
{
	DoubleVec pca;
	double d0,phi;
	if (_useD0Start)
		d0=_d0Start;
	else
		d0=seedTrack->getD0();
	if (_usePhiStart)
		phi=_phiStart;
	else
		phi=seedTrack->getPhi();

	pca.push_back(-d0*sin(phi)+seedTrack->getReferencePoint()[0]);
	pca.push_back(d0*cos(phi)+seedTrack->getReferencePoint()[1]);
	return pca;
}

double TrackFitterSimpleHelix::calculateS(EVENT::TrackerHit * aHit, EVENT::DoubleVec pca, EVENT::Track const * seedTrack ) const
{
	double s_xy = 0.;

	double phi,d0,omega;
	if (_usePhiStart)
		phi=_phiStart;
	else
		phi=seedTrack->getPhi();
	if (_useD0Start)
		d0=_d0Start;
	else
		d0=seedTrack->getD0();
	if (_useOmegaStart)
		omega=_omegaStart;
	else
		omega=seedTrack->getOmega();

	//if curvature is zero calculate s as if track is a straight line
	if(omega != 0.)
	{
		double r = 1 / omega;

		//center of circle
		double xm, ym;
		xm = (r - d0) * sin(phi)+seedTrack->getReferencePoint()[0];
		ym = -(r - d0) * cos(phi)+seedTrack->getReferencePoint()[1];

		//cout<<"pca: "<<pca[0]<<" "<<pca[1]<<endl;
		//cout<<"center: "<<xm<<" "<<ym<<endl;

		s_xy = r * (
				atan2(pca[1] - ym , pca[0] - xm)
				-
				atan2(aHit->getPosition()[1] - ym , aHit->getPosition()[0] - xm)
		);


		double s_xy_1, s_xy_2;
		/*
	                    if(s_xy>=0.)
	                      {
	                        s_xy_1 = s_xy;
	                        s_xy_2 = s_xy - fabs(r);
	                      }
	                    else
	                      {
	                            s_xy_1 = s_xy;
	                            s_xy_2 = fabs(r) - s_xy;
	                      }
		 */
		s_xy_1 = s_xy;
		if(s_xy < 0)
		{
			s_xy_2 = s_xy + fabs(2 * M_PI / omega);
		}
		else
		{
			s_xy_2 = s_xy - fabs(2 * M_PI / omega);
		}

		//      s_xy_2 = fabs(fabs(s_xy) - fabs(2*M_PI/omega));

		//      cout<<"s_xy: "<<s_xy_1<<" "<<s_xy_2<<endl;
		if(fabs(s_xy_1) <= fabs(s_xy_2)) s_xy = s_xy_1;
		else s_xy = s_xy_2;

	}//if(omega != 0.)
	else
	{
		s_xy = sqrt(
				(aHit->getPosition()[0] - pca[0]) * (aHit->getPosition()[0] - pca[0]) +
				(aHit->getPosition()[1] - pca[1]) * (aHit->getPosition()[1] - pca[1]));

		if((-1.25 * M_PI <= phi && phi <= -0.75 * M_PI) ||
				(-0.25 * M_PI <= phi && phi <= 0.25 * M_PI) ||
				(0.75 * M_PI <= phi && phi <= 1.25 * M_PI))
		{
			if(pca[0] < aHit->getPosition()[0])
			{
				s_xy = (-1.) * s_xy;
			}
		}
		else
		{
			if(pca[1] < aHit->getPosition()[1])
			{
				s_xy = (-1.) * s_xy;
			}
		}
		s_xy = -s_xy;
	}

	return s_xy;

}

} // close namespace brackets

