#ifndef SIGNALDIGITISATIONPROCESSOR_H
#define SIGNALDIGITISATIONPROCESSOR_H 1

// Marlin
#include <marlin/Processor.h>

namespace EVENT {

	class LCRunHeader;
	class LCEvent;
}

namespace marlintpc {

/** SignalDigitisationProcessor : This processor takes TrackerData from the simulation and turns it into digital TrackerRawData
 *
 *    @author J. Abernathy (UVic)<br>
 */
class SignalDigitisationProcessor : public marlin::Processor {

public:
	/** constructor
	 */
	SignalDigitisationProcessor();

	/** destructor
	 */
	virtual ~SignalDigitisationProcessor();

	/** used by Marlin to create a new processor
	 */
	virtual Processor* newProcessor() { return new SignalDigitisationProcessor() ; }

	/**
	 * Called at the begin of the job before anything is read.
	 * Use to initialize the processor, e.g. book histograms.
	 */
	virtual void init();

	/**
	 * Called for every run
	 */
	virtual void processRunHeader(lcio::LCRunHeader* run );

	/**
	 * Called for every event - the working horse.
	 */
	virtual void processEvent(lcio::LCEvent * evt );

	/**
	 * Called after the event has been processed
	 */
	virtual void check(lcio::LCEvent * evt );

	/**
	 * Called after data processing for clean up.
	 */
	virtual void end();

protected:

	/** Adds the electronic parameters to the LCIO file. These can later be used by LCCD to reconstruct the analog signal which produced the digital signal
	 */
	void add_electronic_parameters( LCEvent *evt );

private:

	/** the name of the input tracker data collection
	 */
	std::string _input_collection_name;

	/** the name of the output tracker collection from this processor
	 */
	std::string _output_collection_name;

	/** the name of the output pedestal collection for this processor
	 */
	std::string _tpc_pedestal_collection_name;

	/** the name of the output ADC electronics parameters collection for this processor
	 */
	std::string _adc_electronics_parameters_collection_name;

	/** the gain of this electronic digitiser
	 */
	double _gain;

	/** the number of bits put into each electronic sample
	 */
	int _bits_per_sample;

	/** the digital value is of the range [0..2^bits - 1], the pedestal is added to allow negative values
	 */
	int _pedestal;

	/** the width of a signal sample (1/Hz) in ns
	 */
	double _time_resolution;
};

} // namespace marlintpc

#endif // SIGNALSHAPERGAUSSIANPROCESSOR_H
