//C++
#include <iostream>
#include <string>
#include <sstream>

#include "TAltroPedestalConverter.hh"

//LCIO
#include "lcio.h"
#include "EVENT/LCCollection.h"

//LCCD
#include "lccd.h"
#include "lccd/DBInterface.hh"

//ilcutil
#define DONT_USE_STREAMLOG
#include "streamlog/streamlog.h"

void printUsage()
{
    streamlog_out(MESSAGE)<<"Usage:"<<std::endl <<
            "To write data to a lcio simple file:"<<std::endl <<
            "  ConvertStoreAltroPedestal -S outfile \\"<<std::endl <<
            "  [-M connectionstring folder timeStamp] \\"<<std::endl <<
            "  pedestalfile.txt [pedestalfile.txt [pedestalfile.txt ... ]]"<<std::endl <<
            "To write data to a database:"<<std::endl <<
            "  ConvertStoreAltroPedestal -D connectionString folder startLCIOTimeStamp endLCIOTimeStamp \\"<<std::endl <<
            "  [-M connectionstring folder timeStamp] \\"<<std::endl <<
            "  pedestalfile.txt [pedestalfile.txt [pedestalfile.txt ... ]]"<<std::endl <<
            "-M option reads in channelmapping from a database and translates "<<std::endl <<
            "   hardware channel and rcu into padID and moduleID from Gear"<<std::endl <<
            "Formats:"<<std::endl <<
            " Text file: hwChannel intPedestal floatPedestal floatPedestalWidth integer" << std::endl <<
            " Connection String: hostname:database:login:password[:port]"<<std::endl <<
            " folder: /database/folder"<<std::endl <<
            " LCIO time stamp: nano seconds since 1.1.1970 00:00:00"<<std::endl;
}



int main(int argc, char **argv)
{
    //Program options
    bool writeSimpleFile=false;
    bool writeToDatabase=false;
    bool doMapping = false;

    std::string outFileName;
    std::vector<std::string> inFileNames;
    
    std::string connectionString;
    std::string folder;
    lccd::LCCDTimeStamp startTimeStamp;
    lccd::LCCDTimeStamp endTimeStamp;

    std::string mappingConnectionString;
    std::string mappingFolder;
    lccd::LCCDTimeStamp mappingTimeStamp;
    
    if ( argc < 2 )
    {
        printUsage();
        return(1);
    }

    int  optionCounter = 1;
    std::string option = argv[optionCounter];
    //DEBUG
    //std::cout<<"option= "<<option<<std::endl;
        
    if ( option == "-S" )
    {
        writeSimpleFile=true;
        outFileName=argv[2];
        optionCounter=3;
    } 
    else if ( option == "-D" )
    {
        writeToDatabase=true;
        connectionString=argv[2];
        folder=argv[3];
        std::stringstream startstr(argv[4]);
        startstr >> startTimeStamp;
        std::stringstream endstr(argv[5]);
        endstr >> endTimeStamp;
        optionCounter=6;
    }
    else
    {
        printUsage();
        return(1);
    }
    
    
    //see if data should be re-mapped, if so, set variables
    std::map< std::pair<int, int>, std::pair<int, int> > mappingMap;
    option = argv[optionCounter];
    if ( option == "-M" )
    {
        doMapping = true;
        optionCounter++;
        mappingConnectionString = argv[optionCounter];
        optionCounter++;
        mappingFolder = argv[optionCounter];
        optionCounter++;
        std::stringstream startstr(argv[optionCounter]);
        startstr >> mappingTimeStamp;
        optionCounter++;
    }
    
    for ( int i=optionCounter; i<argc ; ++i)
    {
        inFileNames.push_back(argv[i]);
    }
    
    /*DEBUG
    for (int i=0; i< argc ; ++i)
    {
        std::cout<<"DEBUG: argument ["<<i<<"]: "<<argv[i]<<std::endl;
    } 
    */

    marlintpc::TAltroPedestalConverter PedestalConverter;
    if(doMapping)
    {
        lccd::DBInterface dbConnection(mappingConnectionString, mappingFolder, false);
        lcio::LCCollection* condMappingCol = dynamic_cast<lcio::LCCollection*>( dbConnection.findCollection(mappingTimeStamp));
        PedestalConverter.Remap(condMappingCol);
    }

    PedestalConverter.Convert(inFileNames);
            
    if ( writeToDatabase )
    {
        lccd::DBInterface dbConnection(connectionString, folder, true );
        PedestalConverter.ToDB(dbConnection, startTimeStamp, endTimeStamp, "TPCPedestal");
    }

    if ( writeSimpleFile )
    {
        PedestalConverter.ToFile(outFileName, "TPCPedestal");
    }
    return 0;
}
