/*
 * AltroConverterExceptions.hh
 *
 *  Created on: Jul 6, 2012
 *      Author: caiazza
 */

#ifndef ALTROCONVERTEREXCEPTIONS_HH_
#define ALTROCONVERTEREXCEPTIONS_HH_

#include <stdexcept>
#include <string>
#include <iostream>
#include <utility>

namespace marlintpc
{
/**
 * Base class for the exceptions thrown by the ALTRO Converter.
 */
class TAltroConverterException : public std::runtime_error
{
public:
    virtual ~TAltroConverterException() throw(){}
    TAltroConverterException() :
        std::runtime_error("ERROR: Runtime exception detected in the ALTRO Converter"){}
    TAltroConverterException(const std::string & err) :
        std::runtime_error(std::string("ERROR: Runtime exception detected in the ALTRO Converter: ") + err){}
    TAltroConverterException(const std::string& ClassName, const std::string& Message):
      std::runtime_error(std::string("ERROR: Runtime exception detected in the ALTRO Converter: ") + ClassName + ": " + Message) { }
};

class TAltroConverterVersionException : public TAltroConverterException
{
public:
    virtual ~TAltroConverterVersionException() throw(){}
    TAltroConverterVersionException():
        TAltroConverterException("The converter doesn't support the required version"){}
    TAltroConverterVersionException(const std::string & err):
        TAltroConverterException(err){}
    TAltroConverterVersionException(const std::string& ClassName, const std::string& Message):
        TAltroConverterException(ClassName, Message){}
};

class TAltroConverterInvalidInputException: public TAltroConverterException
{
public:
    virtual ~TAltroConverterInvalidInputException() throw(){}
    TAltroConverterInvalidInputException():
        TAltroConverterException("The input stream is invalid or corrupted"){}
    TAltroConverterInvalidInputException(const std::string & err):
        TAltroConverterException(err){}
    TAltroConverterInvalidInputException(const std::string& ClassName, const std::string& Message):
        TAltroConverterException(ClassName, Message){}
};

class TAltroConverterCLException: public TAltroConverterException
{
public:
    virtual ~TAltroConverterCLException() throw(){}
    TAltroConverterCLException():
        TAltroConverterException("Invalid command line argument detected."){}
    TAltroConverterCLException(const std::string & err):
        TAltroConverterException(err){}
};

class TAltroConverterDecodeException : public TAltroConverterException
{
public:
    virtual ~TAltroConverterDecodeException() throw(){}
    TAltroConverterDecodeException():
        TAltroConverterException("Decoding error."){}
    TAltroConverterDecodeException(const std::string & err):
        TAltroConverterException(err){}
    TAltroConverterDecodeException(const std::string& ClassName, const std::string& Message):
        TAltroConverterException(ClassName, Message){}
};

class TAltroConverterEOFException : public TAltroConverterException
{
public:
    virtual ~TAltroConverterEOFException() throw(){}
    TAltroConverterEOFException():
        TAltroConverterException("End of file reached"){}
    TAltroConverterEOFException(const std::string & err):
        TAltroConverterException(err){}
    TAltroConverterEOFException(const std::string& ClassName, const std::string& Message):
        TAltroConverterException(ClassName, Message){}
};

}//end namespace

#endif /* ALTROCONVERTEREXCEPTIONS_HH_ */
