/*
 * TIPORBlock.hh
 *
 *  Created on: Jul 10, 2012
 *      Author: caiazza
 */

#ifndef TIPORBLOCK_HH_
#define TIPORBLOCK_HH_

#include "TIAltroBlock.hh"

//LCIO
#include "lcio.h"
#include "UTIL/LCTime.h"

namespace marlintpc
{
class TIPORBlock : public TIAltroBlock
{
public:
    /**
     * Virtual destructor
     */
    virtual ~TIPORBlock(){}

    /**
     * Timestamp of the pause
     * @return Timestamp of the pause
     */
    virtual lcio::LCTime GetPauseTime() const = 0;

    /**
     * Number of events collected at the pause
     * @return Number of events collected at the pause
     */
    virtual uint32_t GetEventsAtPause() const = 0;
}; //End interface

/**
 * Implementation of the Pause of Run class for DAQ version 4.2+
 */
class TPORBlock_V42 : public TIPORBlock
{
public:
    TPORBlock_V42(std::vector<uint32_t>& Content);

    virtual ~TPORBlock_V42();

    /**
     * Get the block lenght in number of 32 bit words
     * @return number of 32 bit words in the block
     */
    virtual uint32_t GetLength() const;

    /**
     * Verifies that the content of the block matches the expected content of the type of block we build.
     * @return True if the content of the block matches the expectation, False otherwise
     */
    virtual bool CheckIntegrity() const;

    /**
     * Returns a constant reference to the content of the block
     * @return vector of 32 bit data words
     */
    virtual const std::vector<uint32_t>& Content() const;

    /**
     * Returns a data word that identifies the block type
     * @return 32 bit data word identifying the block
     */
    virtual uint32_t GetIdentifier() const;

    /**
     * Timestamp of the pause
     * @return Timestamp of the pause
     */
    virtual lcio::LCTime GetPauseTime() const;

    /**
     * Number of events collected at the pause
     * @return Number of events collected at the pause
     */
    virtual uint32_t GetEventsAtPause() const;

    /**
     * Allows to set additional parameters to the RunHeader.
     * @param RunHeader
     */
    virtual void SetAdditionalParameters(lcio::LCRunHeader* RunHeader);

private:
    std::vector<uint32_t> _content;
}; //End implementation V4.2

}

#endif /* TIPORBLOCK_HH_ */
