/*
 * TAltroConverter.cpp
 *
 *  Created on: Jul 6, 2012
 *      Author: caiazza
 */
#include <string>
#include <iostream>
#include <sstream>
#include <utility>
#include <vector>

//Converter
#include "TAltroConverter.hh"
#include "TAltroBlockFactory.hh"
#include "TISORBlock.hh"
#include "TIEORBlock.hh"
#include "TIBOFBlock.hh"
#include "TIEventBlock.hh"
#include "AltroConverterGlobals.hh"

//LCIO
#include "IMPL/LCRunHeaderImpl.h"
#include "IMPL/LCEventImpl.h"

//MarlinTPC
#include "GenericADCElectronicsParameters.h"
#include "ExtendedAltroParameters.h"

namespace marlintpc
{
TAltroConverter::TAltroConverter():
        _input(),
        _output(0),
        _GenericParColName("GenericADCElectronicsParameters"),
        _ExtendedParColName("ExtendedAltroElectronicsParameters"),
        _runStart(0),
        _runEnd(0),
        _DataVersion(std::make_pair(0,0)),
        _Factory(0),
        _DetectorName("LPTPC"),
        _RunDescription("Test beam run"),
        _DataColName("AltroRawData"),
        _GenericElectronicsPar(0),
        _ExtendedElectronicsPar(0),
        _ZSThresholdOverride(-1)
{

}

TAltroConverter::TAltroConverter(std::vector<std::ifstream*>& input, lcio::LCWriter* output):
        _input(),
        _output(0),
        _GenericParColName("GenericADCElectronicsParameters"),
        _ExtendedParColName("ExtendedAltroElectronicsParameters"),
        _runStart(0),
        _runEnd(0),
        _DataVersion(std::make_pair(0,0)),
        _Factory(0),
        _DetectorName("LPTPC"),
        _RunDescription("Test beam run"),
        _DataColName("AltroRawData"),
        _GenericElectronicsPar(0),
        _ExtendedElectronicsPar(0),
        _ZSThresholdOverride(-1)
{
    SetInput(input);
    SetOutput(output);
    std::cout << "Converter initialization completed" << std::endl;
}

TAltroConverter::TAltroConverter(const TAltroConverter& other):
        _input(other._input),
        _output(other._output),
        _GenericParColName(other._GenericParColName),
        _ExtendedParColName(other._ExtendedParColName),
        _runStart(other._runStart),
        _runEnd(other._runEnd),
        _DataVersion(other._DataVersion),
        _Factory(0),
        _DetectorName(other._DetectorName),
        _RunDescription(other._RunDescription),
        _DataColName(other._DataColName),
        _GenericElectronicsPar(0),
        _ExtendedElectronicsPar(0),
        _ZSThresholdOverride(other._ZSThresholdOverride)
{
    if(_DataVersion != std::make_pair(0,0))
    {
        _Factory = new TAltroBlockFactory(_DataVersion);
    }
}

TAltroConverter::~TAltroConverter()
{
    Reset();
}

TAltroConverter&
TAltroConverter::operator=(const TAltroConverter& other)
{
    if(this != &other)
    {
        Reset();
        _input = other._input;
        _output= other._output;
        _GenericParColName = other._GenericParColName;
        _ExtendedParColName = other._ExtendedParColName;
        _runStart = other._runStart;
        _runEnd = other._runEnd;
        _DataVersion = other._DataVersion;
        if(_DataVersion != std::make_pair(0,0))
        {
            _Factory = new TAltroBlockFactory(_DataVersion);
        }
        _DetectorName = other._DetectorName;
        _RunDescription = other._RunDescription;
        _DataColName = other._DataColName;
        _ZSThresholdOverride = other._ZSThresholdOverride;
    }
    return *this;
}


void
TAltroConverter::SetInput(std::vector<std::ifstream*>& input)
{
    if(input.empty())
    {
        std::stringstream message;
        message << "The set of input file is empty " << std::endl;
        throw TAltroConverterInvalidInputException("TAltroConverter", message.str());
    }

    std::vector<std::ifstream*>::iterator it = input.begin();
    for(; it != input.end(); ++it)
    {
        if(!((*it) && (*it)->good()))
        {
            int filenumber = (it - input.begin());
            std::stringstream message;
            message << "Unable to open file number " << filenumber << std::endl;
            throw TAltroConverterInvalidInputException("TAltroConverter", message.str());
        }
    }
    _input = input;

    _DataVersion = DataVersion();

    if(_Factory != 0)
    {
        delete _Factory;
        _Factory = 0;
    }
    _Factory = new TAltroBlockFactory(_DataVersion);

    if(!CheckIntegrity())
    {
        std::stringstream message;
        message << "The content of the input files is invalid" << std::endl;
	throw TAltroConverterInvalidInputException("TAltroConverter", message.str());
    }
   
}

void
TAltroConverter::SetOutput(lcio::LCWriter* output)
{
    if(output == 0)
    {
        std::cerr << "WARNING: The LCWriter is being set to a null pointer." << std::endl;
    }
    _output = output;
}

lcio::LCCollection*
TAltroConverter::GetGenericElectronicsPar() const
{
    std::cout << "DB Collection handling not yet implemented" << std::endl;
    return 0;
}

void
TAltroConverter::SetGenericElectronicsParColName(std::string ColName)
{
    _GenericParColName = ColName;
}

lcio::LCCollection*
TAltroConverter::GetExtendedElectronicsPar() const
{
    std::cout << "DB Collection handling not yet implemented" << std::endl;
    return 0;
}

void
TAltroConverter::SetExtendedElectronicParColName(std::string ColName)
{
    _ExtendedParColName = ColName;
}

void
TAltroConverter::ToDB(lccd::DBInterface* GenericInfo, lccd::DBInterface* ExtendedInfo) const
{
    if(GenericInfo != 0 )
    {
        if(_GenericElectronicsPar != 0)
        {
            std::string GenDescription = "General electronics information";
            GenericInfo->storeCollection(_runStart, _runEnd, _GenericElectronicsPar, GenDescription);
        }
        else
        {
            std::stringstream message;
            message << "The data must be decoded to obtain the information for the DB to write" << std::endl;
            throw TAltroConverterException("TAltroConverter", message.str());
        }
    }

    if(ExtendedInfo != 0 )
    {
        if(_ExtendedElectronicsPar != 0)
        {
            std::string ExtDescription = "ALTRO specific information";
            ExtendedInfo->storeCollection(_runStart, _runEnd, _ExtendedElectronicsPar, ExtDescription);
        }
        else
        {
            std::stringstream message;
            message << "The data must be decoded to obtain the information for the DB to write" << std::endl;
            throw TAltroConverterException("TAltroConverter", message.str());
        }
    }
}

lccd::LCCDTimeStamp
TAltroConverter::RunStart() const
{
    return _runStart;
}

lccd::LCCDTimeStamp
TAltroConverter::RunEnd() const
{
    return _runEnd;
}

bool
TAltroConverter::CheckIntegrity() const noexcept(false)
{
    bool result = true;

    if(_input.empty())
    {
        return false;
    }


    std::ifstream* FirstFile = _input.front();
    std::ifstream* LastFile = _input.back();
    uint32_t oldCursor = FirstFile->tellg();
    FirstFile->seekg(2 * WORD32, std::iostream::beg);
    result = result && (ReadWord32(FirstFile) == BLOCK_SOR);
    FirstFile->seekg(oldCursor, std::iostream::beg);

    oldCursor = LastFile->tellg();
    LastFile->seekg(0, std::iostream::end );
    uint32_t length = LastFile->tellg();
 
    LastFile->seekg(length - (_Factory->GetEORBlockLength()-2)*WORD32, std::iostream::beg);
    result = result && (ReadWord32(LastFile) == BLOCK_EOR);
    LastFile->seekg(oldCursor, std::iostream::beg);

    std::cout << "Input integrity check completed ";
    if(result)
    {
        std::cout << "SUCCESSFULLY" << std::endl;
    }
    else
    {
        std::cout << " UNSUCCESSFULLY" << std::endl;
    }

    return result;
}

void
TAltroConverter::Decode()
{
    std::cout << "Start decoding the input data" << std::endl;
    //Checking input and output
    if(_input.empty())
    {
        std::stringstream message;
        message << "Unable to decode because no input file streams "
                << "are associated with this object." << std::endl;
        throw TAltroConverterInvalidInputException("TAltroConverter", message.str());
    }
    if(_output == 0)
    {
        std::stringstream message;
        message << "Invalid LCWriter associated with this object" << std::endl;
        throw TAltroConverterInvalidInputException("TAltroConverterException", message.str());
    }

    //Setting the cursors at the beginning o every file
    std::vector<std::ifstream*>::iterator itFile = _input.begin();
    for(; itFile != _input.end(); ++itFile)
    {
        (*itFile)->seekg(0, std::ifstream::beg);
        (*itFile)->clear();
    }

    // Create the Run Header
    std::ifstream* CurFile = _input.front();
    TIAltroBlock* TempBlock = _Factory->GetBlock(CurFile);
    TISORBlock* SORBlock = 0;
    if(TempBlock && TempBlock->GetIdentifier() == BLOCK_SOR)
    {
        SORBlock = dynamic_cast<TISORBlock*>(TempBlock);
        TempBlock = 0;
    }
    else
    {
        delete TempBlock;
        TempBlock = 0;
        std::stringstream message;
        message << "The block at the beginning of the file is not an SOR Block" << std::endl;
        throw TAltroConverterDecodeException("TAltroConverter", message.str());
    }

    //Writes the run header to the LCIO file
    lcio::LCRunHeaderImpl* RunHeader = dynamic_cast<lcio::LCRunHeaderImpl*>(SORBlock->CreateRunHeader());
    RunHeader->setDetectorName(_DetectorName);
    RunHeader->setDescription(_RunDescription);
    _runStart = SORBlock->GetRunStart().timeStamp();
  
    //Checking the EOR. 
    CurFile = _input.back();
    CurFile->seekg(0, std::ifstream::end);
    //Length of the block depends on daq version
    int EORByteLength =_Factory->GetEORBlockLength() * WORD32;
    CurFile->seekg(-EORByteLength, std::ifstream::cur);
    TempBlock = _Factory->GetBlock(CurFile);
    TIEORBlock* EORBlock = 0;
    if(TempBlock && TempBlock->GetIdentifier() == BLOCK_EOR)
    {
        EORBlock = dynamic_cast<TIEORBlock*>(TempBlock);
        TempBlock = 0;
    }
    else
    {
        delete TempBlock;
        std::stringstream message;
        message << "The block at the end of the last file is not an EOR Block" << std::endl;
        throw TAltroConverterDecodeException("TAltroConverter", message.str());
    }
    EORBlock->SetAdditionalParameters(RunHeader);
    _runEnd = EORBlock->GetEndTime().timeStamp();

    //Writing the run header to file before writing any event toa llow for a better sequential access
    _output->writeRunHeader(RunHeader);

    //Resetting this file to the beginning
    CurFile->seekg(0, std::ifstream::beg);

    // Go through all the files
    itFile = _input.begin();
    for(; itFile != _input.end(); ++itFile)
    {
        std::cout << "Starting conversion of file " << itFile - _input.begin() << std::endl;
        CurFile = *itFile;
        while(!CurFile->eof())
        {
            try
            {
                delete TempBlock;
                TempBlock = _Factory->GetBlock(CurFile);
                if(TempBlock == 0)
                {
                    std::stringstream message;
                    message << "Conversion unsuccessful due to an unforeseen condition. "
                            <<"The Block factory returned a null pointer. This should not happen" << std::endl;
                    throw TAltroConverterException("TAltroConverter", message.str());
                }
                else if(TempBlock->GetIdentifier() == BLOCK_EVENT)
                {

                    TIEventBlock* EventBlock = dynamic_cast<TIEventBlock*>(TempBlock);
                    TempBlock = 0;
                    EventBlock->SetAdditionalParameters(RunHeader);
                    lcio::LCEventImpl* Event =
                            dynamic_cast<lcio::LCEventImpl*>(EventBlock->CreateLCEvent(_DataColName));
                    if((Event->getEventNumber() % 1000) == 0)
                    {
                        std::cout << "Decoding Event " << Event->getEventNumber() << std::endl;
                    }
                    Event->setRunNumber(SORBlock->GetRunNumber());
                    Event->setDetectorName(_DetectorName);

                    if(Event->getTimeStamp() == 0)
                    {
                        EVENT::long64 timestamp =
                                _runStart + ((_runEnd-1 - _runStart)/EORBlock->GetNEvents() * Event->getEventNumber());
                        Event->setTimeStamp(timestamp);
                    }
                    _output->writeEvent(Event);

                    if(Event->getEventNumber() == 1)
                    {
                        _GenericElectronicsPar = EventBlock->GetGeneralElectronicPar();
                        if(_GenericElectronicsPar != 0)
                        {
                            /*
                             * Zero Suppression and pedestal subtraction are reliably set in the SOR.
                             */
                            std::cout << "Generic Condition Data decoded from the data: " << std::endl;
                            int NElements = _GenericElectronicsPar->getNumberOfElements();
                            for(int idElement = 0; idElement < NElements; ++idElement)
                            {
                                tpcconddata::GenericADCElectronicsParameters* Element =
                                        dynamic_cast< tpcconddata::GenericADCElectronicsParameters*>(_GenericElectronicsPar->getElementAt(idElement));

                                Element->setPedestalSubtracted(RunHeader->parameters().getIntVal("Pedestal Subtraction"));
                                Element->setZeroSupressed(RunHeader->parameters().getIntVal("Zero Suppression"));
                                std::cout << std::endl << std::hex << *Element << std::dec << std::endl;
                            }
                        }
                        _ExtendedElectronicsPar = EventBlock->GetExtendedElectronicPar();
                        if(_ExtendedElectronicsPar != 0)
                        {
                            /*
                             * Setting the values that are store in the SOR block and the RunHeader
                             */
                            std::cout << "Extended ALTRO Parameters decoded from the data: " << std::endl;
                            int NElements = _ExtendedElectronicsPar->getNumberOfElements();
                            for(int idElement = 0; idElement < NElements; ++idElement)
                            {
                                tpcconddata::ExtendedAltroParameters* Element =
                                        dynamic_cast< tpcconddata::ExtendedAltroParameters*>(_ExtendedElectronicsPar->getElementAt(idElement));
                                tpcconddata::ExtendedAltroParameters::ExtendedAltroData Data = Element->GetData();
                                Data.PreampEnable = RunHeader->parameters().getIntVal("preampEnable");
                                Data.Gain = tpcconddata::ExtendedAltroParameters::GainSettings(RunHeader->parameters().getIntVal("Gain"));
                                Data.ShapingTime = tpcconddata::ExtendedAltroParameters::ShapingSettings(RunHeader->parameters().getIntVal("Shaper"));
                                Data.Shutdown = RunHeader->parameters().getIntVal("shutdown");
                                if(_ZSThresholdOverride < 0)
                                {
                                    if(_DataVersion.first >= 460)
                                    {
                                        Data.ZSThreshold = RunHeader->parameters().getIntVal("Zero Suppression Threshold");
                                    }
                                    else
                                    {
                                        Data.ZSThreshold = _ZSThresholdOverride;
                                    }
                                }
                                else
                                {
                                    Data.ZSThreshold = _ZSThresholdOverride;
                                }
                                Element->SetData(Data);
                                std::cout << std::endl << std::hex << *Element << std::dec << std::endl;
                            }
                        }

                    }

                    delete Event;
                    delete EventBlock;
                }
                else if(TempBlock->GetIdentifier() == BLOCK_SOR)
                {
                    continue;
                }
                else if(TempBlock->GetIdentifier() == BLOCK_EOR)
                {
                    break;
                }
                else if(TempBlock->GetIdentifier() == BLOCK_EOF)
                {
                    break;
                }
                else if(TempBlock->GetIdentifier() == BLOCK_BOF)
                {
                    TIBOFBlock* BOFBlock = dynamic_cast<TIBOFBlock*>(TempBlock);
                    TempBlock = 0;
                    if(!(BOFBlock->GetRunNumber() == SORBlock->GetRunNumber()))
                    {
                        delete BOFBlock;
                        std::stringstream message;
                        message << "The run number in the new file doesn't match with the run number in the old one"<< std::endl;
                        throw TAltroConverterDecodeException("TAltroConverter", message.str());
                    }
                    delete BOFBlock;
                }
                else if(TempBlock->GetIdentifier() == BLOCK_POR)
                {
                    continue;
                }
                else if(TempBlock->GetIdentifier() == BLOCK_COR)
                {
                    continue;
                }
            }
            catch(TAltroConverterException &e)
            {
                delete TempBlock;
                TempBlock = 0;
                std::cout<< e.what();
            }
        }
    }

    delete RunHeader;
    std::cout << "Run Header created and written to file" << std::endl;

    delete TempBlock;
    delete SORBlock;
    delete EORBlock;
}

void
TAltroConverter::Print(std::ostream& os) const
{
    os << "Printing the converter status" << std::endl;

    std::pair<int,int> Converter = ConverterVersion();
    os << "Converter version: " << Converter.first << "." << Converter.second << std::endl;

    os << "Data version: " << _DataVersion.first << "." << _DataVersion.second << std::endl;

}

std::pair<int, int>
TAltroConverter::DataVersion() const
{
    if(_input.empty())
    {
        std::stringstream message;
        message << "Unable to detect the data format because no input file streams "
                << "are associated with this object." << std::endl;
        throw TAltroConverterInvalidInputException("TAltroConverter", message.str());
    }

    std::ifstream* FirstFile = _input.front();
    FirstFile->clear();
    uint32_t oldCursor = FirstFile->tellg();
    FirstFile->seekg(3 * WORD32, std::iostream::beg);
    int DAQVer = ReadWord16(FirstFile);
    int DataFormat = ReadWord16(FirstFile);
    std::cout << "Detected data stream with DAQ Version: " << DAQVer
            << " and Data Format " << DataFormat << std::endl;

    FirstFile->seekg(oldCursor, std::iostream::beg);

    return std::make_pair(DAQVer, DataFormat);
}

std::pair<int, int>
TAltroConverter::ConverterVersion() const
{
    return std::make_pair(3, 0);
}

void
TAltroConverter::Reset()
{
    _input = std::vector<std::ifstream*>();
    _output = NULL;
    _GenericParColName = "GenericADCElectronicsParameters";
    _ExtendedParColName = "ExtendedAltroElectronicsParameters";
    _runStart = 0;
    _runEnd = 0;
    _DataVersion = std::make_pair(0,0);
    if(_Factory != 0)
    {
        delete _Factory;
        _Factory = 0;
    }
    _DetectorName = "LPTPC";
    _RunDescription = "Test beam run";
    _DataColName = "AltroRawData";
    if(_GenericElectronicsPar != 0)
    {
        delete _GenericElectronicsPar;
        _GenericElectronicsPar = 0;
    }
    if(_ExtendedElectronicsPar != 0)
    {
        delete _ExtendedElectronicsPar;
        _ExtendedElectronicsPar = 0;
    }
    _ZSThresholdOverride = -1;


}

void
TAltroConverter::SetDetectorName(std::string Name)
{
    _DetectorName = Name;
}

void
TAltroConverter::SetRunDescription(std::string Description)
{
    _RunDescription = Description;
}

void
TAltroConverter::SetDataCollectionName(std::string Name)
{
    _DataColName = Name;
}

void
TAltroConverter::SetZSThresholdOverride(int Threshold)
{
    _ZSThresholdOverride = Threshold;
}
} //End namespace


