/*
 * Altro2LcioConverter.hh
 *
 *  Created on: Jul 14, 2011
 *      Author: caiazza
 */

/**
 * The goal of this program is to convert the raw data file output by the ALTRO electronics setup
 * in the LCIO format storing data and metadata from the former to the latter to be used
 * by the reconstruction and analysis program.
 * This program improves the altro2lcio program removing the dependencies from the eudaq package,
 * including in the LCIO file most of the metadata present in the raw data file and previously ignored.
 * The program will also include some preview feature of the raw data to convert
 */

#ifndef ALTRO2LCIOCONVERTER_HH_
#define ALTRO2LCIOCONVERTER_HH_

//#define NDEBUG
#include <assert.h>

//Standard C++ inclusion
#include <iostream>
#include <fstream>
#include <iterator>
#include <vector>
#include <exception>
#include <stdexcept>
#include <string.h>
#include <sstream>

//ILCSOFT inclusions
#include "lcio.h"
#include "UTIL/LCTime.h"
#include "IO/LCWriter.h"
#include "UTIL/LCSplitWriter.h"
#include "EVENT/LCRunHeader.h"

//Altro Block Library
#include "AltroBlocksLib.h"

namespace A2Lconverter
{
/// Program constants
const uint32_t WORD32 = 4;    ///<-Length in bytes of a 32 bit Data Word

/**
 * Data structure encapsulating the different program options to evaluate to change the operational mode of the converter
 */
struct ProgramOptions
{
    bool printRunHeader;    //Set true to just visualize the run header info
    bool printeXtensiveInfo;    //Set true to just visualize extensive run info
    int timestampmode;    //See usage for available modes of timestamping events
    int whatEventPrint;    //Set to -1 for standard conversion. With valid numbers it dumps event data to the output stream
    bool doConvert;    // Set true to perform the actual data conversion
    std::string baseFileName;          //Contains the input file name
    std::string outputFileName;         //Contains the output file name
    int lastFileNumber;        //Contains the filenumber of the last file opened
};

/**
 * Check that there are no error in the command line.
 * In case of error it throws a BadCommandLineException
 */
void
checkCommandLine(int argc, char* argv[], ProgramOptions* theOptions) throw (std::runtime_error);

/**
 * Prints on the error stream the command line instructions to launch the application
 */
void
printusage(char* exeName);

/**
 * Generate the outputfilename appending .slcio to the end of the file name obtained from command line
 * if this is not already ending with .slcio
 */
void
GenerateOutFileName(ProgramOptions & theOptions);

/**
 * Delete all the pointers in the vector
 */
void
vectorDelete(std::vector<std::ifstream*> theVector);

/**
 * Converter Exceptions
 */
class BadBlockException : public std::runtime_error
{
public:
    BadBlockException()
            : runtime_error("ERROR: Incorrect block or unable to decode")
    {
    }
};

class BadCommandLineException : public std::runtime_error
{
public:
    BadCommandLineException()
            : std::runtime_error("ERROR: Something is missing or is wrong in the command line. \n")
    {
    }

    BadCommandLineException(std::string error)
            : std::runtime_error(error)
    {
    }
};

}

#endif /* ALTRO2LCIOCONVERTER_HH_ */
