/*
 * RCUBlock.hh
 *
 *  Created on: Jul 27, 2011
 *      Author: caiazza
 */

#ifndef RCUBLOCK_HH_
#define RCUBLOCK_HH_

#include "AltroBlocksLib.h"
#include "IMPL/LCFlagImpl.h"
#include "EVENT/LCIntVec.h"

namespace A2Lconverter
{

class RCUBlock : public AltroBlock
{

public:
    /**
     * From stream constructor that works as a default as well
     */
    RCUBlock(std::ifstream* const inputFile = 0);
    RCUBlock(std::ifstream* const inputFile, int start_address,
            uint32_t length = 0);

    /**
     * Construct the derived class from an instance of the mother class
     */
    RCUBlock(const AltroBlock & theMotherBlock);

    /**
     * Construct the class from the data vector
     * This constructor makes a copy of the input vector inside of the member vector
     */
    RCUBlock(const std::vector<uint32_t> & refContent);

    /**
     * Construct the class from the data vector specifying the starting point of the
     * relevant section of the data. The function will identify the correct length of
     * the data vector and copy in the member vector containing the data only the
     * the relevant part
     */
    RCUBlock(const std::vector<uint32_t>::const_iterator refStart);

    /**
     * Destructor
     */
    virtual
    ~RCUBlock()
    {
    }

    /**
     * Derived class implementation of the decode function
     */
    virtual void
    Decode();

    /**
     * Derived class implementation of the writer function
     */
    virtual void
    Write2LCIO(IO::LCWriter* lcwriter) const
    {
    }

    /**
     * Derived class implementation of the printer function
     */
    virtual std::string
    PrintSummary() const;

    /**
     * Derived class implementation of the Check Integrity function
     */
    virtual bool
    CheckIntegrity() const;

    /**
     * Add to the event all the collections that the RCU creates:
     * RCUCollection containing the error registers value
     * Data collection containing the actual ADC values
     */

    void
    AddCollections(EVENT::LCEvent* lcevent) const;

protected:

    /**
     * Add a new collection containing the RCU error registers. If the collection already exists in case of
     * multiple RCUs it will append the data to the existing collection
     */
    void
    AddRCUCollection(EVENT::LCCollection* altrocollection) const;

    /**
     * Add a new collection containing the actual ADC data.If the collection already exists in case of
     * multiple RCUs it will append the data to the existing collection
     * The RCUNum is the value that will be fed in the CellID1 of the TrackerRawData object
     */
    void
    AddDataCollection(EVENT::LCCollection* altrocollection, int RCUNum) const;

    uint32_t RCUID;
    uint32_t StatusErrors;    //The Status & Error field of the RCU header extracted from the dtata word
    struct RCUTrailer
    {
        uint32_t PayloadLength;
        uint32_t ErrorReg1;
        uint32_t ErrorReg2;
        uint32_t ErrorReg3;
        uint32_t ActFECA;
        uint32_t ActFECB;
        uint32_t RDOConfig1;
        uint32_t RDOConfig2;
        uint32_t RCUFirmware;
        uint32_t RCUAddress;
        uint32_t TrailerLength;
    } trailer;

    /**
     * This struct contains the mask used to extract the identification codes
     * of the Altro Trailer Words.
     * The two constant objects of type Trailer Masks contains the different
     * definition of this masks in the different daq versions
     */
    struct TrailerMasks
    {
        uint32_t WORDCODEMASK;
        uint32_t PAYLOADMASK;
        uint32_t ERRORREG1MASK;
        uint32_t ERRORREG2MASK;
        uint32_t ERRORREG3MASK;
        uint32_t ACTFECAMASK;
        uint32_t ACTFECBMASK;
        uint32_t RDOCONFIG1MASK;
        uint32_t RDOCONFIG2MASK;
        uint32_t RCUIDMASK;
    };

    static const TrailerMasks V410MASKS;
    static const TrailerMasks V420MASKS;
    static TrailerMasks const * Masks;

    static const std::string RCUCOLLECTIONNAME;
    static const std::string DATACOLLECTIONNAME;
};
}

#endif /* RCUBLOCK_HH_ */









