/*
 * SORBlock.hh
 *
 *  Created on: Jul 21, 2011
 *      Author: caiazza
 */

#ifndef SORBLOCK_HH_
#define SORBLOCK_HH_

#include "AltroBlocksLib.h"
#include "IMPL/LCRunHeaderImpl.h"
#include "IMPL/LCParametersImpl.h"

namespace A2Lconverter
{
class SORBlock : public AltroBlock
{

public:
    /**
     * From stream constructor that works as a default as well
     */
    SORBlock(std::ifstream* const inputFile = 0);
    SORBlock(std::ifstream* const inputFile, int start_address,
            uint32_t length = 0);

    /**
     * Construct the derived class from an instance of the mother class
     */
    SORBlock(const AltroBlock & theMotherBlock);

    /**
     * Destructor
     */
    virtual ~SORBlock(){};

    /**
     * Construct the class from the data vector
     */
    SORBlock(const std::vector<uint32_t> & refContent);

    /**
     * Derived class implementation of the decode function
     */
    virtual void Decode();

    /**
     * Derived class implementation of the writer function
     */
    virtual void
    Write2LCIO(IO::LCWriter* lcwriter) const;

    /**
     * Derived class implementation of the printer function
     */
    virtual std::string
    PrintSummary() const;

    /**
     * Derived class implementation of the Check Integrity function
     */
    virtual bool
    CheckIntegrity() const;

    /**
     * Get Functions
     */
    static const uint32_t GetRunNumber();
    uint16_t GetPCA16DAC() const;


protected:
    static uint32_t runNumber;
    uint16_t PCA16DAC;
    struct PCA16Registers
    {
        bool preampEnable;
        enum gainSettings {MV12 = 0, MV15 = 1, MV19 = 2, MV27 = 3} gain;
        enum shaperSettings {MS30 = 7, MS60 = 3, MS90 = 1, MS120 = 0} shaper;
        bool shutdown;
        bool polarity;
    } registers;

    struct RunSettings
    {
        bool zeroSuppression;
        bool pedestalSubtraction;
        short overThreshold;
        short threshold;
        short postsamples;
        short presamples;
        short pretrigger;
    } runSets;

    uint32_t EUDAQRunNumber;

};
}

#endif /* SORBLOCK_HH_ */
