/*
 * BOFBlock.cpp
 *
 *  Created on: Jul 26, 2011
 *      Author: caiazza
 */

#include "BOFBlock.hh"
#include "AltroBlocksLib.h"

namespace A2Lconverter
{
BOFBlock::BOFBlock(std::ifstream * const inputFile)
        : AltroBlock(inputFile)
{

}
BOFBlock::BOFBlock(std::ifstream* const inputFile, int start_address,
        uint32_t length)
        : AltroBlock(inputFile, start_address, length)
{

}

BOFBlock::BOFBlock(const AltroBlock & theMotherBlock)
        : AltroBlock(theMotherBlock)
{

    if (this->CheckIntegrity())
    {
        id = AltroBlock::BLOCK_BOF;
    }
    else
    {
        throw AltroBlockException(
                "The Altro Block copied in the BOF block doesn't appear to be a valid BOF block");
    }
}

BOFBlock::BOFBlock(const std::vector<uint32_t> & refContent)
        : AltroBlock(refContent)
{

}

void
BOFBlock::Decode()
{
    try
    {
        AltroBlock::Decode();
        if (content[0] != content[1] + 1)
            throw AltroBlockException(
                    "The total length doesn't match the header length +1");
        if (content[2] != AltroBlock::BLOCK_BOF)
            throw AltroBlockException(
                    "The block identification doesn't match with BLOCK_BOF");
        id = AltroBlock::BlockIds(content[2]);
        if (content[3] != AltroBlock::dataformat)
        {
            std::ostringstream oss;
            oss << "ERROR: the dataformat in the Beginning of File block "
                    << content[3] << " don't match the Start of Run dataformat"
                    << AltroBlock::dataformat << std::endl;
        }
        runNumber = content[4];
        int day = content[5] & 0x000000ff;
        int month = (content[5] & 0x0000ff00) >> 8;
        int year = (content[5] & 0xffff0000) >> 16;
        int second = content[6] & 0x000000ff;
        int minute = (content[6] & 0x0000ff00) >> 8;
        int hour = (content[6] & 0xffff0000) >> 16;
        timestamp = UTIL::LCTime(year, month, day, hour, minute, second);
        fileNumber = content[7];
        lastEventN = content[8];
    }
    catch (std::ifstream::failure &f)
    {
        std::ostringstream err;
        err
                << "ERROR: Exception caught while decoding the BOF block. The file ended before it was supposed to"
                << std::endl;
        err << f.what() << std::endl;
        std::string tmp = err.str();
        throw AltroBlockException(tmp);
    }
    catch (std::exception &e)
    {
        std::ostringstream err;
        err << "ERROR: Exception caught while decoding the BOF block. "
                << std::endl;
        err << e.what() << std::endl;
        std::string tmp = err.str();
        throw AltroBlockException(tmp);
    }
}

std::string
BOFBlock::PrintSummary() const
{
    std::ostringstream oss;
    oss << std::dec << "Beginning of File summary" << std::endl << std::endl;
    oss << "Run Number " << runNumber << std::endl;
    oss << "Starting File Number " << fileNumber << " after event " << lastEventN << std::endl << std::endl;
    return oss.str();
}

bool
BOFBlock::CheckIntegrity() const
{
    if (AltroBlock::CheckIntegrity())
    {
        if (content.size() == 0)
        {
            inputFile->seekg(start, std::ios::beg);
            if (static_cast<int>(inputFile->tellg()) != 0)
            {
                throw AltroBlockException(
                        "ERROR: The BOF file is not starting at the beginning of a file");
                return false;
            }
            inputFile->seekg(start + 2 * WORD32, std::ios::beg);
            return (BOFBlock::ReadWord32() == AltroBlock::BLOCK_BOF);
        }
        else
        {
            return (content[2] == AltroBlock::BLOCK_BOF);
        }
    }
    return false;
}

}

