/*
 * CORBlock.cpp
 *
 *  Created on: Jul 26, 2011
 *      Author: caiazza
 */

#include "CORBlock.hh"
#include "AltroBlocksLib.h"

namespace A2Lconverter
{
CORBlock::CORBlock(std::ifstream * const inputFile)
        : AltroBlock(inputFile)
{

}
CORBlock::CORBlock(std::ifstream* const inputFile, int start_address,
        uint32_t length)
        : AltroBlock(inputFile, start_address, length)
{

}

CORBlock::CORBlock(const AltroBlock & theMotherBlock)
        : AltroBlock(theMotherBlock)
{

    if (this->CheckIntegrity())
    {
        id = AltroBlock::BLOCK_COR;
    }
    else
    {
        throw AltroBlockException(
                "The Altro Block copied in the COR block doesn't appear to be a valid COR block");
    }
}

CORBlock::CORBlock(const std::vector<uint32_t> & refContent)
        : AltroBlock(refContent)
{

}

void
CORBlock::Decode()
{
    try
    {
        AltroBlock::Decode();
        if (content[0] != content[1] + 1)
            throw AltroBlockException(
                    "The total length doesn't match the header length +1");
        if (content[2] != AltroBlock::BLOCK_COR)
            throw AltroBlockException(
                    "The block identification doesn't match with BLOCK_COR");
        id = AltroBlock::BlockIds(content[2]);
        lastEventN= content[3];
        int day = content[4] & 0x000000ff;
        int month = (content[4] & 0x0000ff00) >> 8;
        int year = (content[4] & 0xffff0000) >> 16;
        int second = content[5] & 0x000000ff;
        int minute = (content[5] & 0x0000ff00) >> 8;
        int hour = (content[5] & 0xffff0000) >> 16;
        timestamp = UTIL::LCTime(year, month, day, hour, minute, second);
    }
    catch (std::ifstream::failure &f)
    {
        std::ostringstream err;
        err
                << "ERROR: Exception caught while decoding the COR block. The file ended before it was supposed to"
                << std::endl;
        err << f.what() << std::endl;
        std::string tmp = err.str();
        throw AltroBlockException(tmp);
    }
    catch (std::exception &e)
    {
        std::ostringstream err;
        err << "ERROR: Exception caught while decoding the COR block. "
                << std::endl;
        err << e.what() << std::endl;
        std::string tmp = err.str();
        throw AltroBlockException(tmp);
    }
}

std::string
CORBlock::PrintSummary() const
{
    std::ostringstream oss;
    oss << std::dec << "Run Paused after event" << lastEventN << std::endl << std::endl;
    return oss.str();
}

bool
CORBlock::CheckIntegrity() const
{
    if (AltroBlock::CheckIntegrity())
    {
        if (content.size() == 0)
        {
            inputFile->seekg(start + 2 * WORD32, std::ios::beg);
            return (ReadWord32() == AltroBlock::BLOCK_COR);
        }
        else
        {
            return (content[2] == AltroBlock::BLOCK_COR);
        }
    }
    return false;
}

}



