/*
 * DataBlock.cpp
 *
 *  Created on: Aug 1, 2011
 *      Author: caiazza
 */

#include "DataBlock.hh"
#include "AltroBlocksLib.h"

namespace A2Lconverter
{
//TODO Implement data format 0 handling

const DataBlock::DataMasks DataBlock::DF1MASKS = { 0xC0000000, 0x20000000, 0x1FFFFFFF, 0x0003FF0000,
        0x00000FFF, 0x000003FF, 0x000FFC00, 0x3FF00000 };

DataBlock::DataMasks const * DataBlock::Masks = 0;

//TODO Remove useless contructors
DataBlock::DataBlock(std::ifstream * const inputFile)
        : AltroBlock(inputFile)
{
    if (dataformat == 1)
    {
        if (Masks == 0)
            Masks = &DF1MASKS;
    }
    else
    {
        throw AltroBlockException(
                "ERROR: ALTRO Data format 0 currently not handled by this converter. Please use the previous version");
    }
}

DataBlock::DataBlock(std::ifstream* const inputFile, int start_address, uint32_t length)
        : AltroBlock(inputFile, start_address, length)
{
    if (this->CheckIntegrity())
    {
        id = AltroBlock::BLOCK_DATA;
    }
    else
    {
        throw AltroBlockException(
                "The Altro Block copied in the DATA block doesn't appear to be a valid DATA block");
    }
}

DataBlock::DataBlock(const AltroBlock & theMotherBlock)
        : AltroBlock(theMotherBlock)
{
    if (this->CheckIntegrity())
    {
        id = AltroBlock::BLOCK_DATA;
    }
    else
    {
        throw AltroBlockException(
                "The Altro Block copied in the DATA block doesn't appear to be a valid DATA block");
    }
}

DataBlock::DataBlock(const std::vector<uint32_t> & refContent)
        : AltroBlock(refContent)
{
    length = content.size();
    refStart = content.begin();
    if (this->CheckIntegrity())
    {
        id = AltroBlock::BLOCK_DATA;
    }
    else
    {
        throw AltroBlockException(
                "The Altro Block copied in the DATA block doesn't appear to be a valid DATA block");
    }
}

DataBlock::DataBlock(const std::vector<uint32_t>::const_iterator readStart, uint32_t payload)
        : AltroBlock(), refStart(readStart)
{
    length = payload;
    if (this->CheckIntegrity())
    {
        id = AltroBlock::BLOCK_DATA;
    }
    else
    {
        throw AltroBlockException(
                "The Altro Block copied in the DATA block doesn't appear to be a valid DATA block");
    }
}

void
DataBlock::Decode()
{

}

std::string
DataBlock::PrintSummary() const
{
    std::ostringstream oss;
    oss << std::dec << "Data Block summary" << std::endl << std::endl;
    return oss.str();
}

bool
DataBlock::CheckIntegrity() const
{
    if (inputFile != 0 || content.size() != 0)
        AltroBlock::CheckIntegrity();
    if (dataformat == 1)
    {
        if (Masks == 0)
            Masks = &DF1MASKS;
        return true;
    }
    else
    {
        throw AltroBlockException(
                "ERROR: ALTRO Data format 0 currently not handled by this converter. Please use the previous version");
    }
    return false;
}

void
DataBlock::AddDataCollection(EVENT::LCCollection* altrocollection, int RCUID) const
{
    std::vector<uint32_t>::const_iterator decodePosition = refStart;

    switch (AltroBlock::dataformat)
    {
        case 1:
            while (((*decodePosition) & (Masks->WORDID)) == 0x40000000)
            {
                DecodeDF1(decodePosition, RCUID, altrocollection);
            }
            break;
        default:
            throw AltroBlockException(
                    "ERROR: ALTRO Data format 0 currently not handled by this converter. Please use the previous version");
    }

}

void
DataBlock::DecodeDF1(std::vector<uint32_t>::const_iterator& decodePosition, int RCUID,
        EVENT::LCCollection* altrocollection) const
{
    uint32_t chId = (*decodePosition) & (Masks->CHANNELADDRESS);

    //Read from the data the number of altro words in the next data block and moves to the next element in the data vector
    uint32_t words10bit = (*decodePosition & (Masks->DWNUM)) >> 16;
    decodePosition++;

    //creates a vector to be filled with the 10bit data words of the altro data format and reserves enough space
    std::vector<short> sampleList;
    sampleList.reserve((words10bit));

    //Now I fill the sample list with the data words
    for (uint32_t i = 0; i < words10bit; i++)
    {
        int wordPos = i % 3;
        switch (wordPos)
        {
            case 0:
                sampleList.push_back(((*decodePosition) & Masks->DW3) >> 20);
                break;
            case 1:
                sampleList.push_back(((*decodePosition) & Masks->DW2) >> 10);
                break;
            case 2:
                sampleList.push_back((*decodePosition) & Masks->DW1);
                decodePosition++;
                break;
        }
    }
    if ((words10bit % 3) != 0)
        decodePosition++;

    //Now I will decode the different types of data words and I will fill them in the LCIO collection
    std::vector<short>::const_iterator it = sampleList.begin();
    while (it != sampleList.end())
    {
        //Create the Tracker raw data. It will be destroyed by the LCIO framework.
        IMPL::TrackerRawDataImpl* Pulse = new IMPL::TrackerRawDataImpl();
        //Fill it with the time-space coordinates of the adc sample
        Pulse->setCellID1(RCUID);
        Pulse->setCellID0(chId);

        //store the adc values
        short sampleLength = *it;

        //The timestamp in the LCIO format is the time of the first sample while in the
        //AltroDataFormat is the time of the last of the samples. Thus we need to perform a conversion
        Pulse->setTime((*(it + 1)) - (sampleLength -1));

        std::vector<short> adcValues;
        adcValues.reserve(sampleLength - 2);

        //Moving the iterator to the beginning of the next block
        it += sampleLength;
        //retrieving the samples backwords
        for (int i = 1; i <= (sampleLength - 2); i++)
        {
            adcValues.push_back(*(it - i));
        }
        Pulse->setADCValues(adcValues);
        altrocollection->addElement(Pulse);
    }

}

}
