#####################################
# cmake module for finding Mokka
#
# returns:
#   Mokka_FOUND
#   Mokka_INCLUDE_DIRS
#   Mokka_LIBRARIES
#   Mokka_CGA_FOUND
#
# @author Jan Engels, DESY
#####################################

SET( Mokka_FOUND FALSE )
SET( Mokka_CGA_FOUND FALSE )
MARK_AS_ADVANCED( Mokka_FOUND Mokka_CGA_FOUND )

#INCLUDE( FindGeant4.cmake )

SET( Mokka_INC_DIRS Mokka_INC_DIRS-NOTFOUND )
SET( Mokka_LIBS Mokka_LIBS-NOTFOUND )
SET( Mokka_CGA_LIB Mokka_CGA_LIB-NOTFOUND )
MARK_AS_ADVANCED ( Mokka_INC_DIRS Mokka_LIBS Mokka_CGA_LIB )

# include dirs
IF( EXISTS "${Mokka_HOME}" )
    
    EXECUTE_PROCESS(
        COMMAND find ${Mokka_HOME}/source -name include -type d -printf "%p;"
        OUTPUT_VARIABLE Mokka_INC_DIRS
        RESULT_VARIABLE exit_code
    )
        
    IF( NOT exit_code EQUAL 0 )
        MESSAGE( STATUS "error occurred trying to locate Mokka include dirs with find" )
        SET( Mokka_INC_DIRS Mokka_INC_DIRS-NOTFOUND )
    ENDIF()

    # libraries
    FIND_LIBRARY( Mokka_CGA_LIB
        NAMES CGAPack
        PATHS "${Mokka_HOME}" $ENV{G4LIB}/$ENV{G4SYSTEM}/
        PATH_SUFFIXES lib lib/Linux-g++
        NO_DEFAULT_PATH
    )

    IF( Mokka_CGA_LIB )
        SET( Mokka_CGA_FOUND TRUE )
        SET( Mokka_LIBS ${Mokka_CGA_LIB} )
    ELSE()
        FILE( GLOB_RECURSE Mokka_LIBS "${Mokka_HOME}/tmp/*.so" )
        IF( NOT Mokka_LIBS )
            FILE( GLOB_RECURSE Mokka_LIBS "${Mokka_HOME}/tmp/*.a" )
        ENDIF()
    ENDIF()

    IF( NOT Mokka_FIND_QUIETLY )
        IF( Mokka_CGA_FOUND )
            MESSAGE( STATUS "Check for Mokka CGAPack -- works" )
        ELSE()
            MESSAGE( STATUS "Check for Mokka CGAPack -- failed to find libCGAPack. Did you remember to compile Mokka with \"make MOKKA_PACK_LIBS=1\"?" )
        ENDIF()
    ENDIF()

ENDIF()

# debug
#MESSAGE( STATUS "Mokka auto-detected include dirs: ${Mokka_INC_DIRS}" )
#MESSAGE( STATUS "Mokka auto-detected libraries: ${Mokka_LIBS}" )

# set variables and display results
IF( Mokka_INC_DIRS AND Mokka_LIBS )
    SET( Mokka_FOUND TRUE )
    SET( Mokka_INCLUDE_DIRS ${Mokka_INC_DIRS} )
    SET( Mokka_LIBRARIES ${Mokka_LIBS} )
    IF( Mokka_CGA_FOUND )
        SET( Mokka_DEFINITIONS "-DUSE_CGA" )
        MARK_AS_ADVANCED( Mokka_DEFINITIONS )
    ENDIF()

    MARK_AS_ADVANCED( Mokka_INCLUDE_DIRS Mokka_LIBRARIES )
    MESSAGE( STATUS "Check for Mokka: ${Mokka_HOME} -- works" )
ELSE()
    IF( Mokka_FIND_REQUIRED )
        MESSAGE( FATAL_ERROR "Check for Mokka: ${Mokka_HOME} -- failed!!" )
    ENDIF()
    IF( NOT Mokka_FIND_QUIETLY )
        MESSAGE( STATUS "Check for Mokka: ${Mokka_HOME}"
                " -- failed!! will skip this package..." )
    ENDIF()
ENDIF()

