#ifndef BFIELDMAPLISTENER_H
#define BFIELDMAPLISTENER_H

//C++
#include <iostream>

//LCIO
#include "lcio.h"
#include "EVENT/LCCollection.h"
#include "IMPL/LCCollectionVec.h"
#include "UTIL/LCTime.h"

//LCCD
#include "lccd.h"
#include "lccd/DBInterface.hh"
#include "lccd/IConditionsChangeListener.hh"
#include "lccd/IConditionsHandler.hh"

//Marlin
#include "marlin/ConditionsProcessor.h"
#include "Cylindrical2DMagneticFieldMap.h"

namespace marlintpc
{

    /** listener class derived from LCCD::IConditionsChangeListener
     * accesses the bfield map object
     *
     * @author Ch. Rosemann (DESY)
     */

    class BFieldMapListener : public lccd::IConditionsChangeListener
    {
        public:
            /** Constructor
             */
            BFieldMapListener(std::string collectionName, double, double, double, double, double);

            /// Destructor
            virtual ~BFieldMapListener();

            /// Updates values if conditions have changed
            /** Called by the Marlin ConditionsProcessor, if the condition change
             *  inplemetentation of the IConditionsChangeListener interface
             */
            void conditionsChanged(EVENT::LCCollection* col);

            /// Implementation of the listener class, takes care of default collection settings and registration of the handler.
            void registeredWithHandler(lccd::IConditionsHandler* condHandler);

            /// Implementation of the listener class, cleans up after deregistration and gives warning message.
            void deRegisteredWithHandler(lccd::IConditionsHandler* condHandler);

            /// get the BFieldMap
            gear::BField* getBFieldMap() const
            {
                return _theFieldMap;
            } ;

        private:
            /// one of the most important parts: the name of the collection
            std::string _collectionName;

            /// Default collection that is registered and will be returned from the handler if no data is available.
            lcio::LCCollectionVec* _thisEmptyCollection;

            /// Map of registered default collections for differnet handlers. Will hold here only one entry, since this listener registers itself with only one handler.
            std::map<lcio::LCCollection* , lccd::IConditionsHandler* > _handlerDefaultCollectionMap;

            /// Pointer to hold the address of the handler this listener is registered with.
            lccd::IConditionsHandler* _condHandler;

            /// the wrapper for the actual field map implementation
            Cylindrical2DMagneticFieldMap* _theFieldMap;
            double _coilPositionX;
            double _coilPositionY;
            double _coilPositionZ;
            double _coilRotationX;
            double _coilRotationY;
    };
}
#endif // BFIELDMAPLISTENER_H
