#ifndef CONSTANTUNIDIRECTIONALMAGNETICFIELD_H
#define CONSTANTUNIDIRECTIONALMAGNETICFIELD_H 1

// stl
#include <string>
#include <vector>

// CLHEP

// MarlinTPC
#include "Field.h"

namespace EVENT {

	class LCCollection;
}

namespace marlintpc {

/** ConstantUnidirectionalMagneticField : A constant magnetic field
 *
 *    @author J. Abernathy (UVic)<br>
 */
class ConstantUnidirectionalMagneticField: public Field {

public:
	/** constructor
	 */
	ConstantUnidirectionalMagneticField();

	/** destructor
	 */
	virtual ~ConstantUnidirectionalMagneticField();

	/** returns a new field... similar to how the Marlin processor registration works
	 */
	virtual Field *new_field( void );

	/** constructs the field from an LCCollection of data objects (could be anything... field bin values or just parameters)
	 */
	virtual bool construct( EVENT::LCCollection * );

	/** gets the field value at a specified point
	 */
	CLHEP::Hep3Vector get_field_value( const CLHEP::Hep3Vector & );

	/** returns the typename of the field
	 */
	std::string get_typename();

	struct FieldMapEntry {

		double _z_min;
		double _z_max;
		double _r_min;
		double _r_max;

		double _field_x;
		double _field_y;
		double _field_z;
	};

protected:

private:

	/** the entries of the field map (the field vector at different positions)
	 */
	std::vector<FieldMapEntry> _field_map;
};

} // namespace marlintpc

#endif // CONSTANTUNIDIRECTIONALMAGNETICFIELD_H
