/*
 * TrackCircleIntersectionCalculator.cpp
 *
 *  Created on: April 22, 2013
 *      Author: iheinze, amunnich
 */


#include "TrackCircleIntersectionCalculator.hh"
#include <cmath>

namespace marlintpc
{
  TrackCircleIntersectionCalculator::TrackCircleIntersectionCalculator(std::pair<double,double> circlecenter, 
									   double radius, 
									   double d0, 
									   double phi, 
									   double omega):
    _CircleCenter(circlecenter),
    _Radius(radius),
    _D0(d0),
    _Phi0(phi),
    _Omega(omega)
  {
    CalculateIntersection();
  }

  TrackCircleIntersectionCalculator::~TrackCircleIntersectionCalculator(){}

  void TrackCircleIntersectionCalculator::CalculateIntersection()
  {
    //calculate pca
    double xpca = -_D0*sin(_Phi0);
    double ypca = _D0*cos(_Phi0);

    double phir = atan2(sin(_Phi0) - _Omega*(_CircleCenter.first - xpca) , cos(_Phi0) + _Omega*(_CircleCenter.second - ypca));
    double gamma = (2*(_CircleCenter.first-xpca)*sin(_Phi0) - 2*(_CircleCenter.second-ypca)*cos(_Phi0) - _Omega*_Radius*_Radius - _Omega*( (_CircleCenter.first-xpca)*(_CircleCenter.first-xpca) + (_CircleCenter.second-ypca)*_CircleCenter.second-ypca ))/(2*_Radius*sqrt((sin(_Phi0) - _Omega*(_CircleCenter.first-xpca))*(sin(_Phi0) - _Omega*(_CircleCenter.first-xpca)) + (cos(_Phi0) + _Omega*(_CircleCenter.second-ypca))*(cos(_Phi0) + _Omega*(_CircleCenter.second-ypca))));
    //this decides to take the intersection with positive arc length
    double phic = phir + asin(gamma);

    _Intersection.first = _CircleCenter.first + _Radius*cos(phic);
    _Intersection.second = _CircleCenter.second + _Radius*sin(phic);
  }

}
