#ifndef LOADFIELDFROMCONDITIONSDATAPROCESSOR_H
#define LOADFIELDFROMCONDITIONSDATAPROCESSOR_H

// LCIO
#include "lcio.h"

// Marlin
#include "marlin/Processor.h"

// MarlinTPC
#include "BField.h"
#include "BFieldMapListener.h"

// C++
#include <string>

namespace marlintpc
{

    /** Processor to load the field map from conditions data into GEAR
     *
     *  The map is provided via LCCD (simple file, database file or database) in the conditions processor.
     *  Here the map object is first translated into a field map, which implements the BField from gear.
     *
     *  <h4>Input - Prerequisites</h4>
     *  Name of the BField map collection and the coil position collection as loaded in the conditions processor
     ***/
    /**
    *
    * @author Ch. Rosemann, DESY
    */

    class LoadFieldFromConditionsDataProcessor : public marlin::Processor
    {

        public:

            virtual Processor*  newProcessor()
            {
                return new LoadFieldFromConditionsDataProcessor ;
            }

            LoadFieldFromConditionsDataProcessor() ;

            virtual void init() ;

            virtual void processRunHeader(lcio::LCRunHeader* run) ;

            virtual void processEvent(lcio::LCEvent * evt) ;

            virtual void check(lcio::LCEvent * evt) ;

            virtual void end() ;


        protected:

            /// the name of the bfield map collection
            std::string _bFieldColName ;
            /// the translation and rotation in terms of the coil position -- LP and coil have different origins
            double _coilPositionX;
            double _coilPositionY;
            double _coilPositionZ;
            double _coilRotationX;
            double _coilRotationY;

        private:
            BFieldMapListener* _bFieldListener;
    } ;
}
#endif // LOADFIELDFROMCONDITIONSDATAPROCESSOR_H
