#ifndef TESTTPCDISTORTIONSMAPPROCESSOR_H
#define TESTTPCDISTORTIONSMAPPROCESSOR_H

#include "TPCDistortionsMapListener.h"

#include <marlin/Processor.h>
#include <lcio.h>
#include <string>

namespace marlintpc
{
/**  Processor skeleton for MARLIN.
 * 
 * @author Ch. Rosemann, DESY
 * @author M. Killenberg, CERN
 *
 * Concrete implementation:
 * @author K. Zenker, DESY
 *
 * This processor can be used to visualize the field distortions measured
 * with system of photodots on the TPC cathode. They are stored in a conditions object
 * (TPCDistortionMeasurement) in the data base.
 * Up to now arrows are drawn that point from the true photodot position on the
 * cathode to the reconstructed position on the pad plane.
 */
class TestTPCDistortionsMapProcessor : public marlin::Processor
{
 public:
  
  virtual Processor*  newProcessor() { return new TestTPCDistortionsMapProcessor; }
  
  TestTPCDistortionsMapProcessor();
  
  virtual void init();
  
  virtual void processRunHeader(lcio::LCRunHeader* run);
  
  virtual void processEvent(lcio::LCEvent* evt);
  
  virtual void check(lcio::LCEvent* evt);
  
  virtual void end();
 private:
  bool _draw; ///< Used to draw everything only for one event.
  double _scale; ///< Can be used to scale the length of the arrow for better visibility.
  bool _cartesian; ///< If true everything is shown in the global coordinate system in x-y (else in R-Phi).
  int _showModule; ///< This is the id of the module which is zoomed
  bool _moduleInterpolation; ///< If false only dots from the module with the measured point are considered.
  std::string _inputDistortionsCollectionName; ///< Name of the distortions collection in the database
  TPCDistortionsMapListener* _distortionsListener; ///< The listener getting the information from the data base.
  
};
} // namespace marlintpc
#endif // TESTTPCDISTORTIONSMAPPROCESSOR_H
