#include "LoadFieldFromConditionsDataProcessor.h"

// Marlin
#include <marlin/Global.h>
#include <marlin/ConditionsProcessor.h>

//MarliNTPC
#include "BFieldMapListener.h"

// gear
#include "gearimpl/GearMgrImpl.h"
#include "gearimpl/ConstantBField.h"
// C++
#include <iostream>

using namespace lcio ;
using namespace marlin ;
using namespace std;

namespace marlintpc
{

    LoadFieldFromConditionsDataProcessor aLoadFieldFromConditionsDataProcessor;


    LoadFieldFromConditionsDataProcessor::LoadFieldFromConditionsDataProcessor() : Processor("LoadFieldFromConditionsDataProcessor")
    {

        /* description of the processor which will be displayed in the
         * steering file autometicly generated by
         * marlin (use MarlinTPC -x (XML-format) or MarlinTPXC -l (text format))
         */
        _description = "LoadFieldFromConditionsDataProcessor loads field map(s) from lccd";

        /* register steering parameters: name, description, class-variable, default value
         * the type will by definded by the type of the default value
         * string, double, float and int are possible
         * plaese note to get a float you must cast in C++: (float) 1.3
         */

        registerProcessorParameter("BFieldMapCollectionName" ,
                                   "Name of the collection containing the magnetic field map",
                                   _bFieldColName,
                                   std::string("none")
                                  );

        registerOptionalParameter("CoilPositionX" ,
                                  "X value of the coil position in mm.",
                                  _coilPositionX,
                                  double(1503.62)
                                 );

        registerOptionalParameter("CoilPositionY" ,
                                  "Y value of the coil position in mm.",
                                  _coilPositionY,
                                  double(0.)
                                 );

        registerOptionalParameter("CoilPositionZ" ,
                                  "Z value of the coil position in mm.",
                                  _coilPositionZ,
                                  double(283.8)
                                 );

        registerOptionalParameter("CoilRotationX" ,
                                  "Value of coil rotation around x axis (in rad?)",
                                  _coilRotationX,
                                  double(0.)
                                 );

        registerOptionalParameter("CoilRotationY" ,
                                  "Value of coil rotation around y axis (in rad?)",
                                  _coilRotationY,
                                  double(0.)
                                 );

    }

    void LoadFieldFromConditionsDataProcessor::init()
    {
        printParameters() ;
        try
            {
                _bFieldListener = new BFieldMapListener(_bFieldColName, _coilPositionX, _coilPositionY, _coilPositionZ, _coilRotationX, _coilRotationY);
                gear::BField* const bfieldmap = _bFieldListener->getBFieldMap();
                Global::GEAR->setBField(bfieldmap);
            }
        catch(...)
            {
                streamlog_out(ERROR) << "Couldn't get the field map working." << endl;
            }
    }



    void LoadFieldFromConditionsDataProcessor::processRunHeader(lcio::LCRunHeader* run)
    {
        run->parameters().setValue(_processorName + "_revision", "$Rev: 2711 $");
        for(ProcParamMap::iterator i = _map.begin(); i != _map.end(); i++)
            {
                if(! i->second->isOptional() || i->second->valueSet())
                    {
                        run->parameters().setValue(_processorName + "_" + i->second->name(), i->second->value());
                    }
            }
    }

    void LoadFieldFromConditionsDataProcessor::processEvent(lcio::LCEvent *  /*evt*/)
    {
        // use like this:
        //~ gear::Vector3D here(1563.,0.,0.);
        //~ gear::Vector3D magF = Global::GEAR->getBField().at(here);
    }


    void LoadFieldFromConditionsDataProcessor::check(lcio::LCEvent *  /*evt*/)
    {
        // nothing here currently - could be extended to fill checkplots
    }


    void LoadFieldFromConditionsDataProcessor::end()
    {

    }

}
