#include "SelectNthEventsProcessor.h"
#include <iostream>
#include <cmath>
#include "marlin/Global.h"
#include "marlin/Exceptions.h"

#include <EVENT/LCCollection.h>
#include <EVENT/MCParticle.h>


using namespace lcio ;
using namespace marlin ;


SelectNthEventsProcessor aSelectNthEventsProcessor ;


SelectNthEventsProcessor::SelectNthEventsProcessor() : Processor("SelectNthEventsProcessor") {
  
  // modify processor description
  _description = "SelectNthEventsProcessor Processor selects every nth event from the run and throws a SkipEventException for all other ones. The selection can be inverted." ;
  

  registerProcessorParameter( "nEventToSelect", 
			      "Processor selects every nth event for processing by the chain, all other events are skipped, default: 1"  ,
			      _nEventToSelect ,
			      int(1) ) ;

  registerProcessorParameter( "SelectionOffset" , 
			      "Sets a selection offset so the selection starts with this event number; if this is greater than \"nEventToSelect\" it behaves as if the remainder of their fraction was set; default: 0",
			      _selectionOffset,
			      int(0) );

  registerProcessorParameter( "InvertSelection" , 
			      "Inverts the event selection if set to true, default: false"  ,
			      _invertSelection ,
			      false ) ;
}


void SelectNthEventsProcessor::init() { 

  // usually a good idea to
  printParameters() ;

  _nRun = 0 ;
  _nEvt = 0 ;
  _nEvtProc = 0 ;
  _nEvtSkip = 0 ;
  
}

void SelectNthEventsProcessor::processRunHeader( LCRunHeader*  /*run*/) { 

  _nRun++ ;
} 

void SelectNthEventsProcessor::processEvent( LCEvent * evt )
{ 
  _nEvt = evt->getEventNumber();
  bool skip ( (_nEvt-_selectionOffset)%_nEventToSelect!=0 );
  if( _invertSelection ) skip = !skip;

//  setReturnValue(!skip);
  if (skip)
  {
    ++_nEvtSkip;
    streamlog_out(DEBUG4) << "Skipping event " << _nEvt << std::endl;
    throw SkipEventException(this);
  }
  ++_nEvtProc;
}



void SelectNthEventsProcessor::check( LCEvent *  /*evt*/ ) { 

}


void SelectNthEventsProcessor::end(){ 
  streamlog_out(MESSAGE) << "Total Events: " << _nEvt << ",  Events processed: " << _nEvtProc << ",  Events skipped: " << _nEvtSkip << std::endl;
}

