#ifndef GasConditions_h
#define GasConditions_h 1

//C++
#include "iostream"
#include "string"

//LCIO
#include "lcio.h"
#include "EVENT/LCGenericObject.h"
#include "IMPL/LCGenericObjectImpl.h"

namespace tpcconddata{

class GasConditions ;

/** class to store the condition of the gas in a gaseous detector (e.g. TPC)
 */
class GasConditions : public EVENT::LCGenericObject {
  
public: 
  
    /** constructor wich only sets channel
     */
    GasConditions();

    /** 'Copy constructor' needed to interpret LCCollection read from file/database.
     */
    GasConditions(EVENT::LCObject* obj);
    
    /** Important for memory handling*/
    virtual ~GasConditions();

    /** return fraction of the mixtrue of the gas in of the following contents
     */
    float getMixture( int Content ) const;

    /// return the temperature of the gas in C
    float getTemperature() const;

    /// return the pressure of the gas in hPa
    float getPressure() const;

    /// return the over pressure of the sytem (to the athmosphere) in mbar
    float getOverPressure() const;

    /// return the flow of the gas in l/h
    float getFlow() const;

    /// return the water content in the gas in ppmV
    float getWaterContent() const;

    /// return the oxygen content in the gas in ppmV
    float getOxygenContent() const;

    void setMixture( int Content, float Value );
    void setTemperature( float Value );
    void setPressure( float Value );
    void setOverPressure( float Value );
    void setFlow( float Value );
    void setWaterContent( float Value );
    void setOxygenContent( float Value );

    static std::string getRevision();
    static std::string getDefaultColName();

    void print( std::ostream& os = std::cout ) const;

    IMPL::LCGenericObjectImpl* obj(){return myObj;}
    
    virtual int id(){ return myObj->id();}

    // ---- need to implement LCGenericObject interface:
    
    int getNInt() const;   
    int getNFloat() const; 
    int getNDouble() const;
    
    int getIntVal(int index) const;
    float getFloatVal(int index) const;
    double getDoubleVal(int index) const;

    bool isFixedSize() const {return false;};

    const std::string getTypeName() const {
	return "GasConditions";
    } 
    const std::string getDataDescription() const {
	return std::string("i:NContents,MixtureContents,")+
	    std::string("f:Temperture,Preasure,Overpreasure,Flow,")+
	    std::string("WaterContent,OxygenContent,MixtureFractions");
    }

protected:

    bool createObj;
    IMPL::LCGenericObjectImpl* myObj;
    
    int   getNMixtureContents() const;
    int   getContent( int Index ) const;
    float getFraction( int Index ) const;
    void  setFraction( int Index, float Fraction );
    void  addMixtureContents( int Content, float Fraction );

}; // class

    std::ostream &operator<<(std::ostream &os, const GasConditions &gc);
}

#endif
