/* TPCConditions.cc
 * $Id$ 
 */

#include "TPCConditions.h"

namespace tpcconddata{

std::string TPCConditions::getRevision(){
    return std::string("$Rev: 3427 $");
}

std::string TPCConditions::getDefaultColName(){
    return std::string("TPCConditions");
}

TPCConditions::TPCConditions(){
    setDriftVelocity(0);
    setTimeOffset(0);
    setLongDiffusionCoef(0);
    setLongDefocussing(0);
    setTransDiffusionCoef(0);
    setTransDefocussing(0);
    setAmplification(1);
    setADCtoPrimaryElectronsFactor(0);
}

TPCConditions::~TPCConditions() { /* no op*/  }

float TPCConditions::getDriftVelocity() const {
    return getFloatVal( 0 ) ; 
} 

float TPCConditions::getLongDiffusionCoef() const {
    return getFloatVal( 1 ) ;
}
 
float TPCConditions::getLongDefocussing() const {
    return getFloatVal( 2 ) ;
}
 
float TPCConditions::getTransDiffusionCoef() const {
    return getFloatVal( 3 ) ;
}

float TPCConditions::getTransDefocussing() const {
    return getFloatVal( 4 ) ;
}
    
float TPCConditions::getAmplification() const {
    return getFloatVal( 5 ) ;
} 

float TPCConditions::getADCtoPrimaryElectronsFactor() const {
    return getFloatVal( 6 ) ;
}
    
float TPCConditions::getTimeOffset() const {
    return getFloatVal( 7 ) ; 
} 

void TPCConditions::setDriftVelocity(float Value) {
    obj()->setFloatVal( 0 , Value ) ;
}

void TPCConditions::setLongDiffusionCoef(float Value) {
    obj()->setFloatVal( 1 , Value ) ;
}

void TPCConditions::setLongDefocussing(float Value) {
    obj()->setFloatVal( 2 , Value ) ;
}
    
void TPCConditions::setTransDiffusionCoef(float Value) {
    obj()->setFloatVal( 3 , Value ) ;
}

void TPCConditions::setTransDefocussing(float Value) {
    obj()->setFloatVal( 4 , Value ) ;
}

void TPCConditions::setAmplification(float Value) {
    obj()->setFloatVal( 5 , Value ) ;
}

void TPCConditions::setADCtoPrimaryElectronsFactor(float Value) {
    obj()->setFloatVal( 6 , Value ) ;
}

void TPCConditions::setTimeOffset(float Value) {
    obj()->setFloatVal( 7 , Value ) ;
}

void TPCConditions::print( std::ostream& os ) const {
    
    os<<"amplification:          "<<getAmplification()<<std::endl;
    os<<"drift velocity:         ";
    if (getDriftVelocity()>0){
	os<<getDriftVelocity()<<"mm/us";
    }
    else{
	os<<" not set";
    }
    os<<std::endl;
    os<<"time offset:         ";
    if (getTimeOffset()>0){
	os<<getTimeOffset()<<"us";
    }
    else{
	os<<" not set";
    }
    os<<std::endl;
    os<<"transversal diffusion:  ";
    if (getLongDiffusionCoef()>0){
	os<<"sigma^2 = "<<getLongDiffusionCoef()<<"^2* z (mm) + "<<getLongDefocussing()<<"^2 (mm^2)";
    }
    else{
	os<<"not set";
    }
    os<<std::endl;
    os<<"longitudinal diffusion: ";
    if (getTransDiffusionCoef()>0){
	os<<"sigma^2 = "<<getTransDiffusionCoef()<<"^2* z (mm) + "<<getTransDefocussing()<<"^2 (mm^2)";
    }
    else{
	os<<"not set";
    }
    os<<std::endl;
    os<<"ADC counts to primary electrons conversion factor: ";
    if (getADCtoPrimaryElectronsFactor() > 0){
        os << getADCtoPrimaryElectronsFactor();
    }
    else{
        os<< "not set.";
    }
    os << std::endl;
}

std::ostream &operator<<(std::ostream &os, const TPCConditions &tc) {
    tc.print(os);
    return os;
}

}
