#include "lcio.h"

#include "IO/LCWriter.h"
#include "EVENT/LCIO.h"
#include "DATA/LCFloatVec.h"
#include "DATA/LCIntVec.h"

#include "IMPL/LCEventImpl.h" 
#include "IMPL/LCRunHeaderImpl.h" 
#include "IMPL/LCCollectionVec.h"
#include "IMPL/SimCalorimeterHitImpl.h"
#include "IMPL/SimTrackerHitImpl.h"
#include "IMPL/MCParticleImpl.h" 
#include "IMPL/LCFlagImpl.h" 
#include "IMPL/LCTOOLS.h"
#include "IMPL/TPCHitImpl.h"
#include "UTIL/LCRelationNavigator.h"

#include <cstdlib>
#include <iostream>
#include <sstream>
#include <math.h>
//#include <gsl/gsl_rng.h>
//#include <gsl/gsl_randist.h>
#include <string>

#include <CLHEP/Random/RandGauss.h>

#define MASS 0.10566 // mass of muon

static const int NRUN = 1 ;

using namespace std ;
using namespace lcio ;

int main(int argc, char** argv ){
  
  try{
    if(argc != 5) 
    {
      cout << "Usage:"<<endl;
      cout<< "CreateMuonDataFile \"Number of Muonevents to be craeted\" \"tpc radius\" \"tpc length\" \"muonenergy.dat\" " << endl;
      exit(1);
    }
    
    int events=atoi(argv[1]);
    double radius=atof(argv[2])-1;
    double length=atof(argv[3]);
    const char *muonenergy=argv[4];
    
    //    gsl_rng *r=gsl_rng_alloc(gsl_rng_mt19937);
    
    // loop over runs
    for(int rn=0;rn<NRUN;rn++)
      {
	// create sio writer
	LCWriter* lcWrt = LCFactory::getInstance()->createLCWriter() ;
	char *FILEN = new char[256]; 
	sprintf(FILEN,"MuonEvents_N%i_R%.0f_L%.0f.LCIO",events,
		(radius+1),length); 

	if( rn==0 )
	  lcWrt->open( FILEN , LCIO::WRITE_NEW )  ;
	else
	  lcWrt->open( FILEN , LCIO::WRITE_APPEND )  ;
	
	
	
	LCRunHeaderImpl* runHdr = new LCRunHeaderImpl ; 
	runHdr->setRunNumber( rn ) ;
	string tpcName("PrototypeTPC") ;
	runHdr->setDetectorName( tpcName ) ;
	lcWrt->writeRunHeader( runHdr ) ;
	//read in file for muon energy
	double array[175715];
	FILE* fin = fopen(muonenergy,"r");
	if(! fin) {  
	  printf("File with muon energies could not be opened.\n");
	  return 1;
	}
	int evts=0;
	while(!feof(fin)) {                    
	  int error = fscanf(fin,"%lf", &array[evts] );
       if(error == EOF)
          cerr << "Trouble reading the muon energy input file." << endl;
	  evts++;                            
	  if(evts > 175715) {             
	    printf("Number of energies in file bigger than array! %i\n", 175715);
	    break;                            
	  }
	}
	fclose(fin);

	// EventLoop - create some events and write them to the file
	for(int i=0;i<events;i++)
	  {
	    // we need to use the implementation classes here 
	    LCEventImpl*  evt = new LCEventImpl() ;
	    
	    evt->setRunNumber(  rn   ) ;
	    evt->setEventNumber( i ) ;
	    evt->setDetectorName( tpcName ) ;
	    
	    // create and add some mc particles 
	    LCCollectionVec* mcVec = new LCCollectionVec( LCIO::MCPARTICLE )  ;
	    
	    //dice number of Muons in event
	    //	    double randgaus = gsl_ran_gaussian(r,10);
	    double randgaus = CLHEP::RandGauss::shoot(0, 10);
	    int numberofmuons=(int)(1+randgaus*drand48());
	    //int numberofmuons=1;
	    if(numberofmuons<0 || numberofmuons==0) numberofmuons=1;
	    if(i%100==0) cout<<"Event: "<<i<<" with "<<numberofmuons<<" muons"<<endl;
	    for(int j=0;j<numberofmuons;j++)
	      {
		//dice starting point
		double vx = 0.;//-radius/2+radius*drand48();
		double vy = 0.;//-sqrt(radius*radius-vx*vx);
		double vz = 0;//length*drand48();
		//dice energy
		//		int rand = (int)(175713*drand48());
		double energy=1.0;//array[rand];
		double P = sqrt(energy*energy-MASS*MASS);
		//split momentum in components
		double pz=0.01*P;
		//if(drand48()>0.5)
		//pz=drand48()*P/10;
		//else 
		//pz=-drand48()*P/10;
		double px=0.05*P;
		//if(vx<0)
		// px=sqrt(P*P-pz*pz)*0.4*drand48();
		//else
		//px=-sqrt(P*P-pz*pz)*0.4*drand48();
		double py=sqrt(P*P-px*px-pz*pz);
		
		MCParticleImpl* mom = new MCParticleImpl ;
		mom->setPDG( 13 ) ;
		float p0[3] = { static_cast<float>(px) , 
				static_cast<float>(py),
				static_cast<float>(pz) } ;
		mom->setMomentum( p0 ) ;
		mom->setMass( MASS ) ;
		mom->setGeneratorStatus (1);
		double vtx[3] = { vx, vy, vz } ;
		mom->setVertex (vtx);
		mom->setCharge(-1); 	  
		mcVec->push_back( mom ) ;
	      }
	    evt->addCollection( mcVec , "MCParticle" ) ;
	    
	    // write the event to the file
	    lcWrt->writeEvent( evt ) ;
	    
	    // dump the event to the screen 
	    //LCTOOLS::dumpEvent( evt ) ;
	    
	    // ------------ IMPORTANT ------------- !
	    // we created the event so we need to delete it ...
	    delete evt ;
	    
	  }//event loop
	
	delete runHdr ;
	
	lcWrt->close() ;
	delete lcWrt ;
	
      }// run loop
  } catch( Exception& ex)
    {
      cout << " an excpetion occured: " << endl ;
      cout << "  " << ex.what() << endl ;
      return 1 ;
    }
  
  return 0 ;
}
  
