#ifndef HELIX_H
#define HELIX_H
#include "gearimpl/Vectors.h"
#include "TVector3.h"
#include <vector>

using namespace gear;
using namespace std;

class Circle
{
public:
    Circle();

    Circle(double x0, double y0, double r);
    bool calcXPWithCircle(Circle& c, vector<Vector2D>& xp);    //calculate crossing point of two circles

    inline double   getR(){return _r;}
    inline void     setR(double r){_r = r;}

    inline Vector2D getCenter(){return _c;}
    inline void     setCenter(double x, double y){_c[0] = x; _c[1] = y;}

private:
    Vector2D _c;  // center of circle
    double   _r;  // radius of circle
};

class Helix : public Circle
{
public:
    Helix(double xc, double yc, double r, TVector3 X0, double lambda, double sign);
    Helix();

    bool calcXPWithLayer(Circle& c, vector<TVector3>& xp);

    inline void SetLambda (double l) {_lambda = l;}
    inline void SetSign   (int  s  ) {_sign = s; }

    inline void SetX0     (TVector3  x0) {_X0 = x0;}

    inline void SetLimit  (double minz, double maxz) { _MINZ = minz; _MAXZ = maxz;}

    //inline TVector3 getCenter(){return _c3;}

private:
    double _lambda;
    
    int      _sign;
    TVector3 _X0;

    double _MAXZ;
    double _MINZ;
};
#endif //HELIX_H
