# ifndef ChargeOnPadsVisualisationProcessor_h 
# define ChargeOnPadsVisualisationProcessor_h 1
          
//LCIO
#include <lcio.h>

//MARLIN
#include <marlin/Processor.h>

//GEAR
#include <gear/PadRowLayout2D.h>

namespace AIDA{
  class IHistogram2D;
}


namespace marlintpc{

/* The following comment is tranfered to doxygen documentation
 * which begins with a second asterix, or in case of on line comment an extra third slash 
 */

/** This processor fills an 2D root histogram which represents the
 *  pad plane. The amount of charge deposited in a pixel is given
 *  by the color. If used with a rectangular pad layout the number 
 *  of x-bins is given by the number of pads in the row with most pads.
 *  The number of y-bins is given by the number of rows. If used with
 *  circular pad layout the number of bins in x and y is given by to 
 *  times the outer radius of the pad plane divided by row height.
 *
 *  <h4>Input - Prerequisites</h4>
 *  Collection of TPCVoxels with charges on pads
 *
 *
 * @param InputCollectionName Name of input collection, SimTrackerHits
 * @param ResetHistogram Resets the histogram after each event if turned on (0)
 *
 * @author Thorsten Krautscheid
 */


  class ChargeOnPadsVisualisationProcessor : public marlin::Processor {
    
   public:
    
    virtual Processor*  newProcessor() { return new ChargeOnPadsVisualisationProcessor ; }
    
    
    ChargeOnPadsVisualisationProcessor() ;
    virtual ~ChargeOnPadsVisualisationProcessor(){}
    
    /** Called at the begin of the job before anything is read.
     *  Use to initialize the processor, e.g. book histograms.
     */
    virtual void init() ;
    
    /** Called for every run.
     */
    virtual void processRunHeader( EVENT::LCRunHeader* run ) ;
    
    /** Called for every event - the working horse.
     */
    virtual void processEvent( EVENT::LCEvent * evt ) ; 
     
  
    protected:
  
    std::string _inputCollectionName; ///<Inputcollection name
  
    //For internal bookkeeping only.
    int _nEvt;
    int _nRun;
  
    AIDA::IHistogram2D* _chargeOnPads;

    //gear pad layout
    gear::PadRowLayout2D const* _padLayout;

    //pad layout type.
    int _padLayoutType;

    int _resetHisto;
 
  };  

} //end of namespace marlintpc
#endif
