#include "ADCRawEventDisplayProcessor.h"

// C++
#include <sstream>

//AIDA
#include <AIDA/AIDA.h>
#include <marlin/AIDAProcessor.h>
#include <AIDA/IHistogram1D.h>

// LCIO
#include <EVENT/TrackerData.h>
#include <IMPL/LCCollectionVec.h>

#include <marlin/AIDAProcessor.h>

using namespace lcio ;
using namespace marlin ;
//using namespace tpcconddata;

namespace marlintpc{


    ADCRawEventDisplayProcessor anADCRawEventDisplayProcessor ;
    

    ADCRawEventDisplayProcessor::ADCRawEventDisplayProcessor() : 
      Processor("ADCRawEventDisplayProcessor") {
	/* description of the processor which will be displayed in the
	 * steering file automatically generated by 
	 * "marlin -x"
	 */
	_description = "Attention: ADCRawEventDisplayProcessor creates one histogram per channel and event!" ;	
	
	/* register steering parameters: name, description, class-variable, default value
	 * the type will by definded by the type of the default value
	 * string, double, float and int are possible
	 * please note to get a float you must cast in C++: (float) 1.3 
	 */
	
	registerInputCollection( LCIO::TRACKERDATA,
				 "InputCollectionName" , 
				 "Name of the TrackerData input collection  (default AltroData)" ,
				 _inputColName ,
				 std::string("AltroData") ) ;
	
	registerProcessorParameter( "nTimeSamples" , 
				    "Number of time samples (bins per histogram). Default: 1024"  ,
				    _nTimeSamples ,
				    (int) 1024 ) ;

	registerProcessorParameter( "deleteHistograms" , 
				    "Set true if running with zero suppressed data. Otherwise histograms will only be reset (emptied). Default: false"  ,
				    _deleteHistograms,
				    false ) ;
	
    }
    
    
    void ADCRawEventDisplayProcessor::init() { 
	
	
    }
    
    void ADCRawEventDisplayProcessor::processRunHeader( LCRunHeader* run) { 
	
      
      run->parameters().setValue( _processorName + "_revision", "$Rev: 5472 $");

      for ( ProcParamMap::iterator i = _map.begin(); i != _map.end(); i++ ) {
	if ( ! i->second->isOptional() || i->second->valueSet() ) {
	  run->parameters().setValue( _processorName + "_" + i->second->name(), 
				      i->second->value() );
	}
      }

    } 
    
    void ADCRawEventDisplayProcessor::processEvent( LCEvent * evt )
    { 

	streamlog_out(MESSAGE1)<<"processEvent( "<<evt->getEventNumber()<<" )"<<std::endl;

	LCCollectionVec* TrackerDataCol;
	try{
	    TrackerDataCol = dynamic_cast<LCCollectionVec*>(evt->getCollection( _inputColName ));
	}
	catch(lcio::DataNotAvailableException &)
	{
	    streamlog_out(WARNING0) << "Empty event: Collection  " << _inputColName << " not found in event "
				    <<evt->getEventNumber()<<std::endl;
	    return;
	}
	
	// clear all histograms in the map
//	if (_deleteHistograms)
//	{
	    // ask the aida factory to delete all histograms
	    for (std::map<std::pair<int,int>, AIDA::IHistogram1D*>::iterator histoiter = rawHistos.begin();
		 histoiter != rawHistos.end(); histoiter++ )
	    {
		AIDAProcessor::histogramFactory(this)->destroy(histoiter->second);
	    }
	    // empty the map (n.b. the pointers are invalid anyhow)
	    rawHistos.clear();
//	}
//	else
//	{
//	    for (std::map<int, AIDA::IHistogram1D*>::iterator histoiter = rawHistos.begin();
//		 histoiter < rawHistos.end(); histoiter++ )
//	    {
//		histoiter->second->reset();
//	    }	    
//	}

	
	

//	std::stringstream histoTitle;
//	
//	sEvt << evt->getEventNumber();
//	
//	std::string histoname = "TimePixEvt" + sEvt.str();
//	
//	AIDAProcessor::histogramFactory(this)->destroy(eventHisto);
//	    
//	eventHisto = AIDAProcessor::histogramFactory(this)->
//	    createHistogram2D( histoname.c_str(), histoname.c_str(), 
//			       256, 0, 256, 256, 0, 256);

	    streamlog_out(DEBUG4)<<"loop over "<<TrackerDataCol->getNumberOfElements()
			     <<" Elements in inputCol"<<std::endl;
	
	for ( LCCollectionVec::const_iterator dataIter = TrackerDataCol->begin();
	      dataIter < TrackerDataCol->end(); dataIter++ )
	{
	    TrackerData* data = 
		dynamic_cast<TrackerData*>( *dataIter ) ;
	    
	    // have the readout group first, then the pad ID	    
	    std::pair<int,int> globalCellID(data->getCellID1(), data->getCellID0());
	    
	    std::stringstream histoTitle;
	    histoTitle << "event"<< evt->getEventNumber()<< "AltroData_"
		       << "_" << globalCellID.first << "_"<< globalCellID.second;

	    AIDA::IHistogram1D* eventHisto;
	    if ( rawHistos.find(globalCellID) ==  rawHistos.end() )
	    { // the entry does not exist yet, create it
		eventHisto = AIDAProcessor::histogramFactory(this)->
		    createHistogram1D( histoTitle.str(), histoTitle.str(), 
				       _nTimeSamples, 0, _nTimeSamples);
		rawHistos[globalCellID] = eventHisto;
	    }
	    else
	    {
		eventHisto = rawHistos[globalCellID];
	    }
	  
	    FloatVec const & ADCValues  = data->getChargeValues();
	    
	    for ( size_t i = 0; i <  ADCValues.size(); i++ )
	    {
		eventHisto->fill( data->getTime() + i , ADCValues[i] );
	    }	    


	}// for dataIter
	
    streamlog_out(DEBUG4)<<"end of processEvent( "<<evt->getEventNumber()<<" )"<<std::endl;
}
    
    
//FIXME: not needed: void ADCRawEventDisplayProcessor::check( LCEvent * evt ) {
//
//
//
//    }

    
    
    void ADCRawEventDisplayProcessor::end(){ 
      /*	
      streamlog_out(MESSAGE4) << "ADCRawEventDisplayProcessor::end()  " << name() 
		<< " processed " << evt->getEventNumber() 
		<< " events in " << _nRun << " runs "
		<< std::endl ;
      */	

    }
    
}
