#include "DumpSimTrackerHitsProcessor.h"

//C++
#include <iostream>
using namespace std;

//LCIO
#include <IMPL/LCCollectionVec.h>
#include <EVENT/SimTrackerHit.h>
#include <EVENT/MCParticle.h>
//#include "UTIL/LCRelationNavigator.h"

using namespace lcio ;
using namespace marlin ;

namespace marlintpc
{

DumpSimTrackerHitsProcessor aDumpSimTrackerHitsProcessor;
    
DumpSimTrackerHitsProcessor::DumpSimTrackerHitsProcessor(): Processor("DumpSimTrackerHitsProcessor")
{
    // register steering parameters: name, description, class-variable, default value
    registerInputCollection(    LCIO::SIMTRACKERHIT,
				"InputHitsCollection",
				"Name of the input SimTrackerHits collection, default: TPCSimHits",
				_inputColName ,
				std::string("TPCSimHits") ) ;

    registerProcessorParameter( "PrintMCParticleInfo",
				"Flag whether information about the particle which caused this hit should be printed. Default: false",
				_printMCParticleInfo ,
				bool(false) ) ;

    // modify processor description
    _description = " Dump the coordinates of all SimTrackerHits in the input collection." ;
}
    
void DumpSimTrackerHitsProcessor::processEvent( LCEvent * evt ) 
{ 
    //read input data
    cout << "# Event: " << evt->getEventNumber() << endl;

    // we need a LCCollectionVec to use iterators, as LCCollectios is o not a std::vector
    LCCollectionVec* inputCollection=0;
    try{
	inputCollection = dynamic_cast<LCCollectionVec*>(evt->getCollection( _inputColName )) ;
    }
    catch(DataNotAvailableException &e)
    {
	// no input data
	// just continue ...
	return;
    }


    if( inputCollection != 0 )
    {
	cout << "# hitNumber \t x \t y \t z" << endl;
	int hitNr=0;

	for(LCCollectionVec::iterator hitIter = inputCollection->begin() ; 
	    hitIter < inputCollection->end(); hitIter++ )
	{
	    SimTrackerHit *hit = dynamic_cast<SimTrackerHit *>(*hitIter);
	    if (hit==0)
	    {
		throw DataNotAvailableException("DumpSimTrackerHitsProcessor: Input collection does not contain SimTrackerHits");
	    }

	    const double *pos = hit->getPosition();
	    cout << hitNr++<<"\t "<<pos[0]<<"\t "
		 <<pos[1]<<"\t "<<pos[2]<< endl;

	    if (_printMCParticleInfo)
	    {
	      MCParticle * mcpart=  hit->getMCParticle();

	      std::cout << "# MCParticle ID " << mcpart->getPDG() <<" at "<<  mcpart->getVertex()[0] 
			                                          <<" : " <<  mcpart->getVertex()[1]
                                                                  <<" : " <<  mcpart->getVertex()[2]
			<< "\t" //std::endl
			<< "E = " << mcpart->getEnergy() << " ; q = " << mcpart->getCharge()
			<< "\t" //std::endl
			<< "genStat " << mcpart->getGeneratorStatus() 
			<< " ; simStat " << mcpart->getSimulatorStatus() 
			<< std::endl;	
	      
	    }
	}
    }// if( inputCollection != 0 )

}// processEvent


}// namespace 
