#include "HitDistributor.h"
#include <VoxelTPC.h>

#include <gearxml/GearXML.h>
#include <gear/GearMgr.h>
#include <gear/GEAR.h>

#include <gear/TPCParameters.h>
#include <gear/TPCModule.h>

#include <IMPL/SimTrackerHitImpl.h>
#include <IMPL/TrackerPulseImpl.h>
#include <IMPL/TrackerDataImpl.h>
#include <IMPL/TrackerHitImpl.h>
#include <IMPL/LCFlagImpl.h>


//LCIO
#include "lcio.h"
#include "IO/LCWriter.h"
#include "EVENT/LCIO.h"
#include "IMPL/LCEventImpl.h" 
#include "IMPL/LCRunHeaderImpl.h" 
#include "IMPL/LCCollectionVec.h"
#include "IMPL/LCTOOLS.h"

#include "streamlog/streamlog.h"

#include <iostream>

using namespace std;
using namespace marlintpc;
using namespace gear;
using namespace lcio;


int main()
{
  streamlog::out.init( std::cout , "testSimHitDistributor") ;
  streamlog::out.addLevelName<streamlog::DEBUG>() ;

  
  // we hardcode the gear file name because we hard coded against it's content.
  std::string const gearFilename("gearfile_strips.xml");

  string outFileName="SimHitTest.slcio";

  // read pad layout from gear
  GearMgr* gearMgr;
  try{
    GearXML gearXML(gearFilename);

    gearMgr = gearXML.createGearMgr();
  }
  catch(gear::ParseException &)
  {
    std::cout << "Error: Gear file \"" << gearFilename << "\" not found." << std::endl;
    return -1;
  }

  TPCParameters const& tpcParameters = gearMgr->getTPCParameters();

  HitDistributor *myHitDistributor = new HitDistributor( &tpcParameters, 10 );

  // open outfile			     
  try{
    // create sio writer
    LCWriter* lcWrt = LCFactory::getInstance()->createLCWriter() ;
    
    try{  lcWrt->open( outFileName, LCIO::WRITE_NEW ) ;
    }
    catch( IOException& e ){    
      cout << e.what() << endl ;
      return 0 ;
    }
    
    // create runheader and write it
    // the information should come from a file
    // or be given interactively in a future version
    LCRunHeaderImpl* runHdr = new LCRunHeaderImpl ; 
    int runNr=0;
    runHdr->setRunNumber( runNr) ;
    runHdr->setDetectorName ("LPTPC");      
    lcWrt->writeRunHeader( runHdr ) ; 
    int ev=1;	
    //just one event
    //for (int ev=0; ev<100;ev++)
      {
    // create new lcio event
    LCEventImpl*  evt = new LCEventImpl() ;
	
    evt->setRunNumber(  0   ) ;
    evt->setEventNumber( ev ) ;
      
    // we want to store a collection of trackerRawData
    LCCollectionVec* pulseVec = new LCCollectionVec( LCIO::TRACKERPULSE )  ;
    LCCollectionVec* rawVec = new LCCollectionVec( LCIO::TRACKERDATA )  ;
    LCCollectionVec* hits = new LCCollectionVec( LCIO::TRACKERHIT )  ;
    //hit loop
    double pos_start=1300;

    lcio::LCFlagImpl trkFlag(0) ;
    trkFlag.setBit(lcio::LCIO::TRAWBIT_ID1) ;
    pulseVec->setFlag(trkFlag.getFlag());
    for(int h=0;h<100;h++)
      {
	SimTrackerHitImpl *hit =new SimTrackerHitImpl();
	double pos[3]={pos_start+h*20,50,550};
	hit->setPosition(pos);
	hit->setTime(100);
	hit->setEDep(1000);
	std::cout<<"HIT: "<<h<<" "<<pos[0]<<" "<<pos[1]<<" "<<pos[2]<<std::endl;
	double sigmaTrans=0.4;
	double sigmaLong=0;
	if( tpcParameters.isInsideModule(pos[0],pos[1]) )
	  {
	    const gear::TPCModule & tpcModule = tpcParameters.getNearestModule(pos[0],pos[1] );
	    
	    std::list<TPCVoxel> voxelList = myHitDistributor->distributeHit( hit, sigmaTrans, sigmaLong);
	    std::cout<<"Found "<<voxelList.size()<<" voxels"<<std::endl;
	    
	    double chargeSum=0;
	    int count=0;
	    for ( std::list<TPCVoxel>::iterator voxelIter = voxelList.begin() ;
		  voxelIter != voxelList.end() ; ++voxelIter ) 
	      {
		std::cout<<count<<" "<<tpcModule.getPadCenter(voxelIter->getPadIndex())[0]<<" "
			 <<tpcModule.getPadCenter(voxelIter->getPadIndex())[1]<<" "
			 <<voxelIter->getModuleID()<<" "
			 <<tpcModule.getRowNumber(voxelIter->getPadIndex())<<" "
			 <<tpcModule.getPadNumber(voxelIter->getPadIndex())<<" "
			 <<voxelIter->getTimeIndex()<<" "
			 <<voxelIter->getCharge()<<std::endl;
		chargeSum+=voxelIter->getCharge();
		//create a fake pulse for event display
		TrackerPulseImpl* newPulse = new TrackerPulseImpl();
		newPulse->setCellID0(voxelIter->getPadIndex());
		newPulse->setCellID1(voxelIter->getModuleID());
		newPulse->setCharge(voxelIter->getCharge());
		newPulse->setTime(200);
		
		float data[] = { float(voxelIter->getCharge())  };
		EVENT::FloatVec dataVec(data,data + sizeof(data) / sizeof(float));
		IMPL::TrackerDataImpl *trackerData=new TrackerDataImpl();
		trackerData->setCellID0(voxelIter->getPadIndex());
		trackerData->setCellID1(voxelIter->getModuleID());
		trackerData->setTime(200); 
		trackerData->setChargeValues(dataVec);
		rawVec->addElement(trackerData);
		newPulse->setTrackerData(trackerData);
		
		pulseVec->addElement(newPulse);
		count++;
	      }
	    std::cout<<"Total charge on pads: "<<chargeSum<<std::endl;
	    double integral=hit->getEDep()/4 *((erf((4*sigmaTrans)/(sqrt(2.)*sigmaTrans))
						-erf((-4*sigmaTrans)/(sqrt(2.)*sigmaTrans)))
					       * (erf((4*sigmaTrans)/(sqrt(2.)*sigmaTrans))	
						  -erf((-4*sigmaTrans)/(sqrt(2.)*sigmaTrans))));
	    std::cout<<"Total charge of integral: "<<integral<<std::endl;
	    TrackerHitImpl *rhit =new TrackerHitImpl();
	    rhit->setPosition(pos);
	    rhit->setTime(100);
	    rhit->setEDep(1000);
	    rhit->setCellID0((tpcParameters.getNearestPad( pos[0],pos[1])).getModuleID());
	    rhit->setCellID1((tpcParameters.getNearestPad( pos[0],pos[1])).getPadIndex());

	    hits->addElement(rhit);
	  }
      }
    // add the rawData collection to the event
    evt->addCollection( pulseVec , "TPCPulses" ) ;
    evt->addCollection( rawVec , "TPCData" ) ;
    evt->addCollection( hits , "TPCHits" ) ;
    // write the event to the file
    lcWrt->writeEvent( evt ) ;
    // dump the event to the screen 
    // LCTOOLS::dumpEvent( evt ) ;
    
    // ------------ IMPORTANT ------------- !
    // we created the event so we need to delete it ...
    delete evt ;
    // -------------------------------------
    }//ev loop
    
    lcWrt->close() ;
    
  } catch( std::exception& ex){
    
    cout << " an excpetion occured: " << endl ;
    cout << "  " << ex.what() << endl ;
    return 1 ;
  }
  
  return 0;

}
// // forward declaration so we can declare it friend
// // of TestableDummyDevice.
// class DummyDeviceTest;

// /** The TestableDummyDevice is derived from 
// * DummyDevice to get access to the protected members.
// * This is done by declaring DummyDeviceTest as a friend.
// */
// class TestableDummyDevice : public DummyDevice
// {
// friend class DummyDeviceTest;
// };

// class DummyDeviceTest{
//void testSomething();
